@@ -151,6 +151,16 @@
   PSrectfill(x - 1, y - 1, 2, 2);
 }
 
+- (BOOL) acceptsFirstMouse: (NSEvent *)theEvent
+{
+  return YES;
+}
+
+- (BOOL) acceptsFirstResponder
+{
+  return NO;
+}
+
 - (void) mouseDown: (NSEvent *)theEvent
 {
   unsigned int eventMask = NSLeftMouseDownMask | NSLeftMouseUpMask
@@ -158,7 +168,7 @@
   NSPoint point = [self convertPoint: [theEvent locationInWindow] 
 			fromView: nil];
   NSEventType eventType = [theEvent type];
-  NSEvent *presentEvent;
+  NSEvent *presentEvent = theEvent;
 
   float new_hue, new_saturation;
   float old_x, old_y;
@@ -330,7 +340,7 @@
   [s setAutoresizingMask: NSViewHeightSizable];
   [s setCell: [[GSColorSliderCell alloc] init]];
   [(GSColorSliderCell *)[s cell] _setColorSliderCellMode: 10];
-  [s setContinuous: NO];
+  [s setContinuous: YES];
   [s setMinValue: 0.0];
   [s setMaxValue: 1.0];
   [s setTarget: self];
