@@ -29,13 +29,13 @@
 #ifndef _GNUstep_H_NSAnimation_
 #define _GNUstep_H_NSAnimation_
 
-#include <GNUstepBase/GSVersionMacros.h>
+#import <GNUstepBase/GSVersionMacros.h>
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
 
-#include <Foundation/NSObject.h>
-#include <AppKit/AppKitDefines.h>
-#include <GNUstepGUI/GSAnimator.h>
+#import <Foundation/NSObject.h>
+#import <AppKit/AppKitDefines.h>
+#import <GNUstepGUI/GSAnimator.h>
 
 @class NSString;
 @class NSArray;
@@ -45,22 +45,24 @@
 
 /** These constants describe the curve of an animation,
     that is, the relative speed of an animation from start to finish. */
-typedef enum _NSAnimationCurve
+enum _NSAnimationCurve
 {
   NSAnimationEaseInOut = 0, // default
   NSAnimationEaseIn,
   NSAnimationEaseOut,
   NSAnimationLinear,
   NSAnimationSpeedInOut // GNUstep only
-} NSAnimationCurve;
+};
+typedef NSUInteger NSAnimationCurve;
 
 /** These constants indicate the blocking mode of an NSAnimation object when it is running. */
-typedef enum _NSAnimationBlockingMode
+enum _NSAnimationBlockingMode
 {
   NSAnimationBlocking,
   NSAnimationNonblocking,
   NSAnimationNonblockingThreaded
-} NSAnimationBlockingMode;
+};
+typedef NSUInteger NSAnimationBlockingMode;
 
 typedef float NSAnimationProgress;
 
@@ -97,6 +99,9 @@ APPKIT_EXPORT NSString *NSAnimationProgressMarkNotification;
 /** Key used in the [NSNotification-userInfo] disctionary to access the current progress mark. */
 APPKIT_EXPORT NSString *NSAnimationProgressMark;
 
+APPKIT_EXPORT NSString *NSAnimationTriggerOrderIn; 
+APPKIT_EXPORT NSString *NSAnimationTriggerOrderOut; 
+
 /**
  * Objects of the NSAnimation class manage the timing and progress of
  * animations in the user interface. The class also lets you link together
