@@ -33,13 +33,13 @@
 
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-#include <Foundation/NSAttributedString.h>
-#include <Foundation/NSRange.h>
-#include <AppKit/NSFontManager.h>
+#import <Foundation/NSAttributedString.h>
+#import <Foundation/NSRange.h>
+#import <AppKit/NSFontManager.h>
 // for NSWritingDirection
-#include <AppKit/NSParagraphStyle.h>
-#include <AppKit/NSText.h>
-#include <AppKit/AppKitDefines.h>
+#import <AppKit/NSParagraphStyle.h>
+#import <AppKit/NSText.h>
+#import <AppKit/AppKitDefines.h>
 
 @class NSTextAttachment;
 @class NSFileWrapper;
@@ -190,10 +190,10 @@ enum
 - (BOOL) containsAttachments;
 - (NSDictionary*) fontAttributesInRange: (NSRange)range;
 - (NSDictionary*) rulerAttributesInRange: (NSRange)range;
-- (unsigned) lineBreakBeforeIndex: (unsigned)location
-		      withinRange: (NSRange)aRange;
-- (NSRange) doubleClickAtIndex: (unsigned)location;
-- (unsigned) nextWordFromIndex: (unsigned)location forward: (BOOL)isForward;
+- (NSUInteger) lineBreakBeforeIndex: (NSUInteger)location
+                        withinRange: (NSRange)aRange;
+- (NSRange) doubleClickAtIndex: (NSUInteger)location;
+- (NSUInteger) nextWordFromIndex: (NSUInteger)location forward: (BOOL)isForward;
 
 - (id) initWithRTF: (NSData*)data documentAttributes: (NSDictionary**)dict;
 - (id) initWithRTFD: (NSData*)data documentAttributes: (NSDictionary**)dict;
@@ -232,8 +232,8 @@ enum
             options: (NSDictionary *)options
  documentAttributes: (NSDictionary **)dict;
 
-- (unsigned) lineBreakByHyphenatingBeforeIndex: (unsigned)location
-                                   withinRange: (NSRange)aRange;
+- (NSUInteger) lineBreakByHyphenatingBeforeIndex: (NSUInteger)location
+                                     withinRange: (NSRange)aRange;
 #endif
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
 - (NSData *) dataFromRange: (NSRange)range
@@ -251,14 +251,14 @@ enum
 documentAttributes: (NSDictionary **)dict
              error: (NSError **)error;
 
-- (NSRange) itemNumberInTextList: (NSTextList *)list
-                         atIndex: (unsigned)location;
+- (NSInteger) itemNumberInTextList: (NSTextList *)list
+                           atIndex: (NSUInteger)location;
 - (NSRange) rangeOfTextBlock: (NSTextBlock *)block
-                     atIndex: (unsigned)location;
+                     atIndex: (NSUInteger)location;
 - (NSRange) rangeOfTextList: (NSTextList *)list
-                    atIndex: (unsigned)location;
+                    atIndex: (NSUInteger)location;
 - (NSRange) rangeOfTextTable: (NSTextTable *)table
-                     atIndex: (unsigned)location;
+                     atIndex: (NSUInteger)location;
 #endif
 @end
 
