@@ -11,19 +11,19 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor,
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
    Boston, MA 02110-1301, USA.
 */ 
 
@@ -31,20 +31,21 @@
 #define _GNUstep_H_NSColorPanel
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSBox.h>
-#include <AppKit/NSButton.h>
-#include <AppKit/NSColorPicking.h>
-#include <AppKit/NSColorWell.h>
-#include <AppKit/NSMatrix.h>
-#include <AppKit/NSNibDeclarations.h>
-#include <AppKit/NSPanel.h>
-#include <AppKit/NSSlider.h>
-#include <AppKit/NSSplitView.h>
+#import <AppKit/AppKitDefines.h>
+#import <AppKit/NSApplication.h>
+#import <AppKit/NSColorPicking.h>
+#import <AppKit/NSColorWell.h>
+#import <AppKit/NSNibDeclarations.h>
+#import <AppKit/NSPanel.h>
 
-@class NSView;
+@class NSBox;
+@class NSButton;
 @class NSColorList;
 @class NSEvent;
+@class NSMatrix;
+@class NSSlider;
+@class NSSplitView;
+@class NSView;
 
 enum {
   NSGrayModeColorPanel,
@@ -53,7 +54,8 @@ enum {
   NSHSBModeColorPanel,
   NSCustomPaletteModeColorPanel,
   NSColorListModeColorPanel,
-  NSWheelModeColorPanel 
+  NSWheelModeColorPanel,
+  NSCrayonModeColorPanel 
 };
 
 enum {
@@ -64,7 +66,8 @@ enum {
   NSColorPanelCustomPaletteModeMask = 16,
   NSColorPanelColorListModeMask = 32,
   NSColorPanelWheelModeMask = 64,
-  NSColorPanelAllModesMask = 127 
+  NSColorPanelCrayonModeMask = 128,
+  NSColorPanelAllModesMask = 255
 };
 
 @interface NSApplication (NSColorPanel)
