@@ -31,7 +31,7 @@
 
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-#include <AppKit/AppKitDefines.h>
+#import <AppKit/AppKitDefines.h>
 
 @class NSResponder;
 @class NSString;
@@ -42,11 +42,15 @@ typedef	enum {
   NSMacintoshInterfaceStyle = 2,
   NSWindows95InterfaceStyle = 3,
 
-/*
- * GNUstep specific. Blame: Michael Hanni.
- */ 
+  /*
+   * GNUstep specific. Blame: Michael Hanni.
+   */ 
+  GSWindowMakerInterfaceStyle = 4,
 
-  GSWindowMakerInterfaceStyle = 4
+  /*
+   * GNUstep specific style for native menus.
+   */
+  GSNativeInterfaceStyle = 5
 
 } NSInterfaceStyle;
 
