@@ -34,8 +34,8 @@
 #define _GNUstep_H_NSMenuItem
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <AppKit/NSUserInterfaceValidation.h>
+#import <Foundation/NSObject.h>
+#import <AppKit/NSUserInterfaceValidation.h>
 
 @class NSAttributedString;
 @class NSString;
@@ -48,7 +48,7 @@
  * placed in a menu as a menu item.  The [NSMenuItem] class provides
  * a reference implementation suitable for most uses.
  */
-@protocol NSMenuItem <NSCopying, NSCoding, NSObject>
+@protocol NSMenuItem <NSValidatedUserInterfaceItem, NSCopying, NSCoding, NSObject>
 
 /**
    <p> Returns a seperator.   This is just a blank menu item which serves 
@@ -293,7 +293,7 @@
    Sets the tag of the reciever as <var>anInt</var>.
    </p>
  */
-- (void) setTag: (int)anInt;
+- (void) setTag: (NSInteger)anInt;
 
 /**
    <p>
@@ -332,7 +332,7 @@
    Returns the tag of the receiver.
    </p>
  */
-- (int) tag;
+- (NSInteger) tag;
 
 /**
    <p>
