@@ -30,9 +30,9 @@
 #ifndef _GNUstep_H_NSTableView
 #define _GNUstep_H_NSTableView
 
-#include <AppKit/NSControl.h>
-#include <AppKit/NSDragging.h>
-#include <AppKit/NSUserInterfaceValidation.h>
+#import <AppKit/NSControl.h>
+#import <AppKit/NSDragging.h>
+#import <AppKit/NSUserInterfaceValidation.h>
 
 @class NSArray;
 @class NSIndexSet;
@@ -104,6 +104,7 @@ typedef enum _NSTableViewColumnAutoresizingStyle
   BOOL               _autosaveTableColumns;
   NSString          *_autosaveName;
   BOOL              _verticalMotionDrag;
+  NSArray           *_sortDescriptors;
 
   /*
    * Ivars Acting as Cache 
@@ -237,6 +238,9 @@ typedef enum _NSTableViewColumnAutoresizingStyle
 /* Layout */
 - (NSRect) rectOfColumn: (int)columnIndex;
 - (NSRect) rectOfRow: (int)rowIndex;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (NSIndexSet *) columnIndexesInRect: (NSRect)aRect;
+#endif
 - (NSRange) columnsInRect: (NSRect)aRect;
 - (NSRange) rowsInRect: (NSRect)aRect;
 - (int) columnAtPoint: (NSPoint)aPoint;
@@ -315,8 +319,10 @@ typedef enum _NSTableViewColumnAutoresizingStyle
 #endif
 
 /* sorting */
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 - (void) setSortDescriptors: (NSArray *)array;
 - (NSArray *) sortDescriptors;
+#endif
 
 @end /* interface of NSTableView */
 
@@ -372,12 +378,16 @@ APPKIT_EXPORT NSString *NSTableViewSelectionIsChangingNotification;
 @interface NSObject (NSTableViewDelegate)
 
 - (BOOL) selectionShouldChangeInTableView: (NSTableView *)aTableView;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
+- (void) tableView: (NSTableView*)tableView
+  sortDescriptorsDidChange: (NSArray *)oldSortDescriptors;
 - (void) tableView: (NSTableView*)tableView
 didClickTableColumn: (NSTableColumn *)tableColumn;
 - (void) tableView: (NSTableView*)tableView
 didDragTableColumn: (NSTableColumn *)tableColumn;
 - (void) tableView: (NSTableView*)tableView
 mouseDownInHeaderOfTableColumn: (NSTableColumn *)tableColumn;
+#endif
 - (BOOL)tableView: (NSTableView *)aTableView 
 shouldEditTableColumn: (NSTableColumn *)aTableColumn 
 	      row: (int)rowIndex;
