@@ -35,9 +35,10 @@
 
 #ifndef _GNUstep_H_NSWindow
 #define _GNUstep_H_NSWindow
+#import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/NSResponder.h>
+#import <AppKit/NSGraphicsContext.h>
+#import <AppKit/NSResponder.h>
 
 @class NSArray;
 @class NSData;
@@ -170,9 +171,13 @@ APPKIT_EXPORT NSSize NSTokenSize;
   id            _firstResponder;
   id            _futureFirstResponder;
   NSView        *_initialFirstResponder;
+PACKAGE_SCOPE
   id            _delegate;
+@protected
   id            _fieldEditor;
-  id            _lastView;
+  id            _lastLeftMouseDownView;
+  id            _lastRightMouseDownView;
+  id            _lastOtherMouseDownView;
   id            _lastDragView;
   int           _lastDragOperationMask;
   int           _windowNum;
@@ -186,12 +191,16 @@ APPKIT_EXPORT NSSize NSTokenSize;
   NSString      *_miniaturizedTitle;
   NSImage       *_miniaturizedImage;
   NSString      *_windowTitle;
+PACKAGE_SCOPE
   NSPoint       _lastPoint;
+@protected
   NSBackingStoreType _backingType;
   unsigned      _styleMask;
   int           _windowLevel;
+PACKAGE_SCOPE
   NSRect        _rectNeedingFlush;
   NSMutableArray *_rectsBeingDrawn;
+@protected
   unsigned	_disableFlushWindow;
   
   NSWindowDepth _depthLimit;
@@ -203,7 +212,9 @@ APPKIT_EXPORT NSSize NSTokenSize;
   NSWindow       *_parent;
   NSCachedImageRep *_cachedImage;
   NSPoint        _cachedImageOrigin;
+  NSWindow       *_attachedSheet;
 
+PACKAGE_SCOPE
   struct GSWindowFlagsType {
     unsigned	accepts_drag:1;
     unsigned	is_one_shot:1;
@@ -245,7 +256,7 @@ APPKIT_EXPORT NSSize NSTokenSize;
     unsigned ignores_mouse_events: 1;
     unsigned preserves_content_during_live_resize: 1;
   } _f;
- 
+@protected 
   NSToolbar     *_toolbar;
   void          *_reserved_1;
 }
