@@ -11,29 +11,31 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #ifndef _GNUstep_H_NSWorkspace
 #define _GNUstep_H_NSWorkspace
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSGeometry.h>
-#include <AppKit/AppKitDefines.h>
+#import <Foundation/NSObject.h>
+#import <Foundation/NSGeometry.h>
+#import <AppKit/AppKitDefines.h>
 
 @class NSString;
+@class NSNumber;
 @class NSArray;
 @class NSMutableArray;
 @class NSMutableDictionary;
@@ -42,6 +44,26 @@
 @class NSView;
 @class NSURL;
 
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
+enum {
+  NSWorkspaceLaunchAndPrint                  = 0x2,
+  NSWorkspaceLaunchInhibitingBackgroundOnly  = 0x80,
+  NSWorkspaceLaunchWithoutAddingToRecents    = 0x100,
+  NSWorkspaceLaunchWithoutActivation         = 0x200,
+  NSWorkspaceLaunchAsync                     = 0x10000,
+  NSWorkspaceLaunchAllowingClassicStartup    = 0x20000,
+  NSWorkspaceLaunchPreferringClassic         = 0x40000,
+  NSWorkspaceLaunchNewInstance               = 0x80000,
+  NSWorkspaceLaunchAndHide                   = 0x100000,
+  NSWorkspaceLaunchAndHideOthers             = 0x200000,
+  NSWorkspaceLaunchDefault                   = NSWorkspaceLaunchAsync | NSWorkspaceLaunchAllowingClassicStartup
+};
+
+typedef NSUInteger NSWorkspaceLaunchOptions;
+#endif
+
+
 @interface NSWorkspace : NSObject
 {
   NSMutableDictionary	*_iconMap;
@@ -70,7 +92,7 @@
   withApplication: (NSString*)appName
     andDeactivate: (BOOL)flag;
 - (BOOL) openTempFile: (NSString*)fullPath;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (BOOL) openURL: (NSURL*)url;
 #endif
 
@@ -101,7 +123,7 @@
 - (NSImage*) iconForFile: (NSString*)fullPath;
 - (NSImage*) iconForFiles: (NSArray*)pathArray;
 - (NSImage*) iconForFileType: (NSString*)fileType;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (BOOL) isFilePackageAtPath: (NSString*)fullPath;
 #endif
 
@@ -110,7 +132,7 @@
 //
 - (BOOL) fileSystemChanged;
 - (void) noteFileSystemChanged;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (void) noteFileSystemChanged: (NSString*)path;
 #endif
 
@@ -127,7 +149,18 @@
 - (BOOL) launchApplication: (NSString*)appName
 		  showIcon: (BOOL)showIcon
 		autolaunch: (BOOL)autolaunch;
-#ifndef STRICT_OPENSTEP
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
+// TODO: implement NSAppleEventDescriptor in gnustep-base
+typedef void NSAppleEventDescriptor;
+
+- (BOOL) launchAppWithBundleIdentifier: (NSString *)bundleIdentifier
+			       options: (NSWorkspaceLaunchOptions)options 
+	additionalEventParamDescriptor: (NSAppleEventDescriptor *)descriptor 
+		      launchIdentifier: (NSNumber **)identifier;
+#endif
+
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (NSDictionary*) activeApplication;
 - (NSArray*) launchedApplications;
 #endif
@@ -143,7 +176,7 @@
 - (void) checkForRemovableMedia;
 - (NSArray*) mountNewRemovableMedia;
 - (NSArray*) mountedRemovableMedia;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (NSArray*) mountedLocalVolumePaths;
 #endif
 
@@ -172,7 +205,7 @@
 
 @end
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 @class NSBundle;
 
