@@ -817,22 +817,34 @@ NSFrameLinkRect(NSRect aRect, BOOL isDestination)
 
 void NSSetFocusRingStyle(NSFocusRingPlacement placement)
 {
-	// FIXME: NIMP
-	NSLog(@"*** NSSetFocusRingStyle not implemented ***");
+  // FIXME: NIMP
+  NSLog(@"*** NSSetFocusRingStyle not implemented ***");
 }
 
 void 
 NSConvertGlobalToWindowNumber(int globalNum, unsigned int *winNum)
 {
-// TODO
+  NSArray *windows = GSAllWindows();
+  unsigned count = [windows count];
+  unsigned i;
+
+  for (i = 0; i < count; i++)
+    {
+      NSWindow *win = [windows objectAtIndex: i];
+
+      if (((int)(intptr_t)[win windowRef]) == globalNum)
+        {
+          *winNum = [win windowNumber];
+          return;
+        }
+    }
   *winNum = 0;
 }
 
 void 
 NSConvertWindowNumberToGlobal(int winNum, unsigned int *globalNum)
 {
-// TODO
-  *globalNum = 0;
+  *globalNum = (int)(intptr_t)[GSWindowWithNumber(winNum) windowRef];
 }
 
 void 
