@@ -26,23 +26,24 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSLock.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSGeometry.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSSet.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSGeometry.h>
 
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSWindow.h"
-#include "GNUstepGUI/GSDisplayServer.h"
-#include "GNUstepGUI/GSDragView.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSDisplayServer.h"
+#import "GNUstepGUI/GSDragView.h"
 
-#include "GSSlideView.h"
+#import "GSSlideView.h"
 
 /* Display attributes */
 NSString * GSDisplayName = @"DisplayName";
@@ -72,6 +73,7 @@ GSDisplayServer *
 GSServerForWindow(NSWindow *window)
 {
   int num;
+
   if (windowmaps == NULL)
     {
       NSLog(@"GSServerForWindow: No window server");
@@ -126,12 +128,14 @@ GSCurrentServer(void)
     {
       [gnustep_global_lock lock];
       if (serverLock == nil)
-	{
-	  serverLock = [NSRecursiveLock new];
-	  _globalGSZone = NSDefaultMallocZone();
-	  defaultServerClass = [GSDisplayServer class];
-	  NSCurrentServerThreadKey  = @"NSCurrentServerThreadKey";
-	}
+        {
+          serverLock = [NSRecursiveLock new];
+          _globalGSZone = NSDefaultMallocZone();
+          defaultServerClass = [GSDisplayServer class];
+          NSCurrentServerThreadKey  = @"NSCurrentServerThreadKey";
+          windowmaps = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
+                                        NSNonOwnedPointerMapValueCallBacks, 20);
+        }
       [gnustep_global_lock unlock];
     }
 }
@@ -176,12 +180,6 @@ GSCurrentServer(void)
 {
   GSDisplayServer *server;
 
-  if (windowmaps == NULL)
-    {
-      windowmaps = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
-				    NSNonOwnedPointerMapValueCallBacks, 20);
-    }
-
   if (self == [GSDisplayServer class])
     {
       server = [[defaultServerClass allocWithZone: _globalGSZone]
@@ -251,20 +249,23 @@ GSCurrentServer(void)
   void			*key;
   void			*val;
 
-  /*
-   * Remove the display server from the windows map.
-   * This depends on a property of GNUstep map tables, that an
-   * enumerated object can safely be removed from the map.
-   */
-  enumerator = NSEnumerateMapTable(windowmaps);
-  while (NSNextMapEnumeratorPair(&enumerator, &key, &val))
+  if (windowmaps != NULL)
     {
-      if (val == (void*)self)
-	{
-	  NSMapRemove(windowmaps, key);
-	}
+      /*
+       * Remove the display server from the windows map.
+       * This depends on a property of GNUstep map tables, that an
+       * enumerated object can safely be removed from the map.
+       */
+      enumerator = NSEnumerateMapTable(windowmaps);
+      while (NSNextMapEnumeratorPair(&enumerator, &key, &val))
+        {
+          if (val == (void*)self)
+            {
+              NSMapRemove(windowmaps, key);
+            }
+        }
+      NSEndMapTableEnumeration(&enumerator);
     }
-  NSEndMapTableEnumeration(&enumerator);
 
   DESTROY(server_info);
   DESTROY(event_queue);
@@ -517,7 +518,10 @@ GSCurrentServer(void)
     for a window that has already been created */
 - (void) _setWindowOwnedByServer: (int)win
 {
-  NSMapInsert (windowmaps, (void*)(intptr_t)win,  self);
+  if (windowmaps != NULL)
+    {
+      NSMapInsert(windowmaps, (void*)(intptr_t)win,  self);
+    }
 }
 
 /** Creates a window whose location and size is described by frame and
@@ -606,9 +610,9 @@ GSCurrentServer(void)
     typically by calling [NSGraphicsContext-GSSetDevice:::],
     although depending on the concrete implmentation, more information
     than this may need to be exchanged. */
-- (void) windowdevice: (int) win
+- (void) windowdevice: (int)winNum
 {
-  [self setWindowdevice: win forContext: GSCurrentContext()];
+  [self setWindowdevice: winNum forContext: GSCurrentContext()];
 }
 
 /** Sets the window device information for the NSGraphicsContext,
@@ -857,6 +861,12 @@ GSCurrentServer(void)
   [self subclassResponsibility: _cmd];
 }
 
+/** Set mouse cursor position. */
+- (void) setMouseLocation: (NSPoint)mouseLocation onScreen: (int)aScreen
+{
+  [self subclassResponsibility: _cmd];
+}
+
 /** Hides the cursor */
 - (void) hidecursor
 {
@@ -896,7 +906,7 @@ GSCurrentServer(void)
   [self subclassResponsibility: _cmd];
 }
 
-- (void) setParentWindow: (int)partentWin 
+- (void) setParentWindow: (int)parentWin 
           forChildWindow: (int)childWin
 {
   [self subclassResponsibility: _cmd];
