@@ -31,21 +31,22 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSThread.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSThread.h>
 
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSCell.h>
-#include <AppKit/NSCursor.h>
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/NSImage.h>
-#include <AppKit/NSPasteboard.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSWindow.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSCell.h"
+#import "AppKit/NSCursor.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSWindow.h"
 
-#include "GNUstepGUI/GSDisplayServer.h"
-#include "GNUstepGUI/GSDragView.h"
+#import "GNUstepGUI/GSDisplayServer.h"
+#import "GNUstepGUI/GSDragView.h"
 #include <math.h>
 
 /* Size of the dragged window */
@@ -57,6 +58,10 @@
 @interface GSRawWindow : NSWindow
 @end
 
+@interface NSApplication (GNUstepPrivate)
+- (void) _postAndSendEvent: (NSEvent *)anEvent;
+@end
+
 @interface NSCursor (BackendPrivate)
 - (void *)_cid;
 - (void) _setCid: (void *)val;
@@ -576,7 +581,7 @@ static	GSDragView *sharedDragView = nil;
 	                  subtype: subtype
 	                    data1: dragWindowRef
 	                    data2: action];
-  [dWindow sendEvent: e];
+  [NSApp _postAndSendEvent: e];
 }
 
 /*
@@ -618,7 +623,7 @@ static	GSDragView *sharedDragView = nil;
   startPoint = [eWindow convertBaseToScreen: [theEvent locationInWindow]];
   startPoint.x -= offset.width;
   startPoint.y -= offset.height;
-  NSDebugLLog(@"NSDragging", @"Drag window origin %d %d\n", startPoint.x, startPoint.y);
+  NSDebugLLog(@"NSDragging", @"Drag window origin %@\n", NSStringFromPoint(startPoint));
 
   // Notify the source that dragging has started
   if ([dragSource respondsToSelector:
@@ -689,7 +694,7 @@ static	GSDragView *sharedDragView = nil;
         {
           [self _sendLocalEvent: GSAppKitDraggingDrop
                          action: 0
-                       position: NSZeroPoint
+                       position: dragPosition
 		                  timestamp: [theEvent timestamp]
                        toWindow: destWindow];
         }
@@ -697,7 +702,7 @@ static	GSDragView *sharedDragView = nil;
         {
           [self sendExternalEvent: GSAppKitDraggingDrop
                            action: 0
-		                     position: NSZeroPoint
+		                     position: dragPosition
 		                    timestamp: [theEvent timestamp]
 		                     toWindow: targetWindowRef];
         }
@@ -883,7 +888,7 @@ static	GSDragView *sharedDragView = nil;
       dragPoint = [destWindow convertScreenToBase: dragPosition];
     }
             
-  NSDebugLLog(@"NSDragging", @"mouse window %d (%x) at %@\n",
+  NSDebugLLog(@"NSDragging", @"mouse window %d (%@) at %@\n",
     mouseWindowRef, destWindow, NSStringFromPoint(dragPosition));
             
   //--- send exit message if necessary -------------------------------------
