@@ -26,34 +26,26 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSInvocation.h>
-#include <Foundation/NSObjCRuntime.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSKeyValueCoding.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSArchiver.h>
-#include <Foundation/NSSet.h>
-#include <AppKit/NSMenu.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSTextView.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSNibLoading.h>
-#include <AppKit/NSNibConnector.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSScreen.h>
-#include <GNUstepBase/GSObjCRuntime.h>
-#include <GNUstepGUI/GSGormLoading.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSKeyValueCoding.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSArchiver.h>
+#import <Foundation/NSSet.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSControl.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSNib.h"
+#import "AppKit/NSNibLoading.h"
+#import "AppKit/NSNibConnector.h"
+#import "AppKit/NSScreen.h"
+#import "AppKit/NSTextView.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSGormLoading.h"
 
 static const int currentVersion = 1; // GSNibItem version number...
 
@@ -163,6 +155,11 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
       NSMutableArray    *topObjects; 
       id                 obj;
 
+      // Add these objects with there old names as the code expects them
+      context = AUTORELEASE([context mutableCopyWithZone: [context zone]]);
+      obj = [context objectForKey: NSNibTopLevelObjects];
+      obj = [context objectForKey: NSNibOwner];
+
       isAwake = YES;
       /*
        *	Add local entries into name table.
@@ -222,12 +219,11 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
 
 
       /* 
-       * See if the user has passed in the NSTopLevelObjects key.
-       * This is an implementation of an undocumented, but commonly used feature
-       * of nib files to allow the release of the top level objects in the nib
-       * file.
+       * See if the user has passed in the NSNibTopLevelObjects key.
+       * This is an implementation of a commonly used feature to give access to
+       * all top level objects of a nib file.
        */
-      obj = [context objectForKey: @"NSTopLevelObjects"];
+      obj = [context objectForKey: NSNibTopLevelObjects];
       if ([obj isKindOfClass: [NSMutableArray class]])
 	{
 	  topObjects = obj;
@@ -246,12 +242,12 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
       while ((key = [enumerator nextObject]) != nil)
 	{
 	  if ([context objectForKey: key] == nil || 
-	      [key isEqualToString: @"NSOwner"]) // we want to send the message to the owner
+	      [key isEqualToString: NSNibOwner]) // we want to send the message to the owner
 	    {
 	      // we don't want to send a message to these menus twice, if they're custom classes. 
 	      if ([key isEqualToString: @"NSWindowsMenu"] == NO && 
 		  [key isEqualToString: @"NSServicesMenu"] == NO && 
-		  [key isEqualToString: @"NSTopLevelObjects"] == NO)
+		  [key isEqualToString: NSNibTopLevelObjects] == NO)
 		{
 		  id o = [nameTable objectForKey: key];
 
@@ -268,30 +264,23 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
 		   * We don't want to retain the owner.
 		   *
 		   * Please note: It is encumbent upon the developer of an application to 
-		   * release these objects.   Instantiating a window manually or loading in a .gorm 
-		   * file are equivalent processes.  These objects need to be released in their 
-		   * respective controllers.  If the developer has used the "NSTopLevelObjects" feature, 
-		   * then he will get the objects back in an array which he merely must release in
-		   * order to release the objects held within.  GJC
+		   * release these objects. Instantiating a window manually or loading in a .gorm 
+		   * file are equivalent processes. These objects need to be released in their 
+		   * respective controllers. If the developer has used the NSNibTopLevelObjects feature, 
+		   * then she will get the objects back in an array. She will will have to first release 
+                   * all the objects in the array and then the array itself in order to release the 
+                   * objects held within.
 		   */
-		  if ([key isEqualToString: @"NSOwner"] == NO)
+		  if ([key isEqualToString: NSNibOwner] == NO)
 		    {
 		      if ([topLevelObjects containsObject: o]) // anything already designated a top level item..
 			{
-			  if (topObjects == nil)
-			    {
-			      // It is expected, if the NSTopLevelObjects key is not passed in,
-			      // that the user has opted to either allow these objects to leak or
-			      // to release them explicitly.
-			      RETAIN(o);
-			    }
-			  else
-			    {
-			      // We don't want to do the extra retain if the items are added to the
-			      // array, since the array will do the retain for us.   When the array
-			      // is released, the top level objects should be released as well.
-			      [topObjects addObject: o];
-			    }
+			  [topObjects addObject: o];
+			  // All top level objects must be released by the
+			  // caller to avoid leaking, unless they are going
+			  // to be released by other nib objects on behalf
+			  // of the owner.
+			  RETAIN(o);
 			}
 		    }
 		}
