@@ -9,6 +9,8 @@
 
    Author: Gregory John Casamento
    Date: 2003, 2005
+   Author: Fred Kiefer
+   Date: 2003, 2010
 
    This file is part of the GNUstep GUI Library.
 
@@ -29,40 +31,43 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSClassDescription.h>
-#include <Foundation/NSArchiver.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSByteOrder.h>
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSDecimalNumber.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSInvocation.h>
-#include <Foundation/NSObjCRuntime.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSKeyValueCoding.h>
-#include <Foundation/NSKeyedArchiver.h>
-#include "AppKit/AppKit.h"
-#include <GNUstepBase/GSObjCRuntime.h>
-#include "GSGuiPrivate.h"
+#define EXPOSE_NSKeyedUnarchiver_IVARS
 
+#import <Foundation/NSArchiver.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSByteOrder.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDecimalNumber.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSKeyedArchiver.h>
+#import <Foundation/NSObjCRuntime.h>
+#import <Foundation/NSSet.h>
+#import <Foundation/NSString.h>
+
+// FIXME: Used for NSKeyedArchiver access. All this should be moved into base.
 /*
  *      Setup for inline operation of arrays.
  */
 #define GSI_ARRAY_RETAIN(A, X)	RETAIN((X).obj)
 #define GSI_ARRAY_RELEASE(A, X)	RELEASE((X).obj)
 #define GSI_ARRAY_TYPES GSUNION_OBJ
-#include <GNUstepBase/GSIArray.h>
+#import <GNUstepBase/GSIArray.h>
 
-#include <GNUstepGUI/GSNibLoading.h>
-#include <GNUstepGUI/GSInstantiator.h>
+#import "GNUstepGUI/GSNibLoading.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenuItem.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSNib.h"
+#import "AppKit/NSScreen.h"
+#import "AppKit/NSSound.h"
+#import "GNUstepGUI/GSInstantiator.h"
+#import "GSGuiPrivate.h"
 
 static BOOL _isInInterfaceBuilder = NO;
 
@@ -98,7 +103,7 @@ static BOOL _isInInterfaceBuilder = NO;
 @interface NSMenu (NibCompatibility)
 - (void) _setMain: (BOOL)isMain;
 @end
-@interface	NSMenu (GNUstepPrivate)
+@interface NSMenu (GNUstepPrivate)
 - (void) _setGeometry;
 @end
 
@@ -113,7 +118,7 @@ static BOOL _isInInterfaceBuilder = NO;
       NSInterfaceStyle	newStyle;
       NSString          *processName;
 
-      if([self numberOfItems] == 0)
+      if ([self numberOfItems] == 0)
         return;
 
       oldRep = [self menuRepresentation];
@@ -147,22 +152,59 @@ static BOOL _isInInterfaceBuilder = NO;
       [[self window] setLevel: NSMainMenuWindowLevel];
 
       // if it's a standard menu, transform it to be more NeXT'ish/GNUstep-like
-      if(_menu.horizontal == NO)
+      if (_menu.horizontal == NO)
         {
-          NSString *infoString = NSLocalizedString (@"Info", @"Info");
-          NSString *quitString = [NSString stringWithFormat: @"%@ %@", 
-                                           NSLocalizedString (@"Quit", @"Quit"), processName];
-          NSMenuItem *quitItem = [[NSMenuItem alloc] initWithTitle: quitString
-                                                     action: @selector(terminate:)
-                                                     keyEquivalent: @"q"];
           NSMenuItem *appItem;
+	  NSMenu *sub;
+	  SEL	sel = @selector(terminate:);
           
-          appItem = (NSMenuItem*)[self itemAtIndex: 0]; // Info item.
-
-          [self addItem: quitItem];
+	  /* The title of the main menu should be the process name.
+	   */
           [self setTitle: processName];
-          [appItem setTitle: infoString];
-          [[appItem submenu] setTitle: infoString];
+
+	  /* If there is no 'quite' item (one which sends a -terminate:
+	   * actions) we add one.
+	   */
+	  if ([self indexOfItemWithTarget: nil andAction: sel] < 0
+	    && [self indexOfItemWithTarget: NSApp andAction: sel] < 0)
+	    {
+	      NSString *quitString;
+	      NSMenuItem *quitItem;
+
+	      quitString = [NSString stringWithFormat: @"%@ %@", 
+		NSLocalizedString (@"Quit", @"Quit"), processName];
+	      quitItem = [[NSMenuItem alloc] initWithTitle: quitString
+		action: @selector(terminate:)
+		keyEquivalent: @"q"];
+              [self addItem: quitItem];
+	    }
+
+	  /* An OSX main menu has the first item pointing to a submenu
+	   * whose contents are much the same as a GNUstep info menu.
+	   */
+          appItem = (NSMenuItem*)[self itemAtIndex: 0]; // Info item.
+	  sub = [appItem submenu];
+	  if (sub != nil)
+	    {
+              NSString	*infoString;
+	      NSInteger	index;
+
+	      infoString = NSLocalizedString (@"Info", @"Info");
+	      [appItem setTitle: infoString];
+	      [sub setTitle: infoString];
+	      /* The submenu may contain a 'quit' item ... if so we need to
+	       * remove it as we already added one to the main menu.
+	       */
+	      index = [sub indexOfItemWithTarget: nil andAction: sel];
+	      if (index < 0)
+		{
+	          index = [sub indexOfItemWithTarget: NSApp andAction: sel];
+		}
+	      if (index >= 0)
+		{
+		  [sub removeItemAtIndex: index];
+		}
+	    }
         }
 
       [self _setGeometry];
@@ -250,6 +292,7 @@ static BOOL _isInInterfaceBuilder = NO;
   RELEASE(_windowClass);
   RELEASE(_view);
   RELEASE(_autosaveName);
+  RELEASE(_realObject);
   [super dealloc];
 }
 
@@ -665,13 +708,20 @@ static BOOL _isInInterfaceBuilder = NO;
     }
 }
 
+- (void) dealloc
+{
+  RELEASE(_className);
+  RELEASE(_realObject);
+  [super dealloc];
+}
+
 /**
  * Designated initializer for NSViewTemplate.
  */
 - (id) initWithObject: (id)o
 	    className: (NSString *)name
 {
-  if((self = [super init]) != nil)
+  if ((self = [super init]) != nil)
     {
       [self setRealObject: o];
       [self setClassName: name];
@@ -699,12 +749,12 @@ static BOOL _isInInterfaceBuilder = NO;
 	    }
 	  else
 	    {
-	      _realObject = [[aClass allocWithZone: NSDefaultMallocZone()] initWithCoder: coder];
+	      ASSIGN(_realObject, [[aClass allocWithZone: NSDefaultMallocZone()] initWithCoder: coder]);
 	      [[self superview] replaceSubview: self with: _realObject]; // replace the old view...
 	    }
 	}
-      
-      // FIXME: Should release self
+
+      AUTORELEASE(self);
       return _realObject;
     }
   else
@@ -801,6 +851,13 @@ static BOOL _isInInterfaceBuilder = NO;
     }
 }
 
+- (void) dealloc
+{
+  RELEASE(_menuClass);
+  RELEASE(_realObject);
+  [super dealloc];
+}
+
 - (id) initWithCoder: (NSCoder *)aCoder
 {
   RELEASE(self);
@@ -918,10 +975,10 @@ static BOOL _isInInterfaceBuilder = NO;
                        format: @"Unable to find class '%@'", _className];
         }
 
-      if(GSObjCIsKindOf(aClass, [NSApplication class]) || 
+      if (GSObjCIsKindOf(aClass, [NSApplication class]) || 
 	 [_className isEqual: @"NSApplication"])
 	{
-	  _object = [aClass sharedApplication];
+	  _object = RETAIN([aClass sharedApplication]);
 	}
       else
 	{
@@ -934,7 +991,7 @@ static BOOL _isInInterfaceBuilder = NO;
 - (void) awakeFromNib
 {
   NSDebugLog(@"Called awakeFromNib on an NSCustomObject instance: %@", self);
-  if([_object respondsToSelector: @selector(awakeFromNib)])
+  if ([_object respondsToSelector: @selector(awakeFromNib)])
     {
       [_object awakeFromNib];
     }
@@ -952,6 +1009,7 @@ static BOOL _isInInterfaceBuilder = NO;
 {
   RELEASE(_className);
   RELEASE(_extension);
+  RELEASE(_object);
   [super dealloc];
 }
 @end
@@ -991,7 +1049,7 @@ static BOOL _isInInterfaceBuilder = NO;
     }
   
   // If the class name is nil, assume NSView.
-  if(_className == nil)
+  if (_className == nil)
     {
       aClass = [NSView class];
     }
@@ -1016,7 +1074,7 @@ static BOOL _isInInterfaceBuilder = NO;
 
 - (id) nibInstantiateWithCoder: (NSCoder *)coder
 {
-  if([NSClassSwapper isInInterfaceBuilder])
+  if ([NSClassSwapper isInInterfaceBuilder])
     {
       return _view;
     }
@@ -1074,7 +1132,7 @@ static BOOL _isInInterfaceBuilder = NO;
 - (id) initWithCoder: (NSCoder *)coder
 {
   // if in interface builder, then initialize as normal.
-  if([NSClassSwapper isInInterfaceBuilder])
+  if ([NSClassSwapper isInInterfaceBuilder])
     {
       self = [super initWithCoder: coder];
     }
@@ -1100,12 +1158,12 @@ static BOOL _isInInterfaceBuilder = NO;
           ASSIGN(_className, [coder decodeObjectForKey: @"NSClassName"]);
           ASSIGN(_extension, [coder decodeObjectForKey: @"NSExtension"]);
 
-	  if([self nibInstantiate] != nil)
+	  if ([self nibInstantiate] != nil)
 	    {
 	      [self nibInstantiateWithCoder: coder];
 	    }
 
-	  if(self != _view)
+	  if (self != _view)
 	    {
 	      AUTORELEASE(self);
 	      [(NSKeyedUnarchiver *)coder replaceObject: self withObject: _view];
@@ -1171,7 +1229,7 @@ static BOOL _isInInterfaceBuilder = NO;
       ASSIGN(_className, [coder decodeObjectForKey: @"NSClassName"]);
       ASSIGN(_resourceName, [coder decodeObjectForKey: @"NSResourceName"]);
 
-      // this is a hack, but for now it should do.
+      // FIXME: this is a hack, but for now it should do.
       if ([_className isEqual: @"NSSound"])
         {
           realObject = RETAIN([NSSound soundNamed: _resourceName]);
@@ -1253,12 +1311,15 @@ static BOOL _isInInterfaceBuilder = NO;
   Class aClass;
   if ((aClass = [self classForClassName: className]) == nil)
     {
-      aClass = NSClassFromString(className);
-    }
-  if (aClass == nil)
-    {
-      [NSException raise: NSInternalInconsistencyException
-                   format: @"NSClassSwapper unable to find class '%@'", className];
+      if ((aClass = [[self class] classForClassName: className]) == nil)
+        {
+          aClass = NSClassFromString(className);
+          if (aClass == nil)
+            {
+              [NSException raise: NSInternalInconsistencyException
+                          format: @"NSClassSwapper unable to find class '%@'", className];
+            }
+        }
     }
   return aClass;
 }
@@ -1362,7 +1423,7 @@ static BOOL _isInInterfaceBuilder = NO;
   id object = nil;
   NSKeyedUnarchiver *decoder = (NSKeyedUnarchiver *)coder;
 
-  if([NSClassSwapper isInInterfaceBuilder] == YES)
+  if ([NSClassSwapper isInInterfaceBuilder] == YES)
     {
       newClass = [decoder replacementClassForClassName: _originalClassName];
     }
@@ -1574,7 +1635,7 @@ static BOOL _isInInterfaceBuilder = NO;
   while ((key = [ken nextObject]) != nil && (value = [ven nextObject]) != nil)
     {
       NSMapInsert(mapTable, key, value);
-      if(value == nil)
+      if (value == nil)
 	{
 	  NSLog(@"==> WARNING: Value for key %@ is %@",key , value);
 	}
@@ -1706,7 +1767,7 @@ static BOOL _isInInterfaceBuilder = NO;
       // aren't useful outside of it and only waste memory if
       // unarchived in the live application.
       //
-      if([NSClassSwapper isInInterfaceBuilder])
+      if ([NSClassSwapper isInInterfaceBuilder])
 	{
 	  // Only get these when in the editor...
 	  oidsKeys = (NSArray *)
@@ -1784,7 +1845,7 @@ static BOOL _isInInterfaceBuilder = NO;
   NSFreeMapTable(_names);
   NSFreeMapTable(_classes);
   // these are not allocated when not in interface builder.
-  if([NSClassSwapper isInInterfaceBuilder])
+  if ([NSClassSwapper isInInterfaceBuilder])
     {
       NSFreeMapTable(_oids);
       NSFreeMapTable(_accessibilityOids);
@@ -1819,15 +1880,16 @@ static BOOL _isInInterfaceBuilder = NO;
  */
 - (void) nibInstantiateWithOwner: (id)owner topLevelObjects: (NSMutableArray *)topLevelObjects
 {
-  NSEnumerator *en = [_connections objectEnumerator];
-  NSArray *objs = NSAllMapTableKeys([self names]);
+  NSEnumerator *en;
+  NSArray *objs;
   id obj = nil;
   id menu = nil;
-  
+
   // set the new root object.
   [_root setRealObject: owner];
 
-  // iterate over connections, instantiate, and then establish them.
+  // iterate over connections, instantiate and then establish them.
+  en = [_connections objectEnumerator];
   while ((obj = [en nextObject]) != nil)
     {
       if ([obj respondsToSelector: @selector(instantiateWithInstantiator:)])
@@ -1837,27 +1899,23 @@ static BOOL _isInInterfaceBuilder = NO;
         }
     }
 
-  // iterate over all objects instantiate windows, awaken objects and fill
+  // iterate over all objects, instantiate, awaken objects and fill
   // in top level array.
+  // We use the _names map not the _objects map here. This seems to give better results.
+  objs = NSAllMapTableKeys(_names);
   en = [objs objectEnumerator];
   while ((obj = [en nextObject]) != nil)
     {
-      // instantiate all windows and fill in the top level array.
-      if ([obj isKindOfClass: [NSWindowTemplate class]])
+      id v = NSMapGet(_objects, obj);
+      obj = [self instantiateObject: obj];
+      // Object is top level if it isn't the owner but points to it. 
+      if ((v == owner || v == _root) && (obj != owner) && (obj != _root))
         {
-          if ([obj realObject] == nil)
-            {
-              obj = [self instantiateObject: obj];
-              [topLevelObjects addObject: obj];
-            }
-        }
-      else
-        {
-          id v = NSMapGet(_objects, obj);
-          if (v == nil || v == owner)
-            {
-              [topLevelObjects addObject: obj];
-            }
+          [topLevelObjects addObject: obj];
+          // All top level objects must be released by the caller to avoid
+          // leaking, unless they are going to be released by other nib
+          // objects on behalf of the owner.
+          RETAIN(obj);
         }
 
       // awaken the object.
@@ -1890,19 +1948,8 @@ static BOOL _isInInterfaceBuilder = NO;
  */
 - (void) awakeWithContext: (NSDictionary *)context
 {
-  NSMutableArray *tlo = [context objectForKey: @"NSTopLevelObjects"];
-  id owner = [context objectForKey: @"NSOwner"];
-
-  // get using the alternate names.
-  if(tlo == nil)
-    {
-      tlo = [context objectForKey: @"NSNibTopLevelObjects"];
-    }
-
-  if(owner == nil)
-    {
-      owner = [context objectForKey: @"NSNibOwner"];
-    }
+  NSMutableArray *tlo = [context objectForKey: NSNibTopLevelObjects];
+  id owner = [context objectForKey: NSNibOwner];
 
   // instantiate...
   [self nibInstantiateWithOwner: owner topLevelObjects: tlo];
@@ -2054,7 +2101,7 @@ static BOOL _isInInterfaceBuilder = NO;
                    NSStringFromClass([coder class])];
     }
 
-  RELEASE(self);
+  AUTORELEASE(self);
   return RETAIN([NSImage imageNamed: imageName]);
 }
 
@@ -2105,11 +2152,18 @@ static BOOL _isInInterfaceBuilder = NO;
   if ((self = [super init]) != nil)
     {
       _file = nil;
-      _marker = @"NSToolTipHelpKey";
+      ASSIGN(_marker, @"NSToolTipHelpKey");
     }
   return self;
 }
 
+- (void) dealloc
+{
+  RELEASE(_file);
+  RELEASE(_marker);
+  [super dealloc];
+}
+
 - (id) initWithCoder: (NSCoder *)coder
 {
   if ((self = [super initWithCoder: coder]) != nil)
@@ -2118,17 +2172,17 @@ static BOOL _isInInterfaceBuilder = NO;
         {
           if ([coder containsValueForKey: @"NSFile"])
             {
-              _file = RETAIN([coder decodeObjectForKey: @"NSFile"]);
+              ASSIGN(_file, [coder decodeObjectForKey: @"NSFile"]);
             }
           if ([coder containsValueForKey: @"NSMarker"])
             {
-              _marker = RETAIN([coder decodeObjectForKey: @"NSMarker"]);
+              ASSIGN(_marker, [coder decodeObjectForKey: @"NSMarker"]);
             }
         }
       else
         {
-          _file = RETAIN([coder decodeObject]);
-          _marker = RETAIN([coder decodeObject]);
+          ASSIGN(_file, [coder decodeObject]);
+          ASSIGN(_marker, [coder decodeObject]);
         }
     }
   return self;
@@ -2157,7 +2211,7 @@ static BOOL _isInInterfaceBuilder = NO;
 
 - (void) establishConnection
 {
-  if([_dst respondsToSelector: @selector(setToolTip:)])
+  if ([_dst respondsToSelector: @selector(setToolTip:)])
     {
       [_dst setToolTip: _marker];
     }
@@ -2190,7 +2244,7 @@ static BOOL _isInInterfaceBuilder = NO;
   NSDecimalNumber *dn = nil;
   if ([coder allowsKeyedCoding])
     {
-      unsigned int len = 0;
+      NSUInteger len = 0;
       short exponent = (short)[coder decodeIntForKey: @"NS.exponent"];
       NSByteOrder bo = [coder decodeIntForKey: @"NS.mantissa.bo"];
       BOOL negative = [coder decodeBoolForKey: @"NS.negative"];
@@ -2219,7 +2273,8 @@ static BOOL _isInInterfaceBuilder = NO;
                                     exponent: exponent
                                     isNegative: negative];
     }
-  // FIXME: Needs to release self
+
+  RELEASE(self);
   return dn;
 }
 
@@ -2257,5 +2312,3 @@ static BOOL _isInInterfaceBuilder = NO;
   return self;
 }
 @end
-
-
