@@ -59,6 +59,7 @@
 #include "AppKit/NSDocumentController.h"
 
 #include "GNUstepGUI/GSServicesManager.h"
+#include "GSGuiPrivate.h"
 
 static GSServicesManager	*manager = nil;
 
@@ -508,7 +509,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
  */
 + (GSServicesManager*) newWithApplication: (NSApplication*)app
 {
-  NSString	*str;
+  NSString	*str = nil;
   NSArray       *paths;
   NSString      *path = nil;
 
@@ -535,7 +536,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
   if (str == nil)
     {
       str = [[NSHomeDirectory() stringByAppendingPathComponent:
-			       @"GNUstep"] stringByAppendingPathComponent: @"Library"];
+        @"GNUstep"] stringByAppendingPathComponent: @"Library"];
     }
   str = [str stringByAppendingPathComponent: @"Services"];
   path = [str stringByAppendingPathComponent: servicesName];
@@ -1713,8 +1714,7 @@ NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
    */
   NS_DURING
     {
-      const char	*name = [selName UTF8String];
-      SEL		sel = GSSelectorFromNameAndTypes(name, 0); 
+      SEL		sel = NSSelectorFromString(selName); 
       NSMethodSignature	*sig = [provider methodSignatureForSelector: sel];
 
       if (sig != nil)
@@ -1722,7 +1722,6 @@ NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
 	  NSInvocation	*inv;
 	  NSString	**errPtr = &error;
 
-	  sel = GSSelectorFromNameAndTypes(name, [sig methodType]); 
 	  inv = [NSInvocation invocationWithMethodSignature: sig];
 	  [inv setTarget: provider];
 	  [inv setSelector: sel];
@@ -1787,5 +1786,12 @@ NSShowsServicesMenuItem(NSString *name)
 void
 NSUpdateDynamicServices(void)
 {
+  /* Get the workspace manager to make sure that cached service info is
+   * up to date.
+   */
+  [[NSWorkspace sharedWorkspace] findApplications];
+
+  /* Reload service information from disk cache.
+   */
   [[GSServicesManager manager] loadServices];
 }
