@@ -25,6 +25,7 @@
 */
 
 #include <Foundation/NSException.h>
+#include <Foundation/NSNotification.h>
 
 #include "AppKit/NSApplication.h"
 #include "AppKit/NSAttributedString.h"
@@ -37,17 +38,21 @@
 #include "AppKit/NSWindow.h"
 #include "AppKit/PSOperators.h"
 #include "GNUstepGUI/GSDisplayServer.h"
-#include "GSWindowDecorationView.h"
+#include "GNUstepGUI/GSTheme.h"
+
+#include <GNUstepGUI/GSWindowDecorationView.h>
+
+@interface GSStandardWindowDecorationView (GSTheme)
+- (void) _themeDidActivate: (NSNotification*)notification;
+@end
 
 @implementation GSStandardWindowDecorationView
 
-/* These include the black border. */
-#define TITLE_HEIGHT 23.0
-#define RESIZE_HEIGHT 9.0
-
 + (void) offsets: (float *)l : (float *)r : (float *)t : (float *)b
     forStyleMask: (unsigned int)style
 {
+  GSTheme *theme = [GSTheme theme];
+
   if (style
     & (NSTitledWindowMask | NSClosableWindowMask
       | NSMiniaturizableWindowMask | NSResizableWindowMask))
@@ -62,11 +67,11 @@
   if (style
     & (NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask))
     {
-      *t = TITLE_HEIGHT;
+      *t = [theme titlebarHeight];
     }
   if (style & NSResizableWindowMask)
     {
-      *b = RESIZE_HEIGHT;
+      *b = [theme resizebarHeight];
     }
 }
 
@@ -86,20 +91,20 @@
   return width;
 }
 
-
-static NSDictionary *titleTextAttributes[3];
-static NSColor *titleColor[3];
-
 - (void) updateRects
 {
+  GSTheme *theme = [GSTheme theme];
+
   if (hasTitleBar)
     {
-      titleBarRect = NSMakeRect(0.0, _frame.size.height - TITLE_HEIGHT,
-	_frame.size.width, TITLE_HEIGHT);
+      float titleHeight = [theme titlebarHeight];
+
+      titleBarRect = NSMakeRect(0.0, _frame.size.height - titleHeight,
+	_frame.size.width, titleHeight);
     }
   if (hasResizeBar)
     {
-      resizeBarRect = NSMakeRect(0.0, 0.0, _frame.size.width, RESIZE_HEIGHT);
+      resizeBarRect = NSMakeRect(0.0, 0.0, _frame.size.width, [theme resizebarHeight]);
     }
   if (hasCloseButton)
     {
@@ -119,231 +124,70 @@ static NSColor *titleColor[3];
 - (id) initWithFrame: (NSRect)frame
 	      window: (NSWindow *)w
 {
-  if (!titleTextAttributes[0])
-    {
-      NSMutableParagraphStyle	*p;
-
-      p = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
-      [p setLineBreakMode: NSLineBreakByClipping];
-
-      titleTextAttributes[0] = [[NSMutableDictionary alloc]
-	initWithObjectsAndKeys:
-	  [NSFont titleBarFontOfSize: 0], NSFontAttributeName,
-	  [NSColor windowFrameTextColor], NSForegroundColorAttributeName,
-	  p, NSParagraphStyleAttributeName,
-	  nil];
-
-      titleTextAttributes[1] = [[NSMutableDictionary alloc]
-	initWithObjectsAndKeys:
-	  [NSFont titleBarFontOfSize: 0], NSFontAttributeName,
-	  [NSColor blackColor], NSForegroundColorAttributeName, /* TODO: need a named color for this */
-	  p, NSParagraphStyleAttributeName,
-	  nil];
-
-      titleTextAttributes[2] = [[NSMutableDictionary alloc]
-	initWithObjectsAndKeys:
-	  [NSFont titleBarFontOfSize: 0], NSFontAttributeName,
-	  [NSColor windowFrameTextColor], NSForegroundColorAttributeName,
-	  p, NSParagraphStyleAttributeName,
-	  nil];
-
-      RELEASE(p);
-      titleColor[0] = RETAIN([NSColor windowFrameColor]);
-      titleColor[1] = RETAIN([NSColor lightGrayColor]);
-      titleColor[2] = RETAIN([NSColor darkGrayColor]);
-    }
+  unsigned int styleMask;
 
   self = [super initWithFrame: frame window: w];
   if (!self) return nil;
 
-  if ([w styleMask]
+  styleMask = [w styleMask];
+  if (styleMask
     & (NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask))
     {
       hasTitleBar = YES;
     }
-  if ([w styleMask] & NSTitledWindowMask)
+  if (styleMask & NSTitledWindowMask)
     {
       isTitled = YES;
     }
-  if ([w styleMask] & NSClosableWindowMask)
+  if (styleMask & NSClosableWindowMask)
     {
       hasCloseButton = YES;
 
       closeButton = [NSWindow standardWindowButton: NSWindowCloseButton 
-                              forStyleMask: [w styleMask]];
+                              forStyleMask: styleMask];
       [closeButton setTarget: window];
       [self addSubview: closeButton];
     }
-  if ([w styleMask] & NSMiniaturizableWindowMask)
+  if (styleMask & NSMiniaturizableWindowMask)
     {
       hasMiniaturizeButton = YES;
 
       miniaturizeButton = [NSWindow standardWindowButton: NSWindowMiniaturizeButton 
-                              forStyleMask: [w styleMask]];
+                              forStyleMask: styleMask];
       [miniaturizeButton setTarget: window];
       [self addSubview: miniaturizeButton];
     }
-  if ([w styleMask] & NSResizableWindowMask)
+  if (styleMask & NSResizableWindowMask)
     {
       hasResizeBar = YES;
     }
   [self updateRects];
 
+  [[NSNotificationCenter defaultCenter]
+    addObserver: self
+    selector: @selector(_themeDidActivate:)
+    name: GSThemeDidActivateNotification
+    object: nil];
   return self;
 }
 
-
-- (void) drawTitleBar
+- (void) dealloc
 {
-  static const NSRectEdge edges[4] = {NSMinXEdge, NSMaxYEdge,
-				    NSMaxXEdge, NSMinYEdge};
-  float grays[3][4] =
-    {{NSLightGray, NSLightGray, NSDarkGray, NSDarkGray},
-    {NSWhite, NSWhite, NSDarkGray, NSDarkGray},
-    {NSLightGray, NSLightGray, NSBlack, NSBlack}};
-  NSRect workRect;
-  NSString *title = [window title];
-
-  /*
-  Draw the black border towards the rest of the window. (The outer black
-  border is drawn in -drawRect: since it might be drawn even if we don't have
-  a title bar.
-  */
-  [[NSColor blackColor] set];
-  PSmoveto(0, NSMinY(titleBarRect) + 0.5);
-  PSrlineto(titleBarRect.size.width, 0);
-  PSstroke();
-
-  /*
-  Draw the button-like border.
-  */
-  workRect = titleBarRect;
-  workRect.origin.x += 1;
-  workRect.origin.y += 1;
-  workRect.size.width -= 2;
-  workRect.size.height -= 2;
-
-  workRect = NSDrawTiledRects(workRect, workRect, edges, grays[inputState], 4);
- 
-  /*
-  Draw the background.
-  */
-  [titleColor[inputState] set];
-  NSRectFill(workRect);
-
-  /* Draw the title. */
-  if (isTitled)
-    {
-      NSSize titleSize;
-    
-      if (hasMiniaturizeButton)
-	{
-	  workRect.origin.x += 17;
-	  workRect.size.width -= 17;
-	}
-      if (hasCloseButton)
-	{
-	  workRect.size.width -= 17;
-	}
-  
-      titleSize = [title sizeWithAttributes: titleTextAttributes[inputState]];
-      if (titleSize.width <= workRect.size.width)
-	workRect.origin.x = NSMidX(workRect) - titleSize.width / 2;
-      workRect.origin.y = NSMidY(workRect) - titleSize.height / 2;
-      workRect.size.height = titleSize.height;
-      [title drawInRect: workRect
-	 withAttributes: titleTextAttributes[inputState]];
-    }
-}
-
-- (void) drawResizeBar
-{
-  [[NSColor lightGrayColor] set];
-  PSrectfill(1.0, 1.0, resizeBarRect.size.width - 2.0, RESIZE_HEIGHT - 3.0);
-
-  PSsetlinewidth(1.0);
-
-  [[NSColor blackColor] set];
-  PSmoveto(0.0, 0.5);
-  PSlineto(resizeBarRect.size.width, 0.5);
-  PSstroke();
-
-  [[NSColor darkGrayColor] set];
-  PSmoveto(1.0, RESIZE_HEIGHT - 0.5);
-  PSlineto(resizeBarRect.size.width - 1.0, RESIZE_HEIGHT - 0.5);
-  PSstroke();
-
-  [[NSColor whiteColor] set];
-  PSmoveto(1.0, RESIZE_HEIGHT - 1.5);
-  PSlineto(resizeBarRect.size.width - 1.0, RESIZE_HEIGHT - 1.5);
-  PSstroke();
-
-
-  /* Only draw the notches if there's enough space. */
-  if (resizeBarRect.size.width < 30 * 2)
-    return;
-
-  [[NSColor darkGrayColor] set];
-  PSmoveto(27.5, 1.0);
-  PSlineto(27.5, RESIZE_HEIGHT - 2.0);
-  PSmoveto(resizeBarRect.size.width - 28.5, 1.0);
-  PSlineto(resizeBarRect.size.width - 28.5, RESIZE_HEIGHT - 2.0);
-  PSstroke();
-
-  [[NSColor whiteColor] set];
-  PSmoveto(28.5, 1.0);
-  PSlineto(28.5, RESIZE_HEIGHT - 2.0);
-  PSmoveto(resizeBarRect.size.width - 27.5, 1.0);
-  PSlineto(resizeBarRect.size.width - 27.5, RESIZE_HEIGHT - 2.0);
-  PSstroke();
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
+  [super dealloc];
 }
 
 - (void) drawRect: (NSRect)rect
 {
-  if (hasTitleBar && NSIntersectsRect(rect, titleBarRect))
-    {
-      [self drawTitleBar];
-    }
-
-  if (hasResizeBar && NSIntersectsRect(rect, resizeBarRect))
-    {
-      [self drawResizeBar];
-    }
-
-  if (hasResizeBar || hasTitleBar)
-    {
-      PSsetlinewidth(1.0);
-      [[NSColor blackColor] set];
-      if (NSMinX(rect) < 1.0)
-	{
-	  PSmoveto(0.5, 0.0);
-	  PSlineto(0.5, _frame.size.height);
-	  PSstroke();
-	}
-      if (NSMaxX(rect) > _frame.size.width - 1.0)
-	{
-	  PSmoveto(_frame.size.width - 0.5, 0.0);
-	  PSlineto(_frame.size.width - 0.5, _frame.size.height);
-	  PSstroke();
-	}
-      if (NSMaxY(rect) > _frame.size.height - 1.0)
-	{
-	  PSmoveto(0.0, _frame.size.height - 0.5);
-	  PSlineto(_frame.size.width, _frame.size.height - 0.5);
-	  PSstroke();
-	}
-      if (NSMinY(rect) < 1.0)
-	{
-	  PSmoveto(0.0, 0.5);
-	  PSlineto(_frame.size.width, 0.5);
-	  PSstroke();
-	}
-    }
+  [[GSTheme theme] drawWindowBorder: rect
+                   withFrame: _frame 
+                   forStyleMask: [window styleMask]
+                   state: inputState
+                   andTitle: [window title]];
 
   [super drawRect: rect];
 }
 
-
 - (void) setTitle: (NSString *)newTitle
 {
   if (isTitled)
@@ -376,7 +220,6 @@ static NSColor *titleColor[3];
   [super setDocumentEdited: flag];
 }
 
-
 - (NSPoint) mouseLocationOnScreenOutsideOfEventStream
 {
   int screen = [[window screen] screenNumber];
@@ -411,7 +254,6 @@ static NSColor *titleColor[3];
       if (currentEvent && [currentEvent type] == NSLeftMouseUp)
 	break;
 
-
       currentEvent = [_window nextEventMatchingMask: mask
 			untilDate: [NSDate distantFuture]
 			inMode: NSEventTrackingRunLoopMode
@@ -461,12 +303,14 @@ calc_new_frame(NSRect frame, NSPoint point, NSPoint firstPoint,
 
 - (void) resizeWindowStartingWithEvent: (NSEvent *)event
 {
-  unsigned int mask = NSLeftMouseDraggedMask | NSLeftMouseUpMask;
+  unsigned int mask = NSLeftMouseDraggedMask | NSLeftMouseUpMask | NSPeriodicMask;
   NSEvent *currentEvent = event;
   NSDate *distantPast = [NSDate distantPast];
+  NSDate *distantFuture = [NSDate distantFuture];
   NSPoint firstPoint, point;
   NSRect newFrame, frame;
   NSSize minSize, maxSize;
+  int num = 0;
 
   /*
   0 drag lower left corner
@@ -475,7 +319,6 @@ calc_new_frame(NSRect frame, NSPoint point, NSPoint firstPoint,
   */
   int mode;
 
-
   firstPoint = [event locationInWindow];
   if (resizeBarRect.size.width < 30 * 2
       && firstPoint.x < resizeBarRect.size.width / 2)
@@ -492,6 +335,7 @@ calc_new_frame(NSRect frame, NSPoint point, NSPoint firstPoint,
   maxSize = [window maxSize];
 
   [window _captureMouse: nil];
+  [NSEvent startPeriodicEventsAfterDelay: 0.1 withPeriod: 0.1];
   do
     {
       while (currentEvent && [currentEvent type] != NSLeftMouseUp)
@@ -509,11 +353,19 @@ calc_new_frame(NSRect frame, NSPoint point, NSPoint firstPoint,
       if (currentEvent && [currentEvent type] == NSLeftMouseUp)
 	break;
 
+      num++;
+      if (num == 5)
+	{
+	  [window setFrame: newFrame  display: YES];
+	  num = 0;
+	}
+
       currentEvent = [_window nextEventMatchingMask: mask
-			untilDate: [NSDate distantFuture]
+			untilDate: distantFuture
 			inMode: NSEventTrackingRunLoopMode
 			dequeue: YES];
     } while ([currentEvent type] != NSLeftMouseUp);
+  [NSEvent stopPeriodicEvents];
   [window _releaseMouse: nil];
 
   [window setFrame: newFrame  display: YES];
@@ -552,3 +404,12 @@ calc_new_frame(NSRect frame, NSPoint point, NSPoint firstPoint,
 
 @end
 
+@implementation GSStandardWindowDecorationView (GSTheme)
+
+- (void) _themeDidActivate: (NSNotification*)notification
+{
+  [self updateRects];
+  [self setNeedsDisplay: YES];
+}
+
+@end
