@@ -41,17 +41,19 @@
  *		attributes argument and use the values from the string.
  */
 
-#include <Foundation/NSAttributedString.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSRange.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSZone.h>
-#include <Foundation/NSLock.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSNotification.h>
-#include "AppKit/NSTextStorage.h"
-#include "GSTextStorage.h"
+#import <Foundation/NSAttributedString.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSRange.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSZone.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSProxy.h>
+#import <Foundation/NSInvocation.h>
+#import <Foundation/NSNotification.h>
+#import "AppKit/NSTextStorage.h"
+#import "GSTextStorage.h"
 
 #define		SANITY_CHECKS	0
 
@@ -77,6 +79,103 @@ static IMP		unlockImp;
 #define	ALOCK()	if (attrLock != nil) (*lockImp)(attrLock, lockSel)
 #define	AUNLOCK() if (attrLock != nil) (*unlockImp)(attrLock, unlockSel)
 
+@interface GSTextStorageProxy : NSProxy
+{
+  NSString	*string;
+}
+- (id) _initWithString: (NSString*)s;
+@end
+
+@implementation	GSTextStorageProxy
+
+static Class NSObjectClass = nil;
+static Class NSStringClass = nil;
+
++ (void) initialize
+{
+  NSObjectClass = [NSObject class];
+  NSStringClass = [NSString class];
+}
+
+- (Class) class
+{
+  return NSStringClass;
+}
+
+- (void) dealloc
+{
+  [string release];
+  [super dealloc];
+}
+
+- (void) forwardInvocation: (NSInvocation*)anInvocation
+{
+  SEL	aSel = [anInvocation selector];
+
+  if (YES == [NSStringClass instancesRespondToSelector: aSel])
+    {
+      [anInvocation invokeWithTarget: string];
+    }
+  else
+    {
+      [NSException raise: NSGenericException
+	          format: @"NSString(instance) does not recognize %s",
+	aSel ? GSNameFromSelector(aSel) : "(null)"];
+    }
+}
+
+- (NSUInteger) hash
+{
+  return [string hash];
+}
+
+- (id) _initWithString: (NSString*)s
+{
+  string = [s retain];
+  return self;
+}
+
+- (BOOL) isEqual: (id)other
+{
+  return [string isEqual: other];
+}
+
+- (BOOL) isMemberOfClass: (Class)c
+{
+  return (c == NSStringClass) ? YES : NO;
+}
+
+- (BOOL) isKindOfClass: (Class)c
+{
+  return (c == NSStringClass || c == NSObjectClass) ? YES : NO;
+}
+
+- (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
+{
+  NSMethodSignature	*sig;
+
+  if (YES == [NSStringClass instancesRespondToSelector: aSelector])
+    {
+      sig = [string methodSignatureForSelector: aSelector];
+    }
+  else
+    {
+      sig = [super methodSignatureForSelector: aSelector];
+    }
+  return sig;
+}
+
+- (BOOL) respondsToSelector: (SEL)aSelector
+{
+  if (YES == [NSStringClass instancesRespondToSelector: aSelector])
+    {
+      return YES;
+    }
+  return [super respondsToSelector: aSelector];
+}
+
+@end
+
 /*
  * Add a dictionary to the cache - if it was not already there, return
  * the copy added to the cache, if it was, count it and return retained
@@ -168,8 +267,7 @@ unCacheAttributes(NSDictionary *attrs)
 - (void) dealloc
 {
   [self finalize];
-  NSDeallocateObject(self);
-  GSNOSUPERDEALLOC;
+  [super dealloc];
 }
 
 - (NSString*) description
@@ -464,48 +562,38 @@ _attributesAtIndexEffectiveRange(
     }
 }
 
-- (Class) classForPortCoder
-{
-  return [self class];
-}
-
-- (id) replacementObjectForPortCoder: (NSPortCoder*)aCoder
-{
-  return self;
-}
-
-- (void) encodeWithCoder: (NSCoder*)aCoder
-{
-  [super encodeWithCoder: aCoder];
-  [aCoder encodeValueOfObjCType: @encode(id) at: &_textChars];
-  [aCoder encodeValueOfObjCType: @encode(id) at: &_infoArray];
-}
-
 - (id) initWithCoder: (NSCoder*)aCoder
 {
   self = [super initWithCoder: aCoder];
-  [aCoder decodeValueOfObjCType: @encode(id) at: &_textChars];
-  [aCoder decodeValueOfObjCType: @encode(id) at: &_infoArray];
+  if([aCoder allowsKeyedCoding] == NO)
+    {
+      if ([aCoder versionForClassName: @"GSTextStorage"] != (NSInteger)NSNotFound)
+        {
+          NSLog(@"Warning - decoding archive containing obsolete %@ object - please delete/replace this archive", NSStringFromClass([self class]));
+          [aCoder decodeValueOfObjCType: @encode(id) at: &_textChars];
+          [aCoder decodeValueOfObjCType: @encode(id) at: &_infoArray];
+        }
+    }
   return self;
 }
 
 - (id) initWithString: (NSString*)aString
 	   attributes: (NSDictionary*)attributes
 {
-  NSZone	*z = [self zone];
+  NSZone *z = [self zone];
 
   self = [super initWithString: aString attributes: attributes];
   _infoArray = [[NSMutableArray allocWithZone: z] initWithCapacity: 1];
   if (aString != nil && [aString isKindOfClass: [NSAttributedString class]])
     {
-      NSAttributedString	*as = (NSAttributedString*)aString;
+      NSAttributedString *as = (NSAttributedString*)aString;
 
       aString = [as string];
       _setAttributesFrom(as, NSMakeRange(0, [aString length]), _infoArray);
     }
   else
     {
-      GSTextInfo	*info;
+      GSTextInfo *info;
 
       if (attributes == nil)
         {
@@ -525,9 +613,12 @@ _attributesAtIndexEffectiveRange(
 
 - (NSString*) string
 {
+  /* NB. This method is SUPPOSED to return a proxy to the mutable string!
+   * This is a performance feature documented ifor OSX.
+   */
   if (_textProxy == nil)
     {
-      _textProxy = RETAIN([_textChars immutableProxy]);
+      _textProxy = [[_textChars immutableProxy] retain];
     }
   return _textProxy;
 }
@@ -535,7 +626,7 @@ _attributesAtIndexEffectiveRange(
 - (NSDictionary*) attributesAtIndex: (unsigned)index
 		     effectiveRange: (NSRange*)aRange
 {
-  unsigned	dummy;
+  unsigned dummy;
 
   return _attributesAtIndexEffectiveRange(
     index, aRange, [_textChars length], _infoArray, &dummy);
@@ -828,9 +919,9 @@ changeInLength: [aString length] - range.length];
 
 - (void) dealloc
 {
-  TEST_RELEASE(_textProxy);
   RELEASE(_textChars);
   RELEASE(_infoArray);
+  RELEASE(_textProxy);
   [super dealloc];
 }
 
