@@ -27,7 +27,9 @@
 */
 
 #import <Foundation/NSBundle.h>
+#import <Foundation/NSDebug.h>
 #import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSInvocation.h>
 #import <Foundation/NSMapTable.h>
@@ -37,8 +39,10 @@
 #import <Foundation/NSPathUtilities.h>
 #import <Foundation/NSSet.h>
 #import <Foundation/NSUserDefaults.h>
+#import "GNUstepBase/GSObjCRuntime.h"
 #import "GNUstepGUI/GSTheme.h"
 #import "AppKit/NSApplication.h"
+#import "AppKit/NSButtonCell.h"
 #import "AppKit/NSButton.h"
 #import "AppKit/NSColor.h"
 #import "AppKit/NSColorList.h"
@@ -49,6 +53,7 @@
 #import "AppKit/NSMenu.h"
 #import "AppKit/NSPanel.h"
 #import "AppKit/NSScrollView.h"
+#import "AppKit/NSSegmentedControl.h"
 #import "AppKit/NSTextContainer.h"
 #import "AppKit/NSTextField.h"
 #import "AppKit/NSTextView.h"
@@ -69,6 +74,26 @@ NSString	*GSScrollerUpArrow = @"GSScrollerUpArrow";
 NSString	*GSScrollerVerticalKnob = @"GSScrollerVerticalKnob";
 NSString	*GSScrollerVerticalSlot = @"GSScrollerVerticalSlot";
 
+// Table view part names
+NSString	*GSTableHeader = @"GSTableHeader";
+NSString	*GSTableCorner = @"GSTableCorner";
+
+// Browser part names
+NSString        *GSBrowserHeader = @"GSBrowserHeader";
+
+// Menu part names
+NSString        *GSMenuHorizontalBackground = @"GSMenuHorizontalBackground";
+NSString        *GSMenuVerticalBackground = @"GSMenuVerticalBackground";
+NSString        *GSMenuHorizontalItem = @"GSMenuHorizontalItem";
+NSString        *GSMenuVerticalItem = @"GSMenuVerticalItem";
+
+// Progress indicator part names
+NSString        *GSProgressIndicatorBarDeterminate 
+  = @"GSProgressIndicatorBarDeterminate";
+
+// Color well part names
+NSString	*GSColorWell = @"GSColorWell";
+
 NSString	*GSThemeDidActivateNotification
   = @"GSThemeDidActivateNotification";
 NSString	*GSThemeDidDeactivateNotification
@@ -78,10 +103,136 @@ NSString	*GSThemeWillActivateNotification
 NSString	*GSThemeWillDeactivateNotification
   = @"GSThemeWillDeactivateNotification";
 
+NSString *
+GSThemeStringFromFillStyle(GSThemeFillStyle s)
+{
+  switch (s)
+    {
+      case GSThemeFillStyleNone: return @"None";
+      case GSThemeFillStyleScale: return @"Scale";
+      case GSThemeFillStyleRepeat: return @"Repeat";
+      case GSThemeFillStyleCenter: return @"Center";
+      case GSThemeFillStyleMatrix: return @"Matrix";
+      case GSThemeFillStyleScaleAll: return @"ScaleAll";
+    }
+  return nil;
+}
+
+GSThemeFillStyle
+GSThemeFillStyleFromString(NSString *s)
+{
+  if (s == nil || [s isEqualToString: @"None"])
+    {
+      return GSThemeFillStyleNone;
+    }
+  if ([s isEqualToString: @"Scale"])
+    {
+      return GSThemeFillStyleScale;
+    }
+  if ([s isEqualToString: @"Repeat"])
+    {
+      return GSThemeFillStyleRepeat;
+    }
+  if ([s isEqualToString: @"Center"])
+    {
+      return GSThemeFillStyleCenter;
+    }
+  if ([s isEqualToString: @"Matrix"])
+    {
+      return GSThemeFillStyleMatrix;
+    }
+  if ([s isEqualToString: @"ScaleAll"])
+    {
+      return GSThemeFillStyleScaleAll;
+    }
+  return GSThemeFillStyleNone;
+}
+
+NSString *
+GSStringFromSegmentStyle(NSSegmentStyle segmentStyle)
+{
+  switch (segmentStyle)
+    {
+      case NSSegmentStyleAutomatic:       return @"NSSegmentStyleAutomatic";
+      case NSSegmentStyleRounded:         return @"NSSegmentStyleRounded";
+      case NSSegmentStyleTexturedRounded: return @"NSSegmentStyleTexturedRounded";
+      case NSSegmentStyleRoundRect:       return @"NSSegmentStyleRoundRect";
+      case NSSegmentStyleTexturedSquare:  return @"NSSegmentStyleTexturedSquare";
+      case NSSegmentStyleCapsule:         return @"NSSegmentStyleCapsule";
+      case NSSegmentStyleSmallSquare:     return @"NSSegmentStyleSmallSquare";
+      default:                            return nil;
+    }
+}
+
+NSString *
+GSStringFromBezelStyle(NSBezelStyle bezelStyle)
+{
+  switch (bezelStyle)
+    {
+      case NSRoundedBezelStyle:           return @"NSRoundedBezelStyle";
+      case NSRegularSquareBezelStyle:     return @"NSRegularSquareBezelStyle";
+      case NSThickSquareBezelStyle:       return @"NSThickSquareBezelStyle";
+      case NSThickerSquareBezelStyle:     return @"NSThickerSquareBezelStyle";
+      case NSDisclosureBezelStyle:        return @"NSDisclosureBezelStyle";
+      case NSShadowlessSquareBezelStyle:  return @"NSShadowlessSquareBezelStyle";
+      case NSCircularBezelStyle:          return @"NSCircularBezelStyle";
+      case NSTexturedSquareBezelStyle:    return @"NSTexturedSquareBezelStyle";
+      case NSHelpButtonBezelStyle:        return @"NSHelpButtonBezelStyle";
+      case NSSmallSquareBezelStyle:       return @"NSSmallSquareBezelStyle";
+      case NSTexturedRoundedBezelStyle:   return @"NSTexturedRoundedBezelStyle";
+      case NSRoundRectBezelStyle:         return @"NSRoundRectBezelStyle";
+      case NSRecessedBezelStyle:          return @"NSRecessedBezelStyle";
+      case NSRoundedDisclosureBezelStyle: return @"NSRoundedDisclosureBezelStyle";
+      case NSNeXTBezelStyle:              return @"NSNeXTBezelStyle";
+      case NSPushButtonBezelStyle:        return @"NSPushButtonBezelStyle";
+      case NSSmallIconButtonBezelStyle:   return @"NSSmallIconButtonBezelStyle";
+      case NSMediumIconButtonBezelStyle:  return @"NSMediumIconButtonBezelStyle";
+      case NSLargeIconButtonBezelStyle:   return @"NSLargeIconButtonBezelStyle";
+      default:                            return nil;
+    }
+}
+
+NSString *
+GSStringFromBorderType(NSBorderType borderType)
+{
+  switch (borderType)
+    {
+      case NSNoBorder:                    return @"NSNoBorder";
+      case NSLineBorder:                  return @"NSLineBorder";
+      case NSBezelBorder:                 return @"NSBezelBorder";
+      case NSGrooveBorder:                return @"NSGrooveBorder";
+      default:                            return nil;
+    }
+}
+
+
+
+@interface	NSImage (GSTheme)
++ (NSImage*) _setImage: (NSImage*)image name: (NSString*)name;
+@end
+
 @interface	GSTheme (Private)
 - (void) _revokeOwnerships;
 @end
 
+/* This private internal class is used to store information about a method
+ * in some other class which is overridden while the current theme is
+ * active.
+ */
+@interface GSThemeMethod : NSObject
+{
+@public
+  Class		cls;
+  SEL		sel;
+  IMP		imp;	// The new method implementation
+  IMP		old;	// The original method implementation
+  Method	mth;	// The method information
+}
+@end
+
+@implementation	GSThemeMethod
+@end
+
 @implementation GSTheme
 
 static GSTheme			*defaultTheme = nil;
@@ -96,10 +247,14 @@ typedef	struct {
   NSColorList		*colors;
   NSColorList		*extraColors[GSThemeSelectedState+1];
   NSMutableDictionary	*images;
+  NSMutableDictionary	*oldImages;
   NSMutableDictionary	*tiles[GSThemeSelectedState+1];
   NSMutableSet		*owned;
   NSImage		*icon;
   NSString		*name;
+  Class			colorClass;
+  Class			imageClass;
+  NSMutableArray	*overrides;
 } internal;
 
 #define	_internal 		((internal*)_reserved)
@@ -107,10 +262,14 @@ typedef	struct {
 #define	_colors			_internal->colors
 #define	_extraColors		_internal->extraColors
 #define	_images			_internal->images
+#define	_oldImages		_internal->oldImages
 #define	_tiles			_internal->tiles
 #define	_owned			_internal->owned
 #define	_icon			_internal->icon
 #define	_name			_internal->name
+#define	_colorClass		_internal->colorClass
+#define	_imageClass		_internal->imageClass
+#define	_overrides		_internal->overrides
 
 + (void) defaultsDidChange: (NSNotification*)n
 {
@@ -187,7 +346,7 @@ typedef	struct {
   bundle = [themes objectForKey: theme];
   if (bundle == nil)
     {
-      NSString		*path;
+      NSString		*path = nil;
       NSFileManager	*mgr = [NSFileManager defaultManager];
       BOOL 		isDir;
 
@@ -260,10 +419,11 @@ typedef	struct {
   if (theme != theTheme)
     {
       [theTheme deactivate];
+      DESTROY(currentThemeName);
       ASSIGN (theTheme, theme);
       [theTheme activate];
+      ASSIGN(currentThemeName, [theTheme name]);
     }
-  ASSIGN(currentThemeName, [theTheme name]);
 }
 
 + (GSTheme*) theme 
@@ -283,6 +443,7 @@ typedef	struct {
   NSWindow		*window;
   GSThemeControlState	state;
   
+  NSDebugMLLog(@"GSTheme", @"%@ %p", [self name], self);
   /* Get rid of any cached colors list so that we regenerate it when needed
    */
   [_colors release];
@@ -298,7 +459,7 @@ typedef	struct {
    * to memory, setting their names so that they are visible to
    * [NSImage+imageNamed:] and storing them in our local array.
    */
-  imageTypes = [NSImage imageFileTypes];
+  imageTypes = [_imageClass imageFileTypes];
   imagePaths = [_bundle pathsForResourcesOfType: nil
 				    inDirectory: @"ThemeImages"];
   enumerator = [imagePaths objectEnumerator];
@@ -317,11 +478,43 @@ typedef	struct {
 	  image = [_images objectForKey: imageName];
 	  if (image == nil)
 	    {
-	      image = [[NSImage alloc] initWithContentsOfFile: imagePath];
-	      if (image != nil)
+	      image = [[_imageClass alloc] initWithContentsOfFile: imagePath];
+	      if (image == nil)
 		{
+		  NSLog(@"GSTheme failed to load %@ from %@",
+		    imageName, imagePath);
+		}
+	      else
+		{
+		  NSImage	*old;
+
+		  /* OK, we have loaded a new image, so we cache it for the
+		   * lifetime of the theme, and we also make a record of
+		   * any previous/default image of the same name.
+		   */
 		  [_images setObject: image forKey: imageName];
 		  RELEASE(image);
+		  old = [NSImage imageNamed: imageName];
+		  if (old == nil)
+		    {
+		      /* This could potentially be a real problem ... if the
+		       * image from the current theme with this name is used
+		       * and the theme is unloaded, what happens when the
+		       * app tries to draw using the proxy to the unloaded
+		       * image?
+		       * To avoid that possibility, we save the new image
+		       * as if it were the old one ... so when the theme is
+		       * unloaded the old image persists.
+		       */
+		      [_oldImages setObject: image forKey: imageName];
+		    }
+		  else
+		    {
+		      /* Store the actual image, not the proxy.
+		       */
+		      old = [(id)old _resource];
+		      [_oldImages setObject: old forKey: imageName];
+		    }
 		}
 	    }
 
@@ -329,27 +522,11 @@ typedef	struct {
 	   */
 	  if (image != nil && [[image name] isEqualToString: imageName] == NO)
 	    {
-	      if ([image setName: imageName] == NO)
-	        {
-		  NSImage	*old;
-
-		  /* Couldn't set image name ... presumably already
-		   * in use ... so we remove the name from the old
-		   * image and try again.
-		   */
-		  old = [NSImage imageNamed: imageName];
-		  [old setName: nil];
-	          [image setName: imageName];
-		}
+	      [NSImage _setImage: image name: imageName];
 	    }
 	}
     }
 
-  /*
-   * We could cache tile info here, but it's probably better for the
-   * tilesNamed:state:cache: method to do it lazily.
-   */
-
   /*
    * Use the GSThemeDomain key in the info dictionary of the theme to
    * set a defaults domain which will establish user defaults values
@@ -395,8 +572,21 @@ typedef	struct {
     }
   RELEASE(searchList);
 
+  /* Install any overridden methods.
+   */
+  if (_overrides != nil)
+    {
+      NSEnumerator	*e = [_overrides objectEnumerator];
+      GSThemeMethod	*m;
+
+      while ((m = [e nextObject]) != nil)
+	{
+	  method_setImplementation(m->mth, m->imp);
+	}
+    }
+
   /*
-   * Tell all other classes that new theme information is about to be present.
+   * Tell subclass that basic activation is done and it can do its own.
    */
   [[NSNotificationCenter defaultCenter]
     postNotificationName: GSThemeWillActivateNotification
@@ -436,21 +626,48 @@ typedef	struct {
   return _bundle;
 }
 
+- (Class) colorClass
+{
+  return [NSColorList class];
+}
+
+- (void) colorFlush: (NSString*)aName
+	      state: (GSThemeControlState)elementState
+{
+  int	pos;
+  int	end;
+
+  if (elementState < 0 || elementState > GSThemeSelectedState)
+    {
+      pos = 0;
+      end = GSThemeSelectedState;
+    }
+  else
+    {
+      pos = elementState;
+      end = elementState;
+    }
+  while (pos <= end)
+    {
+      if (_extraColors[pos] != nil)
+	{
+	  [_extraColors[pos] release];
+	  _extraColors[pos] = nil;
+	}
+      pos++;
+    }
+}
+
 - (NSColor*) colorNamed: (NSString*)aName
 		  state: (GSThemeControlState)elementState
-	          cache: (BOOL)useCache
 {
   NSColor	*c = nil;
 
   NSAssert(elementState <= GSThemeSelectedState, NSInvalidArgumentException);
+  NSAssert(elementState >= 0, NSInvalidArgumentException);
 
   if (aName != nil)
     {
-      if (useCache == NO)
-	{
-	  [_extraColors[elementState] release];
-	  _extraColors[elementState] = nil;
-	}
       if (_extraColors[elementState] == nil)
 	{
 	  NSString	*colorsPath;
@@ -462,6 +679,7 @@ typedef	struct {
 	   */
 	  switch (elementState)
 	    {
+              default:
 	      case GSThemeNormalState:
 		listName = @"ThemeExtra";
 		break;
@@ -478,7 +696,7 @@ typedef	struct {
 	  if (colorsPath != nil)
 	    {
 	      _extraColors[elementState]
-		= [[NSColorList alloc] initWithName: listName
+		= [[_colorClass alloc] initWithName: listName
 					   fromFile: colorsPath];
 	      /* If the list is actually empty, we get rid of it to avoid
 	       * unnecessary lookups.
@@ -515,7 +733,7 @@ typedef	struct {
 	}
       else
 	{
-	  _colors = [[NSColorList alloc] initWithName: @"System"
+	  _colors = [[_colorClass alloc] initWithName: @"System"
 					     fromFile: colorsPath];
 	}
     }
@@ -530,6 +748,9 @@ typedef	struct {
 {
   NSEnumerator	*enumerator;
   NSImage	*image;
+  NSString	*name;
+
+  NSDebugMLLog(@"GSTheme", @"%@ %p", [self name], self);
 
   /* Tell everything that we will become inactive.
    */
@@ -538,11 +759,20 @@ typedef	struct {
     object: self
     userInfo: nil];
 
-  /*
-   * Remove all cached bundle images from both NSImage's name dictionary
-   * and our cache dictionary, so that we can be sure we reload afresh
-   * when re-activated (in case the images on disk changed ... eg by a
-   * theme editor modifying the theme).
+  /* Remove any overridden methods.
+   */
+  if (_overrides != nil)
+    {
+      NSEnumerator	*e = [_overrides objectEnumerator];
+      GSThemeMethod	*m;
+
+      while ((m = [e nextObject]) != nil)
+	{
+	  method_setImplementation(m->mth, m->old);
+	}
+    }
+
+  /* Unload all images created by this theme.
    */
   enumerator = [_images objectEnumerator];
   while ((image = [enumerator nextObject]) != nil)
@@ -551,6 +781,21 @@ typedef	struct {
     }
   [_images removeAllObjects];
 
+  /* Restore old images in NSImage's lookup dictionary so that the app
+   * still has images to draw.
+   * The remove all cached bundle images from both NSImage's name dictionary
+   * and our cache dictionary, so that we can be sure we reload afresh
+   * when re-activated (in case the images on disk changed ... eg by a
+   * theme editor modifying the theme).
+   */
+  enumerator = [_oldImages keyEnumerator];
+  while ((name = [enumerator nextObject]) != nil)
+    {
+      image = [_oldImages objectForKey: name];
+      [NSImage _setImage: image name: name];
+    }
+  [_oldImages removeAllObjects];
+
   [self _revokeOwnerships];
 
   /* Tell everything that we have become inactive.
@@ -575,8 +820,10 @@ typedef	struct {
       RELEASE(_bundle);
       RELEASE(_colors);
       RELEASE(_images);
+      RELEASE(_oldImages);
       RELEASE(_icon);
       [self _revokeOwnerships];
+      RELEASE(_overrides);
       RELEASE(_owned);
       NSZoneFree ([self zone], _reserved);
     }
@@ -598,12 +845,12 @@ typedef	struct {
 	  path = [_bundle pathForResource: path ofType: ext];
 	  if (path != nil)
 	    {
-	      _icon = [[NSImage alloc] initWithContentsOfFile: path];
+	      _icon = [[_imageClass alloc] initWithContentsOfFile: path];
 	    }
 	}
       if (_icon == nil)
         {
-	  _icon = RETAIN([NSImage imageNamed: @"GNUstep"]);
+	  _icon = RETAIN([_imageClass imageNamed: @"GNUstep"]);
 	}
       else
 	{
@@ -626,14 +873,24 @@ typedef	struct {
   return _icon;
 }
 
+- (Class) imageClass
+{
+  return [NSImage class];
+}
+
 - (id) initWithBundle: (NSBundle*)bundle
 {
-  GSThemeControlState	state;
+  Class				c = [self class];
+  unsigned int			count;
+  Method			*methods;
+  GSThemeMethod			*mth;
+  GSThemeControlState		state;
 
   _reserved = NSZoneCalloc ([self zone], 1, sizeof(internal));
 
   ASSIGN(_bundle, bundle);
   _images = [NSMutableDictionary new];
+  _oldImages = [NSMutableDictionary new];
   for (state = 0; state <= GSThemeSelectedState; state++)
     {
       _tiles[state] = [NSMutableDictionary new];
@@ -643,6 +900,130 @@ typedef	struct {
   ASSIGN(_name,
     [[[_bundle bundlePath] lastPathComponent] stringByDeletingPathExtension]);
 
+  _colorClass = [self colorClass];
+  _imageClass = [self imageClass];
+
+  /* Now we look through our methods to find those which are actually
+   * replacements to override methods in other classes.
+   * That's determined by method name ... any method of the form
+   * '_override' <classname> 'Method_' <originalmethodname>
+   * is used to replace the original method in the class.
+   * We maintain dictionaries (keyed by class) for instance and class
+   * methods, so we can look up the original methods at runtime if the
+   * replacement methods want to call them.
+   */
+  methods = class_copyMethodList(c, &count);
+  if (methods != NULL)
+    {
+      int counter = 0;
+
+      while (methods[counter] != 0)
+	{
+          Method	method = methods[counter++];
+	  const char	*name = sel_getName(method_getName(method));
+	  const char	*ptr;
+
+	  if (strncmp(name, "_override", 9) == 0
+	    && (ptr = strstr(name, "Method_")) > 0)
+	    {
+	      char		buf[strlen(name)];
+	      const char	*types;
+
+	      mth = [[GSThemeMethod new] autorelease];
+	      types = method_getTypeEncoding(method);
+	      mth->imp = method_getImplementation(method);
+	      memcpy(buf, name + 9, (ptr - name) + 9);
+	      buf[(ptr - name) + 9] = '\0';
+	      mth->cls = objc_lookUpClass(buf);
+	      if (mth->cls == 0)
+		{
+		  NSLog(@"Unable to find class '%s' for '%s'", buf, name);
+		  continue;
+		}
+	      memcpy(buf, ptr + 7, strlen(ptr + 7));
+	      buf[strlen(ptr + 7)] = '\0';
+	      mth->sel = sel_getUid(buf);
+	      if (mth->sel == 0)
+		{
+		  NSLog(@"Unable to find selector '-%s' for '%s'", buf, name);
+		  continue;
+		}
+	      if (NO == [mth->cls instancesRespondToSelector: mth->sel])
+		{
+		  NSLog(@"Instances do not respond for '%s'", name);
+		  continue;
+		}
+	      mth->old = [mth->cls instanceMethodForSelector: mth->sel];
+	      class_addMethod(mth->cls, mth->sel, mth->imp, types);
+	      mth->mth = class_getInstanceMethod(mth->cls, mth->sel);
+
+	      if (_overrides == nil)
+		{
+		  _overrides = [NSMutableArray new];
+		}
+	      [_overrides addObject: mth];
+	    }
+	}
+      free(methods);
+    }
+
+  methods = class_copyMethodList(object_getClass(c), &count);
+  if (methods != NULL)
+    {
+      int counter = 0;
+
+      while (methods[counter] != 0)
+	{
+          Method	method = methods[counter++];
+	  const char	*name = sel_getName(method_getName(method));
+	  const char	*ptr;
+
+	  if (strncmp(name, "_override", 9) == 0
+	    && (ptr = strstr(name, "Method_")) > 0)
+	    {
+	      char		buf[strlen(name)];
+	      const char	*types;
+	      Class		cls;
+
+	      mth = [[GSThemeMethod new] autorelease];
+	      types = method_getTypeEncoding(method);
+		      mth->imp = method_getImplementation(method);
+	      memcpy(buf, name + 9, (ptr - name) + 9);
+	      buf[(ptr - name) + 9] = '\0';
+	      cls = objc_lookUpClass(buf);
+	      if (cls == 0)
+		{
+		  NSLog(@"Unable to find class '%s' for '%s'", buf, name);
+		  continue;
+		}
+	      mth->cls = object_getClass(cls);
+	      memcpy(buf, ptr + 7, strlen(ptr + 7));
+	      buf[strlen(ptr + 7)] = '\0';
+	      mth->sel = sel_getUid(buf);
+	      if (mth->sel == 0)
+		{
+		  NSLog(@"Unable to find selector '-%s' for '%s'", buf, name);
+		  continue;
+		}
+	      if (NO == [cls respondsToSelector: mth->sel])
+		{
+		  NSLog(@"Class does not respond for '%s'", name);
+		  continue;
+		}
+	      mth->old = [cls methodForSelector: mth->sel];
+	      class_addMethod(mth->cls, mth->sel, mth->imp, types);
+	      mth->mth = class_getClassMethod(cls, mth->sel);
+
+	      if (_overrides == nil)
+		{
+		  _overrides = [NSMutableArray new];
+		}
+	      [_overrides addObject: mth];
+	    }
+	}
+      free(methods);
+    }
+
   return self;
 }
 
@@ -667,6 +1048,22 @@ typedef	struct {
   return name;
 }
 
+- (IMP) overriddenMethod: (SEL)selector for: (id)receiver
+{
+  Class		cls = object_getClass(receiver);
+  NSEnumerator	*e = [_overrides objectEnumerator];
+  GSThemeMethod	*m;
+
+  while ((m = [e nextObject]) != nil)
+    {
+      if (m->cls == cls && sel_isEqual(selector, m->sel))
+	{
+	  return m->old;
+	}
+    }
+  return (IMP)0;
+}
+
 - (void) setName: (NSString*)aString
 {
   if (self != defaultTheme)
@@ -717,20 +1114,52 @@ typedef	struct {
   return [GSThemeInspector sharedThemeInspector];
 }
 
+- (void) tilesFlush: (NSString*)aName
+	      state: (GSThemeControlState)elementState
+{
+  int	pos;
+  int	end;
+
+  if (elementState < 0 || elementState > GSThemeSelectedState)
+    {
+      pos = 0;
+      end = GSThemeSelectedState;
+    }
+  else
+    {
+      pos = elementState;
+      end = elementState;
+    }
+  while (pos <= end)
+    {
+      NSMutableDictionary	*cache;
+
+      cache = _tiles[pos++];
+      if (aName == nil)
+	{
+	  return [cache removeAllObjects];
+	}
+      else
+	{
+	  [cache removeObjectForKey: aName];
+	}
+    }
+}
+
 - (GSDrawTiles*) tilesNamed: (NSString*)aName
 		      state: (GSThemeControlState)elementState
-		      cache: (BOOL)useCache
 {
   GSDrawTiles		*tiles;
   NSMutableDictionary	*cache;
 
   NSAssert(elementState <= GSThemeSelectedState, NSInvalidArgumentException);
+  NSAssert(elementState >= 0, NSInvalidArgumentException);
   if (aName == nil)
     {
       return nil;
     }
   cache = _tiles[elementState];
-  tiles = (useCache == YES) ? [cache objectForKey: aName] : nil;
+  tiles = [cache objectForKey: aName];
   if (tiles == nil)
     {
       NSDictionary	*info;
@@ -739,9 +1168,13 @@ typedef	struct {
 
       switch (elementState)
 	{
+          default:
 	  case GSThemeNormalState:
 	    fullName = aName;
 	    break;
+	  case GSThemeDisabledState:
+	    fullName = [aName stringByAppendingString: @"Disabled"];
+	    break;
 	  case GSThemeHighlightedState:
 	    fullName = [aName stringByAppendingString: @"Highlighted"];
 	    break;
@@ -761,12 +1194,17 @@ typedef	struct {
       info = [[info objectForKey: @"GSThemeTiles"] objectForKey: fullName];
       if ([info isKindOfClass: [NSDictionary class]] == YES)
         {
-	  float		x;
-	  float		y;
-	  NSString	*path;
-	  NSString	*file;
-	  NSString	*ext;
+	  float			x;
+	  float			y;
+	  NSString		*name;
+	  NSString		*path;
+	  NSString		*file;
+	  NSString		*ext;
+	  GSThemeFillStyle	style;
 
+	  name = [info objectForKey: @"FillStyle"];
+	  style = GSThemeFillStyleFromString(name);
+	  if (style < GSThemeFillStyleNone) style = GSThemeFillStyleNone;
 	  x = [[info objectForKey: @"HorizontalDivision"] floatValue];
 	  y = [[info objectForKey: @"VerticalDivision"] floatValue];
 	  file = [info objectForKey: @"FileName"];
@@ -781,12 +1219,22 @@ typedef	struct {
 	    }
 	  else
 	    {
-	      image = [[NSImage alloc] initWithContentsOfFile: path];
+	      image = [[_imageClass alloc] initWithContentsOfFile: path];
 	      if (image != nil)
 		{
-		  tiles = [[GSDrawTiles alloc] initWithImage: image
-						  horizontal: x
-						    vertical: y];
+                  if ([[info objectForKey: @"NinePatch"] boolValue])
+                    {
+                      tiles = [[GSDrawTiles alloc]
+			initWithNinePatchImage: image];
+		      [tiles setFillStyle: GSThemeFillStyleScaleAll];
+                    }
+                  else
+                    {
+		      tiles = [[GSDrawTiles alloc] initWithImage: image
+                                                      horizontal: x
+                                                        vertical: y];
+		      [tiles setFillStyle: style];
+                    }
 		  RELEASE(image);
 		}
 	    }
@@ -797,7 +1245,7 @@ typedef	struct {
 	  NSString	*imagePath;
 	  unsigned	count;
 
-	  imageTypes = [NSImage imageFileTypes];
+	  imageTypes = [_imageClass imageFileTypes];
 	  for (count = 0; count < [imageTypes count]; count++)
 	    {
 	      NSString	*ext = [imageTypes objectAtIndex: count];
@@ -807,7 +1255,8 @@ typedef	struct {
 				       inDirectory: @"ThemeTiles"];
 	      if (imagePath != nil)
 		{
-		  image = [[NSImage alloc] initWithContentsOfFile: imagePath];
+		  image
+		    = [[_imageClass alloc] initWithContentsOfFile: imagePath];
 		  if (image != nil)
 		    {
 		      tiles = [[GSDrawTiles alloc] initWithImage: image];
@@ -835,6 +1284,11 @@ typedef	struct {
   return tiles;
 }
 
+- (NSString*) versionString
+{
+  return [[self infoDictionary] objectForKey: @"GSThemeVersion"];
+}
+
 @end
 
 @implementation	GSTheme (Private)
@@ -865,6 +1319,14 @@ typedef	struct {
   DESTROY(_resource);
   [super dealloc];
 }
+- (NSString*) description
+{
+  return [_resource description];
+}
+- (id) forwardingTargetForSelector:(SEL)aSelector
+{
+  return _resource;
+}
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
   [anInvocation invokeWithTarget: _resource];
