@@ -24,18 +24,18 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "GSWindowDecorationView.h"
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
 
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
+#import <GNUstepGUI/GSWindowDecorationView.h>
+#import "AppKit/NSColor.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSDisplayServer.h"
+#import "GNUstepGUI/GSTheme.h"
 
-#include "AppKit/NSColor.h"
-#include "AppKit/NSMenuView.h"
-#include "AppKit/NSWindow.h"
-#include "GNUstepGUI/GSDisplayServer.h"
-#include "GNUstepGUI/GSTheme.h"
-
-#include "NSToolbarFrameworkPrivate.h"
+#import "NSToolbarFrameworkPrivate.h"
 
 @implementation GSWindowDecorationView
 
@@ -108,6 +108,8 @@
   self = [super initWithFrame: frame];
   if (self != nil)
     {
+      hasToolbar = NO;
+      hasMenu = NO;
       window = w;
       // Content rect will be everything apart from the border
       // that is including menu, toolbar and the like.
@@ -117,6 +119,26 @@
   return self;
 }
 
+- (void) setHasMenu: (BOOL) flag
+{
+  hasMenu = flag;
+}
+
+- (BOOL) hasMenu
+{
+  return hasMenu;
+}
+
+- (void) setHasToolbar: (BOOL) flag
+{
+  hasToolbar = flag;
+}
+
+- (BOOL) hasToolbar
+{
+  return hasToolbar;
+}
+
 - (NSRect) contentRectForFrameRect: (NSRect)aRect
                          styleMask: (unsigned int)aStyle
 {
@@ -126,7 +148,11 @@
 
   if ([_window menu] != nil)
     {
-      content.size.height -= [NSMenuView menuBarHeight];
+      float	menubarHeight = [[GSTheme theme] 
+					 menuHeightForWindow: 
+				    _window];
+  
+      content.size.height -= menubarHeight;
     }
 
   if ([tb isVisible])
@@ -146,7 +172,11 @@
 
   if ([_window menu] != nil)
     {
-      aRect.size.height += [NSMenuView menuBarHeight];
+      float	menubarHeight = [[GSTheme theme] 
+				  menuHeightForWindow: 
+				    _window];
+
+      aRect.size.height += menubarHeight;
     }
 
   if ([tb isVisible])
@@ -183,17 +213,9 @@
 
 - (void) setContentView: (NSView *)contentView
 {
-  NSSize oldSize;
-
   [contentView setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
   [self addSubview: contentView];
-
-  oldSize = [contentView frame].size;
-  oldSize.width += _frame.size.width - contentRect.size.width;
-  oldSize.height += _frame.size.height - contentRect.size.height;
-  [contentView resizeWithOldSuperviewSize: oldSize];
-  [contentView setFrameOrigin: NSMakePoint(contentRect.origin.x,
-					   contentRect.origin.y)];
+  [self layout];
 }
 
 - (void) setDocumentEdited: (BOOL)flag
@@ -209,16 +231,18 @@
   NSRect contentViewFrame;
   NSToolbar *tb = [_window toolbar];
   NSRect frame = [window frame];
+  NSView *windowContentView = [_window contentView];
 
   frame.origin = NSZeroPoint;
   contentViewFrame = [isa contentRectForFrameRect: frame
                           styleMask: [window styleMask]];
 
-  if ([_window menu] != nil)
+  if (hasMenu)
     {
       NSMenuView *menuView;
-      float menuBarHeight = [NSMenuView menuBarHeight];
-
+      GSTheme *theme = [GSTheme theme];
+      float menuBarHeight = [theme menuHeightForWindow: _window];
+      
       menuView = [[_window menu] menuRepresentation];
       [menuView setFrame: NSMakeRect(
               contentViewFrame.origin.x,
@@ -228,26 +252,30 @@
       contentViewFrame.size.height -= menuBarHeight;
     }
 
-  if ([tb isVisible])
+  if (hasToolbar)
     {
       GSToolbarView *tv = [tb _toolbarView];
       float newToolbarViewHeight;
-      
+	  
       // If the width changed we may need to recalculate the height
       if (contentViewFrame.size.width != [tv frame].size.width)
-        {
-          [tv setFrameSize: NSMakeSize(contentViewFrame.size.width, 100)];
-          // Will recalculate the layout
-          [tv _reload];
-        }
+	{
+	  [tv setFrameSize: NSMakeSize(contentViewFrame.size.width, 100)];
+	  // Will recalculate the layout
+	  [tv _reload];
+	}
       newToolbarViewHeight = [tv _heightFromLayout];
-      [tv setFrame: NSMakeRect(
-              contentViewFrame.origin.x,
-              NSMaxY(contentViewFrame) - newToolbarViewHeight, 
-              contentViewFrame.size.width, 
-              newToolbarViewHeight)];
+      [tv setFrame: NSMakeRect(contentViewFrame.origin.x,
+			       NSMaxY(contentViewFrame) - newToolbarViewHeight,
+			       contentViewFrame.size.width, 
+			       newToolbarViewHeight)];
       contentViewFrame.size.height -= newToolbarViewHeight;
     }
+
+  if ([windowContentView superview] == self)
+    {
+      [windowContentView setFrame:contentViewFrame];
+    }
 }
 
 - (void) changeWindowHeight: (float)difference
@@ -273,6 +301,7 @@
 
   // then resize the window
   [window setFrame: windowFrame display: YES];
+  [self layout];
 }
 
 /*
@@ -331,6 +360,9 @@
 {
   if (NSIntersectsRect(rect, contentRect))
     {
+      // Since this is the outermost view, we have to clear the contentRect
+      // in case the theme's window background is not opaque.
+      NSRectFillUsingOperation(contentRect, NSCompositeClear);  
       [[GSTheme theme] drawWindowBackground: contentRect view: self];
     }
 }
@@ -353,16 +385,26 @@
 - (void) addToolbarView: (GSToolbarView*)toolbarView
 {
   float newToolbarViewHeight;
+  float contentYOrigin;
 
+  hasToolbar = YES;
   [toolbarView setFrameSize: NSMakeSize(contentRect.size.width, 100)];
   // Will recalculate the layout
   [toolbarView _reload];
   newToolbarViewHeight = [toolbarView _heightFromLayout];
-  
+
+  // take in account of the menubar when calculating the origin
+  contentYOrigin = NSMaxY(contentRect);
+  if (hasMenu)
+    {
+      float menuBarHeight = [[GSTheme theme] menuHeightForWindow: _window];
+      contentYOrigin -= menuBarHeight;
+    }
+
   // Plug the toolbar view
   [toolbarView setFrame: NSMakeRect(
           contentRect.origin.x,
-          NSMaxY(contentRect), 
+          contentYOrigin, 
           contentRect.size.width, 
           newToolbarViewHeight)];
   [self addSubview: toolbarView];
@@ -375,8 +417,9 @@
   float toolbarViewHeight = [toolbarView frame].size.height;
 
   // Unplug the toolbar view
+  hasToolbar = NO;
   [toolbarView removeFromSuperviewWithoutNeedingDisplay];
-  
+
   [self changeWindowHeight: -toolbarViewHeight];  
 }
 
@@ -405,8 +448,10 @@
 
 - (void) addMenuView: (NSMenuView*)menuView
 {
-  float	menubarHeight = [NSMenuView menuBarHeight];
-  
+  float	menubarHeight = [[GSTheme theme] 
+			  menuHeightForWindow: 
+			    _window];
+  hasMenu = YES;
   // Plug the menu view
   [menuView setFrame: NSMakeRect(
           contentRect.origin.x,
@@ -422,22 +467,26 @@
 {
   NSEnumerator	*e = [[self subviews] objectEnumerator];
   NSView	*v;
-
+  float	menubarHeight = [[GSTheme theme] 
+			  menuHeightForWindow: 
+			    _window];
+  
   while ((v = [e nextObject]) != nil)
     {
       if ([v isKindOfClass: [NSMenuView class]] == YES)
 	{
-	  float	menubarHeight = [NSMenuView menuBarHeight];
-
 	  /* Unplug the menu view and return it so that it can be
-           * restored to its original menu if necessary.
+	   * restored to its original menu if necessary.
 	   */
+
+	  hasMenu = NO;
 	  [RETAIN(v) removeFromSuperviewWithoutNeedingDisplay];
-  
-	  [self changeWindowHeight: -menubarHeight];  
+	  
+	  [self changeWindowHeight: -(menubarHeight)];  
 	  return AUTORELEASE(v);
 	}
     }
+
   return nil;
 }
 
