@@ -34,31 +34,42 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
+#import "config.h"
 
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSError.h>
-#include <Foundation/NSString.h>
-#include "AppKit/NSAlert.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSBox.h"
-#include "AppKit/NSButton.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSHelpManager.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSPanel.h"
-#include "AppKit/NSScreen.h"
-#include "AppKit/NSScrollView.h"
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSError.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSThread.h>
+#import "AppKit/NSAlert.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSBox.h"
+#import "AppKit/NSBrowser.h"
+#import "AppKit/NSBrowserCell.h"
+#import "AppKit/NSButton.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSHelpManager.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMatrix.h"
+#import "AppKit/NSPanel.h"
+#import "AppKit/NSScreen.h"
+#import "AppKit/NSScroller.h"
+#import "AppKit/NSScrollView.h"
+#import <AppKit/NSStringDrawing.h>
+#import "AppKit/NSTextField.h"
+#import "AppKit/NSTextView.h"
 
-#include "GNUstepGUI/IMLoading.h"
-#include "GNUstepGUI/GMAppKit.h"
-#include "GNUstepGUI/GMArchiver.h"
-#include "GSGuiPrivate.h"
+#import "GNUstepGUI/GMAppKit.h"
+#import "GNUstepGUI/GMArchiver.h"
+#import "GSGuiPrivate.h"
 
 extern NSThread *GSAppKitThread;
 
+static NSNotificationCenter *nc = nil;
+
 #ifdef ALERT_TITLE
 static NSString	*defaultTitle = @"Alert";
 #else
@@ -158,9 +169,15 @@ static NSString	*defaultTitle = @" ";
     The width of the scroll is the rest of the width of the content rect
     minus the margins.
 
-    ((wsize.width <= ssize.width)
+    In order to prevent alert panels from obscuring the whole screen, we
+    limit the width and height of a panel to at most a factor of SIZE_SCALE
+    of the screen size. At present, we use 60% for SIZE_SCALE, which means
+    that the limit is greater than the minimum width and height even on a
+    640x400 screen.
+
+    ((wsize.width <= ssize.width * SIZE_SCALE)
      and ([messageField frame].size.width+2*MessageHorzMargin <= wsize.width))
-    or ((wsize.width == ssize.width)
+    or ((wsize.width == ssize.width * SIZE_SCALE)
         and ([scroll frame].size.width = wsize.width-2*MessageHorzMargin));
 
     ...
@@ -226,6 +243,7 @@ static const float ButtonInterspace = 10.0;
 static const float ButtonMinHeight = 24.0;
 static const float ButtonMinWidth = 72.0;
 
+#define SIZE_SCALE 0.6
 #define MessageFont [NSFont messageFontOfSize: 14]
 
 + (void) initialize
@@ -280,7 +298,7 @@ makeScrollViewWithRect(NSRect rect)
 
   [scroll setBorderType: NSLineBorder];
   [scroll setBackgroundColor: [NSColor controlBackgroundColor]];
-  [scroll setHasHorizontalScroller: YES];
+  [scroll setHasHorizontalScroller: NO];
   [scroll setHasVerticalScroller: YES];
   [scroll setScrollsDynamically: YES];
   [scroll setLineScroll: lineHeight];
@@ -474,6 +492,9 @@ setControl(NSView* content, id control, NSString *title)
   bounds = [screen frame];
   bounds = [NSWindow contentRectForFrameRect: bounds styleMask: mask];
   ssize = bounds.size;
+  /* Do not let a panel grow beyond a factor of SIZE_SCALE of the screen */
+  ssize.width = SIZE_SCALE * ssize.width;
+  ssize.height = SIZE_SCALE * ssize.height;
 
   // Let's size the title.
   if (useControl(titleField))
@@ -547,8 +568,17 @@ setControl(NSView* content, id control, NSString *title)
 	{
 	  wsize.width = width;
 	}
-      // The title could be large too, without implying a scroll view.
-      needsScroll = (ssize.width < width);
+
+      /* If the message is too wide, wrap its text. Apparently, we cannot
+	 use -sizeToFit to compute the height of the message. */
+      [messageField setAlignment: NSLeftTextAlignment];
+      width = ssize.width - 2*MessageHorzMargin;
+      rect.size =
+	[[messageField attributedStringValue]
+	  boundingRectWithSize: NSMakeSize(width, 1e6)
+		       options: 0].size;
+      [messageField setFrame: rect];
+
       /*
        * But only the messageField can impose a great height, therefore
        * we check it along in the next paragraph.
@@ -619,6 +649,7 @@ setControl(NSView* content, id control, NSString *title)
       if (needsScroll)
 	{
 	  NSRect	srect;
+	  float		width;
 
 	  // The scroll view takes all the space that is available.
 	  srect.origin.x = bounds.origin.x + MessageHorzMargin;
@@ -639,17 +670,23 @@ setControl(NSView* content, id control, NSString *title)
 	    {
 	      [content addSubview: scroll];
 	    }
+
+	  /* Adjust the message field's width again so that it does not
+	     exceed the scroll view's visible rectangle and we do not
+	     need a horizontal scroller. */
 	  [messageField removeFromSuperview];
-	  mrect.origin.x
-	    = srect.origin.x + srect.size.width - mrect.size.width;
-	  mrect.origin.y
-	    = srect.origin.y + srect.size.height - mrect.size.height;
+	  width =
+	    [NSScrollView contentSizeForFrameSize: srect.size
+			    hasHorizontalScroller: NO
+			      hasVerticalScroller: YES
+				       borderType: [scroll borderType]].width;
+	  mrect.origin = NSZeroPoint;
+	  mrect.size =
+	    [[messageField attributedStringValue]
+	      boundingRectWithSize: NSMakeSize(width, 1e6)
+			   options: 0].size;
 	  [messageField setFrame: mrect];
 	  [scroll setDocumentView: messageField];
-	  [[scroll contentView] scrollToPoint:
-	    NSMakePoint(mrect.origin.x, mrect.origin.y + mrect.size.height
-	      - [[scroll contentView] bounds].size.height)];
-	  [scroll reflectScrolledClipView: [scroll contentView]];
 	}
       else
 	{
@@ -657,7 +694,7 @@ setControl(NSView* content, id control, NSString *title)
 
 	  /*
 	   * We must center vertically the messageField because
-	   * the window has a minimum size, thus may be greated
+	   * the window has a minimum size, thus may be greater
 	   * than expected.
 	   */
 	  mrect.origin.x = (wsize.width - mrect.size.width)/2;
@@ -892,6 +929,156 @@ setControl(NSView* content, id control, NSString *title)
 
 @end /* GSAlertPanel GMArchiverMethods */
 
+/*
+  GSAlertSheet.  This class provides a borderless window which is
+  attached to the parent window.
+ */
+
+@interface GSAlertSheet : GSAlertPanel
+@end
+
+@implementation GSAlertSheet
++ (void) initialize
+{
+  if (self == [GSAlertSheet class])
+    {
+      if (nc == nil)
+	{
+	  nc = [NSNotificationCenter defaultCenter];
+	}
+      [self setVersion: 0];
+    }
+}
+
+
+- (id) initWithContentRect: (NSRect)contentRect
+                 styleMask: (unsigned int)aStyle
+                   backing: (NSBackingStoreType)bufferingType
+                     defer: (BOOL)flag
+                    screen: (NSScreen*)aScreen
+{
+  if(NSIsEmptyRect(contentRect))
+    {
+      contentRect = NSMakeRect(0,0,100,100);
+    }
+
+  self = [super initWithContentRect: contentRect
+		styleMask: NSBorderlessWindowMask
+		backing: bufferingType
+		defer: flag
+		screen: aScreen];
+
+  if (self != nil)
+    {
+      // 
+    }
+  return self;
+}
+
+- (NSRect) frameFromParentWindowFrame
+{
+  id parent = [self parentWindow];
+  NSRect frame = [self frame];
+  NSRect newFrame = NSZeroRect;  // return zero rect, if parent isn't set.
+  
+  if(parent != nil)
+    {
+      NSRect contentRect = [[parent contentView] frame];
+      
+      //
+      // The calculation is based on the contentRect of the parent window
+      // since we want the sheet to appear just inside of it.
+      //
+      newFrame = [parent frame];
+      newFrame.origin.x += ((newFrame.size.width - frame.size.width) / 2); 
+      newFrame.origin.y += (contentRect.size.height - frame.size.height) + 5;
+    }
+
+  return newFrame;
+}
+
+- (void) resetWindow
+{
+  NSRect frame = [self frameFromParentWindowFrame]; 
+  NSWindow *parent = nil;
+
+  if((parent = [self parentWindow]) != nil)
+    {
+      [self setBackgroundColor: 
+	      [[parent backgroundColor] 
+		highlightWithLevel: 0.4]];
+    }
+
+  [self setFrame: frame display: YES];
+}
+
+- (void) setParentWindow: (NSWindow *)window
+{
+  [super setParentWindow: window];
+  [self resetWindow];
+  /*
+  [nc removeObserver: self];
+
+  if (parent != nil)
+    {
+      // add observers....
+      [nc addObserver: self
+	selector: @selector(handleWindowClose:)
+	name: NSWindowWillCloseNotification
+	object: parent];
+	
+      [nc addObserver: self
+	selector: @selector(handleWindowMiniaturize:)
+	name: NSWindowWillMiniaturizeNotification
+	object: parent];
+	
+      [nc addObserver: self
+	selector: @selector(handleWindowMove:)
+	name: NSWindowWillMoveNotification
+	object: parent];
+	
+      [nc addObserver: self
+	selector: @selector(handleWindowMove:)
+	name: NSWindowDidResizeNotification
+	object: parent];
+	
+      [nc addObserver: self
+	selector: @selector(handleWindowDidBecomeKey:)
+	name: NSWindowDidBecomeKeyNotification
+	object: parent];
+    }
+  */
+}
+
+/*
+- (void) handleWindowClose: (NSNotification *)notification
+{
+  [self close];
+}
+
+- (void) handleWindowMiniaturize: (NSNotification *)notification
+{
+  [self close];
+}
+
+- (void) handleWindowMove: (NSNotification *)notification
+{
+  [self _resetWindowPosition];
+}
+
+- (void) handleWindowDidBecomeKey: (NSNotification *)notification
+{
+  [self _resetWindowPosition];
+}
+
+- (void) dealloc
+{
+  [nc removeObserver: self];
+  [super dealloc];
+}
+*/
+@end
+
 /*
   These functions may be called "recursively". For example, from a
   timed event. Therefore, there  may be several alert panel active
@@ -936,6 +1123,7 @@ setControl(NSView* content, id control, NSString *title)
 	alternateButton: (NSString*)_alternateButton
 	    otherButton: (NSString*)_otherButton;
 - (void) makePanel;
+- (void) makeSheet;
 - (GSAlertPanel*) panel;
 @end
 
@@ -970,7 +1158,7 @@ setControl(NSView* content, id control, NSString *title)
 
 - (void) makePanel
 {
-  if (*instance != 0)
+  if (*instance != 0 && [*instance isMemberOfClass: [GSAlertPanel class]])
     {
       if ([*instance isActivePanel])
 	{				// c:
@@ -996,6 +1184,34 @@ setControl(NSView* content, id control, NSString *title)
 	       other: otherButton];
 }
 
+- (void) makeSheet
+{
+  if (*instance != 0 && [*instance isMemberOfClass: [GSAlertSheet class]])
+    {
+      if ([*instance isActivePanel])
+	{				// c:
+	  panel = [[GSAlertSheet alloc] init];
+	}
+      else
+	{				// b:
+	  panel = *instance;
+	}
+    }
+  else
+    { 					// a:
+      panel = [[GSAlertSheet alloc] init];
+      *instance = panel;
+    }
+
+  [panel setTitleBar: defaultTitle
+		icon: nil
+	       title: title
+	     message: message
+		 def: defaultButton
+		 alt: alternateButton
+	       other: otherButton];
+}
+
 - (GSAlertPanel*) panel
 {
   return panel;
@@ -1037,7 +1253,7 @@ getSomePanel(
     }
   else
     {
-      if (*instance != 0)
+      if (*instance != 0 && [*instance isMemberOfClass: [GSAlertPanel class]])
 	{
 	  if ([*instance isActivePanel])
 	    {				// c:
@@ -1065,6 +1281,69 @@ getSomePanel(
   return panel;
 }
 
+static GSAlertPanel*
+getSomeSheet(
+  GSAlertPanel **instance,
+  NSString *defaultTitle,
+  NSString *title,
+  NSString *message,
+  NSString *defaultButton,
+  NSString *alternateButton,
+  NSString *otherButton)
+{
+  GSAlertSheet	*panel;
+
+  if (GSCurrentThread() != GSAppKitThread)
+    {
+      _GSAlertCreation	*c;
+
+      NSWarnFLog(@"Alert Sheet functionality called from a thread other than"
+        @" the main one, this may not work on MacOS-X and could therefore be"
+	@" a portability problem in your code");
+      c = [_GSAlertCreation alloc];
+      c = [c initWithInstance: instance
+		 defaultTitle: defaultTitle
+			title: title
+		      message: message
+		defaultButton: defaultButton
+	      alternateButton: alternateButton
+		  otherButton: otherButton];
+      [c performSelectorOnMainThread: @selector(makeSheet)
+			  withObject: nil
+		       waitUntilDone: YES];
+      panel = (GSAlertSheet *)[c panel];
+      RELEASE(c);
+    }
+  else
+    {
+      if (*instance != 0 && [*instance isMemberOfClass: [GSAlertSheet class]])
+	{
+	  if ([*instance isActivePanel])
+	    {				// c:
+	      panel = [[GSAlertSheet alloc] init];
+	    }
+	  else
+	    {				// b:
+	      panel = (GSAlertSheet *)*instance;
+	    }
+	}
+      else
+	{ 					// a:
+	  panel = [[GSAlertSheet alloc] init];
+	  *instance = panel;
+	}
+
+      [panel setTitleBar: defaultTitle
+		    icon: nil
+		   title: title
+		 message: message
+		     def: defaultButton
+		     alt: alternateButton
+		   other: otherButton];
+    }
+  return panel;
+}
+
 id
 NSGetAlertPanel(
   NSString *title,
@@ -1101,6 +1380,12 @@ NSRunAlertPanel(
   message = [NSString stringWithFormat: msg arguments: ap];
   va_end(ap);
 
+  if (NSApp == nil)
+    {
+      // No NSApp ... not running in a gui application so just log.
+      NSLog(@"%@", message);
+      return NSAlertDefaultReturn;
+    }
   if (defaultButton == nil)
     {
       defaultButton = @"OK";
@@ -1292,23 +1577,25 @@ void NSBeginAlertSheet(NSString *title,
       defaultButton = @"OK";
     }
 
-  panel = getSomePanel(&standardAlertPanel, defaultTitle, title, message,
+  panel = getSomeSheet(&standardAlertPanel, defaultTitle, title, message,
     defaultButton, alternateButton, otherButton);
+
   // FIXME: We should also change the button action to call endSheet:
   [NSApp beginSheet: panel
 	 modalForWindow: docWindow
 	 modalDelegate: modalDelegate
 	 didEndSelector: willEndSelector
 	 contextInfo: contextInfo];
+
   [panel close];
   if (modalDelegate && [modalDelegate respondsToSelector: didEndSelector])
     {
       void (*didEnd)(id, SEL, id, int, void*);
-
-      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate methodForSelector: 
-								 didEndSelector];
+      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate
+        methodForSelector: didEndSelector];
       didEnd(modalDelegate, didEndSelector, panel, [panel result], contextInfo);
     }
+
   NSReleaseAlertPanel(panel);
 }
 
@@ -1331,7 +1618,7 @@ void NSBeginCriticalAlertSheet(NSString *title,
   message = [NSString stringWithFormat: msg arguments: ap];
   va_end(ap);
 
-  panel = getSomePanel(&criticalAlertPanel, @"Critical", title, message,
+  panel = getSomeSheet(&criticalAlertPanel, @"Critical", title, message,
     defaultButton, alternateButton, otherButton);
   // FIXME: We should also change the button action to call endSheet:
   [NSApp beginSheet: panel
@@ -1343,11 +1630,11 @@ void NSBeginCriticalAlertSheet(NSString *title,
   if (modalDelegate && [modalDelegate respondsToSelector: didEndSelector])
     {
       void (*didEnd)(id, SEL, id, int, void*);
-
-      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate methodForSelector: 
-								 didEndSelector];
+      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate
+        methodForSelector: didEndSelector];
       didEnd(modalDelegate, didEndSelector, panel, [panel result], contextInfo);
     }
+
   NSReleaseAlertPanel(panel);
 }
 
@@ -1370,7 +1657,7 @@ void NSBeginInformationalAlertSheet(NSString *title,
   message = [NSString stringWithFormat: msg arguments: ap];
   va_end(ap);
 
-  panel = getSomePanel(&informationalAlertPanel,
+  panel = getSomeSheet(&informationalAlertPanel,
 		      @"Information",
 		      title, message,
 		      defaultButton, alternateButton, otherButton);
@@ -1384,11 +1671,11 @@ void NSBeginInformationalAlertSheet(NSString *title,
   if (modalDelegate && [modalDelegate respondsToSelector: didEndSelector])
     {
       void (*didEnd)(id, SEL, id, int, void*);
-
-      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate methodForSelector: 
-								 didEndSelector];
+      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate
+        methodForSelector: didEndSelector];
       didEnd(modalDelegate, didEndSelector, panel, [panel result], contextInfo);
     }
+
   NSReleaseAlertPanel(panel);
 }
 
