@@ -52,6 +52,7 @@
 #include "AppKit/NSFontManager.h"
 // For the colour name spaces
 #include "AppKit/NSGraphics.h"
+#import "AppKit/NSTextTable.h"
 
 #include "GNUstepGUI/GSTextConverter.h"
 #include "GSGuiPrivate.h"
@@ -247,16 +248,22 @@ create_error(int code, NSString* desc)
 
 + (NSArray *) textFileTypes
 {
-  // FIXME
+  // FIXME: Apply service filters
   return [self textUnfilteredFileTypes];
 }
 
 + (NSArray *) textPasteboardTypes
 {
-  // FIXME
+  // FIXME: Apply service filters
   return [self textUnfilteredPasteboardTypes];
 }
 
++ (NSArray *) textTypes
+{
+  // FIXME: Apply service filters
+  return [self textUnfilteredTypes];
+}
+
 + (NSArray *) textUnfilteredFileTypes
 {
   return [NSArray arrayWithObjects: @"txt",  @"rtf", @"rtfd", @"html", nil];
@@ -268,12 +275,6 @@ create_error(int code, NSString* desc)
 		  NSRTFDPboardType, NSHTMLPboardType, nil];
 }
 
-+ (NSArray *) textTypes
-{
-  // FIXME
-  return [self textUnfilteredTypes];
-}
-
 + (NSArray *) textUnfilteredTypes
 {
   return [NSArray arrayWithObjects: @"public.plain-text",
@@ -400,7 +401,7 @@ create_error(int code, NSString* desc)
 {
   // FIXME
   return [self lineBreakBeforeIndex: location
-                      withinRange: aRange];
+                        withinRange: aRange];
 }
 
 - (NSUInteger) lineBreakBeforeIndex: (NSUInteger)location
@@ -448,7 +449,7 @@ create_error(int code, NSString* desc)
     }
 }
 
-- (NSRange) doubleClickAtIndex: (unsigned)location
+- (NSRange) doubleClickAtIndex: (NSUInteger)location
 {
   NSString *str = [self string];
   unsigned length = [str length];
@@ -554,7 +555,7 @@ create_error(int code, NSString* desc)
   return NSMakeRange (location, length);
 }
 
-- (unsigned) nextWordFromIndex: (unsigned)location
+- (NSUInteger) nextWordFromIndex: (NSUInteger)location
 		       forward: (BOOL)isForward
 {
   NSString *str = [self string];
@@ -576,18 +577,18 @@ create_error(int code, NSString* desc)
 
   if (isForward)
     {
-      /* What we want to do is: move forward to the next chunk of word
-	 separator characters, skip them all, and return the location
-	 just after them.  */
+      /* What we want to do is: move forward to the next chunk of
+	 non-word separator characters, skip them all, and return the
+	 location just after them.  */
 
       if (location == length)
 	{
 	  return length;
 	}
 
-      /* Move forward to the next word-separator.  */
+      /* Move forward to the next non-word separator.  */
       range = NSMakeRange (location, length - location);
-      range = [str rangeOfCharacterFromSet: wordBreakCSet
+      range = [str rangeOfCharacterFromSet: wordCSet
 		                   options: NSLiteralSearch
                           	     range: range];
       if (range.location == NSNotFound)
@@ -595,12 +596,12 @@ create_error(int code, NSString* desc)
 	  return length;
 	}
       /* rangeOfCharacterFromSet: options: range: only returns the range
-	 of the first word-separator character ... we want to skip
+	 of the first non-word-separator character ... we want to skip
 	 them all!  So we need to search again, this time for the
-	 first non-word-separator character, and return the first such
+	 first word-separator character, and return the first such
 	 character.  */
       range = NSMakeRange (range.location, length - range.location);
-      range = [str rangeOfCharacterFromSet: wordCSet
+      range = [str rangeOfCharacterFromSet: wordBreakCSet
 		                   options: NSLiteralSearch
                           	     range: range];
       if (range.location == NSNotFound)
@@ -651,61 +652,34 @@ create_error(int code, NSString* desc)
 - (id) initWithRTFDFileWrapper: (NSFileWrapper *)wrapper
             documentAttributes: (NSDictionary **)dict
 {
-  NSAttributedString *new;
-
-  if (wrapper == nil)
-    {
-      RELEASE (self);
-      return nil;
-    }
-
-  new = [converter_class(@"RTFD", NO) 
-			parseFile: wrapper
-			documentAttributes: dict
-			class: [self class]];
-  // We do not return self but the newly created object
-  RELEASE (self);
-  return RETAIN (new); 
+  return [self initWithRTFD: [wrapper serializedRepresentation]
+                 documentAttributes: dict];
 }
 
 - (id) initWithRTFD: (NSData*)data
  documentAttributes: (NSDictionary**)dict
 {
-  NSAttributedString *new;
+  NSDictionary *options;
 
-  if (data == nil)
-    {
-      RELEASE (self);
-      return nil;
-    }
-
-  new = [converter_class(@"RTFD", NO)
-			parseData: data
-			documentAttributes: dict
-			class: [self class]];
-  // We do not return self but the newly created object
-  RELEASE (self);
-  return RETAIN (new); 
+  options = [NSDictionary dictionaryWithObject: NSRTFDTextDocumentType
+                          forKey: NSDocumentTypeDocumentOption];
+  return [self initWithData: data
+               options: options
+               documentAttributes: dict
+               error: NULL];
 }
 
 - (id) initWithRTF: (NSData *)data
   documentAttributes: (NSDictionary **)dict
 {
-  NSAttributedString *new;
+  NSDictionary *options;
 
-  if (data == nil)
-    {
-      RELEASE (self);
-      return nil;
-    }
-
-  new = [converter_class(@"RTF", NO) 
-			parseData: data
-			documentAttributes: dict
-			class: [self class]];
-  // We do not return self but the newly created object
-  RELEASE (self);
-  return RETAIN (new); 
+  options = [NSDictionary dictionaryWithObject: NSRTFTextDocumentType
+                          forKey: NSDocumentTypeDocumentOption];
+  return [self initWithData: data
+               options: options
+               documentAttributes: dict
+               error: NULL];
 }
 
 - (id) initWithHTML: (NSData *)data
@@ -727,7 +701,7 @@ create_error(int code, NSString* desc)
 					  forKey: NSBaseURLDocumentOption];
 
   return [self initWithHTML: data
-		    options: options
+               options: options
                documentAttributes: dict];
 }
 
@@ -735,34 +709,41 @@ create_error(int code, NSString* desc)
             options: (NSDictionary *)options
  documentAttributes: (NSDictionary **)dict
 {
-  if (data == nil)
+  if (options == nil)
     {
-      RELEASE (self);
-      return nil;
+      options = [NSDictionary dictionaryWithObject: NSHTMLTextDocumentType
+                                 forKey: NSDocumentTypeDocumentOption];
+    }
+  else if ([options objectForKey: NSDocumentTypeDocumentOption] == nil)
+    {
+      options = AUTORELEASE([options mutableCopy]);
+      [(NSMutableDictionary*)options setObject: NSHTMLTextDocumentType 
+                             forKey: NSDocumentTypeDocumentOption];
     }
 
-  // FIXME: Not implemented
-  return self;
+  /*
+    The converter should support:
+    NSHTMLTextDocumentType
+    @"public.html"
+    @"html"
+   */
+  return [self initWithData: data
+               options: options
+               documentAttributes: dict
+               error: NULL];
 }
 
 - (id) initWithDocFormat: (NSData *)data
       documentAttributes: (NSDictionary **)dict
 {
-  NSAttributedString *new;
+  NSDictionary *options;
 
-  if (data == nil)
-    {
-      RELEASE (self);
-      return nil;
-    }
-
-  new = [converter_class(@"DOC", NO)
-			parseData: data
-			documentAttributes: dict
-			class: [self class]];
-  // We do not return self but the newly created object
-  RELEASE (self);
-  return RETAIN (new); 
+  options = [NSDictionary dictionaryWithObject: NSDocFormatTextDocumentType
+                          forKey: NSDocumentTypeDocumentOption];
+  return [self initWithData: data
+               options: options
+               documentAttributes: dict
+               error: NULL];
 }
 
 - (id) initWithData: (NSData *)data
@@ -771,11 +752,13 @@ create_error(int code, NSString* desc)
               error: (NSError **)error
 {
   NSString *type = [options objectForKey: NSDocumentTypeDocumentOption];
+  Class converter;
 
   if (data == nil)
     {
-      *error = create_error(0, NSLocalizedString(@"No data specified for data loading.", 
-                                                 @"Error description"));
+      if (error)
+	*error = create_error(0, NSLocalizedString(@"No data specified for data loading.",
+						   @"Error description"));
       RELEASE(self);
       return nil;
     }
@@ -783,57 +766,49 @@ create_error(int code, NSString* desc)
   if (type == nil)
     {
       // FIXME: try to determine type
-      *error = create_error(0, NSLocalizedString(@"No type specified for data.", 
-                                                 @"Error description"));
+      if (error)
+	*error = create_error(0, NSLocalizedString(@"No type specified for data.",
+						   @"Error description"));
       RELEASE(self);
       return nil;
     }
 
-  if ([type isEqualToString: NSDocFormatTextDocumentType])
+  converter = converter_class(type, NO);
+  if (converter != Nil)
     {
-      return [self initWithDocFormat: data
-                   documentAttributes: dict];
-    }
-  else if ([type isEqualToString: NSHTMLTextDocumentType]
-           || [type isEqualToString: @"public.html"]
-           || [type isEqualToString: @"html"])
-    {
-      return [self initWithHTML: data
-                   options: options
-                   documentAttributes: dict];
-    }
-  else if ([type isEqualToString: NSRTFDTextDocumentType]
-           || [type isEqualToString: @"com.apple.rtfd"]
-           || [type isEqualToString: @"rtfd"])
-    {
-      return [self initWithRTFD: data
-                   documentAttributes: dict];
-    }
-  else if ([type isEqualToString: NSRTFTextDocumentType]
-           || [type isEqualToString: @"public.rtf"]
-           || [type isEqualToString: @"rtf"])
-    {
-      return [self initWithRTF: data
-                   documentAttributes: dict];
+      NSAttributedString *new;
+
+      new = [converter
+              parseData: data
+              options: options
+              documentAttributes: dict
+              error: error
+              class: [self class]];
+      // We do not return self but the newly created object
+      RELEASE(self);
+      return RETAIN(new); 
     }
   else if ([type isEqualToString: NSPlainTextDocumentType]
            || [type isEqualToString: @"public.plain-text"]
            || [type isEqualToString: @"text"])
     {
+      // FIXME: Should we have a proper converter for this type?
       NSStringEncoding encoding = [[options objectForKey: @"CharacterEncoding"] 
 				      intValue];
       NSDictionary *defaultAttrs = [options objectForKey: @"DefaultAttributes"];
-      NSString *str = [[NSString alloc] initWithData: data 
-                                        encoding: encoding];
+      NSString *str;
 
+      str = [[NSString alloc] initWithData: data 
+                              encoding: encoding];
       self = [self initWithString: str
                    attributes: defaultAttrs];
       RELEASE(str);
       return self;
     }
 
-  *error = create_error(0, NSLocalizedString(@"Could not load data.", 
-                                             @"Error description"));
+  if (error)
+    *error = create_error(0, NSLocalizedString(@"Could not load data.", 
+					       @"Error description"));
   RELEASE(self);
   return nil;
 }
@@ -862,7 +837,7 @@ create_error(int code, NSString* desc)
     }
   else
    {
-     return [self initWithURL:  [NSURL fileURLWithPath: path]
+     return [self initWithURL: [NSURL fileURLWithPath: path]
 		  documentAttributes: dict];
    }
 }
@@ -870,14 +845,13 @@ create_error(int code, NSString* desc)
 - (id) initWithURL: (NSURL *)url 
 documentAttributes: (NSDictionary **)dict
 {
-  NSError *error = nil;
   NSDictionary *options = [NSDictionary dictionaryWithObject: [url baseURL]
                                         forKey: NSBaseURLDocumentOption];
 
   return [self initWithURL: url
                options: options
                documentAttributes: dict
-               error: &error];
+               error: NULL];
 }
 
 - (id) initWithURL: (NSURL *)url
@@ -889,8 +863,9 @@ documentAttributes: (NSDictionary **)dict
 
   if (data == nil)
     {
-      *error = create_error(0, NSLocalizedString(@"Could not load data from URL.", 
-                                                 @"Error description"));
+      if (error)
+	*error = create_error(0, NSLocalizedString(@"Could not load data from URL.", 
+						   @"Error description"));
       RELEASE(self);
       return nil;
     }
@@ -915,35 +890,70 @@ documentAttributes: (NSDictionary **)dict
 - (NSData *) RTFFromRange: (NSRange)range
        documentAttributes: (NSDictionary *)dict
 {
-  return [converter_class(@"RTF", YES) 
-			 produceDataFrom: 
-			   [self attributedSubstringFromRange: range]
-			 documentAttributes: dict];
+  if (dict == nil)
+    {
+      dict = [NSDictionary dictionaryWithObject: NSRTFTextDocumentType
+                                 forKey: NSDocumentTypeDocumentOption];
+    }
+  else if ([dict objectForKey: NSDocumentTypeDocumentOption] == nil)
+    {
+      dict = AUTORELEASE([dict mutableCopy]);
+      [(NSMutableDictionary*)dict setObject: NSRTFTextDocumentType 
+                             forKey: NSDocumentTypeDocumentOption];
+    }
+
+  return [self dataFromRange: range
+               documentAttributes: dict
+               error: NULL];
 }
 
 - (NSData *) RTFDFromRange: (NSRange)range
 	documentAttributes: (NSDictionary *)dict
 {
-  return [converter_class(@"RTFD", YES)  
-			 produceDataFrom: 
-			   [self attributedSubstringFromRange: range]
-			 documentAttributes: dict];
+  if (dict == nil)
+    {
+      dict = [NSDictionary dictionaryWithObject: NSRTFDTextDocumentType
+                                 forKey: NSDocumentTypeDocumentOption];
+    }
+  else if ([dict objectForKey: NSDocumentTypeDocumentOption] == nil)
+    {
+      dict = AUTORELEASE([dict mutableCopy]);
+      [(NSMutableDictionary*)dict setObject: NSRTFDTextDocumentType 
+                             forKey: NSDocumentTypeDocumentOption];
+    }
+
+  return [self dataFromRange: range
+               documentAttributes: dict
+               error: NULL];
 }
 
 - (NSFileWrapper *) RTFDFileWrapperFromRange: (NSRange)range
 			  documentAttributes: (NSDictionary *)dict
 {
-  return [converter_class(@"RTFD", YES)
-			 produceFileFrom: 
-			   [self attributedSubstringFromRange: range]
-			 documentAttributes: dict];
+  return AUTORELEASE([[NSFileWrapper alloc]
+                       initWithSerializedRepresentation: 
+                         [self RTFDFromRange: range
+                               documentAttributes: dict]]);
 }
 
 - (NSData *) docFormatFromRange: (NSRange)range
              documentAttributes: (NSDictionary *)dict
 {
-  // FIXME
-  return nil;
+  if (dict == nil)
+    {
+      dict = [NSDictionary dictionaryWithObject: NSDocFormatTextDocumentType
+                                 forKey: NSDocumentTypeDocumentOption];
+    }
+  else if ([dict objectForKey: NSDocumentTypeDocumentOption] == nil)
+    {
+      dict = AUTORELEASE([dict mutableCopy]);
+      [(NSMutableDictionary*)dict setObject: NSDocFormatTextDocumentType 
+                             forKey: NSDocumentTypeDocumentOption];
+    }
+
+  return [self dataFromRange: range
+               documentAttributes: dict
+               error: NULL];
 }
 
 - (NSData *) dataFromRange: (NSRange)range
@@ -951,34 +961,28 @@ documentAttributes: (NSDictionary **)dict
                      error: (NSError **)error
 {
   NSString *type = [dict objectForKey: NSDocumentTypeDocumentOption];
+  Class converter;
 
   if (type == nil)
     {
-      *error = create_error(0, NSLocalizedString(@"No type specified for data.", 
-                                                 @"Error description"));
+      if (error)
+	*error = create_error(0, NSLocalizedString(@"No type specified for data.",
+						   @"Error description"));
       return nil;
     }
 
-  if ([type isEqualToString: NSDocFormatTextDocumentType])
+  converter = converter_class(type, YES);
+  if (converter != Nil)
     {
-      return [self docFormatFromRange: range
-                   documentAttributes: dict];
+      return [converter
+               produceDataFrom: 
+                 [self attributedSubstringFromRange: range]
+               documentAttributes: dict
+               error: error];
     }
-  else if ([type isEqualToString: NSHTMLTextDocumentType])
-    {
-      // FIXME
-    }
-  else if ([type isEqualToString: NSRTFDTextDocumentType])
-    {
-      return [self RTFDFromRange: range
-                   documentAttributes: dict];
-    }
-  else if ([type isEqualToString: NSRTFTextDocumentType])
-    {
-      return [self RTFFromRange: range
-                   documentAttributes: dict];
-    }
-  else if ([type isEqualToString: NSPlainTextDocumentType])
+  else if ([type isEqualToString: NSPlainTextDocumentType]
+           || [type isEqualToString: @"public.plain-text"]
+           || [type isEqualToString: @"text"])
     {
       NSStringEncoding encoding = [[dict objectForKey: @"CharacterEncoding"] 
                                       intValue];
@@ -988,8 +992,9 @@ documentAttributes: (NSDictionary **)dict
       return [[self string] dataUsingEncoding: encoding];
     }
 
-  *error = create_error(0, NSLocalizedString(@"Could not create data for type.", 
-                                             @"Error description"));
+  if (error)
+    *error = create_error(0, NSLocalizedString(@"Could not create data for type.",
+					       @"Error description"));
   return nil;
 }
 
@@ -1000,48 +1005,223 @@ documentAttributes: (NSDictionary **)dict
   NSFileWrapper *wrapper;
   NSData *data;
 
-  // FIXME: This wont work for directory bundles.
   data = [self dataFromRange: range
 	  documentAttributes: dict
 		       error: error];
   if (data != nil)
     {
+      // FIXME: This wont work for directory bundles.
       wrapper = [[NSFileWrapper alloc] initRegularFileWithContents: data];
       return AUTORELEASE(wrapper);
     }
 
-  if (*error == nil)
-      *error = create_error(0, NSLocalizedString(@"Could not create data for type.",
-                                                 @"Error description"));
+  if (error)
+    *error = create_error(0, NSLocalizedString(@"Could not create data for type.",
+					       @"Error description"));
 
   return nil;
 }
 
-- (NSRange) itemNumberInTextList: (NSTextList *)list
-                         atIndex: (unsigned)location
+- (NSInteger) itemNumberInTextList: (NSTextList *)list
+                           atIndex: (NSUInteger)location
 {
-  // FIXME
-  return NSMakeRange(NSNotFound, 0);
+  NSParagraphStyle *style = [self attribute: NSParagraphStyleAttributeName
+                                  atIndex: location
+                                  effectiveRange: NULL];
+  if (style != nil)
+    {
+      NSArray *textLists = [style textLists];
+
+      if (textLists != nil)
+        {
+          return [textLists indexOfObject: list];
+        }
+    }
+
+  return NSNotFound;
 }
 
 - (NSRange) rangeOfTextBlock: (NSTextBlock *)block
-                     atIndex: (unsigned)location
+                     atIndex: (NSUInteger)location
 {
-  // FIXME
+  NSRange effRange;
+  NSParagraphStyle *style = [self attribute: NSParagraphStyleAttributeName
+                                  atIndex: location
+                                  effectiveRange: &effRange];
+  if (style != nil)
+    {
+      NSArray *textBlocks = [style textBlocks];
+
+      if ((textBlocks != nil) && [textBlocks containsObject: block])
+        {
+          NSRange newEffRange;
+          unsigned len = [self length];
+
+          while ((effRange.location > 0) && style && textBlocks)
+            {
+              style = [self attribute: NSParagraphStyleAttributeName
+                            atIndex: effRange.location - 1
+                            effectiveRange: &newEffRange];
+              if (style != nil)
+                {
+                  textBlocks = [style textBlocks];
+                  
+                  if ((textBlocks != nil) && [textBlocks containsObject: block])
+                    {
+                      effRange.location = newEffRange.location;
+                      effRange.length += newEffRange.length;
+                    }
+                }
+            }
+
+          while (NSMaxRange(effRange) < len && style && textBlocks) 
+            {
+              style = [self attribute: NSParagraphStyleAttributeName
+                            atIndex: NSMaxRange(effRange)
+                            effectiveRange: &newEffRange];
+              if (style != nil)
+                {
+                  textBlocks = [style textBlocks];
+                  
+                  if ((textBlocks != nil) && [textBlocks containsObject: block])
+                    {
+                      effRange.length += newEffRange.length;
+                    }
+                }
+            }
+
+          return effRange;
+        }
+    }
+
   return NSMakeRange(NSNotFound, 0);
 }
 
 - (NSRange) rangeOfTextList: (NSTextList *)list
-                    atIndex: (unsigned)location
+                    atIndex: (NSUInteger)location
 {
-  // FIXME
+  NSRange effRange;
+  NSParagraphStyle *style = [self attribute: NSParagraphStyleAttributeName
+                                  atIndex: location
+                                  effectiveRange: &effRange];
+  if (style != nil)
+    {
+      NSArray *textLists = [style textLists];
+
+      if ((textLists != nil) && [textLists containsObject: list])
+        {
+          NSRange newEffRange;
+          unsigned len = [self length];
+
+          while ((effRange.location > 0) && style && textLists)
+            {
+              style = [self attribute: NSParagraphStyleAttributeName
+                            atIndex: effRange.location - 1
+                            effectiveRange: &newEffRange];
+              if (style != nil)
+                {
+                  textLists = [style textLists];
+                  
+                  if ((textLists != nil) && [textLists containsObject: list])
+                    {
+                      effRange.location = newEffRange.location;
+                      effRange.length += newEffRange.length;
+                    }
+                }
+            }
+
+          while (NSMaxRange(effRange) < len && style && textLists) 
+            {
+              style = [self attribute: NSParagraphStyleAttributeName
+                            atIndex: NSMaxRange(effRange)
+                            effectiveRange: &newEffRange];
+              if (style != nil)
+                {
+                  textLists = [style textLists];
+                  
+                  if ((textLists != nil) && [textLists containsObject: list])
+                    {
+                      effRange.length += newEffRange.length;
+                    }
+                }
+            }
+
+          return effRange;
+        }
+    }
+
   return NSMakeRange(NSNotFound, 0);
 }
 
-- (NSRange) rangeOfTextTable: (NSTextTable *)table
-                     atIndex: (unsigned)location
+static inline
+BOOL containsTable(NSArray *textBlocks, NSTextTable *table)
 {
-  // FIXME
+  NSEnumerator *benum = [textBlocks objectEnumerator];
+  NSTextTableBlock *block;
+
+  while ((block = [benum nextObject]))
+    {
+      if ([table isEqual: [block table]])
+	{
+	  return YES;
+	}
+    }
+  return NO;
+}
+
+- (NSRange) rangeOfTextTable: (NSTextTable *)table
+                     atIndex: (NSUInteger)location
+{
+  NSRange effRange;
+  NSParagraphStyle *style = [self attribute: NSParagraphStyleAttributeName
+                                  atIndex: location
+                                  effectiveRange: &effRange];
+  if (style != nil)
+    {
+      NSArray *textBlocks = [style textBlocks];
+
+      if ((textBlocks != nil) && containsTable(textBlocks, table))
+        {
+	  NSRange newEffRange;
+	  unsigned len = [self length];
+	  
+	  while ((effRange.location > 0) && style && textBlocks)
+	    {
+	      style = [self attribute: NSParagraphStyleAttributeName
+			      atIndex: effRange.location - 1
+		       effectiveRange: &newEffRange];
+	      if (style != nil)
+		{
+		  textBlocks = [style textBlocks];
+		  
+		  if ((textBlocks != nil) && containsTable(textBlocks, table))
+                    {
+                      effRange.location = newEffRange.location;
+                      effRange.length += newEffRange.length;
+                    }
+                }
+            }
+
+          while (NSMaxRange(effRange) < len && style && textBlocks) 
+            {
+              style = [self attribute: NSParagraphStyleAttributeName
+                            atIndex: NSMaxRange(effRange)
+                            effectiveRange: &newEffRange];
+              if (style != nil)
+                {
+                  textBlocks = [style textBlocks];
+                  
+                  if ((textBlocks != nil) && containsTable(textBlocks, table))
+                    {
+                      effRange.length += newEffRange.length;
+                    }
+                }
+            }
+
+          return effRange;
+        }
+    }
+
   return NSMakeRange(NSNotFound, 0);
 }
 
@@ -1308,7 +1488,7 @@ static NSMutableDictionary *cachedCSets = nil;
 - (void) fixFontAttributeInRange: (NSRange)range
 {
   NSString *string;
-  NSFont *font;
+  NSFont *font = nil;
   NSCharacterSet *charset = nil;
   NSRange fontRange = NSMakeRange(NSNotFound, 0);
   unsigned int i;
@@ -1495,6 +1675,7 @@ static NSMutableDictionary *cachedCSets = nil;
 
   // Check for attachment characters without attachments
   location = range.location;
+  string = [self string];
   while (location < end)
     {
       NSRange eRange = [string rangeOfString: attachmentString
@@ -1514,6 +1695,8 @@ static NSMutableDictionary *cachedCSets = nil;
           [self deleteCharactersInRange: NSMakeRange (eRange.location, 1)];
           eRange.length--;
           end--;
+          // Need to reset this after every character change
+          string = [self string];
         }
 
       location = NSMaxRange (eRange);
@@ -1619,9 +1802,9 @@ static NSMutableDictionary *cachedCSets = nil;
                            range: (NSRange)range
 {
 	[self setAttributes: [NSDictionary dictionaryWithObject: 
-                                         [NSNumber numberWithInt: writingDirection]
-													forKey: @"WritingDirection"]
-				  range: range];
+                                             [NSNumber numberWithInt: writingDirection]
+                                           forKey: @"WritingDirection"]
+              range: range];
 }
 
 @end
