@@ -28,13 +28,14 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "AppKit/NSBezierPath.h"
-
-#include "AppKit/NSAffineTransform.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/PSOperators.h"
-#include "GNUstepGUI/GSFontInfo.h"
+#import <Foundation/NSDebug.h>
+#import "AppKit/NSAffineTransform.h"
+#import "AppKit/NSBezierPath.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/PSOperators.h"
+#import "GNUstepGUI/GSFontInfo.h"
+#import "GSGuiPrivate.h"
 
 #include <math.h>
 
@@ -51,7 +52,7 @@ static Class NSBezierPath_concrete_class = nil;
 
 static NSWindingRule default_winding_rule = NSNonZeroWindingRule;
 static float default_line_width = 1.0;
-static float default_flatness = 1.0;
+static float default_flatness = 0.6;
 static NSLineJoinStyle default_line_join_style = NSMiterLineJoinStyle;
 static NSLineCapStyle default_line_cap_style = NSButtLineCapStyle;
 static float default_miter_limit = 10.0;
@@ -78,7 +79,7 @@ typedef struct _PathElement
 #ifdef GSIArray
 #undef GSIArray
 #endif
-#include <GNUstepBase/GSIArray.h>
+#import <GNUstepBase/GSIArray.h>
 
 @interface GSBezierPath : NSBezierPath
 {
@@ -1884,7 +1885,7 @@ static NSPoint point_on_curve(double t, NSPoint a, NSPoint b, NSPoint c,
   NSZone *zone;
 
   self = [super init];
-  zone = GSObjCZone(self);
+  zone = [self zone];
   pathElements = NSZoneMalloc(zone, sizeof(GSIArray_t));
   GSIArrayInitWithZoneAndCapacity(pathElements, zone, 8);
   flat = YES;
@@ -1895,7 +1896,7 @@ static NSPoint point_on_curve(double t, NSPoint a, NSPoint b, NSPoint c,
 - (void)dealloc
 {
   GSIArrayEmpty(pathElements);
-  NSZoneFree(GSObjCZone(self), pathElements);
+  NSZoneFree([self zone], pathElements);
   [super dealloc];
 }
 
