@@ -36,6 +36,7 @@
 #include <Foundation/NSException.h>
 #include <Foundation/NSValue.h>
 #include "AppKit/NSGraphics.h"
+#include "GSGuiPrivate.h"
 
 #include <jerror.h>
 #if defined(__MINGW32__)
@@ -349,6 +350,8 @@ static void gs_jpeg_memory_dest_destroy (j_compress_ptr cinfo)
   struct jpeg_decompress_struct  cinfo;
   struct gs_jpeg_error_mgr  jerrMgr;
 
+  memset((void*)&cinfo, 0, sizeof(struct jpeg_decompress_struct));
+
   /* Be sure imageData contains data */
   if (![imageData length])
     {
@@ -399,6 +402,8 @@ static void gs_jpeg_memory_dest_destroy (j_compress_ptr cinfo)
   if (!(self = [super init]))
     return nil;
 
+  memset((void*)&cinfo, 0, sizeof(struct jpeg_decompress_struct));
+
   /* Establish the our custom error handler */
   gs_jpeg_error_mgr_init(&jerrMgr);
   cinfo.err = jpeg_std_error(&jerrMgr.parent);
@@ -486,7 +491,7 @@ static void gs_jpeg_memory_dest_destroy (j_compress_ptr cinfo)
 
   if (jerrMgr.parent.num_warnings)
     {
-      NSLog(@"NSBitmapImageRep+JPEG: %d warnings during jpeg decompression, "
+      NSLog(@"NSBitmapImageRep+JPEG: %ld warnings during jpeg decompression, "
         @"image may be corrupted", jerrMgr.parent.num_warnings);
     }
 
@@ -544,10 +549,12 @@ static void gs_jpeg_memory_dest_destroy (j_compress_ptr cinfo)
       if (errorMsg != NULL)
         *errorMsg = em;
       else
-        NSLog (em);
+        NSLog (@"JPEG image rep: Planar Image, not handled yet !");
       return nil;
     }
 
+  memset((void*)&cinfo, 0, sizeof(struct jpeg_compress_struct));
+
   imageSource = [self bitmapData];
   sPP = [self samplesPerPixel];
   width = [self size].width;
