@@ -332,7 +332,7 @@
 -(NSSize) minimumSize
 {
   NSRect rect;
-  NSSize borderSize = _sizeForBorderType (_border_type);
+  NSSize borderSize = [[GSTheme theme] sizeForBorderType: _border_type];
 
   if ([_content_view respondsToSelector: @selector(minimumSize)])
     {
@@ -460,27 +460,26 @@
   // Draw border
   switch (_border_type)
     {
-    case NSNoBorder: 
-      break;
-    case NSLineBorder: 
-      [[NSColor controlDarkShadowColor] set];
-      NSFrameRect(_border_rect);
-      break;
-    case NSBezelBorder:
-      [[GSTheme theme] drawDarkBezel: _border_rect withClip: rect];
-      break;
-    case NSGrooveBorder: 
-      [[GSTheme theme] drawGroove: _border_rect withClip: rect];
-      break;
+      case NSNoBorder: 
+	break;
+      case NSLineBorder: 
+	[[NSColor controlDarkShadowColor] set];
+	NSFrameRect(_border_rect);
+	break;
+      case NSBezelBorder:
+	[[GSTheme theme] drawDarkBezel: _border_rect withClip: rect];
+	break;
+      case NSGrooveBorder: 
+	[[GSTheme theme] drawGroove: _border_rect withClip: rect];
+	break;
     }
 
   // Draw title
   if (_title_position != NSNoTitle)
     {
       // If the title is on the border, clip a hole in the later 
-      if ((_border_type != NSNoBorder) &&
-          ((_title_position == NSAtTop) || 
-           (_title_position == NSAtBottom)))
+      if ((_border_type != NSNoBorder)
+	&& ((_title_position == NSAtTop) || (_title_position == NSAtBottom)))
         {
           [color set];
           NSRectFill(_title_rect);
@@ -598,262 +597,277 @@
 
 - (NSRect) calcSizesAllowingNegative: (BOOL)aFlag
 {
+  GSTheme	*theme = [GSTheme theme];
   NSRect r = NSZeroRect;
 
   switch (_title_position)
     {
-    case NSNoTitle: 
-      {
-        NSSize borderSize = _sizeForBorderType (_border_type);
-        _border_rect = _bounds;
-        _title_rect = NSZeroRect;
+      case NSNoTitle: 
+	{
+	  NSSize borderSize = [theme sizeForBorderType: _border_type];
+	  _border_rect = _bounds;
+	  _title_rect = NSZeroRect;
 
-        // Add the offsets to the border rect
-        r.origin.x = _offsets.width + borderSize.width;
-        r.origin.y = _offsets.height + borderSize.height;
-        r.size.width = _border_rect.size.width - (2 * _offsets.width)
-          - (2 * borderSize.width);
-        r.size.height = _border_rect.size.height - (2 * _offsets.height)
-          - (2 * borderSize.height);
+	  // Add the offsets to the border rect
+	  r.origin.x = _offsets.width + borderSize.width;
+	  r.origin.y = _offsets.height + borderSize.height;
+	  r.size.width = _border_rect.size.width - (2 * _offsets.width)
+	    - (2 * borderSize.width);
+	  r.size.height = _border_rect.size.height - (2 * _offsets.height)
+	    - (2 * borderSize.height);
 
-        break;
-      }
-    case NSAboveTop: 
-      {
-        NSSize titleSize = [_cell cellSize];
-        NSSize borderSize = _sizeForBorderType (_border_type);
-        float c;
+	  break;
+	}
+      case NSAboveTop: 
+	{
+	  NSSize titleSize = [_cell cellSize];
+	  NSSize borderSize = [theme sizeForBorderType: _border_type];
+	  float c;
 
-        // Add spacer around title
-        titleSize.width += 6;
-        titleSize.height += 2;
+	  // Add spacer around title
+	  titleSize.width += 6;
+	  titleSize.height += 2;
 
-        // Adjust border rect by title cell
-        _border_rect = _bounds;
-        _border_rect.size.height -= titleSize.height + borderSize.height;
+	  // Adjust border rect by title cell
+	  _border_rect = _bounds;
+	  _border_rect.size.height -= titleSize.height + borderSize.height;
 
-        // Add the offsets to the border rect
-        r.origin.x = _border_rect.origin.x + _offsets.width + borderSize.width;
-        r.origin.y = _border_rect.origin.y + _offsets.height + borderSize.height;
-        r.size.width = _border_rect.size.width - (2 * _offsets.width)
-          - (2 * borderSize.width);
-        r.size.height = _border_rect.size.height - (2 * _offsets.height)
-          - (2 * borderSize.height);
+	  // Add the offsets to the border rect
+	  r.origin.x
+	    = _border_rect.origin.x + _offsets.width + borderSize.width;
+	  r.origin.y
+	    = _border_rect.origin.y + _offsets.height + borderSize.height;
+	  r.size.width = _border_rect.size.width - (2 * _offsets.width)
+	    - (2 * borderSize.width);
+	  r.size.height = _border_rect.size.height - (2 * _offsets.height)
+	    - (2 * borderSize.height);
 
-        // center the title cell
-        c = (_bounds.size.width - titleSize.width) / 2;
-        if (c < 0) c = 0;
-        _title_rect.origin.x = _bounds.origin.x + c;
-        _title_rect.origin.y = _bounds.origin.y + _border_rect.size.height
-          + borderSize.height;
-        _title_rect.size = titleSize;
+	  // center the title cell
+	  c = (_bounds.size.width - titleSize.width) / 2;
+	  if (c < 0) c = 0;
+	  _title_rect.origin.x = _bounds.origin.x + c;
+	  _title_rect.origin.y = _bounds.origin.y + _border_rect.size.height
+	    + borderSize.height;
+	  _title_rect.size = titleSize;
 
-        break;
-      }
-    case NSBelowTop: 
-      {
-        NSSize titleSize = [_cell cellSize];
-        NSSize borderSize = _sizeForBorderType (_border_type);
-        float c;
+	  break;
+	}
+      case NSBelowTop: 
+	{
+	  NSSize titleSize = [_cell cellSize];
+	  NSSize borderSize = [theme sizeForBorderType: _border_type];
+	  float c;
 
-        // Add spacer around title
-        titleSize.width += 6;
-        titleSize.height += 2;
+	  // Add spacer around title
+	  titleSize.width += 6;
+	  titleSize.height += 2;
 
-        // Adjust border rect by title cell
-        _border_rect = _bounds;
+	  // Adjust border rect by title cell
+	  _border_rect = _bounds;
 
-        // Add the offsets to the border rect
-        r.origin.x = _border_rect.origin.x + _offsets.width + borderSize.width;
-        r.origin.y = _border_rect.origin.y + _offsets.height + borderSize.height;
-        r.size.width = _border_rect.size.width - (2 * _offsets.width)
-          - (2 * borderSize.width);
-        r.size.height = _border_rect.size.height - (2 * _offsets.height)
-          - (2 * borderSize.height);
+	  // Add the offsets to the border rect
+	  r.origin.x
+	    = _border_rect.origin.x + _offsets.width + borderSize.width;
+	  r.origin.y
+	    = _border_rect.origin.y + _offsets.height + borderSize.height;
+	  r.size.width = _border_rect.size.width - (2 * _offsets.width)
+	    - (2 * borderSize.width);
+	  r.size.height = _border_rect.size.height - (2 * _offsets.height)
+	    - (2 * borderSize.height);
 
-        // Adjust by the title size
-        r.size.height -= titleSize.height + borderSize.height;
+	  // Adjust by the title size
+	  r.size.height -= titleSize.height + borderSize.height;
 
-        // center the title cell
-        c = (_border_rect.size.width - titleSize.width) / 2;
-        if (c < 0) c = 0;
-        _title_rect.origin.x = _border_rect.origin.x + c;
-        _title_rect.origin.y = _border_rect.origin.y + _border_rect.size.height
-          - titleSize.height - borderSize.height;
-        _title_rect.size = titleSize;
+	  // center the title cell
+	  c = (_border_rect.size.width - titleSize.width) / 2;
+	  if (c < 0) c = 0;
+	  _title_rect.origin.x = _border_rect.origin.x + c;
+	  _title_rect.origin.y
+	    = _border_rect.origin.y + _border_rect.size.height
+	    - titleSize.height - borderSize.height;
+	  _title_rect.size = titleSize;
 
-	break;
-      }
-    case NSAtTop: 
-      {
-	NSSize titleSize = [_cell cellSize];
-	NSSize borderSize = _sizeForBorderType (_border_type);
-	float c;
-	float topMargin;
-	float topOffset;
+	  break;
+	}
+      case NSAtTop: 
+	{
+	  NSSize titleSize = [_cell cellSize];
+	  NSSize borderSize = [theme sizeForBorderType: _border_type];
+	  float c;
+	  float topMargin;
+	  float topOffset;
 
-	// Add spacer around title
-	titleSize.width += 6;
-	titleSize.height += 2;
+	  // Add spacer around title
+	  titleSize.width += 6;
+	  titleSize.height += 2;
 
-	_border_rect = _bounds;
+	  _border_rect = _bounds;
 
-	topMargin = ceil(titleSize.height / 2);
-	topOffset = titleSize.height - topMargin;
-	
-	// Adjust by the title size
-	_border_rect.size.height -= topMargin;
-	
-	// Add the offsets to the border rect
-	r.origin.x = _border_rect.origin.x + _offsets.width + borderSize.width;
-	r.size.width = _border_rect.size.width - (2 * _offsets.width)
-	  - (2 * borderSize.width);
-	
-	if (topOffset > _offsets.height)
-	  {
-	    r.origin.y = _border_rect.origin.y + _offsets.height + borderSize.height;
-	    r.size.height = _border_rect.size.height - _offsets.height
-	      - (2 * borderSize.height) - topOffset;
-	  }
-	else
-	  {
-	    r.origin.y = _border_rect.origin.y + _offsets.height + borderSize.height;
-	    r.size.height = _border_rect.size.height - (2 * _offsets.height)
-	      - (2 * borderSize.height);
-	  }
+	  topMargin = ceil(titleSize.height / 2);
+	  topOffset = titleSize.height - topMargin;
+	  
+	  // Adjust by the title size
+	  _border_rect.size.height -= topMargin;
+	  
+	  // Add the offsets to the border rect
+	  r.origin.x
+	    = _border_rect.origin.x + _offsets.width + borderSize.width;
+	  r.size.width = _border_rect.size.width - (2 * _offsets.width)
+	    - (2 * borderSize.width);
+	  
+	  if (topOffset > _offsets.height)
+	    {
+	      r.origin.y
+		= _border_rect.origin.y + _offsets.height + borderSize.height;
+	      r.size.height = _border_rect.size.height - _offsets.height
+		- (2 * borderSize.height) - topOffset;
+	    }
+	  else
+	    {
+	      r.origin.y
+		= _border_rect.origin.y + _offsets.height + borderSize.height;
+	      r.size.height = _border_rect.size.height - (2 * _offsets.height)
+		- (2 * borderSize.height);
+	    }
 
-	// Adjust by the title size
-	//	r.size.height -= titleSize.height + borderSize.height;
+	  // Adjust by the title size
+	  //	r.size.height -= titleSize.height + borderSize.height;
 
-	// center the title cell
-	c = (_border_rect.size.width - titleSize.width) / 2;
-	if (c < 0) c = 0;
-	_title_rect.origin.x = _border_rect.origin.x + c;
-	_title_rect.origin.y = _border_rect.origin.y + _border_rect.size.height
-	  - topMargin;
-	_title_rect.size = titleSize;
+	  // center the title cell
+	  c = (_border_rect.size.width - titleSize.width) / 2;
+	  if (c < 0) c = 0;
+	  _title_rect.origin.x = _border_rect.origin.x + c;
+	  _title_rect.origin.y
+	    = _border_rect.origin.y + _border_rect.size.height - topMargin;
+	  _title_rect.size = titleSize;
 
-	break;
-      }
-    case NSAtBottom: 
-      {
-	NSSize titleSize = [_cell cellSize];
-	NSSize borderSize = _sizeForBorderType (_border_type);
-	float c;
-	float bottomMargin;
-	float bottomOffset;
+	  break;
+	}
+      case NSAtBottom: 
+	{
+	  NSSize titleSize = [_cell cellSize];
+	  NSSize borderSize = [theme sizeForBorderType: _border_type];
+	  float c;
+	  float bottomMargin;
+	  float bottomOffset;
 
-	// Add spacer around title
-	titleSize.width += 6;
-	titleSize.height += 2;
+	  // Add spacer around title
+	  titleSize.width += 6;
+	  titleSize.height += 2;
 
-	_border_rect = _bounds;
+	  _border_rect = _bounds;
 
-	bottomMargin = ceil(titleSize.height / 2);
-	bottomOffset = titleSize.height - bottomMargin;
+	  bottomMargin = ceil(titleSize.height / 2);
+	  bottomOffset = titleSize.height - bottomMargin;
 
-	// Adjust by the title size
-	_border_rect.origin.y += bottomMargin;
-	_border_rect.size.height -= bottomMargin;
+	  // Adjust by the title size
+	  _border_rect.origin.y += bottomMargin;
+	  _border_rect.size.height -= bottomMargin;
 
-	// Add the offsets to the border rect
-	r.origin.x = _border_rect.origin.x + _offsets.width + borderSize.width;
-	r.size.width = _border_rect.size.width - (2 * _offsets.width)
-	  - (2 * borderSize.width);
+	  // Add the offsets to the border rect
+	  r.origin.x = _border_rect.origin.x + _offsets.width + borderSize.width;
+	  r.size.width = _border_rect.size.width - (2 * _offsets.width)
+	    - (2 * borderSize.width);
 
-	if (bottomOffset > _offsets.height)
-	  {
-	    r.origin.y = _border_rect.origin.y + bottomOffset + borderSize.height;
-	    r.size.height = _border_rect.size.height - _offsets.height
-	      - bottomOffset
-	      - (2 * borderSize.height);
-	  }
-	else
-	  {
-	    r.origin.y = _border_rect.origin.y + _offsets.height + borderSize.height;
-	    r.size.height = _border_rect.size.height - (2 * _offsets.height)
-	      - (2 * borderSize.height);
-	  }
+	  if (bottomOffset > _offsets.height)
+	    {
+	      r.origin.y
+		= _border_rect.origin.y + bottomOffset + borderSize.height;
+	      r.size.height = _border_rect.size.height - _offsets.height
+		- bottomOffset
+		- (2 * borderSize.height);
+	    }
+	  else
+	    {
+	      r.origin.y
+		= _border_rect.origin.y + _offsets.height + borderSize.height;
+	      r.size.height = _border_rect.size.height - (2 * _offsets.height)
+		- (2 * borderSize.height);
+	    }
 
-	// Adjust by the title size
-	/*
-	r.origin.y += (titleSize.height / 2) + borderSize.height;
-	r.size.height -= (titleSize.height / 2) + borderSize.height;
-	*/
-	// center the title cell
-	c = (_border_rect.size.width - titleSize.width) / 2;
-	if (c < 0) c = 0;
-	_title_rect.origin.x = c;
-	_title_rect.origin.y = 0;
-	_title_rect.size = titleSize;
+	  // Adjust by the title size
+	  /*
+	  r.origin.y += (titleSize.height / 2) + borderSize.height;
+	  r.size.height -= (titleSize.height / 2) + borderSize.height;
+	  */
+	  // center the title cell
+	  c = (_border_rect.size.width - titleSize.width) / 2;
+	  if (c < 0) c = 0;
+	  _title_rect.origin.x = c;
+	  _title_rect.origin.y = 0;
+	  _title_rect.size = titleSize;
 
-	break;
-      }
-    case NSBelowBottom: 
-      {
-	NSSize titleSize = [_cell cellSize];
-	NSSize borderSize = _sizeForBorderType (_border_type);
-	float c;
+	  break;
+	}
+      case NSBelowBottom: 
+	{
+	  NSSize titleSize = [_cell cellSize];
+	  NSSize borderSize = [theme sizeForBorderType: _border_type];
+	  float c;
 
-	// Add spacer around title
-	titleSize.width += 6;
-	titleSize.height += 2;
+	  // Add spacer around title
+	  titleSize.width += 6;
+	  titleSize.height += 2;
 
-	// Adjust by the title
-	_border_rect = _bounds;
-	_border_rect.origin.y += titleSize.height + borderSize.height;
-	_border_rect.size.height -= titleSize.height + borderSize.height;
+	  // Adjust by the title
+	  _border_rect = _bounds;
+	  _border_rect.origin.y += titleSize.height + borderSize.height;
+	  _border_rect.size.height -= titleSize.height + borderSize.height;
 
-	// Add the offsets to the border rect
-	r.origin.x = _border_rect.origin.x + _offsets.width + borderSize.width;
-	r.origin.y = _border_rect.origin.y + _offsets.height + borderSize.height;
-	r.size.width = _border_rect.size.width - (2 * _offsets.width)
-	  - (2 * borderSize.width);
-	r.size.height = _border_rect.size.height - (2 * _offsets.height)
-	  - (2 * borderSize.height);
+	  // Add the offsets to the border rect
+	  r.origin.x
+	    = _border_rect.origin.x + _offsets.width + borderSize.width;
+	  r.origin.y
+	    = _border_rect.origin.y + _offsets.height + borderSize.height;
+	  r.size.width = _border_rect.size.width - (2 * _offsets.width)
+	    - (2 * borderSize.width);
+	  r.size.height = _border_rect.size.height - (2 * _offsets.height)
+	    - (2 * borderSize.height);
 
-	// center the title cell
-	c = (_border_rect.size.width - titleSize.width) / 2;
-	if (c < 0) c = 0;
-	_title_rect.origin.x = c;
-	_title_rect.origin.y = 0;
-	_title_rect.size = titleSize;
+	  // center the title cell
+	  c = (_border_rect.size.width - titleSize.width) / 2;
+	  if (c < 0) c = 0;
+	  _title_rect.origin.x = c;
+	  _title_rect.origin.y = 0;
+	  _title_rect.size = titleSize;
 
-	break;
-      }
-    case NSAboveBottom: 
-      {
-	NSSize titleSize = [_cell cellSize];
-	NSSize borderSize = _sizeForBorderType (_border_type);
-	float c;
+	  break;
+	}
+      case NSAboveBottom: 
+	{
+	  NSSize titleSize = [_cell cellSize];
+	  NSSize borderSize = [theme sizeForBorderType: _border_type];
+	  float c;
 
-	// Add spacer around title
-	titleSize.width += 6;
-	titleSize.height += 2;
+	  // Add spacer around title
+	  titleSize.width += 6;
+	  titleSize.height += 2;
 
-	_border_rect = _bounds;
+	  _border_rect = _bounds;
 
-	// Add the offsets to the border rect
-	r.origin.x = _border_rect.origin.x + _offsets.width + borderSize.width;
-	r.origin.y = _border_rect.origin.y + _offsets.height + borderSize.height;
-	r.size.width = _border_rect.size.width - (2 * _offsets.width)
-	  - (2 * borderSize.width);
-	r.size.height = _border_rect.size.height - (2 * _offsets.height)
-	  - (2 * borderSize.height);
+	  // Add the offsets to the border rect
+	  r.origin.x
+	    = _border_rect.origin.x + _offsets.width + borderSize.width;
+	  r.origin.y
+	    = _border_rect.origin.y + _offsets.height + borderSize.height;
+	  r.size.width = _border_rect.size.width - (2 * _offsets.width)
+	    - (2 * borderSize.width);
+	  r.size.height = _border_rect.size.height - (2 * _offsets.height)
+	    - (2 * borderSize.height);
 
-	// Adjust by the title size
-	r.origin.y += titleSize.height + borderSize.height;
-	r.size.height -= titleSize.height + borderSize.height;
+	  // Adjust by the title size
+	  r.origin.y += titleSize.height + borderSize.height;
+	  r.size.height -= titleSize.height + borderSize.height;
 
-	// center the title cell
-	c = (_border_rect.size.width - titleSize.width) / 2;
-	if (c < 0) c = 0;
-	_title_rect.origin.x = _border_rect.origin.x + c;
-	_title_rect.origin.y = _border_rect.origin.y + borderSize.height;
-	_title_rect.size = titleSize;
+	  // center the title cell
+	  c = (_border_rect.size.width - titleSize.width) / 2;
+	  if (c < 0) c = 0;
+	  _title_rect.origin.x = _border_rect.origin.x + c;
+	  _title_rect.origin.y = _border_rect.origin.y + borderSize.height;
+	  _title_rect.size = titleSize;
 
-	break;
-      }
+	  break;
+	}
     }
 
   if (!aFlag)
