@@ -35,25 +35,26 @@
 */
 
 #include <math.h>                  // (float)rintf(float x)
-#include "config.h"
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSException.h>
-#include "AppKit/NSBrowser.h"
-#include "AppKit/NSBrowserCell.h"
-#include "AppKit/AppKitExceptions.h"
-#include "AppKit/NSScroller.h"
-#include "AppKit/NSCell.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSScrollView.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSTableHeaderCell.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSBezierPath.h"
-#include "GNUstepGUI/GSTheme.h"
+#import "config.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSUserDefaults.h>
+#import "AppKit/NSBrowser.h"
+#import "AppKit/NSBrowserCell.h"
+#import "AppKit/AppKitExceptions.h"
+#import "AppKit/NSScroller.h"
+#import "AppKit/NSCell.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSScrollView.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSMatrix.h"
+#import "AppKit/NSTableHeaderCell.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSBezierPath.h"
+#import "GNUstepGUI/GSTheme.h"
 
 DEFINE_RINT_IF_MISSING
 
@@ -205,14 +206,21 @@ static NSTextFieldCell *titleCell;
 - (NSRect) drawingRectForBounds: (NSRect)theRect
 {
   // This adjustment must match the drawn border
-  return NSInsetRect(theRect, 2, 2);
+  return [[GSTheme theme] browserHeaderDrawingRectForCell: self
+						withFrame: theRect];
 }
 
 - (void) _drawBorderAndBackgroundWithFrame: (NSRect)cellFrame 
                                     inView: (NSView*)controlView
 {
-  [[GSTheme theme] drawGrayBezel: cellFrame withClip: NSZeroRect];
-  [self _drawBackgroundWithFrame: cellFrame inView: controlView];
+  [[GSTheme theme] drawBrowserHeaderCell: self
+			       withFrame: cellFrame
+				  inView: controlView];
+}
+
+- (BOOL) isOpaque
+{
+  return NO;
 }
 
 @end
@@ -533,7 +541,7 @@ static NSTextFieldCell *titleCell;
  * implements it).  If this call to the delegate returns NO then
  * the attempt to set the path fails.<br />
  * If the delegate does not implement the method, the browser attempts
- * to locate and select the cell itsself, and the method fails if it
+ * to locate and select the cell itself, and the method fails if it
  * is unable to locate the cell by matching its [NSCell-stringValue] with
  * the component of the path.
  * </p>
@@ -1342,11 +1350,11 @@ static NSTextFieldCell *titleCell;
   return browserColumn->_width;
 }
 
-- (void) setWidth: (float)columnWidth ofColumn: (int)column
+- (void) setWidth: (float)columnWidth ofColumn: (int)columnIndex
 {
   NSBrowserColumn *browserColumn;
 
-  browserColumn = [_browserColumns objectAtIndex: column];
+  browserColumn = [_browserColumns objectAtIndex: columnIndex];
 
   browserColumn->_width = columnWidth;
   // FIXME: Send a notifiaction
@@ -2130,19 +2138,21 @@ static NSTextFieldCell *titleCell;
 
   selectedCellsCount = [selectedCells count];
 
-/* FIXME why were we reselecting already selected cells?
- * this caused bug #18881 because -selectCell: will scroll the row to visible.
- */
-#if 0
-
-  // Select cells that should be selected
-  if (selectedCellsCount > 0)
+  /* If some branch cells were selected but branch selection is not allowed
+     reset the selection and select only the leaf cells. It is a pity that
+     we cannot deselect cells individually. */
+  if (selectedCellsCount != aCount)
     {
+      BOOL autoscroll = [sender isAutoscroll];
+
+      /* Note: Temporarily disable autoscrolling to prevent bug #18881 */
+      [sender setAutoscroll: NO];
+      [sender deselectAllCells];
       enumerator = [selectedCells objectEnumerator];
       while ((cell = [enumerator nextObject]))
         [sender selectCell: cell];
+      [sender setAutoscroll: autoscroll];
     }
-#endif
 
   [self setLastColumn: column];
   // Single selection
@@ -3072,6 +3082,11 @@ static NSTextFieldCell *titleCell;
       [matrix setIntercellSpacing: matrixIntercellSpace];
       [matrix setAllowsEmptySelection: _allowsEmptySelection];
       [matrix setAutoscroll: YES];
+
+      // Set up background colors.
+      [matrix setBackgroundColor: [NSColor controlColor]];
+      [matrix setDrawsBackground: YES];
+
       if (!_allowsMultipleSelection)
         {
           [matrix setMode: NSRadioModeMatrix];
