@@ -30,19 +30,22 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include "config.h"
-#include <Foundation/NSArray.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSKeyValueCoding.h>
-#include <AppKit/NSControl.h>
-#include "AppKit/NSNibConnector.h"
-#include "AppKit/NSNibLoading.h"
-#include "GNUstepGUI/GSModelLoaderFactory.h"
+#import "config.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSURL.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSKeyValueCoding.h>
+#import "AppKit/NSControl.h"
+#import "AppKit/NSNib.h"
+#import "AppKit/NSNibConnector.h"
+#import "AppKit/NSNibLoading.h"
+#import "GNUstepGUI/GSModelLoaderFactory.h"
 
 @implementation	NSNibConnector
 
@@ -183,7 +186,36 @@
     {
       if (_src != nil)
 	{
-	  [_src setValue: _dst forKey: _tag];
+          NSString *selName;
+          SEL sel; 	 
+          
+          selName = [NSString stringWithFormat: @"set%@%@:", 	 
+                       [[_tag substringToIndex: 1] uppercaseString], 	 
+                      [_tag substringFromIndex: 1]]; 	 
+          sel = NSSelectorFromString(selName); 	 
+          
+          if (sel && [_src respondsToSelector: sel]) 	 
+            { 	 
+              [_src performSelector: sel withObject: _dst]; 	 
+            } 	 
+          else 	 
+            { 	 
+              const char *nam = [_tag cString]; 	 
+              const char *type; 	 
+              unsigned int size; 	 
+              int offset; 	 
+              
+              /* 	 
+               * Use the GNUstep additional function to set the instance 	 
+               * variable directly. 	 
+               * FIXME - need some way to do this for libFoundation and 	 
+               * Foundation based systems. 	 
+               */ 	 
+              if (GSObjCFindVariable(_src, nam, &type, &size, &offset)) 	 
+                { 	 
+                  GSObjCSetVariable(_src, offset, size, (void*)&_dst); 	 
+                } 	 
+            }
 	}
     }
   NS_HANDLER
@@ -199,10 +231,11 @@
    externalNameTable: (NSDictionary*)context
 	    withZone: (NSZone*)zone
 {
-  GSModelLoader *loader = [GSModelLoaderFactory modelLoaderForFileName: fileName];
-  BOOL loaded = [loader loadModelFile: fileName
-			externalNameTable: context
-			withZone: zone];
+  NSNib *nib = [[NSNib alloc] initWithContentsOfURL: [NSURL fileURLWithPath: fileName]];
+  BOOL loaded = [nib instantiateNibWithExternalNameTable: context
+                                                withZone: zone];
+
+  RELEASE(nib);
   return loaded;
 }
 
@@ -216,7 +249,7 @@
     {
       return NO;
     }
-  table = [NSDictionary dictionaryWithObject: owner forKey: @"NSOwner"];
+  table = [NSDictionary dictionaryWithObject: owner forKey: NSNibOwner];
 
   /*
    * First look for the NIB in the bundle corresponding to the owning class,
