@@ -31,29 +31,31 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include <Foundation/NSLock.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSValue.h>
-#include <GNUstepBase/GSCategories.h>
+#import "config.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSAttributedString.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
 
-#include "AppKit/AppKitExceptions.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSBezierPath.h"
-#include "AppKit/NSButtonCell.h"
-#include "AppKit/NSButton.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSSound.h"
-#include "AppKit/NSWindow.h"
-#include "GNUstepGUI/GSTheme.h"
-#include "GNUstepGUI/GSNibLoading.h"
+#import "AppKit/AppKitExceptions.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSBezierPath.h"
+#import "AppKit/NSButtonCell.h"
+#import "AppKit/NSButton.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSSound.h"
+#import "AppKit/NSStringDrawing.h"
+#import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GNUstepGUI/GSNibLoading.h"
+#import "GSGuiPrivate.h"
 
 #include <math.h>
 
@@ -131,7 +133,7 @@ typedef struct _GSButtonCellFlags
   [self setButtonType: NSMomentaryPushInButton];
   _delayInterval = 0.4;
   _repeatInterval = 0.075;
-  _keyEquivalentModifierMask = NSCommandKeyMask;
+  _keyEquivalentModifierMask = 0;
   _keyEquivalent = @"";
   _altContents = @"";
   _gradient_type = NSGradientNone;
@@ -195,11 +197,18 @@ typedef struct _GSButtonCellFlags
 }
 
 /** <p>Returns the NSButtonCell's title.</p>
-    <p>See Also: -setTitle: [NSCell-stringValue]</p>
  */
 - (NSString*) title
 {
-  return [super stringValue];
+  if (_cell.contents_is_attributed_string == NO)
+    {
+      // If we have a formatter this is also the string of the _object_value
+      return _contents;
+    }
+  else
+    {
+      return [(NSAttributedString *)_contents string];
+    }
 }
 
 /** <p>Returns the NSButtonCell's alternate title ( used when highlighted ).
@@ -323,11 +332,11 @@ typedef struct _GSButtonCellFlags
 }
 
 /**<p>Sets the NSButtonCell's title to <var>aString</var>.</p>
-   <p>See Also: -title [NSCell-setStringValue:]</p>
  */
 - (void) setTitle: (NSString*)aString
 {
-  [super setStringValue: aString];
+  ASSIGNCOPY(_contents, aString);
+  _cell.contents_is_attributed_string = NO;
 }
 
 /**<p>Sets the NSButtonCell's alternate title ( used when highlighted ) 
@@ -369,12 +378,27 @@ typedef struct _GSButtonCellFlags
 
 - (NSAttributedString *)attributedTitle
 {
-  return [super attributedStringValue];
+  if (_cell.contents_is_attributed_string)
+    {
+      return (NSAttributedString *)_contents;
+    }
+  else
+    {
+      NSDictionary *dict;
+      NSAttributedString *attrStr;
+
+      dict = [self _nonAutoreleasedTypingAttributes];
+      attrStr = [[NSAttributedString alloc] initWithString: _contents 
+                                            attributes: dict];
+      RELEASE(dict);
+      return AUTORELEASE(attrStr);
+    }
 }
 
 - (void)setAttributedTitle:(NSAttributedString *)aString
 {
-  [super setAttributedStringValue: aString];
+  ASSIGNCOPY(_contents, aString);
+  _cell.contents_is_attributed_string = YES;
 }
 
 - (void)setTitleWithMnemonic:(NSString *)aString
@@ -458,35 +482,18 @@ typedef struct _GSButtonCellFlags
    
   // In the GNUstep NSButtonCell implementation, the cell type depends only on
   // the image position. 
-    
+  /* NOTE: We set the cell type attribute directly here instead of calling
+     NSCell's -setType: method because it may change the title or image of
+     the button cell. This is to make our implementation compatible with
+     the behavior of Mac OS X, which does not change the cell's type and
+     hence does not involve any of the side effects of -setType: either. */
   if (_cell.image_position == NSNoImage)
     {
-      /* NOTE: If we always call -setType: on superclass, the cell _content will
-         be reset each time. When we alter a button displaying both an image and
-         a title, by just calling -setImagePosition: with 'NSNoImage' value,
-         the current title is reset to NSCell default one. That's why we have to
-         set cell type ourself when a custom title (or attributed title) is
-         already in use. 
-         Take note that [self title] is able to return the attributed title in
-         NSString form.
-         We precisely match Mac OS X behavior currently. That means... When you
-         switch from 'NSNoImage' option to another one, the title will be the
-         one in use before you switched to 'NSNoImage'. The reverse with the
-         image isn't true, when you switch to 'NSNoImage' option, the current
-         image is lost (image value being reset to nil).
-      */
-      if ([self title] == nil || [[self title] isEqualToString: @""])
-        {
-          [super setType: NSTextCellType];
-        }
-      else
-        { 
-          _cell.type = NSTextCellType;
-        }
+      _cell.type = NSTextCellType;
     }
   else
     {
-      [super setType: NSImageCellType];
+      _cell.type = NSImageCellType;
     }
 }
 
@@ -537,7 +544,7 @@ typedef struct _GSButtonCellFlags
 /** <p>Returns the modifier mask of the NSButtonCell's key equivalent. 
     The key equivalent and its modifier mask are used to simulate the click
     of the button in  [NSButton-performKeyEquivalent:]. The default mask is 
-    NSCommandKeyMask.</p><p>See Also: -setKeyEquivalentModifierMask:
+    0.</p><p>See Also: -setKeyEquivalentModifierMask:
     -keyEquivalent [NSButton-performKeyEquivalent:]</p>
  */
 - (unsigned int) keyEquivalentModifierMask
@@ -559,7 +566,7 @@ typedef struct _GSButtonCellFlags
 /** <p>Sets the modifier mask of the NSButtonCell's key equivalent to
     <var>mask</var>. The key equivalent and its modifier mask are used to
     simulate the click of the button in [NSButton-performKeyEquivalent:]. 
-    By default the mask is NSCommandKeyMask.</p>
+    By default the mask is 0.</p>
     <p>See Also: -keyEquivalentModifierMask  
     -setKeyEquivalent: [NSButton-performKeyEquivalent:]</p>
 */
@@ -602,11 +609,13 @@ typedef struct _GSButtonCellFlags
 }
 
 /**<p>Returns whether the NSButtonCell is opaque. Returns YES if the button 
-   cell is not transparent and if the cell is bordered. NO otherwise</p>
+   cell is not transparent and if the cell is bordered and if there is no
+   bezel style, NO otherwise</p>
  */
 - (BOOL) isOpaque
 {
-  return !_buttoncell_is_transparent && _cell.is_bordered;
+  return !_buttoncell_is_transparent && _cell.is_bordered &&
+    _bezel_style == 0;
 }
 
 - (NSBezelStyle) bezelStyle
@@ -886,6 +895,11 @@ typedef struct _GSButtonCellFlags
       buttonState = GSThemeSelectedState;
     }
 
+  if (_cell.is_disabled && buttonState != GSThemeHighlightedState)
+    {
+      buttonState = GSThemeDisabledState;
+    }
+
   [[GSTheme theme] drawButton: cellFrame 
                    in: self 
                    view: controlView
@@ -893,19 +907,19 @@ typedef struct _GSButtonCellFlags
                    state: buttonState];
 }
 
-- (void) drawImage: (NSImage*)anImage 
-         withFrame: (NSRect)aRect 
+- (void) drawImage: (NSImage*)imageToDisplay 
+         withFrame: (NSRect)cellFrame 
             inView: (NSView*)controlView
 {
   // Draw image
-  if (anImage != nil)
+  if (imageToDisplay != nil)
     {
       NSSize size;
       NSPoint position;
       
-      size = [anImage size];
-      position.x = MAX(NSMidX(aRect) - (size.width / 2.), 0.);
-      position.y = MAX(NSMidY(aRect) - (size.height / 2.), 0.);
+      size = [imageToDisplay size];
+      position.x = MAX(NSMidX(cellFrame) - (size.width / 2.), 0.);
+      position.y = MAX(NSMidY(cellFrame) - (size.height / 2.), 0.);
       
       /*
        * Images are always drawn with their bottom-left corner at the origin
@@ -915,25 +929,20 @@ typedef struct _GSButtonCellFlags
         {
           position.y += size.height;
         }
-      
-      if (_cell.is_disabled && _image_dims_when_disabled)
-        {
-          [anImage dissolveToPoint: position fraction: 0.5];
-        }
-      else
-        {
-          [anImage compositeToPoint: position 
-                          operation: NSCompositeSourceOver];
-        }
+
+      [[GSTheme theme] drawImage: imageToDisplay
+		    inButtonCell: self
+		       withFrame: cellFrame
+		        position: position];
     }
 }
 
 - (void) drawTitle: (NSAttributedString*)titleToDisplay 
-         withFrame: (NSRect)frame 
-            inView: (NSView*)control
+         withFrame: (NSRect)cellFrame 
+            inView: (NSView*)controlView
 {
   [self _drawAttributedText: titleToDisplay
-        inFrame: frame];
+		    inFrame: cellFrame];
 }
 
 // Private helper method overridden in subclasses
@@ -1468,11 +1477,11 @@ typedef struct _GSButtonCellFlags
   NSButtonCell *c = [super copyWithZone: zone];
   
   c->_altContents = [_altContents copyWithZone: zone];
-  TEST_RETAIN(_altImage);
-  TEST_RETAIN(_keyEquivalent);
-  TEST_RETAIN(_keyEquivalentFont);
-  TEST_RETAIN(_sound);
-  TEST_RETAIN(_backgroundColor);
+  _altImage = TEST_RETAIN(_altImage);
+  _keyEquivalent = TEST_RETAIN(_keyEquivalent);
+  _keyEquivalentFont = TEST_RETAIN(_keyEquivalentFont);
+  _sound = TEST_RETAIN(_sound);
+  _backgroundColor = TEST_RETAIN(_backgroundColor);
 
   return c;
 }
@@ -1541,8 +1550,9 @@ typedef struct _GSButtonCellFlags
       [aCoder encodeInt: bFlags forKey: @"NSButtonFlags"];
 
       // style and border.
-      bFlags2 != [self showsBorderOnlyWhileMouseInside] ? 0x8 : 0;
+      bFlags2 |= [self showsBorderOnlyWhileMouseInside] ? 0x8 : 0;
       bFlags2 |= (([self bezelStyle] & 0x7) | (([self bezelStyle] & 0x18) << 2));
+      bFlags2 |= [self keyEquivalentModifierMask] << 8;
       [aCoder encodeInt: bFlags2 forKey: @"NSButtonFlags2"];
 
       // alternate image encoding...
@@ -1698,6 +1708,8 @@ typedef struct _GSButtonCellFlags
           bFlags2 = [aDecoder decodeIntForKey: @"NSButtonFlags2"];
           [self setShowsBorderOnlyWhileMouseInside: (bFlags2 & 0x8)];
           [self setBezelStyle: (bFlags2 & 0x7) | ((bFlags2 & 0x20) >> 2)];
+          [self setKeyEquivalentModifierMask: ((bFlags2 >> 8) & 
+                                               NSDeviceIndependentModifierFlagsMask)];
         }
       if ([aDecoder containsValueForKey: @"NSAlternateImage"])
         {
