@@ -32,37 +32,39 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include <Foundation/NSString.h>
-#include <Foundation/NSGeometry.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSValue.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSNumberFormatter.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSFormatter.h>
-#include <Foundation/NSRunLoop.h>
-#include <GNUstepBase/GSCategories.h>
+#import "config.h"
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFormatter.h>
+#import <Foundation/NSValue.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSNumberFormatter.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSGeometry.h>
 
-#include "AppKit/AppKitExceptions.h"
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSControl.h"
-#include "AppKit/NSCell.h"
-#include "AppKit/NSClipView.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSCursor.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSParagraphStyle.h"
-#include "AppKit/NSTextView.h"
-#include "AppKit/NSTextContainer.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSWindow.h"
-#include "GNUstepGUI/GSTheme.h"
+#import "AppKit/AppKitExceptions.h"
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSControl.h"
+#import "AppKit/NSCell.h"
+#import "AppKit/NSClipView.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSCursor.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSParagraphStyle.h"
+#import "AppKit/NSStringDrawing.h"
+#import "AppKit/NSTextView.h"
+#import "AppKit/NSTextContainer.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GSGuiPrivate.h"
 
 static Class colorClass;
 static Class cellClass;
@@ -173,6 +175,7 @@ static NSColor *dtxtCol;
   //_cell.is_scrollable = NO;
   //_cell.is_selectable = NO;
   //_cell.state = 0;
+  //_cell.line_break_mode = NSLineBreakByWordWrapping;
   _action_mask = NSLeftMouseUpMask;
   _menu = [isa defaultMenu];
   [self setFocusRingType: [isa defaultFocusRingType]];
@@ -203,6 +206,7 @@ static NSColor *dtxtCol;
   //_cell.is_bezeled = NO;
   //_cell.is_scrollable = NO;
   //_cell.is_selectable = NO;
+  //_cell.line_break_mode = NSLineBreakByWordWrapping;
   _action_mask = NSLeftMouseUpMask;
   _menu = [isa defaultMenu];
   [self setFocusRingType: [isa defaultFocusRingType]];
@@ -291,6 +295,22 @@ static NSColor *dtxtCol;
     }
 }
 
+/**<p>Returns the cell's value as an NSInteger. </p>
+ *<p>See Also: -setIntegerValue:</p>
+ */
+- (NSInteger) integerValue
+{
+  if ((_cell.has_valid_object_value == YES) &&
+      ([_object_value respondsToSelector: @selector(integerValue)]))
+    {
+      return [_object_value integerValue];
+    }
+  else
+    {
+      return [[self stringValue] integerValue];
+    }
+}
+
 /**<p>Returns the cell's value as a NSString.</p>
  *<p>See Also: -setStringValue: </p>
  */
@@ -315,7 +335,7 @@ static NSColor *dtxtCol;
   
   if (_formatter == nil) 
     {
-      if ([object isKindOfClass: [NSString class]] == YES)
+      if (object == nil || [object isKindOfClass: [NSString class]] == YES)
         {
           newContents = object;
           _cell.contents_is_attributed_string = NO;
@@ -327,13 +347,13 @@ static NSColor *dtxtCol;
           _cell.contents_is_attributed_string = YES;
           _cell.has_valid_object_value = YES;
         }
-      else if([_object_value respondsToSelector: @selector(attributedStringValue)])
+      else if ([_object_value respondsToSelector: @selector(attributedStringValue)])
         {
           newContents = [_object_value attributedStringValue];
           _cell.contents_is_attributed_string = YES;
           _cell.has_valid_object_value = YES;
         }
-      else if([_object_value respondsToSelector: @selector(stringValue)])
+      else if ([_object_value respondsToSelector: @selector(stringValue)])
         {
           // If the thing that was assigned is not a string, but 
           // responds to stringValue then get that.
@@ -410,6 +430,20 @@ static NSColor *dtxtCol;
   [self setObjectValue: number];
 }
 
+/**
+ *<p>Sets the NSCell's value to anInt.</p>
+ *<p>See Also: -integerValue</p> 
+ */
+- (void) setIntegerValue: (NSInteger)anInt
+{
+  NSNumber *number;
+
+  // NB: GNUstep can set an int value for an image cell. 
+
+  number = [NSNumber numberWithInteger: anInt];
+  [self setObjectValue: number];
+}
+
 /**<p>Sets the cell's value to a NSString. 
    The NSCell's type is set to NSTextCellType if needed</p>
    <p>See Also: -stringValue</p> 
@@ -431,18 +465,7 @@ static NSColor *dtxtCol;
 
   if (_formatter == nil)
     {
-      /* if we are a string, we do an optimization and set the value here instead of using setObjectValue
-       * also, we check for nil, since isKindOfClass fails for nil objects */
-      if([aString isKindOfClass: [NSString class]] == YES || aString == nil)
-        {
-          ASSIGN (_contents, aString);
-          ASSIGN (_object_value, aString);
-          _cell.has_valid_object_value = YES;
-        }
-      else
-        {
-          [self setObjectValue: aString];
-        }
+      [self setObjectValue: aString];
     }
   else
     {
@@ -933,22 +956,22 @@ static NSColor *dtxtCol;
 
 - (void) setWraps: (BOOL)flag
 {
-  _cell.wraps = flag;  
-
   if (flag)
     {
-      _cell.is_scrollable = NO;
-      [self setLineBreakMode: NSLineBreakByWordWrapping];
+      if (![self wraps])
+	[self setLineBreakMode: NSLineBreakByWordWrapping];
     }
   else
     {
-      [self setLineBreakMode: NSLineBreakByClipping];
+      if ([self wraps])
+	[self setLineBreakMode: NSLineBreakByClipping];
     }
 }
 
 - (BOOL) wraps
 {
-  return _cell.wraps;
+  return _cell.line_break_mode == NSLineBreakByWordWrapping
+      || _cell.line_break_mode == NSLineBreakByCharWrapping;
 }
 
 - (void) setAttributedStringValue: (NSAttributedString*)attribStr
@@ -966,11 +989,15 @@ static NSColor *dtxtCol;
           /* What about the attributed string ?  We are loosing it. */
           return;
         }
+      _cell.has_valid_object_value = NO;
+    }
+  else
+    {
+      _cell.has_valid_object_value = YES;
+      ASSIGN (_object_value, attribStr);
     }
 
-  /* In all other cases */
   ASSIGN (_contents, attribStr);
-  _cell.has_valid_object_value = NO;
   _cell.contents_is_attributed_string = YES;
 }
 
@@ -1050,6 +1077,10 @@ static NSColor *dtxtCol;
 
 - (void) setLineBreakMode: (NSLineBreakMode)mode
 {
+  if (mode == NSLineBreakByCharWrapping || mode == NSLineBreakByWordWrapping)
+    {
+      _cell.is_scrollable = NO;
+    }
   _cell.line_break_mode = mode;
 }
 
@@ -1177,7 +1208,7 @@ static NSColor *dtxtCol;
     The NSCell implementation raises an NSInvalidArgumentException.</p>
     <p>See Also: -tag</p>
  */
-- (void) setTag: (int)anInt
+- (void) setTag: (NSInteger)anInt
 {
   [NSException raise: NSInternalInconsistencyException
               format: @"attempt to set a tag in an NSCell"];
@@ -1186,7 +1217,7 @@ static NSColor *dtxtCol;
 /**<p>Implemented by subclasses to Return the tag. 
    The NSCell implementation returns -1 </p><p>See Also: -setTag:</p>
  */
-- (int) tag
+- (NSInteger) tag
 {
   return -1;
 }
@@ -1336,12 +1367,19 @@ static NSColor *dtxtCol;
 
 - (void) setTitleWithMnemonic: (NSString*)aString
 {
-  unsigned int location = [aString rangeOfString: @"&"].location;
+  NSRange r = [aString rangeOfString: @"&"];
 
-  [self setTitle: [aString stringByReplacingString: @"&"
-                                 withString: @""]];
-  // TODO: We should underline this character
-  [self setMnemonicLocation: location];
+  if (r.length > 0)
+    {
+      unsigned int location = r.location;
+      
+      
+      [self setTitle: [[aString substringToIndex: location] 
+                        stringByAppendingString: 
+                          [aString substringFromIndex: NSMaxRange(r)]]];
+      // TODO: We should underline this character
+      [self setMnemonicLocation: location];
+    }
 }
 
 - (NSString*) mnemonic
@@ -1395,7 +1433,7 @@ static NSColor *dtxtCol;
 {
   SEL action = [self action];
 
-  if ([sender respondsTo: @selector(sendAction:to:)])
+  if ([sender respondsToSelector: @selector(sendAction:to:)])
     {
       return [sender sendAction: action to: [self target]];
     }
@@ -1480,6 +1518,14 @@ static NSColor *dtxtCol;
   [self setIntValue: [sender intValue]];
 }
 
+/** <p>Sets the NSCell's NSInteger value to sender's NSInteger value</p>
+    <p>See Also: -setIntegerValue:</p>
+ */
+- (void) takeIntegerValueFrom: (id)sender
+{
+  [self setIntegerValue: [sender integerValue]];
+}
+
 /** <p>Sets the NSCell's NSString value to sender's NSSting value</p>
     <p>See Also: -setStringValue:</p>
  */
@@ -1553,7 +1599,7 @@ static NSColor *dtxtCol;
     return NO;
 }
 
-/**<p>TODO</p>
+/**<p>Default implementation of this method in NSCell does nothing.</p>
  */
 - (void) stopTracking: (NSPoint)lastPoint
                    at: (NSPoint)stopPoint
@@ -1944,23 +1990,26 @@ static NSColor *dtxtCol;
  */
 - (void) drawInteriorWithFrame: (NSRect)cellFrame inView: (NSView*)controlView
 {
-  cellFrame = [self drawingRectForBounds: cellFrame];
+  NSRect drawingRect = [self drawingRectForBounds: cellFrame];
 
   //FIXME: Check if this is also neccessary for images,
   // Add spacing between border and inside 
   if (_cell.is_bordered || _cell.is_bezeled)
     {
-      cellFrame.origin.x += 3;
-      cellFrame.size.width -= 6;
-      cellFrame.origin.y += 1;
-      cellFrame.size.height -= 2;
+      drawingRect.origin.x += 3;
+      drawingRect.size.width -= 6;
+      drawingRect.origin.y += 1;
+      drawingRect.size.height -= 2;
     }
 
   switch (_cell.type)
     {
       case NSTextCellType:
-        [self _drawAttributedText: [self _drawAttributedString]
-              inFrame: cellFrame];
+	if (_cell.in_editing)
+	  [self _drawEditorWithFrame: cellFrame inView: controlView];
+	else
+	  [self _drawAttributedText: [self _drawAttributedString]
+			    inFrame: drawingRect];
         break;
 
       case NSImageCellType:
@@ -1970,8 +2019,8 @@ static NSColor *dtxtCol;
             NSPoint position;
             
             size = [_cell_image size];
-            position.x = MAX(NSMidX(cellFrame) - (size.width/2.),0.);
-            position.y = MAX(NSMidY(cellFrame) - (size.height/2.),0.);
+            position.x = MAX(NSMidX(drawingRect) - (size.width/2.),0.);
+            position.y = MAX(NSMidY(drawingRect) - (size.height/2.),0.);
             /*
              * Images are always drawn with their bottom-left corner
              * at the origin so we must adjust the position to take
@@ -2067,6 +2116,8 @@ static NSColor *dtxtCol;
 
 - (NSText*) setUpFieldEditorAttributes: (NSText*)textObject
 {
+  NSDictionary *attr;
+
   // Reset the string to have a well defined state. The real string gets set later on.
   [textObject setString: @""];
 
@@ -2089,6 +2140,9 @@ static NSColor *dtxtCol;
   [textObject setRichText: [self allowsEditingTextAttributes]];
   [textObject setImportsGraphics: [self importsGraphics]];
   [(NSTextView*)textObject setAllowsUndo: [self allowsUndo]];
+  attr = [self _nonAutoreleasedTypingAttributes];
+  [(NSTextView*)textObject setTypingAttributes: attr];
+  RELEASE(attr);
 
   return textObject;
 }
@@ -2099,62 +2153,68 @@ static NSColor *dtxtCol;
                     delegate: (id)anObject
                        range: (NSRange)selection
 {
+  BOOL needsClipView;
+  BOOL wraps = [self wraps];
+  NSTextContainer *ct;
+  NSSize maxSize;
   NSRect titleRect = [self titleRectForBounds: aRect];
-  NSClipView *cv = [[NSClipView alloc] initWithFrame: titleRect];
-  NSTextContainer *ct = [(NSTextView*)textObject textContainer];
-  NSRect maxRect;
 
-  // A clip view should is only created for scrollable text
-  if ([self isScrollable])
+  /* We always add a clip view if the cell is editable so that the user
+     can edit the whole contents even if the cell's contents normally is
+     clipped. */
+  needsClipView = [self isScrollable] || [self isEditable];
+  if (needsClipView)
     {
-      /* See comments in NSStringDrawing.m about the choice of maximum size. */
-      maxRect = NSMakeRect(0, 0, 1e6, titleRect.size.height);
+      NSClipView *cv;
+
+      cv = [[NSClipView alloc] initWithFrame: titleRect];
+      [cv setDocumentView: textObject];
+      [controlView addSubview: cv];
+      RELEASE(cv);
     }
   else
-    {
-      maxRect = NSMakeRect(0, 0, titleRect.size.width, titleRect.size.height);
-    }
+    [controlView addSubview: textObject];
 
-  [controlView addSubview: cv];
-  RELEASE(cv);
-  [cv setAutoresizesSubviews: NO];
-  [cv setDocumentView: textObject];
-  [ct setContainerSize: maxRect.size];
+  /* Note: The order of statements matters here. We must set the text object's
+     horizontallyResizable and verticallyResizable attributes before setting
+     its frame size. Otherwise, the text object's width and/or height might
+     incorrectly be reduced to zero (since the text object has no contents at
+     this point) if the text object was resizable by its text container before.
+     Of course we could also have set the text object's minimum size to the
+     intended frame size, but then we must update the minimum size whenever
+     the field editor's frame is changed in -_drawEditorWithFrame:inView:.
+     Note that the minimum size is not relevant when a clip view is used. */
+  [textObject setMinSize: NSZeroSize];
+  [textObject setMaxSize: NSMakeSize(1e6, 1e6)];
+  [textObject setHorizontallyResizable: needsClipView && !wraps];
+  [textObject setVerticallyResizable: needsClipView];
+  [textObject setFrame: titleRect];
+  if (needsClipView)
+    [textObject setAutoresizingMask: NSViewWidthSizable + NSViewHeightSizable];
+  else
+    [textObject setAutoresizingMask: NSViewNotSizable];
+
+  /* Note: Order of statements matters again. The heightTracksTextView
+     and widthTracksTextView container attributes must be set after setting
+     the horizontallyResizable and verticallyResizable text view attributes
+     because NSTextView's setter methods include "safety code", which
+     always updates the container attributes along with the text view
+     attributes.
+     FIXME Fix NSTextView to only reset the text container attributes, but
+     never set them. However note that this may break some sloppily written
+     code which forgets to set the text container attributes. */
+  /* See comments in NSStringDrawing.m about the choice of maximum size. */
+  ct = [(NSTextView*)textObject textContainer];
+  if (wraps)
+    maxSize = NSMakeSize(NSWidth(titleRect), 1e6);
+  else
+    maxSize = NSMakeSize(1e6, 1e6);
+  [ct setContainerSize: maxSize];
+  [ct setWidthTracksTextView: wraps];
   [ct setHeightTracksTextView: NO];
-  [ct setWidthTracksTextView: NO];
 
-  [textObject setFrame: maxRect];
-  [textObject setHorizontallyResizable: NO];
-  [textObject setVerticallyResizable: NO];
-  [textObject setMaxSize: maxRect.size];
-  [textObject setMinSize: titleRect.size];
+  [self _updateFieldEditor: textObject];
 
-  if (_formatter != nil)
-    {
-      NSString *contents; 
-
-      contents = [_formatter editingStringForObjectValue: _object_value];
-      if (contents == nil)
-        {
-          contents = _contents;
-        }
-      [textObject setText: contents];
-    }
-  else
-    {
-      if (_cell.contents_is_attributed_string == NO)
-        {
-          [textObject setText: _contents];
-        }
-      else
-        {
-          // The curent text has size 0, so this replaces the whole text.
-          [textObject replaceCharactersInRange: NSMakeRange(0, 0)
-                      withAttributedString: (NSAttributedString *)_contents];
-        }
-    }
-
-  [textObject sizeToFit];
   [textObject setSelectedRange: selection];
   [textObject scrollRangeToVisible: selection];
 
@@ -2176,8 +2236,13 @@ static NSColor *dtxtCol;
   [textObject setDelegate: nil];
   
   clipView = (NSClipView*)[textObject superview];
-  [textObject removeFromSuperview];
-  [clipView removeFromSuperview];
+  if ([clipView isKindOfClass: [NSClipView class]])
+    {
+      [clipView setDocumentView: nil];
+      [clipView removeFromSuperview];
+    }
+  else
+    [textObject removeFromSuperview];
 }
 
 /*
@@ -2261,12 +2326,12 @@ static NSColor *dtxtCol;
   /* Because of performance issues (and because so the doc says) only
      pointers to the objects are copied.  We need to RETAIN them all
      though. */
-  TEST_RETAIN (_font);
-  TEST_RETAIN (_object_value);
-  TEST_RETAIN (_menu);
-  TEST_RETAIN (_cell_image);
-  TEST_RETAIN (_formatter);
-  TEST_RETAIN (_represented_object);
+  _font = TEST_RETAIN (_font);
+  _object_value = TEST_RETAIN (_object_value);
+  _menu = TEST_RETAIN (_menu);
+  _cell_image = TEST_RETAIN (_cell_image);
+  _formatter = TEST_RETAIN (_formatter);
+  _formatter = TEST_RETAIN (_represented_object);
 
   return c;
 }
@@ -2514,7 +2579,7 @@ static NSColor *dtxtCol;
     }
   else
     {
-      BOOL flag;
+      BOOL flag, wraps;
       unsigned int tmp_int;
       id formatter, menu;
       int version = [aDecoder versionForClassName: @"NSCell"];
@@ -2555,7 +2620,9 @@ static NSColor *dtxtCol;
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &flag];
       _cell.allows_mixed_state = flag;
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &flag];
-      _cell.wraps = flag;
+      /* The wraps attribute has been superseded by lineBreakMode. However,
+	 we may need it to set lineBreakMode when reading old archives. */
+      wraps = flag;
       [aDecoder decodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
       _cell.text_align = tmp_int;
       [aDecoder decodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
@@ -2697,6 +2764,12 @@ static NSColor *dtxtCol;
           [aDecoder decodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
           _cell.base_writing_direction = tmp_int;
         }
+      else
+	{
+	  /* Backward compatibility: Derive lineBreakMode from the superseded
+	     wraps attribute. */
+	  [self setWraps: wraps];
+	}
     }
   return self;
 }
@@ -2871,9 +2944,70 @@ static NSColor *dtxtCol;
     }
 }
 
+- (void) _drawEditorWithFrame: (NSRect)cellFrame
+		       inView: (NSView *)controlView
+{
+  /* Look Ma', no drawing here... */
+
+  if ([controlView isKindOfClass: [NSControl class]])
+    {
+      /* Adjust the text editor's frame to match cell's frame (w/o border) */
+      NSRect titleRect = [self titleRectForBounds: cellFrame];
+      NSText *textObject = [(NSControl*)controlView currentEditor];
+      NSView *clipView = [textObject superview];
+
+      if ([clipView isKindOfClass: [NSClipView class]])
+	{
+	  [clipView setFrame: titleRect];
+	}
+      else
+	{
+	  [textObject setFrame: titleRect];
+	}
+    }
+}
+
 - (BOOL) _sendsActionOn:(int)eventTypeMask
 {
   return (_action_mask & eventTypeMask);
 }
 
+- (void) _setInEditing: (BOOL)flag
+{
+  _cell.in_editing = flag;
+}
+
+- (void) _updateFieldEditor: (NSText*)textObject
+{
+  if (_formatter != nil)
+    {
+      NSString *contents; 
+
+      contents = [_formatter editingStringForObjectValue: _object_value];
+      if (contents == nil)
+        {
+          contents = _contents;
+        }
+      if (![contents isEqualToString: [textObject string]])
+	[textObject setText: contents];
+    }
+  else
+    {
+      if (_cell.contents_is_attributed_string == NO)
+        {
+	  if (![_contents isEqualToString:[textObject string]])
+	    [textObject setText: _contents];
+        }
+      else
+        {
+	  // FIXME what about attribute changes?
+          NSRange range = NSMakeRange(0, [[textObject string] length]);
+	  if (![[(NSAttributedString *)_contents string] isEqualToString:
+		 [textObject string]])
+	    [textObject replaceCharactersInRange: range
+			withAttributedString: (NSAttributedString *)_contents];
+        }
+    }
+}
+
 @end
