@@ -28,27 +28,28 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include <Foundation/NSString.h>
-#include <Foundation/NSArchiver.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSScanner.h>
+#import "config.h"
+#import <Foundation/NSString.h>
+#import <Foundation/NSArchiver.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSScanner.h>
 
-#include "AppKit/NSColor.h"
-#include "AppKit/NSColorList.h"
-#include "AppKit/NSColorSpace.h"
-#include "AppKit/NSPasteboard.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/PSOperators.h"
-#include "GNUstepGUI/GSTheme.h"
-#include "GSGuiPrivate.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSColorList.h"
+#import "AppKit/NSColorSpace.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/PSOperators.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GSGuiPrivate.h"
 
 static Class	NSColorClass;
 
@@ -283,12 +284,11 @@ void initSystemColors(void)
 static NSColor*
 systemColorWithName(NSString *name)
 {
-  NSColor* col = [systemDict objectForKey: name];
+  NSColor	*col = [systemDict objectForKey: name];
 
   if (col == nil)
     {
-      col = [NSColor colorWithCatalogName: @"System"
-		     colorName: name];
+      col = [NSColor colorWithCatalogName: @"System" colorName: name];
       [systemDict setObject: col forKey: name];
     }
 
@@ -502,7 +502,7 @@ systemColorWithName(NSString *name)
 		}
 }
 
-+ (NSControlTint) currentControlTint;
++ (NSControlTint) currentControlTint
 {
 	// FIXME: should be made a system setting
 	return NSBlueControlTint;
@@ -1167,6 +1167,13 @@ systemColorWithName(NSString *name)
                             count: (int)number
 {
   // FIXME
+  if (space == [NSColorSpace genericRGBColorSpace] && (number == 4)) 
+    {
+      return [self colorWithCalibratedRed: comp[0]
+                   green: comp[1]
+                   blue: comp[2]
+                   alpha: comp[3]];
+    }
   if (space == [NSColorSpace deviceRGBColorSpace] && (number == 4)) 
     {
       return [self colorWithDeviceRed: comp[0]
@@ -1174,10 +1181,22 @@ systemColorWithName(NSString *name)
                    blue: comp[2]
                    alpha: comp[3]];
     }
+  if (space == [NSColorSpace genericGrayColorSpace] && (number == 2)) 
+    {
+      return [NSColor colorWithCalibratedWhite: comp[0] alpha: comp[1]];
+    }
   if (space == [NSColorSpace deviceGrayColorSpace] && (number == 2)) 
     {
       return [NSColor colorWithDeviceWhite: comp[0] alpha: comp[1]];
     }
+  if (space == [NSColorSpace genericCMYKColorSpace] && (number == 5)) 
+    {
+      return [NSColor colorWithDeviceCyan: comp[0] 
+                      magenta: comp[1]
+                      yellow: comp[2] 
+                      black: comp[3] 
+                      alpha: comp[4]];
+    }
   if (space == [NSColorSpace deviceCMYKColorSpace] && (number == 5)) 
     {
       return [NSColor colorWithDeviceCyan: comp[0] 
@@ -1187,7 +1206,7 @@ systemColorWithName(NSString *name)
                       alpha: comp[4]];
     }
 
-	return nil;
+  return nil;
 }
 				
 /*
@@ -1198,26 +1217,65 @@ systemColorWithName(NSString *name)
 {
   NSString *name = [self colorSpaceName];
 
-	if ([name isEqualToString: NSCalibratedRGBColorSpace])
+  if ([name isEqualToString: NSCalibratedRGBColorSpace])
     return [NSColorSpace genericRGBColorSpace];
-	if ([name isEqualToString: NSDeviceRGBColorSpace])
+  if ([name isEqualToString: NSDeviceRGBColorSpace])
     return [NSColorSpace deviceRGBColorSpace];
-	if ([name isEqualToString: NSCalibratedBlackColorSpace]
-      || [name isEqualToString: NSCalibratedWhiteColorSpace])
+  if ([name isEqualToString: NSCalibratedBlackColorSpace]
+    || [name isEqualToString: NSCalibratedWhiteColorSpace])
     return [NSColorSpace genericGrayColorSpace];
-	if ([name isEqualToString: NSDeviceBlackColorSpace]
-      || [name isEqualToString: NSDeviceWhiteColorSpace])
+  if ([name isEqualToString: NSDeviceBlackColorSpace]
+    || [name isEqualToString: NSDeviceWhiteColorSpace])
     return [NSColorSpace deviceGrayColorSpace];
-	if ([name isEqualToString: NSDeviceCMYKColorSpace])
+  if ([name isEqualToString: NSDeviceCMYKColorSpace])
     return [NSColorSpace deviceCMYKColorSpace];
 
-	return nil;
+  return nil;
 }
 
 - (NSColor *) colorUsingColorSpace: (NSColorSpace *)space
 {
   // FIXME
-	return nil;
+  NSString *colorSpaceName;
+
+  if (space == [self colorSpace])
+    {
+      return self;
+    }
+
+  switch ([space colorSpaceModel])
+    {
+      default:
+      case NSUnknownColorSpaceModel: 
+        return nil;
+      case NSGrayColorSpaceModel:
+        colorSpaceName = NSDeviceWhiteColorSpace;
+      case NSRGBColorSpaceModel:
+        colorSpaceName = NSDeviceRGBColorSpace;
+      case NSCMYKColorSpaceModel:
+        colorSpaceName = NSDeviceCMYKColorSpace;
+      case NSLABColorSpaceModel:
+        return nil;
+      case NSDeviceNColorSpaceModel:
+        return nil;
+    }
+  return [self colorUsingColorSpaceName: colorSpaceName
+				 device: nil];
+}
+
+- (NSUInteger) hash
+{
+  int nums = [self numberOfComponents];
+  float floats[nums];
+  NSUInteger	h = 0;
+  unsigned	i;
+
+  [self getComponents: &floats[0]];
+  for (i = 0; i < sizeof(floats); i++)
+    {
+      h = (h << 5) + h + *(uint8_t*)(((uintptr_t)&floats[0])+(i*sizeof(uint8_t)));
+    }
+  return h;
 }
 
 - (int) numberOfComponents
@@ -1348,7 +1406,7 @@ systemColorWithName(NSString *name)
 
       if ((colorSpace == 1) || (colorSpace == 2))
         {
-	  unsigned length;
+	  NSUInteger length;
 	  const uint8_t *data;
 	  float red = 0.0;
 	  float green = 0.0;
@@ -1388,7 +1446,7 @@ systemColorWithName(NSString *name)
 	}
       else if ((colorSpace == 3) || (colorSpace == 4))
         {
-	  unsigned length;
+	  NSUInteger length;
 	  const uint8_t *data;
 	  float white = 0.0;
 	  float alpha = 1.0;
@@ -1420,7 +1478,7 @@ systemColorWithName(NSString *name)
 	}
       else if (colorSpace == 5)
         {
-	  unsigned length;
+	  NSUInteger length;
 	  const uint8_t *data;
 	  float cyan = 0.0;
 	  float yellow = 0.0;
@@ -1765,6 +1823,8 @@ systemColorWithName(NSString *name)
 {
   GSTheme	*theme = [notification object];
   NSColorList	*list = [theme colors];
+  NSEnumerator	*enumerator;
+  NSString	*name;
 
   if (list == nil)
     {
@@ -1779,7 +1839,11 @@ systemColorWithName(NSString *name)
    */
   list = [NSColorList colorListNamed: @"System"];
   ASSIGN(systemColors, list);
-  [systemDict removeAllObjects];
+  enumerator = [systemDict keyEnumerator];
+  while ((name = [enumerator nextObject]) != nil)
+    {
+      [[systemDict objectForKey: name] recache];
+    }
   [[NSNotificationCenter defaultCenter]
     postNotificationName: NSSystemColorsDidChangeNotification object: nil];
 }
@@ -1790,12 +1854,42 @@ systemColorWithName(NSString *name)
 // Named colours
 @implementation GSNamedColor
 
+static	NSMutableDictionary	*namedColors = nil;
+static	NSRecursiveLock		*namedColorLock = nil;
+
++ (void) initialize
+{
+  namedColorLock = [NSRecursiveLock new];
+  namedColors = [NSMutableDictionary new];
+}
+
 - (NSColor*) initWithCatalogName: (NSString *)listName
 		       colorName: (NSString *)colorName
 {
-  ASSIGN(_catalog_name, listName);
-  ASSIGN(_color_name, colorName);
+  NSMutableDictionary	*d;
+  NSColor		*c;
 
+  _catalog_name = [listName copy];
+  _color_name = [colorName copy];
+  [namedColorLock lock];
+  d = [namedColors objectForKey: _catalog_name];
+  if (d == nil)
+    {
+      d = [NSMutableDictionary new];
+      [namedColors setObject: d forKey: _catalog_name];
+      [d release];
+    }
+  c = [d objectForKey: _color_name];
+  if (c == nil)
+    {
+      [d setObject: self forKey: _color_name];
+    }
+  else
+    {
+      [self release];
+      self = [c retain];
+    }
+  [namedColorLock unlock];
   return self;
 }
 
@@ -1815,20 +1909,7 @@ systemColorWithName(NSString *name)
 
 - (id) copyWithZone: (NSZone*)aZone
 {
-  if (NSShouldRetainWithZone(self, aZone))
-    {
-      return RETAIN(self);
-    }
-  else
-    {
-      GSNamedColor *aCopy = (GSNamedColor*)NSCopyObject(self, 0, aZone);
-
-      aCopy->_catalog_name = [_catalog_name copyWithZone: aZone];
-      aCopy->_color_name = [_color_name copyWithZone: aZone];
-      aCopy->_cached_name_space = nil;
-      aCopy->_cached_color = nil;
-	 return aCopy;
-    }
+  return RETAIN(self);
 }
 
 - (NSString*) description
@@ -1869,30 +1950,29 @@ systemColorWithName(NSString *name)
   return NSLocalizedString(_color_name, @"colour name");
 }
 
+- (NSUInteger) hash
+{
+  return [_catalog_name hash] + [_color_name hash];
+}
+
 - (BOOL) isEqual: (id)other
 {
   if (other == self)
     return YES;
-  if ([other isKindOfClass: [self class]] == NO)
-    return NO;
-  else
+  if ([other isKindOfClass: [self class]] == NO
+    || [[other catalogNameComponent] isEqualToString: _catalog_name] == NO
+    || [[other colorNameComponent] isEqualToString: _color_name] == NO)
     {
-      GSNamedColor *col = (GSNamedColor*)other;
-
-      if (![[col catalogNameComponent] isEqualToString: _catalog_name])
-	return NO;
-      if (![[col colorNameComponent] isEqualToString: _color_name])
-	return NO;
-      return YES;
+      return NO;
     }
-  return NO;
+  return YES;
 }
 
 - (NSColor*) colorUsingColorSpaceName: (NSString *)colorSpace
 			       device: (NSDictionary *)deviceDescription
 {
-  NSColorList *list;
-  NSColor *real;
+  NSColorList	*list;
+  NSColor	*real;
 
   if (colorSpace == nil)
     {
@@ -1908,29 +1988,30 @@ systemColorWithName(NSString *name)
       return self;
     }
 
-
   // Is there a cache hit?
   // FIXME How would we detect that the cache has become invalid by a
   // change to the colour list?
-  if ([colorSpace isEqualToString: _cached_name_space])
+  [namedColorLock lock];
+  if (NO == [colorSpace isEqualToString: _cached_name_space])
     {
-      return _cached_color;
+      list = [NSColorList colorListNamed: _catalog_name];
+      real = [list colorWithKey: _color_name];
+      ASSIGN(_cached_color, [real colorUsingColorSpaceName: colorSpace
+	device: deviceDescription]);
+      ASSIGN(_cached_name_space, colorSpace);
     }
+  real = [[_cached_color retain] autorelease];
+  [namedColorLock unlock];
 
-  list = [NSColorList colorListNamed: _catalog_name];
-  real = [list colorWithKey: _color_name];
-
-  ASSIGN(_cached_color, [real colorUsingColorSpaceName: colorSpace
-		       device: deviceDescription]);
-  ASSIGN(_cached_name_space, colorSpace);
-
-  return _cached_color;
+  return real;
 }
 
 - (void) recache
 {
+  [namedColorLock lock];
   DESTROY(_cached_name_space);
   DESTROY(_cached_color);
+  [namedColorLock unlock];
 }
 
 //
@@ -1954,10 +2035,13 @@ systemColorWithName(NSString *name)
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_catalog_name];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_color_name];
-
-  return self;
+  NSString	*listName;
+  NSString	*colorName;
+  
+  listName = [aDecoder decodeObject];
+  colorName = [aDecoder decodeObject];
+  return [self initWithCatalogName: listName
+		         colorName: colorName];
 }
 
 @end
@@ -2016,18 +2100,13 @@ systemColorWithName(NSString *name)
 {
   if (other == self)
     return YES;
-  if ([other isKindOfClass: [self class]] == NO)
-    return NO;
-  else
+  if ([other isKindOfClass: [self class]] == NO
+    || [other whiteComponent] != _white_component
+    || [other alphaComponent] != _alpha_component)
     {
-      GSWhiteColor *col = (GSWhiteColor*)other;
-
-      if (col->_white_component != _white_component ||
-	  col->_alpha_component != _alpha_component)
-	  return NO;
-      return YES;
+      return NO;
     }
-  return NO;
+  return YES;
 }
 
 - (NSColor*) colorWithAlphaComponent: (float)alpha
@@ -2327,23 +2406,16 @@ systemColorWithName(NSString *name)
 {
   if (other == self)
     return YES;
-  if ([other isKindOfClass: [self class]] == NO)
-    return NO;
-  else
+  if ([other isKindOfClass: [self class]] == NO
+    || [other cyanComponent] != _cyan_component
+    || [other magentaComponent] != _magenta_component
+    || [other yellowComponent] != _yellow_component
+    || [other blackComponent] != _black_component
+    || [other alphaComponent] != _alpha_component)
     {
-      GSDeviceCMYKColor *col = (GSDeviceCMYKColor*)other;
-
-      if (col->_cyan_component != _cyan_component
-	|| col->_magenta_component != _magenta_component
-	|| col->_yellow_component != _yellow_component
-	|| col->_black_component != _black_component
-	|| col->_alpha_component != _alpha_component)
-	{
-	  return NO;
-	}
-      return YES;
+      return NO;
     }
-  return NO;
+  return YES;
 }
 
 - (NSColor*) colorWithAlphaComponent: (float)alpha
@@ -2529,12 +2601,12 @@ systemColorWithName(NSString *name)
   components[0] = _red_component;
   components[1] = _green_component;
   components[2] = _blue_component;
-  components[4] = _alpha_component;
+  components[3] = _alpha_component;
 }
 
 - (int) numberOfComponents
 {
-	return 4;
+  return 4;
 }
 
 - (void) getHue: (float*)hue
@@ -2573,22 +2645,15 @@ systemColorWithName(NSString *name)
 {
   if (other == self)
     return YES;
-  if ([other isKindOfClass: [self class]] == NO)
-    return NO;
-  else
+  if ([other isKindOfClass: [self class]] == NO
+    || [other redComponent] != _red_component
+    || [other greenComponent] != _green_component
+    || [other blueComponent] != _blue_component
+    || [other alphaComponent] != _alpha_component)
     {
-      GSRGBColor *col = (GSRGBColor*)other;
-
-      if (col->_red_component != _red_component
-	|| col->_green_component != _green_component
-	|| col->_blue_component != _blue_component)
-	{
-	  return NO;
-	}
-      return YES;
+      return NO;
     }
-
-  return NO;
+  return YES;
 }
 
 - (NSColor*) colorWithAlphaComponent: (float)alpha
@@ -3081,22 +3146,21 @@ systemColorWithName(NSString *name)
   return desc;
 }
 
+- (NSUInteger) hash
+{
+  return [_pattern hash];
+}
+
 - (BOOL) isEqual: (id)other
 {
   if (other == self)
     return YES;
-  if ([other isKindOfClass: [self class]] == NO)
-    return NO;
-  else
+  if ([other isKindOfClass: [self class]] == NO
+    || [[other patternImage] isEqual: _pattern] == NO)
     {
-      GSPatternColor *col = (GSPatternColor*)other;
-
-      if ([col->_pattern isEqual: _pattern] == NO)
-	return NO;
-      return YES;
+      return NO;
     }
-
-  return NO;
+  return YES;
 }
 
 - (id) copyWithZone: (NSZone*)aZone
