@@ -31,6 +31,7 @@
 #include "config.h"
 #include "AppKit/NSActionCell.h"
 #include "AppKit/NSApplication.h"
+#include "AppKit/NSBezierPath.h"
 #include "AppKit/NSColorPanel.h"
 #include "AppKit/NSColorWell.h"
 #include "AppKit/NSColor.h"
@@ -42,6 +43,7 @@
 #include "GNUstepGUI/GSTheme.h"
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSNotification.h>
+#include <math.h>
 
 static NSString *GSColorWellDidBecomeExclusiveNotification =
                     @"GSColorWellDidBecomeExclusiveNotification";
@@ -145,6 +147,10 @@ static NSString *GSColorWellDidBecomeExclusiveNotification =
   NSDragOperation sourceDragMask;
        
   NSDebugLLog(@"NSColorWell", @"%@: draggingEntered", self);
+
+  if ([self isEnabled] == NO)
+    return NSDragOperationNone;
+ 
   sourceDragMask = [sender draggingSourceOperationMask];
   pb = [sender draggingPasteboard];
  
@@ -166,60 +172,15 @@ static NSString *GSColorWellDidBecomeExclusiveNotification =
 
 - (void) drawRect: (NSRect)clipRect
 {
-  NSRect aRect = _bounds;
-  
-  if (NSIntersectsRect(aRect, clipRect) == NO)
+  if (NSIntersectsRect(_bounds, clipRect) == NO)
     {
       return;
     }
 
-  if (_is_bordered == YES)
-    {
-      /*
-       * Draw border.
-       */
-      [[GSTheme theme] drawButton: aRect withClip: clipRect];
-
-      /*
-       * Fill in control color.
-       */
-      aRect = NSInsetRect(aRect, 2.0, 2.0);
-      if (_is_active == YES)
-	{
-	  [[NSColor selectedControlColor] set];
-	}
-      else
-	{
-	  [[NSColor controlColor] set];
-	}
-      NSRectFill(NSIntersectionRect(aRect, clipRect));
-
-      /*
-       * Set an inset rect for the color area
-       */
-      _wellRect = NSInsetRect(_bounds, 8.0, 8.0);
-    }
-  else
-    {
-      _wellRect = _bounds;
-    }
-
-  aRect = _wellRect;
-
-  /*
-   * OpenStep 4.2 behavior is to omit the inner border for
-   * non-enabled NSColorWell objects.
-   */
-  if ([self isEnabled])
-    {
-      /*
-       * Draw inner frame.
-       */
-      [[GSTheme theme] drawGrayBezel: aRect withClip: clipRect];
-      aRect = NSInsetRect(aRect, 2.0, 2.0);
-    }
-
-  [self drawWellInside: NSIntersectionRect(aRect, clipRect)];
+  _wellRect = [[GSTheme theme] drawColorWellBorder: self
+                               withBounds: _bounds
+                               withClip: clipRect];
+  [self drawWellInside: NSIntersectionRect(_wellRect, clipRect)];
 }
 
 /**<p>Draws the NSColorWell inside the rectangle <var>insideRect</var>.</p>
@@ -231,6 +192,22 @@ static NSString *GSColorWellDidBecomeExclusiveNotification =
     {
       return;
     }
+  if ([_the_color alphaComponent] < 1.0)
+    {
+      NSBezierPath *triangle = [NSBezierPath bezierPath];
+
+      [[NSColor whiteColor] set];
+      NSRectFill(insideRect);
+
+      [triangle moveToPoint: NSMakePoint(insideRect.origin.x,
+		insideRect.origin.y + insideRect.size.height)];
+      [triangle lineToPoint: NSMakePoint(insideRect.origin.x + insideRect.size.width, 
+		insideRect.origin.y + insideRect.size.height)];
+      [triangle lineToPoint: insideRect.origin];
+      [triangle closePath];
+      [[NSColor blackColor] set];
+      [triangle fill];
+    }
   [_the_color drawSwatchInRect: insideRect];
 }
 
@@ -332,18 +309,74 @@ static NSString *GSColorWellDidBecomeExclusiveNotification =
   // OPENSTEP 4.2 and OSX behavior indicates that the colorwell doesn't
   // work when the widget is marked as disabled.
   //
-  if ([self isEnabled])
-    {
-      NSPoint point = [self convertPoint: [theEvent locationInWindow]
-                            fromView: nil];
+  if ([self isEnabled] == NO)
+    return;
 
-      if ([self mouse: point inRect: _wellRect])
-	{
-	  [NSColorPanel dragColor: _the_color
-			withEvent: theEvent
-			fromView: self];
-	}
-      else if (_is_active == NO)
+  // Unbordered color wells start a drag immediately upon mouse down
+  if ([self isBordered] == NO)
+    {
+      [NSColorPanel dragColor: _the_color
+		    withEvent: theEvent
+		    fromView: self];
+      return;
+    }
+
+  _mouseDownPoint = [self convertPoint: [theEvent locationInWindow]
+			  fromView: nil];
+  [[self cell] setHighlighted: YES];
+  [self setNeedsDisplay: YES];
+}
+
+- (void) mouseDragged: (NSEvent *)theEvent
+{
+  NSPoint point = [self convertPoint: [theEvent locationInWindow]
+			fromView: nil];
+  BOOL inside = [self mouse: point inRect: [self bounds]];
+  BOOL startedInWell = [self mouse: _mouseDownPoint inRect: _wellRect];
+
+  NSSize delta = NSMakeSize(_mouseDownPoint.x - point.x,
+                            _mouseDownPoint.y - point.y);
+  double distance = sqrt(delta.width*delta.width + delta.height*delta.height);
+
+  // FIXME: Make the dragging threshold a user default
+  if (distance < 4)
+    return;
+
+  if ([self isEnabled] == NO)
+    return;
+
+  if (startedInWell)
+    {
+      [[self cell] setHighlighted: NO];
+      [self setNeedsDisplay: YES];
+
+      [NSColorPanel dragColor: _the_color
+		    withEvent: theEvent
+		    fromView: self];
+      return;
+    }
+  else
+    {
+      [[self cell] setHighlighted: inside];
+      [self setNeedsDisplay: YES];
+    }
+}
+
+- (void) mouseUp: (NSEvent *)theEvent
+{
+  NSPoint point = [self convertPoint: [theEvent locationInWindow]
+			fromView: nil];
+  BOOL inside = [self mouse: point inRect: [self bounds]];
+
+  if ([self isEnabled] == NO)
+    return;
+
+  [[self cell] setHighlighted: NO];
+  [self setNeedsDisplay: YES];
+
+  if (inside)
+    {
+      if (_is_active == NO)
 	{
 	  [self activate: YES];
 	}
