@@ -168,7 +168,7 @@ static NSNotificationCenter *nc;
 /** <p>Returns the tag of the NSControl's selected cell (if exists).
     -1 otherwise.</p><p>See Also: [NSCell-tag]</p>
  */
-- (int) selectedTag
+- (NSInteger) selectedTag
 {
   NSCell *selected = [self selectedCell];
 
@@ -625,23 +625,6 @@ static NSNotificationCenter *nc;
 
 - (void) drawRect: (NSRect)aRect
 {
-  /* Do nothing if there is already a text editor doing the drawing;
-   * otherwise, we draw everything twice.  That is bad if there are
-   * any transparency involved (eg, even an anti-alias font!) because
-   * if the semi-transparent pixels are drawn over themselves they
-   * become less transparent (eg, an anti-alias font becomes darker
-   * and gives the impression of being bold).
-   */
-  if ([self currentEditor] != nil)
-    {
-      /* Make sure the cell's control view is always set up.
-       * FIXME Need to draw the cell's border which is not covered by
-       *       the field editor if the cell is bordered.
-       */
-      [_cell setControlView: self];
-      return;
-    }
-
   [self drawCell: _cell];
 }
 
@@ -801,7 +784,7 @@ static NSNotificationCenter *nc;
 
 /** Assigning a Tag
  */
-- (void) setTag: (int)anInt
+- (void) setTag: (NSInteger)anInt
 {
   _tag = anInt;
 }
@@ -809,7 +792,7 @@ static NSNotificationCenter *nc;
 /**<p>Returns the NSControl tag</p>
    <p>See Also: -setTag:</p>
  */
-- (int) tag
+- (NSInteger) tag
 {
   return _tag;
 }
@@ -828,12 +811,12 @@ static NSNotificationCenter *nc;
   [_cell performClickWithFrame: [self bounds] inView: self];
 }
 
-- (BOOL)refusesFirstResponder
+- (BOOL) refusesFirstResponder
 {
   return [[self selectedCell] refusesFirstResponder];
 }
 
-- (void)setRefusesFirstResponder:(BOOL)flag
+- (void) setRefusesFirstResponder:(BOOL)flag
 {
   [[self selectedCell] setRefusesFirstResponder: flag];
 }
