@@ -30,6 +30,7 @@
 
 #include <Foundation/NSArray.h>
 #include <Foundation/NSDebug.h>
+#include <Foundation/NSDictionary.h>
 #include <Foundation/NSKeyedArchiver.h>
 
 #include "AppKit/NSColor.h"
@@ -547,7 +548,8 @@ backgroundColorHint:(NSColor *)bg
 	  // FIXME
 	  self = [NSCursor arrowCursor];
 	}
-      
+      RETAIN(self);
+
       if ([aDecoder containsValueForKey: @"NSHotSpot"])
         {
 	  hotSpot = [aDecoder decodePointForKey: @"NSHotSpot"];
