@@ -28,19 +28,25 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSData.h>
-#include "AppKit/NSBox.h"
-#include "AppKit/NSDocument.h"
-#include "AppKit/NSFileWrapper.h"
-#include "AppKit/NSSavePanel.h"
-#include "AppKit/NSPageLayout.h"
-#include "AppKit/NSPrintInfo.h"
-#include "AppKit/NSPrintOperation.h"
-#include "AppKit/NSPopUpButton.h"
-#include "AppKit/NSView.h"
-#include "NSDocumentFrameworkPrivate.h"
+#import <Foundation/NSData.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSProcessInfo.h>
+#import <Foundation/NSUndoManager.h>
+#import <Foundation/NSURL.h>
+#import "AppKit/NSBox.h"
+#import "AppKit/NSDocument.h"
+#import "AppKit/NSFileWrapper.h"
+#import "AppKit/NSSavePanel.h"
+#import "AppKit/NSPageLayout.h"
+#import "AppKit/NSPopUpButton.h"
+#import "AppKit/NSPrintInfo.h"
+#import "AppKit/NSPrintOperation.h"
+#import "AppKit/NSView.h"
+#import "NSDocumentFrameworkPrivate.h"
 
-#include "GSGuiPrivate.h"
+#import "GSGuiPrivate.h"
 
 @implementation NSDocument
 
@@ -48,14 +54,14 @@
 {
   // FIXME: Should allow for filterable types
   return [[NSDocumentController sharedDocumentController]
-          _editorAndViewerTypesForClass: self];
+          _readableTypesForClass: self];
 }
 
 + (NSArray *) writableTypes
 {
   // FIXME: Should allow for filterable types
   return [[NSDocumentController sharedDocumentController] 
-          _editorTypesForClass: self];
+          _writableTypesForClass: self];
 }
 
 + (BOOL) isNativeType: (NSString *)type
@@ -228,7 +234,19 @@ withContentsOfURL: (NSURL *)url
 
 - (void) setFileName: (NSString *)fileName
 {
-  NSURL *fileUrl = fileName ? [NSURL fileURLWithPath: fileName] : nil;
+  NSURL *fileUrl;
+
+  if (fileName && ![fileName isAbsolutePath])
+    {
+      NSString *dir = [[NSFileManager defaultManager] currentDirectoryPath];
+      
+      if (dir)
+	{
+	  fileName = [dir stringByAppendingPathComponent: fileName];
+	}
+    }
+
+  fileUrl = fileName ? [NSURL fileURLWithPath: fileName] : nil;
 
   // This check is to prevent super calls from recursing.
   if (!OVERRIDDEN(setFileName:))
@@ -569,7 +587,8 @@ withContentsOfURL: (NSURL *)url
 {
   if (OVERRIDDEN(dataRepresentationOfType:))
     {
-      *error = nil; 
+      if (error)
+	*error = nil; 
       return [self dataRepresentationOfType: type];
     }
 
@@ -602,7 +621,8 @@ withContentsOfURL: (NSURL *)url
   
   if (OVERRIDDEN(fileWrapperRepresentationOfType:))
     {
-      *error = nil; 
+      if (error)
+	*error = nil; 
       return [self fileWrapperRepresentationOfType: type];
     }
 
@@ -681,7 +701,8 @@ withContentsOfURL: (NSURL *)url
 {
   if (OVERRIDDEN(loadDataRepresentation:ofType:))
     {
-      *error = nil; 
+      if (error)
+	*error = nil; 
       return [self loadDataRepresentation: data
                    ofType: type];
     }
@@ -697,7 +718,8 @@ withContentsOfURL: (NSURL *)url
 {
   if (OVERRIDDEN(loadFileWrapperRepresentation:ofType:))
     {
-      *error = nil; 
+      if (error)
+	*error = nil; 
       return [self loadFileWrapperRepresentation: wrapper ofType: type];
     }
 
@@ -709,7 +731,8 @@ withContentsOfURL: (NSURL *)url
     }
 
   // FIXME: Set error
-  *error = nil;
+  if (error)
+    *error = nil;
   return NO;
 }
 
@@ -723,7 +746,8 @@ withContentsOfURL: (NSURL *)url
       
       if (OVERRIDDEN(readFromFile:ofType:))
         {
-          *error = nil;
+	  if (error)
+	    *error = nil;
           return [self readFromFile: fileName ofType: type];
         }
       else
@@ -846,7 +870,7 @@ withContentsOfURL: (NSURL *)url
 
 - (BOOL) writeSafelyToURL: (NSURL *)url
                    ofType: (NSString *)type
-         forSaveOperation: (NSSaveOperationType)saveOp
+         forSaveOperation: (NSSaveOperationType)op
                     error: (NSError **)error
 {
   NSURL *original = [self fileURL];
@@ -856,25 +880,40 @@ withContentsOfURL: (NSURL *)url
       
   if (OVERRIDDEN(writeWithBackupToFile:ofType:saveOperation:))
     {
-      if (saveOp == NSAutosaveOperation)
+      BOOL isAutosave = NO;
+
+      if (op == NSAutosaveOperation)
         {
-          saveOp = NSSaveToOperation;
+          op = NSSaveToOperation;
+          isAutosave = YES;
         }
 
-      *error = nil; 
-      return [self writeWithBackupToFile: [url path] 
-                   ofType: type 
-                   saveOperation: saveOp];
+      if (error)
+	*error = nil; 
+      if (![self writeWithBackupToFile: [url path] 
+                 ofType: type 
+                 saveOperation: op])
+	{
+	  return NO;
+	}
+
+      if (isAutosave)
+	{
+	  [self setAutosavedContentsFileURL: url];
+	  [self updateChangeCount: NSChangeAutosaved];
+	}
+      return YES;
     }
 
   if (!isNativeType || (url == nil))
     {
       // FIXME: Set error
-      *error = nil; 
+      if (error)
+	*error = nil; 
       return NO;
     }
   
-  if (saveOp == NSSaveOperation)
+  if (op == NSSaveOperation)
     {
       if ([url isFileURL])
         {
@@ -889,7 +928,8 @@ withContentsOfURL: (NSURL *)url
                          toFile: backupFilename])
                 {
                   // FIXME: Set error.
-                  *error = nil; 
+		  if (error)
+		    *error = nil; 
                   return NO;
                 }
             }
@@ -898,7 +938,7 @@ withContentsOfURL: (NSURL *)url
       
   if (![self writeToURL: url 
              ofType: type 
-             forSaveOperation: saveOp 
+             forSaveOperation: op 
              originalContentsURL: original
              error: error])
     {
@@ -911,18 +951,18 @@ withContentsOfURL: (NSURL *)url
 
       attrs = [self fileAttributesToWriteToURL: url
                     ofType: type 
-                    forSaveOperation: saveOp 
+                    forSaveOperation: op 
                     originalContentsURL: original
                     error: error];
       [fileManager changeFileAttributes: attrs atPath: [url path]];
     }
 
-  if (saveOp == NSAutosaveOperation)
+  if (op == NSAutosaveOperation)
     {
       [self setAutosavedContentsFileURL: url];
       [self updateChangeCount: NSChangeAutosaved];
     }
-  else if (saveOp != NSSaveToOperation)
+  else if (op != NSSaveToOperation)
     {
       [self _removeAutosavedContentsFile];
       [self setFileURL: url];
@@ -948,7 +988,8 @@ withContentsOfURL: (NSURL *)url
 
       if (OVERRIDDEN(writeToFile:ofType:))
         {
-          *error = nil; 
+	  if (error)
+	    *error = nil; 
           return [self writeToFile: [url path] ofType: type];
         }
 
@@ -957,11 +998,13 @@ withContentsOfURL: (NSURL *)url
       if (wrapper == nil)
         {
           // FIXME: Set error
-          *error = nil; 
+	  if (error)
+	    *error = nil; 
           return NO;
         }
-   
-      *error = nil; 
+
+      if (error)
+	*error = nil; 
       return [wrapper writeToFile: [url path] atomically: YES updateFilenames: YES];
     }
   else
@@ -977,22 +1020,23 @@ withContentsOfURL: (NSURL *)url
 
 - (BOOL) writeToURL: (NSURL *)url
              ofType: (NSString *)type
-   forSaveOperation: (NSSaveOperationType)saveOp
+   forSaveOperation: (NSSaveOperationType)op
 originalContentsURL: (NSURL *)orig
               error: (NSError **)error
 {
   if (OVERRIDDEN(writeToFile:ofType:originalFile:saveOperation:))
     {
-      if (saveOp == NSAutosaveOperation)
+      if (op == NSAutosaveOperation)
         {
-          saveOp = NSSaveToOperation;
+          op = NSSaveToOperation;
         }
 
-      *error = nil; 
+      if (error)
+	*error = nil; 
       return [self writeToFile: [url path] 
                    ofType: type 
                    originalFile: [orig path] 
-                   saveOperation: saveOp];
+                   saveOperation: op];
     }
 
   return [self writeToURL: url
@@ -1004,10 +1048,9 @@ originalContentsURL: (NSURL *)orig
 { 
   NSDocumentController *controller = 
     [NSDocumentController sharedDocumentController];
-  NSArray *extensions = nil;
+  NSArray *extensions;
 
-  ASSIGN(_save_type, [controller _nameForHumanReadableType: 
-                                  [sender titleOfSelectedItem]]);
+  ASSIGN(_save_type, [[sender selectedItem] representedObject]);
   extensions = [controller fileExtensionsFromType: _save_type];
   [(NSSavePanel *)[sender window] setAllowedFileTypes: extensions];
 }
@@ -1050,7 +1093,7 @@ originalContentsURL: (NSURL *)orig
       NSRect spaFrame = NSMakeRect(115,14,150,22);
 
       _save_panel_accessory = [[NSBox alloc] initWithFrame: accessoryFrame];
-      [(NSBox *)_save_panel_accessory setTitle: @"File Type"];
+      [(NSBox *)_save_panel_accessory setTitle: _(@"File Type")];
       [_save_panel_accessory setAutoresizingMask: 
                             NSViewWidthSizable | NSViewHeightSizable];
       _spa_button = [[NSPopUpButton alloc] initWithFrame: spaFrame];
@@ -1064,32 +1107,27 @@ originalContentsURL: (NSURL *)orig
 
 - (void) _addItemsToSpaButtonFromArray: (NSArray *)types
 {
-  NSEnumerator *en = [types objectEnumerator];
-  NSString *title = nil;
-  int i = 0;
+  NSString *type, *title;
+  int i, count = [types count];
 
-  while ((title = [en nextObject]) != nil)
+  [_spa_button removeAllItems];
+  for (i = 0; i < count; i++)
     {
+      type = [types objectAtIndex: i];
+      title = [[NSDocumentController sharedDocumentController]
+		displayNameForType: type];
       [_spa_button addItemWithTitle: title];
-      i++;
+      [[_spa_button itemAtIndex: i] setRepresentedObject: type];
     }
 
   // if it's more than one, then
-  [_spa_button setEnabled: (i > 0)];
+  [_spa_button setEnabled: (count > 0)];
   
   // if we have some items, select the current filetype.
-  if (i > 0)
+  if (count > 0)
     {
-      NSString *title = [[NSDocumentController sharedDocumentController] 
-                          displayNameForType: [self fileType]];
-      if ([_spa_button itemWithTitle: title] != nil)
-        {
-          [_spa_button selectItemWithTitle: title];
-        }
-      else
-        {
-          [_spa_button selectItemAtIndex: 0];
-        }
+      [_spa_button selectItemAtIndex:
+	[_spa_button indexOfItemWithRepresentedObject: [self fileType]]];
     }
 }
 
@@ -1098,25 +1136,25 @@ originalContentsURL: (NSURL *)orig
   NSView *accessory = nil;
   NSString *title;
   NSString *directory;
-  NSArray *displayNames;
+  NSArray *types;
   NSDocumentController *controller;
   NSSavePanel *savePanel = [NSSavePanel savePanel];
 
   ASSIGN(_save_type, [self fileType]); 
   controller = [NSDocumentController sharedDocumentController];
-  displayNames = [controller _displayNamesForClass: [self class]];
-  
+  types = [self writableTypesForSaveOperation: saveOperation];
+
   if ([self shouldRunSavePanelWithAccessoryView])
     {
       if (_save_panel_accessory == nil)
         [self _createPanelAccessory];
       
-      [self _addItemsToSpaButtonFromArray: displayNames];
+      [self _addItemsToSpaButtonFromArray: types];
       
       accessory = _save_panel_accessory;
     }
 
-  if ([displayNames count] > 0)
+  if ([types count] > 0)
     {
       NSArray *extensions = [controller fileExtensionsFromType: [self fileType]];
       [savePanel setAllowedFileTypes: extensions];
@@ -1171,29 +1209,27 @@ originalContentsURL: (NSURL *)orig
                                contextInfo: (void *)contextInfo
 {
   // FIXME: Commit registered editors
+  NSSavePanel *savePanel = [self _runSavePanelForSaveOperation: saveOperation];
 
-  if (OVERRIDDEN(saveToFile:saveOperation:delegate:didSaveSelector:contextInfo:))
+  if (savePanel)
     {
-      NSString *fileName;
-
-      fileName = [self fileNameFromRunningSavePanelForSaveOperation: saveOperation];
-      [self saveToFile: fileName 
-            saveOperation: saveOperation 
-            delegate: delegate
-            didSaveSelector: didSaveSelector 
-            contextInfo: contextInfo];
-    }
-  else
-    {
-      NSSavePanel *savePanel = [self _runSavePanelForSaveOperation: saveOperation];
-      NSURL *url = [savePanel URL];
-
-      [self saveToURL: url 
-            ofType: [self fileTypeFromLastRunSavePanel]
-            forSaveOperation: saveOperation 
-            delegate: delegate
-            didSaveSelector: didSaveSelector 
-            contextInfo: contextInfo];
+      if (OVERRIDDEN(saveToFile:saveOperation:delegate:didSaveSelector:contextInfo:))
+        {
+          [self saveToFile: [savePanel filename] 
+                saveOperation: saveOperation 
+                delegate: delegate
+                didSaveSelector: didSaveSelector 
+                contextInfo: contextInfo];
+        }
+      else
+        {
+          [self saveToURL: [savePanel URL] 
+                ofType: [self fileTypeFromLastRunSavePanel]
+                forSaveOperation: saveOperation 
+                delegate: delegate
+                didSaveSelector: didSaveSelector 
+                contextInfo: contextInfo];
+        }
     }
 }
 
@@ -1230,11 +1266,11 @@ originalContentsURL: (NSURL *)orig
   return NO;
 }
 
-- (NSString *) fileNameExtensionForType: (NSString *)type
+- (NSString *) fileNameExtensionForType: (NSString *)typeName
                           saveOperation: (NSSaveOperationType)saveOperation
 {
   NSArray *exts = [[NSDocumentController sharedDocumentController]
-                      fileExtensionsFromType: type];
+                      fileExtensionsFromType: typeName];
   
   if ([exts count])
     return (NSString *)[exts objectAtIndex: 0];
@@ -1302,7 +1338,7 @@ originalContentsURL: (NSURL *)orig
         contextInfo: printInfo];
 }
 
-- (void) runModalPageLayoutWithPrintInfo: (NSPrintInfo *)printInfo
+- (void) runModalPageLayoutWithPrintInfo: (NSPrintInfo *)info
                                 delegate: (id)delegate
                           didRunSelector: (SEL)sel
                              contextInfo: (void *)context
@@ -1312,7 +1348,7 @@ originalContentsURL: (NSURL *)orig
   pageLayout = [NSPageLayout pageLayout];
   if ([self preparePageLayout: pageLayout])
     {
-      [pageLayout beginSheetWithPrintInfo: printInfo
+      [pageLayout beginSheetWithPrintInfo: info
                   modalForWindow: [self windowForSheet]
                   delegate: delegate
                   didEndSelector: sel
@@ -1376,7 +1412,8 @@ originalContentsURL: (NSURL *)orig
 - (NSPrintOperation *) printOperationWithSettings: (NSDictionary *)settings
                                             error: (NSError **)error
 {
-  *error = nil; 
+  if (error)
+    *error = nil; 
   return nil;
 }
 
@@ -1446,7 +1483,8 @@ originalContentsURL: (NSURL *)orig
                                         error: (NSError **)error
 {
   // FIXME: Implement. Should set NSFileExtensionHidden
-  *error = nil; 
+  if (error)
+    *error = nil; 
 
   return [NSDictionary dictionary];
 }
@@ -1617,6 +1655,7 @@ originalContentsURL: (NSURL *)orig
             [array[count] close];
         }
       [self _removeAutosavedContentsFile];
+      AUTORELEASE(RETAIN(self));
       [[NSDocumentController sharedDocumentController] removeDocument: self];
     }
 }
