@@ -56,6 +56,7 @@ static NSString *NSDOSExtensionsKey     = @"NSDOSExtensions";
 //static NSString *NSMacOSTypesKey        = @"NSMacOSTypes";
 //static NSString *NSMIMETypesKey         = @"NSMIMETypes";
 static NSString *NSDocumentClassKey     = @"NSDocumentClass";
+static NSString *NSExportableAsKey      = @"NSExportableAs";
 
 static NSString *CFBundleDocumentTypes  = @"CFBundleDocumentTypes";
 static NSString *CFBundleTypeExtensions = @"CFBundleTypeExtensions";
@@ -72,12 +73,16 @@ static NSString *NSViewerRole = @"Viewer";
 
 static NSDocumentController *sharedController = nil;
 
+static NSArray	*allTypes = nil;
+static NSArray	*classNames = nil;
+
 #define TYPE_INFO(name) TypeInfoForName(_types, name)
 #define HR_TYPE_INFO(name) TypeInfoForHumanReadableName(_types, name)
 
 static NSDictionary *TypeInfoForName (NSArray *types, NSString *typeName)
 {
   int i, count = [types count];
+
   for (i = 0; i < count; i++)
     {
       NSDictionary *dict = [types objectAtIndex: i];
@@ -86,7 +91,8 @@ static NSDictionary *TypeInfoForName (NSArray *types, NSString *typeName)
         {
           return dict;
         }
-      else if ([[dict objectForKey: CFBundleTypeName] isEqualToString: typeName])
+      else if ([[dict objectForKey: CFBundleTypeName]
+	isEqualToString: typeName])
         {
           return dict;
         }
@@ -95,18 +101,22 @@ static NSDictionary *TypeInfoForName (NSArray *types, NSString *typeName)
   return nil;
 }
 
-static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
+static NSDictionary *
+TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
 {
   int i, count = [types count];
+
   for (i = 0; i < count; i++)
     {
       NSDictionary *dict = [types objectAtIndex: i];
 
-      if ([[dict objectForKey: NSHumanReadableNameKey] isEqualToString: typeName])
+      if ([[dict objectForKey: NSHumanReadableNameKey]
+	isEqualToString: typeName])
         {
           return dict;
         }
-      else if ([[dict objectForKey: CFBundleTypeName] isEqualToString: typeName])
+      else if ([[dict objectForKey: CFBundleTypeName]
+	isEqualToString: typeName])
         {
           return dict;
         }
@@ -167,6 +177,81 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 */
 @implementation NSDocumentController
 
+
++ (void) initialize
+{
+  static BOOL	beenHere = NO;
+
+  if (beenHere == NO)
+    {
+      NSArray	*types;
+
+      beenHere = YES;
+      types = [[[NSBundle mainBundle] infoDictionary]
+        objectForKey: NSTypesKey];
+      if (types == nil)
+	{
+          types = [[[NSBundle mainBundle] infoDictionary]
+            objectForKey: CFBundleDocumentTypes];
+	}
+      if (YES == [types isKindOfClass: [NSArray class]])
+	{
+	  unsigned	count = [types count];
+	  unsigned	i;
+	  unsigned	nNames = 0;
+	  unsigned	nValid = 0;
+	  NSString	*names[count];
+	  NSDictionary	*valid[count];
+
+	  for (i = 0; i < count; i++)
+	    {
+	      NSDictionary	*d = [types objectAtIndex: i];
+
+	      if (YES == [d isKindOfClass: [NSDictionary class]])
+		{
+		  NSString	*name = [d objectForKey: NSDocumentClassKey];
+
+		  valid[nValid++] = d;
+		  /* Is this type handled by an NSDocument subclass?
+		   */
+	          if (YES == [name isKindOfClass: [NSString class]])
+		    {
+		      Class	c = NSClassFromString(name);
+
+		      if (YES == [c isSubclassOfClass: [NSDocument class]])
+			{
+			  names[nNames++] = name;
+			}
+		      else if (c == 0)
+			{
+			  NSLog(@"NSDocumentClass %@ not found", name);
+			}
+		      else
+			{
+			  NSLog(@"NSDocumentClass %@ not NSDocument subclass",
+			    name);
+			}
+		    }
+		}
+	      else
+		{
+		  NSLog(@"Bad item at index %u in %@",
+		    i, CFBundleDocumentTypes);
+		}
+	    }
+	  if (nNames > 0)
+	    {
+	      classNames = [[NSArray alloc] initWithObjects: names
+						      count: nNames];
+	    }
+	  if (nValid > 0)
+	    {
+	      allTypes = [[NSArray alloc] initWithObjects: valid count: nValid];
+	    }
+	}
+    }
+}
+
 /** Returns the shared instance of the document controller class. You
     should always use this method to get the NSDocumentController. */
 + (id) sharedDocumentController
@@ -180,30 +265,13 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
   return sharedController;
 }
 
-/* Private method for use by NSApplication to determine if it should
-   instantiate an NSDocumentController.
-*/
-+ (BOOL) isDocumentBasedApplication
-{
-  return ([[[NSBundle mainBundle] infoDictionary] objectForKey: NSTypesKey] ||
-          [[[NSBundle mainBundle] infoDictionary] objectForKey: CFBundleDocumentTypes]) 
-    ? YES : NO;
-}
-
 /** </init>Initializes the document controller class. The first
     instance of a document controller class that gets initialized
     becomes the shared instance.
  */
 - init
 {
-  NSDictionary *customDict = [[NSBundle mainBundle] infoDictionary];
-        
-  ASSIGN (_types, [customDict objectForKey: NSTypesKey]);
-
-  if(_types == nil)
-    {
-      ASSIGN(_types, [customDict objectForKey: CFBundleDocumentTypes]);
-    }
+  _types = RETAIN(allTypes);
 
   _documents = [[NSMutableArray alloc] init];
   
@@ -385,39 +453,39 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
     }
 }
 
-- (BOOL) presentError: (NSError *)error
+- (BOOL) presentError: (NSError *)err
 {
-  error = [self willPresentError: error];
-  return [NSApp presentError: error];
+  err = [self willPresentError: err];
+  return [NSApp presentError: err];
 }
 
-- (void) presentError: (NSError *)error
-       modalForWindow: (NSWindow *)window
+- (void) presentError: (NSError *)err
+       modalForWindow: (NSWindow *)win
              delegate: (id)delegate 
    didPresentSelector: (SEL)sel
           contextInfo: (void *)context
 {
-  error = [self willPresentError: error];
-  [NSApp presentError: error
-         modalForWindow: window
+  err = [self willPresentError: err];
+  [NSApp presentError: err
+         modalForWindow: win
          delegate: delegate
          didPresentSelector: sel
          contextInfo: context];
 }
 
-- (NSError *) willPresentError: (NSError *)error
+- (NSError *) willPresentError: (NSError *)err
 {
-  return error;
+  return err;
 }
 
 - (NSString*) defaultType
 {
   NSString *defaultName = nil;
-	int i, count = [_types count];
+  int i, count = [_types count];
 
-	for (i = 0; i < count; i++)
-		{
-      NSDictionary *typeInfo = (NSDictionary*)[_types objectAtIndex: i];		
+  for (i = 0; i < count; i++)
+    {
+      NSDictionary *typeInfo = (NSDictionary*)[_types objectAtIndex: i];
       NSString *role;
 
       role = [typeInfo objectForKey: NSRoleKey];
@@ -434,10 +502,10 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 
           return defaultName;
         }
-		}
+    }
 
-	// none found
-	return nil;
+  // none found
+  return nil;
 }
 
 - (void) addDocument: (NSDocument *)document
@@ -541,7 +609,7 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
     }
   
   // remember this document as opened
-  [self noteNewRecentDocumentURL: url];
+  [self noteNewRecentDocument: document];
 
   if (display && [self shouldCreateUI])
     {
@@ -551,7 +619,7 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
   return document;
 }
 
-- (id) openUntitledDocumentAndDisplay: (BOOL)display 
+- (id) openUntitledDocumentAndDisplay: (BOOL)flag 
                                 error: (NSError **)err
 {
   NSString *type;
@@ -560,7 +628,7 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 
   if (OVERRIDDEN(openUntitledDocumentOfType:display:))
     {
-      return [self openUntitledDocumentOfType: type display: display];
+      return [self openUntitledDocumentOfType: type display: flag];
     }
   else
     {
@@ -576,7 +644,7 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
       if ([self shouldCreateUI])
         {
           [document makeWindowControllers];
-          if (display)
+          if (flag)
             {
               [document showWindows];
             }
@@ -587,13 +655,13 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 }
 
 - (id) openDocumentWithContentsOfURL: (NSURL *)url
-                             display: (BOOL)display
+                             display: (BOOL)flag
                                error: (NSError **)err
 {
   if (OVERRIDDEN(openDocumentWithContentsOfFile:display:) && [url isFileURL])
     {
       return [self openDocumentWithContentsOfFile: [url path] 
-                   display: display];
+                   display: flag];
     }
   else
     {
@@ -626,9 +694,9 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
         }
       
       // remember this document as opened
-      [self noteNewRecentDocumentURL: url];
+      [self noteNewRecentDocument: document];
       
-      if (display && [self shouldCreateUI])
+      if (flag && [self shouldCreateUI])
         {
           [document showWindows];
         }
@@ -1026,14 +1094,14 @@ static BOOL _shouldClose = YES;
       directory = [[document fileName] stringByDeletingLastPathComponent];
     }
   if (directory == nil || [directory isEqual: @""]
-      || [manager fileExistsAtPath: directory  isDirectory: &isDir] == NO
-      || isDir == NO)
+    || [manager fileExistsAtPath: directory  isDirectory: &isDir] == NO
+    || isDir == NO)
     {
       directory = [[NSOpenPanel openPanel] directory];
     }
   if (directory == nil || [directory isEqual: @""]
-      || [manager fileExistsAtPath: directory  isDirectory: &isDir] == NO
-      || isDir == NO)
+    || [manager fileExistsAtPath: directory  isDirectory: &isDir] == NO
+    || isDir == NO)
     {
       directory = NSHomeDirectory ();
     }
@@ -1151,15 +1219,15 @@ static BOOL _shouldClose = YES;
       NSDictionary *typeInfo = [_types objectAtIndex: i];
       
       if ([[typeInfo objectForKey: NSUnixExtensionsKey] 
-            containsObject: fileExtension] ||
-          [[typeInfo objectForKey: NSDOSExtensionsKey]  
-            containsObject: fileExtension] ||
-          [[typeInfo objectForKey: CFBundleTypeExtensions]
-            containsObject: fileExtension])
+          containsObject: fileExtension]
+	|| [[typeInfo objectForKey: NSDOSExtensionsKey]  
+          containsObject: fileExtension]
+	|| [[typeInfo objectForKey: CFBundleTypeExtensions]
+          containsObject: fileExtension])
         {
           NSString *type = [typeInfo objectForKey: NSNameKey];
 
-          if(type == nil)
+          if (type == nil)
             {
               type = [typeInfo objectForKey: CFBundleTypeName];
             }
@@ -1174,7 +1242,7 @@ static BOOL _shouldClose = YES;
 {
   NSString *type = [self typeFromFileExtension: [[url path] pathExtension]];
 
-  if([url isFileURL] == NO && type == nil)
+  if ([url isFileURL] == NO && type == nil)
     {
       return [self defaultType];
     }
@@ -1201,30 +1269,9 @@ static BOOL _shouldClose = YES;
   return className ? NSClassFromString(className) : Nil;
 }
 
-- (NSArray *)documentClassNames
+- (NSArray*) documentClassNames
 {
-  int i, count = [_types count];
-  NSMutableArray *classNames;
-
-  classNames = [[NSMutableArray alloc] initWithCapacity: count];
-        
-  for (i = 0; i < count; i++)
-    {
-      NSDictionary *typeInfo = [_types objectAtIndex: i];
-      NSString *className = [typeInfo objectForKey: NSDocumentClassKey];
-
-      if(className != nil)
-        {
-          [classNames addObject: className];
-        }
-      else
-        {
-          NSLog(@"WARN: The entry does not have an NSDocumentClass name defined: %@",
-                typeInfo);
-        }
-    }
- 
-  return AUTORELEASE(classNames);
+  return classNames;
 }
 
 
@@ -1305,7 +1352,7 @@ static BOOL _shouldClose = YES;
 
 @implementation NSDocumentController (Private)
 
-- (NSArray *) _editorAndViewerTypesForClass: (Class)documentClass
+- (NSArray *) _readableTypesForClass: (Class)documentClass
 {
   int i, count = [_types count];
   NSMutableArray *types = [NSMutableArray arrayWithCapacity: count];
@@ -1330,7 +1377,7 @@ static BOOL _shouldClose = YES;
         {
           NSString *name = [typeInfo objectForKey: NSNameKey];
 
-          if(name == nil)
+          if (name == nil)
             {
               name = [typeInfo objectForKey: CFBundleTypeName];
             }
@@ -1341,7 +1388,7 @@ static BOOL _shouldClose = YES;
   return types;
 }
 
-- (NSArray *) _editorTypesForClass: (Class)documentClass
+- (NSArray *) _writableTypesForClass: (Class)documentClass
 {
   int i, count = [_types count];
   NSMutableArray *types = [NSMutableArray arrayWithCapacity: count];
@@ -1352,6 +1399,7 @@ static BOOL _shouldClose = YES;
       NSDictionary *typeInfo = [_types objectAtIndex: i];
       NSString     *className = [typeInfo objectForKey: NSDocumentClassKey];
       NSString     *role      = [typeInfo objectForKey: NSRoleKey];
+      NSArray      *exportableAs = [typeInfo objectForKey: NSExportableAsKey];
       
       // if the standard one isn't filled... check the CF key.
       if (role == nil)
@@ -1364,54 +1412,30 @@ static BOOL _shouldClose = YES;
         {
           NSString *name = [typeInfo objectForKey: NSNameKey];
 
-          if(name == nil)
+          if (name == nil)
             {
               name = [typeInfo objectForKey: CFBundleTypeName];
             }
           [types addObject: name];
         }
+
+      if ([exportableAs isKindOfClass: [NSArray class]])
+        {
+	  int i, count = [exportableAs count];
+	  NSString *name;
+
+	  for (i = 0; i < count; i++)
+	    {
+	      name = [exportableAs objectAtIndex: i];
+	      if ([name isKindOfClass: [NSString class]])
+		[types addObject: name];
+	    }
+	}
     }
   
   return types;
 }
 
-- (NSArray *) _exportableTypesForClass: (Class)documentClass
-{
-  // Dunno what this method is for; maybe looks for filter types
-  return [self _editorTypesForClass: documentClass];
-}
-
-- (NSString *) _nameForHumanReadableType: (NSString *)typeHR
-{
-  NSDictionary *typeInfo = HR_TYPE_INFO(typeHR);
-  NSString *type = [typeInfo objectForKey: NSNameKey];
-
-  if(type == nil)
-    {
-      type = [typeInfo objectForKey: CFBundleTypeName];
-    }
-  return type;
-}
-
-- (NSArray *) _displayNamesForTypes: (NSArray *)types
-{
-  NSEnumerator *en = [types objectEnumerator];
-  NSString *type = nil;
-  NSMutableArray *result = [NSMutableArray arrayWithCapacity: 10];
-  while ((type = (NSString *)[en nextObject]) != nil)
-    {
-      NSString *name = [self displayNameForType: type];
-      [result addObject: name];
-    }
-  return result;
-}
-
-- (NSArray *) _displayNamesForClass: (Class)documentClass
-{
-  return [self _displayNamesForTypes: 
-                 [self _editorTypesForClass: documentClass]];
-}
-
 static NSMapTable *autosavedDocuments = NULL;
 static NSString *processName = nil;
 
@@ -1572,29 +1596,30 @@ static NSString *processName = nil;
 // should be handled by making us the delegate of the recent's menu
 - (void) _updateOpenRecentMenu
 {
-	NSMenu *recentMenu;
-	int i;
+  NSMenu *recentMenu;
+  int i;
 
-	recentMenu = [self _recentMenu];
-	if (!recentMenu)
-		{
+  recentMenu = [self _recentMenu];
+  if (!recentMenu)
+    {
       return;
-		}
+    }
 
-	// don't update (to keep Clear List status consistent)
-	[recentMenu setAutoenablesItems: NO];
-	[recentMenu setMenuChangedMessagesEnabled: NO];
+  // don't update (to keep Clear List status consistent)
+  [recentMenu setAutoenablesItems: NO];
+  [recentMenu setMenuChangedMessagesEnabled: NO];
 
-	while ([recentMenu numberOfItems] > 0)
-		[recentMenu removeItemAtIndex: 0];	// remove them all
-
-	for (i = [_recent_documents count]; i >= -2; i--)
-		{
+  while ([recentMenu numberOfItems] > 0)
+    {
+      [recentMenu removeItemAtIndex: 0];	// remove them all
+    }
+  for (i = [_recent_documents count] - 1; i >= -2; i--)
+    {
       // add all items incl. a Clear List item if needed
       NSMenuItem *item;
 
       if (i == -1)
-			  {
+	{
           if ([_recent_documents count] == 0)
             continue;	// skip if menu is empty
           item = (NSMenuItem *) [NSMenuItem separatorItem];
@@ -1602,7 +1627,7 @@ static NSString *processName = nil;
           RETAIN(item);
         }
       else if (i == -2)
-			  {
+	{
           item = [[NSMenuItem alloc] initWithTitle: _(@"Clear List")
                                      action: @selector(clearRecentDocuments:) 
                                      keyEquivalent: nil];
@@ -1610,11 +1635,12 @@ static NSString *processName = nil;
           [item setEnabled: [_recent_documents count] > 0];
         }
       else
-			  {
+	{
           // standard item
           NSURL *u = [_recent_documents objectAtIndex: i];	// get URL
           if ([u isFileURL])
-            item = [[NSMenuItem alloc] initWithTitle: [[u path] lastPathComponent] 
+            item = [[NSMenuItem alloc]
+	      initWithTitle: [[u path] lastPathComponent] 
                                        action: @selector(_openRecentDocument:)
                                        keyEquivalent:nil];
           else
@@ -1626,24 +1652,24 @@ static NSString *processName = nil;
       [item setTarget: self];
       [recentMenu addItem: item];
       RELEASE(item);
-		}
+    }
 
-	[recentMenu setMenuChangedMessagesEnabled: YES];
+  [recentMenu setMenuChangedMessagesEnabled: YES];
 }
 
 - (IBAction) _openRecentDocument: (id)sender
 {
   // action to open recent document by tag index
-	NSURL *url;
-	int idx = [sender tag];
+  NSURL *url;
+  int idx = [sender tag];
 
-	if (idx < 0 || idx >= [_recent_documents count])
-		{
+  if (idx < 0 || idx >= [_recent_documents count])
+    {
       // something went wrong, ignore
       [self _updateOpenRecentMenu];
       return;
-		}
-	url = (NSURL *)[_recent_documents objectAtIndex: idx];
+    }
+  url = (NSURL *)[_recent_documents objectAtIndex: idx];
   [self openDocumentWithContentsOfURL: url display: YES];
 }
 
