@@ -28,31 +28,32 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSGeometry.h> 
-#include <Foundation/NSString.h> 
-#include <Foundation/NSArray.h> 
-#include <Foundation/NSValue.h> 
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSLock.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSSet.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSZone.h>
-#include "AppKit/AppKitExceptions.h"
-#include "AppKit/NSGraphicsContext.h"
-#include "AppKit/NSAffineTransform.h"
-#include "AppKit/NSBezierPath.h"
-#include "AppKit/NSPrintInfo.h"
-#include "AppKit/NSPrintOperation.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSView.h"
-#include "AppKit/DPSOperators.h"
-#include "GNUstepGUI/GSVersion.h"
-#include "GNUstepGUI/GSDisplayServer.h"
+#define EXPOSE_NSThread_IVARS
 
-typedef struct { @defs(NSThread) } *TInfo;
+#import <Foundation/NSGeometry.h> 
+#import <Foundation/NSString.h> 
+#import <Foundation/NSArray.h> 
+#import <Foundation/NSBundle.h> 
+#import <Foundation/NSValue.h> 
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSSet.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSZone.h>
+#import "AppKit/AppKitExceptions.h"
+#import "AppKit/NSGraphicsContext.h"
+#import "AppKit/NSAffineTransform.h"
+#import "AppKit/NSBezierPath.h"
+#import "AppKit/NSPrintInfo.h"
+#import "AppKit/NSPrintOperation.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSView.h"
+#import "AppKit/DPSOperators.h"
+#import "GNUstepGUI/GSVersion.h"
+#import "GNUstepGUI/GSDisplayServer.h"
 
 /* The memory zone where all global objects are allocated from (Contexts
    are also allocated from this zone) */
@@ -96,7 +97,7 @@ NSGraphicsContext	*GSCurrentContext(void)
  */
   NSThread *th = GSCurrentThread();
 
-  return (NSGraphicsContext*) ((TInfo)th)->_gcontext;
+  return (NSGraphicsContext*) th->_gcontext;
 #else
   NSMutableDictionary *dict = [[NSThread currentThread] threadDictionary];
 
@@ -165,7 +166,7 @@ NSGraphicsContext	*GSCurrentContext(void)
  */
   NSThread *th = GSCurrentThread();
 
-  ASSIGN(((TInfo)th)->_gcontext, context);
+  ASSIGN(th->_gcontext, context);
 #else
   NSMutableDictionary *dict = [[NSThread currentThread] threadDictionary];
 
@@ -375,7 +376,7 @@ NSGraphicsContext	*GSCurrentContext(void)
 
 - (void) setFocusStack: (void *)stack
 {
-  ASSIGN(focus_stack, stack);
+  ASSIGN(focus_stack, (id)stack);
 }
 
 - (void) setImageInterpolation: (NSImageInterpolation)interpolation
@@ -1780,3 +1781,24 @@ NSGraphicsContext	*GSCurrentContext(void)
 }
 
 @end
+
+@implementation NSGraphicsContext (NSGradient)
+- (void) drawGradient: (NSGradient*)gradient
+           fromCenter: (NSPoint)startCenter
+               radius: (CGFloat)startRadius
+             toCenter: (NSPoint)endCenter 
+               radius: (CGFloat)endRadius
+              options: (NSUInteger)options
+{
+  [self subclassResponsibility: _cmd];
+}
+
+- (void) drawGradient: (NSGradient*)gradient
+            fromPoint: (NSPoint)startPoint
+              toPoint: (NSPoint)endPoint
+              options: (NSUInteger)options
+{
+  [self subclassResponsibility: _cmd];
+}
+
+@end
