@@ -28,31 +28,32 @@
 #include <string.h>
 #include <math.h>
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSKeyedArchiver.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSValue.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSKeyedArchiver.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
 
-#include "AppKit/NSImage.h"
+#import "AppKit/NSImage.h"
 
-#include "AppKit/AppKitExceptions.h"
-#include "AppKit/NSAffineTransform.h"
-#include "AppKit/NSBitmapImageRep.h"
-#include "AppKit/NSCachedImageRep.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSPasteboard.h"
-#include "AppKit/NSPrintOperation.h"
-#include "AppKit/NSScreen.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/PSOperators.h"
-#include "GNUstepGUI/GSDisplayServer.h"
-#include "GSThemePrivate.h"
+#import "AppKit/AppKitExceptions.h"
+#import "AppKit/NSAffineTransform.h"
+#import "AppKit/NSBitmapImageRep.h"
+#import "AppKit/NSCachedImageRep.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSPrintOperation.h"
+#import "AppKit/NSScreen.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/PSOperators.h"
+#import "GNUstepGUI/GSDisplayServer.h"
+#import "GSThemePrivate.h"
 
 
 /* Helpers.  Would be nicer to use the C99 fmin/fmax functions, but that
@@ -128,15 +129,15 @@ BOOL NSImageForceCaching = NO; /* use on missmatch */
 {
   TEST_RELEASE(rep);
   TEST_RELEASE(bg);
-  NSDeallocateObject(self);
-  GSNOSUPERDEALLOC;
+  [super dealloc];
 }
 @end
 
 /* Class variables and functions for class methods */
-static NSMutableDictionary *nameDict = nil;
-static NSDictionary *nsmapping = nil;
-static NSColor *clearColor = nil;
+static NSRecursiveLock		*imageLock = nil;
+static NSMutableDictionary	*nameDict = nil;
+static NSDictionary		*nsmapping = nil;
+static NSColor			*clearColor = nil;
 static Class cachedClass = 0;
 static Class bitmapClass = 0;
 
@@ -175,49 +176,60 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
 + (void) initialize
 {
-  if (self == [NSImage class])
+  if (imageLock == nil)
     {
-      NSString *path = [NSBundle pathForLibraryResource: @"nsmapping"
-                                                 ofType: @"strings"
-                                             inDirectory: @"Images"];
+      NSString *path;
+
+      imageLock = [NSRecursiveLock new];
+      [imageLock lock];
 
       // Initial version
       [self setVersion: 1];
 
       // initialize the class variables
       nameDict = [[NSMutableDictionary alloc] initWithCapacity: 10];
+      path = [NSBundle pathForLibraryResource: @"nsmapping"
+				       ofType: @"strings"
+				  inDirectory: @"Images"];
       if (path)
         nsmapping = RETAIN([[NSString stringWithContentsOfFile: path]
                                propertyListFromStringsFileFormat]);
       clearColor = RETAIN([NSColor clearColor]);
       cachedClass = [NSCachedImageRep class];
       bitmapClass = [NSBitmapImageRep class];
+      [imageLock unlock];
     }
 }
 
 + (id) imageNamed: (NSString *)aName
 {
-  NSString *realName = [nsmapping objectForKey: aName];
-  NSImage *image;
-
-  if (realName)
-    aName = realName;
-
-  image = (NSImage*)[nameDict objectForKey: aName];
+  NSImage	*image;
  
-  if (image == nil)
+  /* 2009-09-10 changed operation of nsmapping so that the loaded
+   * image is stored under the key 'aName', not under the mapped
+   * name.  That way the image is created with the correct name and
+   * a later call to -setName: will work properly.
+   */
+  [imageLock lock];
+  image = (NSImage*)[nameDict objectForKey: aName];
+  if (image == nil || [(id)image _resource] == nil)
     {
-      NSString *ext;
-      NSString *path = nil;
-      NSBundle *main_bundle;
-      NSArray  *array;
-      NSString *the_name = aName;
+      NSString	*realName = [nsmapping objectForKey: aName];
+      NSString	*ext;
+      NSString	*path = nil;
+      NSBundle	*main_bundle;
+      NSArray	*array;
 
+      if (realName == nil)
+	{
+          realName = aName;
+	}
+ 
       // FIXME: This should use [NSBundle pathForImageResource], but this will 
       // only allow imageUnfilteredFileTypes.
       /* If there is no image with that name, search in the main bundle */
       main_bundle = [NSBundle mainBundle];
-      ext = [aName pathExtension];
+      ext = [realName pathExtension];
       if (ext != nil && [ext length] == 0)
         {
           ext = nil;
@@ -225,23 +237,22 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
       /* Check if extension is one of the image types */
       array = [self imageFileTypes];
-      if ([array indexOfObject: ext] != NSNotFound)
+      if (ext != nil && [array indexOfObject: ext] != NSNotFound)
         {
           /* Extension is one of the image types
              So remove from the name */
-          the_name = [aName stringByDeletingPathExtension];
+          realName = [realName stringByDeletingPathExtension];
         }
       else
         {
           /* Otherwise extension is not an image type
              So leave it alone */
-          the_name = aName;
           ext = nil;
         }
 
       /* First search locally */
       if (ext)
-        path = [main_bundle pathForResource: the_name ofType: ext];
+        path = [main_bundle pathForResource: realName ofType: ext];
       else 
         {
           id o, e;
@@ -249,19 +260,39 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
           e = [array objectEnumerator];
           while ((o = [e nextObject]))
             {
-              path = [main_bundle pathForResource: the_name 
-                        ofType: o];
+              path = [main_bundle pathForResource: realName 
+					   ofType: o];
               if (path != nil && [path length] != 0)
                 break;
             }
         }
 
+      /* Second search on theme bundle */
+      if (!path)
+	{
+	  if (ext)
+	    path = [[[GSTheme theme] bundle] pathForResource: realName ofType: ext];
+	  else 
+	    {
+	      id o, e;
+	      
+	      e = [array objectEnumerator];
+	      while ((o = [e nextObject]))
+		{
+		  path = [[[GSTheme theme] bundle] pathForResource: realName 
+				                            ofType: o];
+		  if (path != nil && [path length] != 0)
+		    break;
+		}
+	    }
+	}
+
       /* If not found then search in system */
       if (!path)
         {
           if (ext)
             {
-              path = [NSBundle pathForLibraryResource: the_name
+              path = [NSBundle pathForLibraryResource: realName
                                                ofType: ext
                                           inDirectory: @"Images"];
             }
@@ -272,8 +303,8 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
               e = [array objectEnumerator];
               while ((o = [e nextObject]))
                 {
-                  path = [NSBundle pathForLibraryResource: the_name
-                                                          ofType: o
+                  path = [NSBundle pathForLibraryResource: realName
+						   ofType: o
                                               inDirectory: @"Images"];
                   if (path != nil && [path length] != 0)
                     break;
@@ -291,10 +322,11 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
               AUTORELEASE(image);
               image->_flags.archiveByName = YES;
             }
-          return image;
+          image = (NSImage*)[nameDict objectForKey: aName];
         }
     }
-  
+  IF_NO_GC([[image retain] autorelease]);
+  [imageLock unlock];
   return image;
 }
 
@@ -420,7 +452,7 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
   return self;
 }
 
-- (id)initWithContentsOfURL:(NSURL *)anURL
+- (id) initWithContentsOfURL: (NSURL *)anURL
 {
   NSArray *array = [NSImageRep imageRepsWithContentsOfURL: anURL];
 
@@ -468,16 +500,18 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
 - (void) dealloc
 {
-  RELEASE(_reps);
-  /* Make sure we don't remove name from the nameDict if we are just a copy
-     of the named image, not the original image */
-  if (_name && self == [[nameDict objectForKey: _name] _resource]) 
-    [[nameDict objectForKey: _name] _setResource: nil];
-  RELEASE(_name);
-  TEST_RELEASE(_fileName);
-  RELEASE(_color);
-
-  [super dealloc];
+  if (_name == nil)
+    {
+      RELEASE(_reps);
+      TEST_RELEASE(_fileName);
+      RELEASE(_color);
+      [super dealloc];
+    }
+  else
+    {
+      [self retain];
+      NSLog(@"Warning ... attempt to deallocate image with name: %@", _name);
+    }
 }
 
 - (id) copyWithZone: (NSZone *)zone
@@ -489,7 +523,7 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
   copy = (NSImage*)NSCopyObject (self, 0, zone);
 
-  RETAIN(_name);
+  copy->_name = nil;
   RETAIN(_fileName);
   RETAIN(_color);
   copy->_lockedView = nil;
@@ -508,21 +542,54 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
   return copy;
 }
 
+/* This methd sets the name of an image, updating the global name dictionary
+ * to point to the image (or removing an image from the dictionary if the
+ * new name is nil).
+ * The images are actually accessed via proxy objects, so that when a
+ * new system image is set (using [NSImage+_setImage:name:]), the proxy
+ * for that image just starts using the new version.
+ */
 - (BOOL) setName: (NSString *)aName
 {
   GSThemeProxy	*proxy = nil;
   
-  if (!aName || [[nameDict objectForKey: aName] _resource] != nil)
+  [imageLock lock];
+
+  /* The name is already set... nothing to do.
+   */
+  if (aName == _name || [aName isEqual: _name] == YES)
     {
+      [imageLock unlock];
+      return YES;
+    }
+
+  /* If the new name is already in use by another image,
+   * we must do nothing.
+   */
+  if (aName != nil && [[nameDict objectForKey: aName] _resource] != nil)
+    {
+      [imageLock unlock];
       return NO;
     }
-  if (_name && self == [(proxy = [nameDict objectForKey: _name]) _resource])
+
+  /* If this image had another name, we remove it.
+   */
+  if (_name != nil)
     {
       /* We retain self in case removing from the dictionary releases us */
       IF_NO_GC([[self retain] autorelease]);
-      [proxy _setResource: nil];
+      [nameDict removeObjectForKey: _name];
+      DESTROY(_name);
     }
   
+  /* If the new name is null, there is nothing more to do.
+   */
+  if (aName == nil)
+    {
+      [imageLock unlock];
+      return NO;
+    }
+
   ASSIGN(_name, aName);
   
   if ((proxy = [nameDict objectForKey: _name]) == nil)
@@ -533,12 +600,18 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
     }
   [proxy _setResource: self];
   
+  [imageLock unlock];
   return YES;
 }
 
 - (NSString *) name
 {
-  return _name;
+  NSString	*name;
+
+  [imageLock lock];
+  name = [[_name retain] autorelease];
+  [imageLock unlock];
+  return name;
 }
 
 - (void) setSize: (NSSize)aSize
@@ -729,11 +802,10 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 - (void) compositeToPoint: (NSPoint)aPoint 
                 operation: (NSCompositingOperation)op
 {
-  NSRect rect;
   // Might not be computed up to now
   NSSize size = [self size];
+  NSRect rect = NSMakeRect(0, 0, size.width, size.height);
 
-  rect = NSMakeRect(0, 0, size.width, size.height);
   [self compositeToPoint: aPoint fromRect: rect operation: op];
 }
 
@@ -741,64 +813,10 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
                  fromRect: (NSRect)aRect
                 operation: (NSCompositingOperation)op
 {
-#if 0
   [self compositeToPoint: aPoint
         fromRect: aRect
         operation: op
         fraction: 1.0];
-#else 
-  NS_DURING
-    { 
-      NSImageRep *rep = [self bestRepresentationForDevice: nil];
-      NSCachedImageRep *cache = nil;
-
-      if (rep == nil)
-        NS_VOIDRETURN;
-
-      if (([GSCurrentContext() isDrawingToScreen] == YES)
-          && _cacheMode != NSImageCacheNever)
-        cache = [self _doImageCache: rep];
-
-      if (cache != nil)
-        {
-          NSRect rect;
-
-          rect = [cache rect];
-          NSDebugLLog(@"NSImage", @"composite rect %@ in %@", 
-                      NSStringFromRect(rect), NSStringFromRect(aRect));
-          // Move the drawing rectangle to the origin of the image rep
-          // and intersect the two rects.
-          aRect.origin.x += rect.origin.x;
-          aRect.origin.y += rect.origin.y;
-          rect = NSIntersectionRect(aRect, rect);
-          
-          PScomposite(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect),
-                      [[cache window] gState], aPoint.x, aPoint.y, op);
-        }
-      else        
-        {
-          NSRect rect;
-
-          rect = NSMakeRect(aPoint.x, aPoint.y, _size.width, _size.height);
-          [self drawRepresentation: rep inRect: rect];
-        }
-    }
-  NS_HANDLER
-    {
-      NSLog(@"NSImage: compositeToPoint:fromRect:operation: failed due to %@: %@", 
-            [localException name], [localException reason]);
-      if ([_delegate respondsToSelector: @selector(imageDidNotDraw:inRect:)])
-        {
-          NSImage *image = [_delegate imageDidNotDraw: self inRect: aRect];
-
-          if (image != nil)
-            [image compositeToPoint: aPoint
-                   fromRect: aRect 
-                   operation: op];
-        }
-    }
-  NS_ENDHANDLER
-#endif
 }
 
 - (void) compositeToPoint: (NSPoint)aPoint
@@ -832,9 +850,8 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
       if (cache != nil)
         {
-          NSRect rect;
+          NSRect rect = [cache rect];
 
-          rect = [cache rect];
           NSDebugLLog(@"NSImage", @"composite rect %@ in %@", 
                       NSStringFromRect(rect), NSStringFromRect(srcRect));
           // Move the drawing rectangle to the origin of the image rep
@@ -889,63 +906,10 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
                 fromRect: (NSRect)aRect 
                 fraction: (float)aFloat
 {
-#if 0
   [self compositeToPoint: aPoint
         fromRect: aRect
         operation: NSCompositeSourceOver
         fraction: aFloat];
-#else 
-  NS_DURING
-    {
-      NSImageRep *rep = [self bestRepresentationForDevice: nil];
-      NSCachedImageRep *cache = nil;
-
-      if (rep == nil)
-        NS_VOIDRETURN;
-
-      if (([GSCurrentContext() isDrawingToScreen] == YES)
-          && _cacheMode != NSImageCacheNever)
-        cache = [self _doImageCache: rep];
-
-      if (cache != nil)
-        {
-          NSRect rect;
-          
-          rect = [cache rect];
-          // Move the drawing rectangle to the origin of the image rep
-          // and intersect the two rects.
-          aRect.origin.x += rect.origin.x;
-          aRect.origin.y += rect.origin.y;
-          rect = NSIntersectionRect(aRect, rect);
-          PSdissolve(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect),
-                     [[cache window] gState], aPoint.x, aPoint.y, aFloat);
-        }
-      else
-        {
-          NSRect rect;
-
-          /* FIXME: Here we are supposed to composite directly from the source
-             but how do you do that? */
-          rect = NSMakeRect(aPoint.x, aPoint.y, _size.width, _size.height);
-          [self drawRepresentation: rep inRect: rect];
-        }
-    }
-  NS_HANDLER
-    {
-      NSLog(@"NSImage: dissolve failed due to %@: %@", 
-            [localException name], [localException reason]);
-      if ([_delegate respondsToSelector: @selector(imageDidNotDraw:inRect:)])
-        {
-          NSImage *image = [_delegate imageDidNotDraw: self inRect: aRect];
-
-          if (image != nil)
-            [image dissolveToPoint: aPoint
-                   fromRect: aRect 
-                   fraction: aFloat];
-        }
-    }
-  NS_ENDHANDLER
-#endif
 }
 
 - (BOOL) drawRepresentation: (NSImageRep *)imageRep inRect: (NSRect)aRect
@@ -1848,7 +1812,7 @@ iterate_reps_for_types(NSArray* imageReps, SEL method)
       return NO;
     }
 
-  ext = [fileName pathExtension];
+  ext = [[fileName pathExtension] lowercaseString];
   if (!ext)
     return NO;
   array = [isa imageFileTypes];
@@ -1935,19 +1899,19 @@ iterate_reps_for_types(NSArray* imageReps, SEL method)
                 {
                   if (repd->bg == nil)
                     {
-                      NSDebugLLog(@"NSImage", @"Invalid %@ ... %@ %d", 
+                      NSDebugLLog(@"NSImage", @"Invalid %@ ... %@ %@", 
                                   repd->bg, _color, repd->rep);
                       invalidCache = repd;
                     }
                   else if (opaque == YES || [repd->bg isEqual: _color] == YES)
                     {
-                      NSDebugLLog(@"NSImage", @"Exact %@ ... %@ %d", 
+                      NSDebugLLog(@"NSImage", @"Exact %@ ... %@ %@", 
                                   repd->bg, _color, repd->rep);
                       return repd;
                     }
                   else
                     {
-                      NSDebugLLog(@"NSImage", @"Partial %@ ... %@ %d", 
+                      NSDebugLLog(@"NSImage", @"Partial %@ ... %@ %@", 
                                   repd->bg, _color, repd->rep);
                       partialCache = repd;
                       partialCount++;
@@ -2006,3 +1970,44 @@ iterate_reps_for_types(NSArray* imageReps, SEL method)
 }
 
 @end
+
+@implementation	NSImage (GSTheme)
+
+/* This method is used by the theming system to replace a named image
+ * without disturbing the proxy ... so that all views and cells using
+ * the named image are automatically updated to use the new image.
+ * This is the counterpart to the -setName: method, which replaces the
+ * proxy (to change a named image without updating the image used by
+ * existing views and cells).
+ */
++ (NSImage*) _setImage: (NSImage*)image name: (NSString*)name
+{
+  GSThemeProxy	*proxy = nil;
+  
+  NSAssert([image isKindOfClass: [NSImage class]], NSInvalidArgumentException);
+  NSAssert(![image isProxy], NSInvalidArgumentException);
+  NSAssert([name isKindOfClass: [NSString class]], NSInvalidArgumentException);
+  NSAssert([name length] > 0, NSInvalidArgumentException);
+  NSAssert([image name] == nil, NSInvalidArgumentException);
+
+  [imageLock lock];
+  ASSIGNCOPY(image->_name, name);
+  if ((proxy = [nameDict objectForKey: image->_name]) == nil)
+    {
+      proxy = [GSThemeProxy alloc];
+      [nameDict setObject: proxy forKey: image->_name];
+      [proxy release]; 
+    }
+  else
+    {
+      /* Remove the name from the old image.
+       */
+      DESTROY(((NSImage*)[proxy _resource])->_name);
+    }
+  [proxy _setResource: image];
+  IF_NO_GC([[proxy retain] autorelease]);
+  [imageLock unlock];
+  return (NSImage*)proxy;
+}
+
+@end
