@@ -33,30 +33,36 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include "config.h"
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSProcessInfo.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSNotification.h>
+#import "config.h"
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSProcessInfo.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSNotificationQueue.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSValue.h>
 
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSMenuItem.h"
-#include "AppKit/NSMenuView.h"
-#include "AppKit/NSPanel.h"
-#include "AppKit/NSPopUpButton.h"
-#include "AppKit/NSPopUpButtonCell.h"
-#include "AppKit/NSScreen.h"
-#include "AppKit/NSAttributedString.h"
+#import "AppKit/NSMatrix.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSMenuItem.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSPanel.h"
+#import "AppKit/NSPopUpButton.h"
+#import "AppKit/NSPopUpButtonCell.h"
+#import "AppKit/NSScreen.h"
+#import "AppKit/NSAttributedString.h"
 
-#include "GSGuiPrivate.h"
+#import "GSGuiPrivate.h"
+#import "GNUstepGUI/GSTheme.h"
 
 /*
   Drawing related:
@@ -108,6 +114,9 @@
 
 /* Subclass of NSPanel since menus cannot become key */
 @interface NSMenuPanel : NSPanel
+{
+  NSMenu *_the_menu;
+}
 @end
 
 @interface NSMenuView (GNUstepPrivate)
@@ -133,6 +142,11 @@ static BOOL menuBarVisible = YES;
 
 
 @implementation NSMenuPanel
+- (void) _setmenu: (NSMenu *)menu
+{
+  _the_menu = menu;
+}
+
 - (BOOL) canBecomeKeyWindow
 {
   /* See [NSWindow-_lossOfKeyOrMainWindow] */
@@ -140,6 +154,20 @@ static BOOL menuBarVisible = YES;
     return YES;
   return NO;
 }
+
+- (void) orderFrontRegardless
+{
+  NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
+  if (style == NSWindows95InterfaceStyle)
+    {
+      // if we're the top level menu in Windows mode, don't show it.
+      if([_the_menu supermenu] == nil && [_the_menu _ownedByPopUp] == NO)
+	{
+	  return;
+	}
+    }
+  [super orderFrontRegardless];
+}
 @end
 
 @implementation	NSMenu (GNUstepPrivate)
@@ -182,9 +210,11 @@ static BOOL menuBarVisible = YES;
 		     styleMask: NSBorderlessWindowMask
 		     backing: NSBackingStoreBuffered
 		     defer: YES];
+  [win setBackgroundColor: [NSColor clearColor]];
   [win setLevel: NSSubmenuWindowLevel];
   [win setWorksWhenModal: NO];
   [win setBecomesKeyOnlyIfNeeded: YES];
+  [win _setmenu: self];
 
   return win;
 }
@@ -589,7 +619,9 @@ static BOOL menuBarVisible = YES;
 {
   NSMenuView *menuRep;
 
-  [super init];
+  self = [super init];
+  if (!self)
+    return nil;
 
   // Keep the title.
   ASSIGN(_title, aTitle);
@@ -627,7 +659,7 @@ static BOOL menuBarVisible = YES;
      up menus in NSApplicationWillFinishLaunching.
   */
   [nc addObserver: self
-      selector: @selector(_showTornOffMenuIfAny:)
+      selector: @selector(applicationDidFinishLaunching:)
       name: NSApplicationDidFinishLaunchingNotification 
       object: NSApp];
 
@@ -652,6 +684,16 @@ static BOOL menuBarVisible = YES;
 /*
  * Setting Up the Menu Commands
  */
+
+ - (void) menuChanged
+{
+  // propagate notification up to the main menu
+  if (self == [NSApp mainMenu])
+    _menu.mainMenuChanged = YES;
+  else
+    [[self supermenu] menuChanged];
+}
+
 - (void) insertItem: (id <NSMenuItem>)newItem
 	    atIndex: (int)index
 {
@@ -676,6 +718,7 @@ static BOOL menuBarVisible = YES;
   
   [_items insertObject: newItem atIndex: index];
   _menu.needsSizing = YES;
+  [(NSMenuView*)_view setNeedsSizing: YES];
   
   // Create the notification for the menu representation.
   d = [NSDictionary
@@ -690,6 +733,7 @@ static BOOL menuBarVisible = YES;
     [nc postNotification: inserted];
   else
     [_notifications addObject: inserted];
+  [self menuChanged];
 
   // Set this after the insert notification has been sent.
   [newItem setMenu: self];
@@ -748,6 +792,7 @@ static BOOL menuBarVisible = YES;
   [anItem setMenu: nil];
   [_items removeObjectAtIndex: index];
   _menu.needsSizing = YES;
+  [(NSMenuView*)_view setNeedsSizing: YES];
   
   d = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: index]
 		    forKey: @"NSMenuItemIndex"];
@@ -760,6 +805,7 @@ static BOOL menuBarVisible = YES;
     [nc postNotification: removed];
   else
     [_notifications addObject: removed];
+  [self menuChanged];
 }
 
 - (void) itemChanged: (id <NSMenuItem>)anObject
@@ -772,6 +818,7 @@ static BOOL menuBarVisible = YES;
     return;
 
   _menu.needsSizing = YES;
+  [(NSMenuView*)_view setNeedsSizing: YES];
 
   d = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: index]
 		    forKey: @"NSMenuItemIndex"];
@@ -784,6 +831,7 @@ static BOOL menuBarVisible = YES;
     [nc postNotification: changed];
   else
     [_notifications addObject: changed];
+  [self menuChanged];
 
   // Update the menu.
   [self update];
@@ -889,7 +937,7 @@ static BOOL menuBarVisible = YES;
       if (actionSelector == 0 || sel_eq([menuItem action], actionSelector))
         {
           // There are different possibilities to implement the check here
-            if ([menuItem target] == anObject)
+          if ([menuItem target] == anObject)
             {
               return i;
             }
@@ -1013,31 +1061,6 @@ static BOOL menuBarVisible = YES;
 
 - (void) update
 {
-  if (self == [NSApp mainMenu])
-    {
-      /* For microsoft style menus, we attempt to notice that we have
-       * a main window, and make sure that the menu appears in it.
-       */
-      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == 
-        NSWindows95InterfaceStyle)
-	{
-	  NSWindow	*w = [NSApp mainWindow];
-
-	  if (w == nil)
-	    {
-	      /* Make sure we are set up properly and displayed.
-	       */
-	      if ([_aWindow isVisible] == NO)
-		{
-	          [self setMain: YES];
-		}
-	    }
-	  else if ([w menu] != self)
-	    {
-	      [w setMenu: self];
-	    }
-	}
-    }
   if (_delegate)
     {
       if ([_delegate respondsToSelector:@selector(menuNeedsUpdate:)])
@@ -1092,70 +1115,86 @@ static BOOL menuBarVisible = YES;
       
       // Temporary disable automatic displaying of menu.
       [self setMenuChangedMessagesEnabled: NO];
-      
-      for (i = 0; i < count; i++)
+
+      NS_DURING
 	{
-	  NSMenuItem *item = [_items objectAtIndex: i];
-	  SEL	      action = [item action];
-	  id	      validator = nil;
-	  BOOL	      wasEnabled = [item isEnabled];
-	  BOOL	      shouldBeEnabled;
-
-	  // Update the submenu items if any.
-	  if ([item hasSubmenu])
-	    [[item submenu] update];
-
-	  // If there is no action - there can be no validator for the item.
-	  if (action)
+	  for (i = 0; i < count; i++)
 	    {
-	      validator = [NSApp targetForAction: action 
-				 to: [item target]
-				 from: item];
-	    }
-	  else if (_popUpButtonCell != nil)
-	    {
-	      if (NULL != (action = [_popUpButtonCell action]))
+	      NSMenuItem *item = [_items objectAtIndex: i];
+	      SEL	      action = [item action];
+	      id	      validator = nil;
+	      BOOL	      wasEnabled = [item isEnabled];
+	      BOOL	      shouldBeEnabled;
+
+	      // Update the submenu items if any.
+	      if ([item hasSubmenu])
+	        [[item submenu] update];
+
+	      // If there is no action - there can be no validator for the item.
+	      if (action)
 	        {
-		  validator = [NSApp targetForAction: action
-				     to: [_popUpButtonCell target]
-				     from: [_popUpButtonCell controlView]];
-		}
-	    }
+	          validator = [NSApp targetForAction: action 
+				     to: [item target]
+				     from: item];
+	        }
+	      else if (_popUpButtonCell != nil)
+	        {
+	          if (NULL != (action = [_popUpButtonCell action]))
+	            {
+		      validator = [NSApp targetForAction: action
+				         to: [_popUpButtonCell target]
+				         from: [_popUpButtonCell controlView]];
+		    }
+	        }
 
-	  if (validator == nil)
-	    {
-	      if ((action == NULL) && (_popUpButtonCell != nil))
-		{
-		  shouldBeEnabled = YES;
-		}
-	      else 
-		{
-		  shouldBeEnabled = NO;
-		}
-	    }
-	  else if ([validator
-		     respondsToSelector: @selector(validateMenuItem:)])
-	    {
-	      shouldBeEnabled = [validator validateMenuItem: item];
-	    }
-	  else if ([validator
-		     respondsToSelector: @selector(validateUserInterfaceItem:)])
-	    {
-	      shouldBeEnabled = [validator validateUserInterfaceItem: item];
-	    }
-	  else
-	    {
-	      shouldBeEnabled = YES;
-	    }
+	      if (validator == nil)
+	        {
+	          if ((action == NULL) && (_popUpButtonCell != nil))
+		    {
+		      shouldBeEnabled = YES;
+		    }
+	          else 
+		    {
+		      shouldBeEnabled = NO;
+		    }
+	        }
+	      else if ([validator
+		         respondsToSelector: @selector(validateMenuItem:)])
+	        {
+	          shouldBeEnabled = [validator validateMenuItem: item];
+	        }
+	      else if ([validator
+		         respondsToSelector: @selector(validateUserInterfaceItem:)])
+	        {
+	          shouldBeEnabled = [validator validateUserInterfaceItem: item];
+	        }
+	      else
+	        {
+	          shouldBeEnabled = YES;
+	        }
 
-	  if (shouldBeEnabled != wasEnabled)
-	    {
-	      [item setEnabled: shouldBeEnabled];
+	      if (shouldBeEnabled != wasEnabled)
+	        {
+	          [item setEnabled: shouldBeEnabled];
+	        }
 	    }
-	}
-          
+          }
+	NS_HANDLER
+	  {
+	    NSLog(@"Error Occurred While Updating Menu %@: %@", [self title], localException);
+	  }
+	NS_ENDHANDLER
       // Reenable displaying of menus
-      [self setMenuChangedMessagesEnabled: YES];
+      [self setMenuChangedMessagesEnabled: YES]; // this will send pending _notifications
+    }
+
+  if (_menu.mainMenuChanged)
+    {
+      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == NSWindows95InterfaceStyle)
+        {
+          [[GSTheme theme] updateAllWindowsWithMenu: self];
+        }
+      _menu.mainMenuChanged = NO;
     }
 
   if (_menu.needsSizing && ([_aWindow isVisible] || [_bWindow isVisible]))
@@ -1177,7 +1216,7 @@ static BOOL menuBarVisible = YES;
   NSEventType   type = [theEvent type];
   unsigned	modifiers = [theEvent modifierFlags];
   NSString	*keyEquivalent = [theEvent charactersIgnoringModifiers];
-         
+
   if (type != NSKeyDown && type != NSKeyUp) 
     return NO;
              
@@ -1264,6 +1303,7 @@ static BOOL menuBarVisible = YES;
   ASSIGN(_title, aTitle);
 
   _menu.needsSizing = YES;
+  [(NSMenuView*)_view setNeedsSizing: YES];
   if ([_aWindow isVisible] || [_bWindow isVisible])
     {
       [self sizeToFit];
@@ -1324,10 +1364,12 @@ static BOOL menuBarVisible = YES;
        * menu, we will still have it recorded as ours, but it won't be
        * in our view hierarchy, so we have to re-add it.
        */
+      /*
       if (contentView != [menuRep superview])
 	{
           [contentView addSubview: menuRep];
 	}
+      */
       return;
     }
 
@@ -1538,11 +1580,11 @@ static BOOL menuBarVisible = YES;
       //
       if ([aDecoder containsValueForKey: @"NSNoAutoenable"])
         {
-	  dAuto = ([aDecoder decodeBoolForKey: @"NSNoAutoenable"] == NO) ? YES : NO;
+	  dAuto = ![aDecoder decodeBoolForKey: @"NSNoAutoenable"];
 	}
       else
 	{
-	  dAuto = NO;
+	  dAuto = YES;
 	}
       dTitle = [aDecoder decodeObjectForKey: @"NSTitle"];
       dItems = [aDecoder decodeObjectForKey: @"NSMenuItems"];
@@ -1563,16 +1605,7 @@ static BOOL menuBarVisible = YES;
   for (i = 0; i < [dItems count]; i++)
     {
       NSMenuItem	*item = [dItems objectAtIndex: i];
-      NSMenu		*sub = [item submenu];
-
       [self addItem: item];
-      // FIXME: We propably don't need this, as all the fields are
-      // already set up for the submenu item.
-      if (sub != nil)
-	{
-	  [sub setSupermenu: nil];
-	  [self setSubmenu: sub  forItem: item];
-	}
     }
   [self setMenuChangedMessagesEnabled: YES];
 
@@ -1656,6 +1689,12 @@ static BOOL menuBarVisible = YES;
     }
 }
 
+- (void) applicationDidFinishLaunching:(NSNotification *)notification
+{
+  [[GSTheme theme] updateAllWindowsWithMenu: [NSApp mainMenu]];
+  [self _showTornOffMenuIfAny:notification];
+}
+
 - (void) _showOnActivateApp: (NSNotification*)notification
 {
   if ([NSApp mainMenu] == self)
@@ -1695,7 +1734,7 @@ static BOOL menuBarVisible = YES;
   if (_menu.transient)
     {
       NSDebugLLog (@"NSMenu", 
-                   @"trying to display while alreay displayed transient");
+                   @"trying to display while already displayed transient");
     }
 
   if (_menu.needsSizing)
@@ -1855,7 +1894,7 @@ static BOOL menuBarVisible = YES;
       oldRep = [self menuRepresentation];
       oldStyle = [oldRep interfaceStyle];
       newStyle = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
-
+	  
       /*
        * If necessary, rebuild menu for (different) style
        */
@@ -1906,6 +1945,7 @@ static BOOL menuBarVisible = YES;
         {
           [self display];
         }
+      [[GSTheme theme] updateAllWindowsWithMenu: self];
     }
   else 
     {
