@@ -28,19 +28,19 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include "config.h"
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSException.h>
-#include <GNUstepBase/GSCategories.h>
-#include "AppKit/NSCell.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSKeyValueBinding.h"
-#include "AppKit/NSMenuItem.h"
-#include "AppKit/NSMenu.h"
-#include "GSBindingHelpers.h"
+#import "config.h"
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+
+#import "AppKit/NSCell.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSKeyValueBinding.h"
+#import "AppKit/NSMenuItem.h"
+#import "AppKit/NSMenu.h"
+#import "GSBindingHelpers.h"
 
 static BOOL usesUserKeyEquivalents = NO;
 static Class imageClass;
@@ -56,6 +56,9 @@ static Class imageClass;
   self = [super initWithTitle: @"-----------"
 		action: NULL
 		keyEquivalent: @""];
+  if (self == nil)
+    return nil;
+
   _enabled = NO;
   _changesState = NO;
   return self;
@@ -66,7 +69,7 @@ static Class imageClass;
   return YES;
 }
 
-// FIXME: We need a lot of methods to switch of changes for a separator
+// FIXME: We need a lot of methods to switch off changes for a separator
 @end
 
 
@@ -127,6 +130,9 @@ static Class imageClass;
        keyEquivalent: (NSString*)charCode
 {
   self = [super init];
+  if (self == nil)
+    return nil;
+
   //_menu = nil;
   [self setTitle: aString];
   [self setKeyEquivalent: charCode];
@@ -168,6 +174,9 @@ static Class imageClass;
 
 - (void) setSubmenu: (NSMenu*)submenu
 {
+  if (submenu == _submenu)
+    return; // no change
+	
   if ([submenu supermenu] != nil)
     {
       [NSException raise: NSInvalidArgumentException
@@ -194,7 +203,10 @@ static Class imageClass;
 {
   if (nil == aString)
     aString = @"";
-
+	
+  if ([_title isEqualToString:aString])
+    return; // no change
+	
   ASSIGNCOPY(_title,  aString);
   [_menu itemChanged: self];
 }
@@ -219,7 +231,9 @@ static Class imageClass;
                    @"Attempt to use nil as key equivalent");
       aKeyEquivalent = @"";
     }
-
+  if ([_keyEquivalent isEqualToString:aKeyEquivalent])
+    return; // no change
+	
   ASSIGNCOPY(_keyEquivalent,  aKeyEquivalent);
   [_menu itemChanged: self];
 }
@@ -234,7 +248,10 @@ static Class imageClass;
 
 - (void) setKeyEquivalentModifierMask: (unsigned int)mask
 {
+  if (_keyEquivalentModifierMask == mask)
+    return; // no change
   _keyEquivalentModifierMask = mask;
+  [_menu itemChanged: self];
 }
 
 - (unsigned int) keyEquivalentModifierMask
@@ -263,6 +280,9 @@ static Class imageClass;
 
 - (void) setMnemonicLocation: (unsigned)location
 {
+  if (_mnemonicLocation == location)
+    return; // no change
+	
   _mnemonicLocation = location;
   [_menu itemChanged: self];
 }
@@ -296,7 +316,10 @@ static Class imageClass;
 {
   NSAssert(image == nil || [image isKindOfClass: imageClass],
     NSInvalidArgumentException);
-
+	
+  if (_image == image)
+    return; // no change
+	
   ASSIGN(_image, image);
   [_menu itemChanged: self];
 }
@@ -325,7 +348,10 @@ static Class imageClass;
 {
   NSAssert(image == nil || [image isKindOfClass: imageClass],
     NSInvalidArgumentException);
-
+	
+  if (_onStateImage == image)
+    return; // no change
+	
   ASSIGN(_onStateImage, image);
   [_menu itemChanged: self];
 }
@@ -339,6 +365,9 @@ static Class imageClass;
 {
   NSAssert(image == nil || [image isKindOfClass: imageClass],
     NSInvalidArgumentException);
+	
+  if (_offStateImage == image)
+    return; // no change
 
   ASSIGN(_offStateImage, image);
   [_menu itemChanged: self];
@@ -353,7 +382,10 @@ static Class imageClass;
 {
   NSAssert(image == nil || [image isKindOfClass: imageClass],
     NSInvalidArgumentException);
-
+	
+  if (_mixedStateImage == image)
+    return; // no change
+	
   ASSIGN(_mixedStateImage, image);
   [_menu itemChanged: self];
 }
@@ -405,12 +437,12 @@ static Class imageClass;
   return _action;
 }
 
-- (void) setTag: (int)anInt
+- (void) setTag: (NSInteger)anInt
 {
   _tag = anInt;
 }
 
-- (int) tag
+- (NSInteger) tag
 {
   return _tag;
 }
@@ -506,9 +538,17 @@ static Class imageClass;
       [aCoder encodeObject: _onStateImage forKey: @"NSOnImage"];
       [aCoder encodeObject: _offStateImage forKey: @"NSOffImage"]; // ???????
       [aCoder encodeObject: _mixedStateImage forKey: @"NSMixedImage"]; 
-      [aCoder encodeObject: _target forKey: @"NSTarget"]; 
+      [aCoder encodeObject: _target forKey: @"NSTarget"];
       [aCoder encodeObject: _menu forKey: @"NSMenu"];
-      [aCoder encodeObject: _submenu forKey: @"NSSubmenu"];
+
+      // If the menu is owned by a popup, then don't encode the children.
+      // This prevents an assertion error in IB as these keys should not 
+      // be present in a menu item when it's encoded as part of a popup. 
+      if([_menu _ownedByPopUp] == NO)
+	{
+	  [aCoder encodeObject: _submenu forKey: @"NSSubmenu"];
+	}
+
       [aCoder encodeInt: _keyEquivalentModifierMask forKey: @"NSKeyEquivModMask"];
       [aCoder encodeInt: _mnemonicLocation forKey: @"NSMnemonicLoc"];
       [aCoder encodeInt: _state forKey: @"NSState"];
@@ -552,6 +592,7 @@ static Class imageClass;
       NSString *action;
       NSString *key;
       BOOL isSeparator = NO;
+      int keyMask;
 
       if ([aDecoder containsValueForKey: @"NSIsSeparator"])
         {
@@ -603,11 +644,12 @@ static Class imageClass;
           NSMenu *submenu = [aDecoder decodeObjectForKey: @"NSSubmenu"];
           [self setSubmenu: submenu];
         }
-      if ([aDecoder containsValueForKey: @"NSKeyEquivModMask"])
-        {
-          int keyMask = [aDecoder decodeIntForKey: @"NSKeyEquivModMask"];
-          [self setKeyEquivalentModifierMask: keyMask];
-        }
+
+      // Set the key mask regardless of whether it is present;
+      // i.e. set it to 0 if it is not present in the nib.
+      keyMask = [aDecoder decodeIntForKey: @"NSKeyEquivModMask"];
+      [self setKeyEquivalentModifierMask: keyMask];
+
       if ([aDecoder containsValueForKey: @"NSMnemonicLoc"])
         {
           int loc = [aDecoder decodeIntForKey: @"NSMnemonicLoc"];
