@@ -46,19 +46,16 @@
 #include "AppKit/NSMenuView.h"
 #include "AppKit/NSParagraphStyle.h"
 #include "GNUstepGUI/GSTheme.h"
+#include "GSGuiPrivate.h"
 
 
 @implementation NSMenuItemCell
 
-static NSImage *arrowImage = nil; /* Cache arrow image. */
-
-
 + (void) initialize
 {
   if (self == [NSMenuItemCell class])
     {
       [self setVersion: 2];
-      arrowImage = [[NSImage imageNamed: @"NSMenuArrow"] copy];
     }
 }
 
@@ -98,7 +95,7 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
 
 - (NSColor *)textColor
 {
-  if(_cell.is_highlighted && [self isEnabled])
+  if (_cell.is_highlighted && [self isEnabled])
     {
       return [NSColor selectedMenuItemTextColor];
     }
@@ -108,8 +105,9 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
 
 - (NSColor *) backgroundColor
 {
-  NSColor *backgroundColor = nil;
-  unsigned  mask;
+  unsigned	mask;
+  NSColor	*color;
+  GSThemeControlState state = GSThemeNormalState;
 
   if (_cell.is_highlighted)
     {
@@ -126,14 +124,28 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
   // Determine the background color
   if (mask & (NSChangeGrayCellMask | NSChangeBackgroundCellMask))
     {
-      backgroundColor = [NSColor selectedMenuItemColor];
+      state = GSThemeHighlightedState;
     }
 
-  // TODO: Should we also check the ivar _backgroundColor?
-  if (backgroundColor == nil)
-    backgroundColor = [NSColor controlBackgroundColor];
-  
-  return backgroundColor;
+  if (mask & NSPushInCellMask)
+    {
+      state = GSThemeSelectedState;
+    }
+
+  color = [[GSTheme theme] colorNamed: @"NSMenuItem" state: state];
+  if (color == nil)
+    {
+      if ((state == GSThemeHighlightedState) || (state == GSThemeSelectedState))
+	{
+	  color = [NSColor selectedMenuItemColor];
+	}
+      else
+	{
+	  color = [NSColor controlBackgroundColor];
+	}
+    }
+
+  return color;
 }
 
 - (void) setMenuItem: (NSMenuItem *)item
@@ -296,7 +308,16 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
   // Submenu Arrow
   if ([_menuItem hasSubmenu])
     {
-      componentSize = [arrowImage size];
+      NSImage	*arrow = [NSImage imageNamed: @"NSMenuArrow"];
+
+      if (arrow != nil)
+	{
+          componentSize = [arrow size];
+	}
+      else
+	{
+	  componentSize = NSMakeSize(0, 0);
+	}
       _keyEquivalentWidth = componentSize.width;
       if (componentSize.height > neededMenuItemHeight)
         neededMenuItemHeight = componentSize.height;
@@ -545,6 +566,9 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
 
 - (NSRect) drawingRectForBounds: (NSRect)theRect
 {
+  if (_needs_sizing)
+    [self calcSize];
+
   if ([_menuView isHorizontal] == YES)
     {
       /* A horizontal menu does not have borders drawn by the cell,
@@ -603,31 +627,45 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
 // Drawing.
 //
 - (void) drawBorderAndBackgroundWithFrame: (NSRect)cellFrame
-                                  inView: (NSView *)controlView
+                                   inView: (NSView *)controlView
 {
-  if ([_menuView isHorizontal] == YES)
+  unsigned mask;
+  GSThemeControlState state = GSThemeNormalState;
+
+  // set the mask
+  if (_cell.is_highlighted)
     {
-      cellFrame = [self drawingRectForBounds: cellFrame];
-      [[self backgroundColor] set];
-      NSRectFill(cellFrame);
-      return;
-    }
-
-  // Set cell's background color
-  [[self backgroundColor] set];
-  NSRectFill(cellFrame);
-
-  if (!_cell.is_bordered)
-    return;
-
-  if (_cell.is_highlighted && (_highlightsByMask & NSPushInCellMask))
-    {
-      [[GSTheme theme] drawGrayBezel: cellFrame withClip: NSZeroRect];
+      mask = _highlightsByMask;
+      if (_cell.state)
+        {
+          mask &= ~_showAltStateMask;
+        }
     }
+  else if (_cell.state)
+    mask = _showAltStateMask;
   else
+    mask = NSNoCellMask;
+
+  /* Determine the background color. 
+     We draw when there is a border or when highlightsByMask
+     is NSChangeBackgroundCellMask or NSChangeGrayCellMask,
+     as required by our nextstep-like look and feel.  */
+  if (mask & (NSChangeGrayCellMask | NSChangeBackgroundCellMask))
     {
-      [[GSTheme theme] drawButton: cellFrame withClip: NSZeroRect];
+      state = GSThemeHighlightedState;
     }
+
+  /* Pushed in buttons contents are displaced to the bottom right 1px.  */
+  if (mask & NSPushInCellMask)
+    {
+      state = GSThemeSelectedState;
+    }
+
+  [[GSTheme theme] drawBorderAndBackgroundForMenuItemCell: self
+                   withFrame: cellFrame
+                   inView: controlView
+                   state: state
+                   isHorizontal: [_menuView isHorizontal]];
 }
 
 - (void) drawImageWithFrame: (NSRect)cellFrame
@@ -637,17 +675,19 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
   [self drawImage: _imageToDisplay withFrame: cellFrame inView: controlView];
 }
 
-- (void) drawKeyEquivalentWithFrame:(NSRect)cellFrame
-                            inView:(NSView *)controlView
+- (void) drawKeyEquivalentWithFrame: (NSRect)cellFrame
+			     inView: (NSView *)controlView
 {
+  NSImage	*arrow = [NSImage imageNamed: @"NSMenuArrow"];
+
   cellFrame = [self keyEquivalentRectForBounds: cellFrame];
 
-  if ([_menuItem hasSubmenu] && arrowImage != nil)
+  if ([_menuItem hasSubmenu] && arrow != nil)
     {
       NSSize size;
       NSPoint position;
 
-      size = [arrowImage size];
+      size = [arrow size];
       position.x = cellFrame.origin.x + cellFrame.size.width - size.width;
       position.y = MAX(NSMidY(cellFrame) - (size.height/2.), 0.);
       /*
@@ -657,7 +697,7 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
       if ([controlView isFlipped])
         position.y += size.height;
 
-      [arrowImage compositeToPoint: position operation: NSCompositeSourceOver];
+      [arrow compositeToPoint: position operation: NSCompositeSourceOver];
     }
   /* FIXME/TODO here - decide a consistent policy for images.
    *
@@ -686,8 +726,6 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
 - (void) drawStateImageWithFrame: (NSRect)cellFrame
                           inView: (NSView*)controlView
 {
-  NSSize size;
-  NSPoint position;
   NSImage *imageToDisplay;
 
   switch ([_menuItem state])
@@ -712,51 +750,14 @@ static NSImage *arrowImage = nil; /* Cache arrow image. */
     }
   
   cellFrame = [self stateImageRectForBounds: cellFrame];
-  // FIXME: We could call drawImage:withFrame:inView: here, if we knew that the code there is correct.
-  size = [imageToDisplay size];
-  position.x = MAX(NSMidX(cellFrame) - (size.width/2.),0.);
-  position.y = MAX(NSMidY(cellFrame) - (size.height/2.),0.);
-  /*
-   * Images are always drawn with their bottom-left corner at the origin
-   * so we must adjust the position to take account of a flipped view.
-   */
-  if ([controlView isFlipped])
-    {
-      position.y += size.height;
-    }
-  
-  [imageToDisplay compositeToPoint: position operation: NSCompositeSourceOver];
+  [self drawImage: imageToDisplay withFrame: cellFrame inView: controlView];
 }
 
-- (void) drawTitleWithFrame:(NSRect)cellFrame
-                    inView:(NSView *)controlView
+- (void) drawTitleWithFrame: (NSRect)cellFrame
+                     inView: (NSView *)controlView
 {
-  if ([_menuView isHorizontal] == YES)
-    {
-      // FIXME: Why do we have separate drawing coce here. The alignement already
-      // gets adjusted when the menu view is set and the text colour looks wrong here.
-      id value = [NSMutableParagraphStyle defaultParagraphStyle];
-      NSDictionary *attr;
-      NSRect cf = [self titleRectForBounds: cellFrame];
-
-      if (!_imageWidth)
-        [value setAlignment: NSCenterTextAlignment];
-
-      attr = [[NSDictionary alloc] initWithObjectsAndKeys:
-        value, NSParagraphStyleAttributeName,
-        _font, NSFontAttributeName,
-        [NSColor controlTextColor], NSForegroundColorAttributeName,
-        nil];
-
-      [[_menuItem title] drawInRect: cf withAttributes: attr];
-
-      RELEASE(attr);
-    }
-  else
-    {
-      [self _drawText: [_menuItem title]
-              inFrame: [self titleRectForBounds: cellFrame]];
-    }
+  [self _drawText: [_menuItem title]
+        inFrame: [self titleRectForBounds: cellFrame]];
 }
 
 - (void) _drawBorderAndBackgroundWithFrame: (NSRect)cellFrame inView: (NSView*)controlView
