@@ -28,23 +28,32 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSDebug.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSDate.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
 
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSMenuView.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSButton.h"
-#include "AppKit/NSPopUpButtonCell.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/PSOperators.h"
-#include "AppKit/NSImage.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSButton.h"
+#import "AppKit/NSPopUpButtonCell.h"
+#import "AppKit/NSScreen.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/PSOperators.h"
 
-#include "GNUstepGUI/GSTitleView.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GNUstepGUI/GSTitleView.h"
 
-#include <Foundation/Foundation.h>
 
 typedef struct _GSCellRect {
   NSRect rect;
@@ -59,12 +68,14 @@ typedef struct _GSCellRect {
 #ifdef GSIArray
 #undef GSIArray
 #endif
-#include <GNUstepBase/GSIArray.h>
+#import <GNUstepBase/GSIArray.h>
 
 static NSMapTable *viewInfo = 0;
 
 #define cellRects ((GSIArray)NSMapGet(viewInfo, self))
 
+#define HORIZONTAL_MENU_LEFT_PADDING 8
+
 /*
   NSMenuView contains:
 
@@ -598,7 +609,8 @@ static NSMapTable *viewInfo = 0;
 
   if (needTitleView == YES && _titleView == nil)
     {
-      _titleView = [[GSTitleView alloc] initWithOwner: _attachedMenu];
+      Class titleViewClass = [[GSTheme theme] titleViewClassForMenuView: self];
+      _titleView = [[titleViewClass alloc] initWithOwner: _attachedMenu];
       [self addSubview: _titleView];
       RELEASE(_titleView);
     }
@@ -641,7 +653,7 @@ static NSMapTable *viewInfo = 0;
     {
       unsigned i;
       unsigned howMany = [_itemCells count];
-      float currentX = 8;
+      float currentX = HORIZONTAL_MENU_LEFT_PADDING;
 //      NSRect scRect = [[NSScreen mainScreen] frame];
 
       GSIArrayRemoveAllItems(cellRects);
@@ -952,6 +964,14 @@ static NSMapTable *viewInfo = 0;
       aRect.origin.x -= _leftBorderOffset;
       aRect.size.width +=  _leftBorderOffset;
 
+      // For horizontal menus, clicking in the left padding should be treated
+      // as hitting the first menu item.
+      if (_horizontal == YES && i == 0)
+        {
+          aRect.origin.x -= HORIZONTAL_MENU_LEFT_PADDING;
+          aRect.size.width += HORIZONTAL_MENU_LEFT_PADDING;
+        }
+
       if (NSMouseInRect(point, aRect, NO))
         return (int)i;
     }
@@ -1204,30 +1224,19 @@ static NSMapTable *viewInfo = 0;
  */
 - (BOOL) isOpaque
 {
-  return YES;
+  return NO;
 }
 
 - (void) drawRect: (NSRect)rect
 {
   int        i;
   int        howMany = [_itemCells count];
-  NSRectEdge sides[2]; 
-  float      grays[] = {NSDarkGray, NSDarkGray};
 
-  if (_horizontal == YES)
-    {
-      sides[0] = NSMinYEdge;
-      sides[1] = NSMinYEdge;
-      NSDrawTiledRects(_bounds, rect, sides, grays, 2);
-    }
-  else
-    {
-      sides[0] = NSMinXEdge;
-      sides[1] = NSMaxYEdge;
-      // Draw the dark gray upper left lines.
-      NSDrawTiledRects(_bounds, rect, sides, grays, 2);
-    }
-  
+  [[GSTheme theme] drawBackgroundForMenuView: self
+                                   withFrame: _bounds
+                                   dirtyRect: rect
+                                  horizontal: _horizontal];
+ 
   // Draw the menu cells.
   for (i = 0; i < howMany; i++)
     {
