@@ -7,10 +7,10 @@
    Author: Scott Christley <scottc@net-community.com>
    Date: 1996
 
-   Author: Daniel Bhringer <boehring@biomed.ruhr-uni-bochum.de>
+   Author: Daniel Boehringer <boehring@biomed.ruhr-uni-bochum.de>
    Date: August 1998
 
-   Source by Daniel Bhringer integrated into Scott Christley's preliminary
+   Source by Daniel Boehringer integrated into Scott Christley's preliminary
    implementation by Felipe A. Rodriguez <far@ix.netcom.com>
 
    Author: Nicola Pero <n.pero@mi.flashnet.it>
@@ -38,19 +38,22 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSString.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSURL.h>
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSBrowser.h"
-#include "AppKit/NSBrowserCell.h"
-#include "AppKit/NSButton.h"
-#include "AppKit/NSForm.h"
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSOpenPanel.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSURL.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSBrowser.h"
+#import "AppKit/NSBrowserCell.h"
+#import "AppKit/NSButton.h"
+#import "AppKit/NSForm.h"
+#import "AppKit/NSMatrix.h"
+#import "AppKit/NSOpenPanel.h"
+
+#include "GSGuiPrivate.h"
+#include "GNUstepGUI/GSTheme.h"
 
 static NSString	*
 pathToColumn(NSBrowser *browser, int column)
@@ -87,11 +90,11 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 - (void) _resetDefaults
 {
   [super _resetDefaults];
-  [self setTitle: @"Open"];
+  [self setTitle: _(@"Open")];
   [self setCanChooseFiles: YES];
   [self setCanChooseDirectories: YES];
   [self setAllowsMultipleSelection: NO];
-  [_okButton setEnabled: YES];
+  [_okButton setEnabled: NO];
 }
 
 - (BOOL) _shouldShowExtension: (NSString *)extension
@@ -124,17 +127,12 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 	     [[matrix selectedCell] isLeaf] == YES)
 	    [super _selectTextInColumn: column];
 	  else
-	    {
-	      [self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
-	      [_okButton setEnabled: 
-			   [self _shouldShowExtension:
-				   [[_browser path] pathExtension]]]; 
-	    }
+	    [self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
 	}
       else
 	{
 	  [_form abortEditing];
-	  [[_form cellAtIndex: 0] setStringValue:@""];
+	  [[_form cellAtIndex: 0] setStringValue: @""];
 	  [_form setNeedsDisplay: YES];
 	  [_okButton setEnabled: YES];
 	}
@@ -144,17 +142,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
       if (_canChooseDirectories == NO || [[matrix selectedCell] isLeaf] == YES)
 	[super _selectTextInColumn: column];
       else
-	{
-	  if ([[[_form cellAtIndex: 0] stringValue] length] > 0)
-	    {
-	      [self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
-	      [_form setNeedsDisplay: YES];
-	    }
-
-	  [_okButton setEnabled: 
-		       [self _shouldShowExtension:
-			       [[_browser path] pathExtension]]]; 
-	}
+	[self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
     }
 }
 
@@ -174,7 +162,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   titleLength = [title length];
   if (!titleLength)
     {
-      [_okButton setEnabled: NO];
+      [_okButton setEnabled: _canChooseDirectories];
       return;
     }
 
@@ -205,6 +193,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
       else if (result == NSOrderedDescending)
 	break;
     }
+  [_okButton setEnabled: NO];
 }
 
 - (void) _setupForDirectory: (NSString *)path file: (NSString *)filename
@@ -274,8 +263,10 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 + (NSOpenPanel *) openPanel
 {
   if (!_gs_gui_open_panel)
-    _gs_gui_open_panel = [[NSOpenPanel alloc] init];
-
+    {
+      Class openPanelClass = [[GSTheme theme] openPanelClass];
+      _gs_gui_open_panel = [[openPanelClass alloc] init];
+    }
   [_gs_gui_open_panel _resetDefaults];
 
   return _gs_gui_open_panel;
@@ -326,8 +317,20 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 */
 - (void) setCanChooseDirectories: (BOOL)flag
 {
-  _canChooseDirectories = flag;
-  [_browser setAllowsBranchSelection: flag];
+  if (flag != _canChooseDirectories)
+    {
+      _canChooseDirectories = flag;
+      [_browser setAllowsBranchSelection: flag];
+      if (!flag)
+	{
+	  /* FIXME If the user disables directory selection we should deselect
+	     any directories that are currently selected. This is achieved by
+	     calling _reloadBrowser, but this may be considered overkill, since
+	     the displayed files are the same whether canChooseDirectories is
+	     enabled or not. */
+	  [self _reloadBrowser];
+	}
+    }
 }
 
 /** <p>Returns YES if the user is allowed to choose directories  The
@@ -516,12 +519,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 	}
       else
 	{
-	  if (_canChooseDirectories == NO)
-	    {
-	      if (selectedColumn == lastColumn)
-		selectedCell = [matrix selectedCell];
-	    }
-	  else if (selectedColumn == lastColumn)
+	  if (selectedColumn == lastColumn)
 	    selectedCell = [matrix selectedCell];
 	}
     }
@@ -539,8 +537,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 	}
     }
   else if (_canChooseDirectories == NO
-    && (![_browser allowsMultipleSelection] || !selectedCells
-	 || selectedColumn != lastColumn || ![selectedCells count]))
+    && (selectedColumn != lastColumn || ![selectedCells count]))
     {
       [_form selectTextAtIndex: 0];
       [_form setNeedsDisplay: YES];
@@ -653,8 +650,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   if (sLength == 0)
     {
       [matrix deselectAllCells];
-      if (_canChooseDirectories == NO)
-	[_okButton setEnabled: NO];
+      [_okButton setEnabled: _canChooseDirectories];
       return;
     }
 
@@ -732,7 +728,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
     }
 
   [matrix deselectAllCells];
-  [_okButton setEnabled: YES];
+  [_okButton setEnabled: NO];
 }
 
 @end /* NSOpenPanel */
