@@ -29,32 +29,33 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSNumberFormatter.h>
-#include <Foundation/NSDecimalNumber.h>
-#include <Foundation/NSUserDefaults.h>
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSTextField.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSImageView.h"
-#include "AppKit/NSBezierPath.h"
-#include "AppKit/NSBox.h"
-#include "AppKit/NSButton.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSComboBox.h"
-#include "AppKit/NSPopUpButton.h"
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSNibLoading.h"
-#include "AppKit/NSForm.h"
-#include "AppKit/NSFormCell.h"
-#include "AppKit/NSPrintInfo.h"
-#include "AppKit/NSPageLayout.h"
-#include "AppKit/NSTableView.h"
-#include "AppKit/NSTabView.h"
-#include "AppKit/NSTabViewItem.h"
-#include "AppKit/NSPrinter.h"
-#include "GSGuiPrivate.h"
-#include "GNUstepGUI/GSPrinting.h"
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSNumberFormatter.h>
+#import <Foundation/NSDecimalNumber.h>
+#import <Foundation/NSUserDefaults.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSTextField.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSImageView.h"
+#import "AppKit/NSBezierPath.h"
+#import "AppKit/NSBox.h"
+#import "AppKit/NSButton.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSComboBox.h"
+#import "AppKit/NSPopUpButton.h"
+#import "AppKit/NSMatrix.h"
+#import "AppKit/NSNibLoading.h"
+#import "AppKit/NSForm.h"
+#import "AppKit/NSFormCell.h"
+#import "AppKit/NSPrintInfo.h"
+#import "AppKit/NSPageLayout.h"
+#import "AppKit/NSTableView.h"
+#import "AppKit/NSTabView.h"
+#import "AppKit/NSTabViewItem.h"
+#import "AppKit/NSPrinter.h"
+#import "GSGuiPrivate.h"
+#import "GNUstepGUI/GSPrinting.h"
 
 static NSPageLayout *shared_instance;
 
@@ -404,25 +405,14 @@ enum {
 @implementation GSPageLayoutController
 - (id) init
 {
-  NSString *panelPath;
-  NSDictionary *table;
   NSImage *image;
   NSNumberFormatter *sizeFormatter;
   NSNumberFormatter *scaleFormatter;
 
-  self = [super init];
+  if ((self = [super init]) == nil)
+    return nil;
   
-  panelPath = [GSGuiBundle() pathForResource: @"GSPageLayout" 
-                                      ofType: @"gorm"
-                                 inDirectory: nil];
-                                 
-  NSLog(@"Panel path=%@",panelPath);
-  table = [NSDictionary dictionaryWithObject: self 
-                                      forKey: @"NSOwner"];
-                                      
-  if ([NSBundle loadNibFile: panelPath 
-          externalNameTable: table
-                   withZone: [self zone]] == NO)
+  if ([NSBundle loadNibNamed: @"GSPageLayout" owner: self] == NO)
     {
       NSRunAlertPanel(@"Error", @"Could not load page layout panel resource", 
                       @"OK", NULL, NULL);
@@ -751,9 +741,9 @@ enum {
   printerNames = [NSPrinter printerNames];
   if ([printerNames count] == 0)  //NO PRINTERS
     {
-      [printerPopUp addItemWithTitle: @"(none)"];
+      [printerPopUp addItemWithTitle: _(@"(none)")];
       [printerPopUp setEnabled: NO];
-      [standardPaperSizePopUp addItemWithTitle: @"(none)"];
+      [standardPaperSizePopUp addItemWithTitle: _(@"(none)")];
       [standardPaperSizePopUp setEnabled: NO];
       radioButton = [paperRadioMatrix cellAtRow: 0
                                          column: 0];
@@ -978,15 +968,15 @@ enum {
         {
           switch(index)
             {
-              case 0:  return @"Name";
-              case 1:  return @"Dimensions";
-              case 2:  return @"Orientation";
-              case 3:  return @"Scale";
-              case 4:  return @"Top Margin";
-              case 5:  return @"Bottom Margin";
-              case 6:  return @"Left Margin";
-              case 7:  return @"Right Margin";
-              default: return @"Unknown";
+              case 0:  return _(@"Name");
+              case 1:  return _(@"Dimensions");
+              case 2:  return _(@"Orientation");
+              case 3:  return _(@"Scale");
+              case 4:  return _(@"Top Margin");
+              case 5:  return _(@"Bottom Margin");
+              case 6:  return _(@"Left Margin");
+              case 7:  return _(@"Right Margin");
+              default: return _(@"Unknown");
             }
         }
       else //The value column
@@ -1035,11 +1025,11 @@ enum {
               case 2:
                 if ([paperOrientationMatrix selectedColumn] == 0)
                   {
-                    return @"Portrait";
+                    return _(@"Portrait");
                   }
                 else
                   {
-                    return @"Landscape" ;
+                    return _(@"Landscape") ;
                   }
               case 3:
                 return [NSString stringWithFormat: @"%@%%",
@@ -1057,7 +1047,7 @@ enum {
                 return [NSString stringWithFormat: @"%.2f %@", rightMargin,
                         measurementString];
               default: 
-                return @"Unknown";
+                return _(@"Unknown");
             }
         }
     }
