@@ -548,7 +548,7 @@
 #include <Foundation/NSRunLoop.h>
 #include <Foundation/NSSet.h>
 #include <Foundation/NSTask.h>
-#include <GNUstepBase/NSTask+GS.h>
+#include <GNUstepBase/NSTask+GNUstepBase.h>
 #include <Foundation/NSTimer.h>
 
 #include "GNUstepGUI/GSServicesManager.h"
@@ -970,19 +970,16 @@ static NSString	*namePrefix = @"NSTypedFilenamesPboardType:";
        */
       NS_DURING
 	{
-	  const char		*cName;
 	  SEL			sel;
 	  NSMethodSignature	*sig;
 
-	  cName = [selName UTF8String];
-	  sel = GSSelectorFromNameAndTypes(cName, 0); 
+	  sel = NSSelectorFromString(selName);
 	  sig = [provider methodSignatureForSelector: sel];
 	  if (sig != nil)
 	    {
 	      NSInvocation	*inv;
 	      NSString		**errPtr = &error;
 
-	      sel = GSSelectorFromNameAndTypes(cName, [sig methodType]); 
 	      inv = [NSInvocation invocationWithMethodSignature: sig];
 	      [inv setTarget: provider];
 	      [inv setSelector: sel];
@@ -1493,12 +1490,12 @@ static  NSMapTable              *mimeMap = NULL;
   if ([self retainCount] == 2)
     {
       [dictionary_lock lock];
-      RETAIN(super);
+      [super retain];
       [pasteboards removeObjectForKey: name];
-      RELEASE(super);
+      [super release];
       [dictionary_lock unlock];
     }
-  RELEASE(super);
+  [super release];
 }
 
 /**
@@ -1533,6 +1530,12 @@ static  NSMapTable              *mimeMap = NULL;
   return ok;
 }
 
+- (BOOL)writeObjects: (NSArray*)objects
+{
+  // FIXME: not implemented
+  return NO;
+}
+
 /**
  * <p>Serialises the data in the supplied property list and writes it to the
  * pasteboard server using the -setData:forType: method.
