@@ -28,16 +28,19 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/Foundation.h>
-#include <Foundation/NSKeyValueObserving.h>
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSKeyValueBinding.h"
-#include "AppKit/NSPopUpButton.h"
-#include "AppKit/NSPopUpButtonCell.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSMenuItem.h"
-#include "AppKit/NSMenuView.h"
+#import <Foundation/NSKeyValueCoding.h>
+#import <Foundation/NSKeyValueObserving.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSKeyValueBinding.h"
+#import "AppKit/NSPopUpButton.h"
+#import "AppKit/NSPopUpButtonCell.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSMenuItem.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSWindow.h"
 
 /*
  * class variables
@@ -251,6 +254,14 @@ this to return nil to indicate that we have no context menu.
   return [_cell indexOfSelectedItem];
 }
 
+/**<p>Returns the tag of the selected item</p>
+   <p>See Also: [NSPopUpButtonCell-indexOfSelectedItem]</p>
+*/
+- (int) selectedTag
+{
+  return [[_cell selectedItem] tag];
+}
+
 - (void) selectItem: (id <NSMenuItem>)anObject
 {
   [self willChangeValueForKey: NSSelectedIndexBinding];
