@@ -26,17 +26,19 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSMenuView.h"
-#include "AppKit/NSPopUpButton.h"
-#include "AppKit/NSPopUpButtonCell.h"
-#include "AppKit/NSWindow.h"
-#include "GNUstepGUI/GSTheme.h"
+#import "config.h"
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSPopUpButton.h"
+#import "AppKit/NSPopUpButtonCell.h"
+#import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSTheme.h"
 
 /* The image to use in a specific popupbutton depends on type and
  * preferred edge; that is, _pbc_image[0] if it is a
@@ -547,6 +549,7 @@ static NSImage *_pbc_image[5];
       int i = [object intValue];
         
       [self selectItemAtIndex: i];
+      [self synchronizeTitleAndSelectedItem];
     }
 }
 
@@ -616,6 +619,8 @@ static NSImage *_pbc_image[5];
 
 - (void) selectItem: (id <NSMenuItem>)item
 {
+  id<NSMenuItem> oldSelectedItem = _selectedItem;
+
   if (_selectedItem == item)
     return;
 
@@ -640,6 +645,9 @@ static NSImage *_pbc_image[5];
   /* Set the item in the menu */
   [[_menu menuRepresentation] setHighlightedItemIndex: 
                    [_menu indexOfItem: _selectedItem]];
+
+  [[_menu menuRepresentation] setNeedsDisplayForItemAtIndex:
+                   [_menu indexOfItem: oldSelectedItem]];
 }
 
 - (void) selectItemAtIndex: (int)index
