@@ -33,6 +33,7 @@
    Boston, MA 02110-1301, USA.
 */ 
 
+#include <limits.h>
 #include <math.h>
 #include "config.h"
 #include <Foundation/NSString.h>
@@ -59,6 +60,7 @@
 #include "GNUstepGUI/GSEPSPrintOperation.h"
 #include "GNUstepGUI/GSPDFPrintOperation.h"
 #include "GNUstepGUI/GSPrintOperation.h"
+#include "GSGuiPrivate.h"
 
 #define NSNUMBER(a) [NSNumber numberWithInt: (a)]
 #define NSFNUMBER(a) [NSNumber numberWithFloat: (a)]
@@ -675,8 +677,8 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
     {
       [_view _cleanupPrinting];
       [NSGraphicsContext setCurrentContext: oldContext];
-      NSRunAlertPanel(@"Error", @"Printing error: %@", 
-                      @"OK", NULL, NULL, localException);
+      NSRunAlertPanel(_(@"Error"), _(@"Printing error: %@"), 
+                      _(@"OK"), NULL, NULL, localException);
     }
   NS_ENDHANDLER
   [self destroyContext];
@@ -760,11 +762,18 @@ scaleRect(NSRect rect, double scale)
 - (void) _printPaginateWithInfo: (page_info_t *)info knowsRange: (BOOL)knowsRange
 {
   NSMutableDictionary *dict;
+  NSNumber *value;
+
   dict = [_print_info dictionary];
 
   info->paperSize = [_print_info paperSize];
   info->orient = [_print_info orientation];
-  info->printScale = [[dict objectForKey: NSPrintScalingFactor] doubleValue];
+  value = [dict objectForKey: NSPrintScalingFactor];
+  if (value != nil)
+    info->printScale = [value doubleValue];
+  else
+    info->printScale = 1.0;
+
   info->nup = [[dict objectForKey: NSPrintPagesPerSheet] intValue];
   info->nupScale = 1;
   if (info->nup < 1 || (info->nup > 1 && (((info->nup) & 0x1) == 1)))
@@ -939,6 +948,7 @@ scaleRect(NSRect rect, double scale)
   BOOL knowsPageRange, allPages;
   NSRange viewPageRange;
   NSMutableDictionary *dict;
+  NSNumber *value;
   page_info_t info;
   
   dict = [_print_info dictionary];
@@ -959,7 +969,11 @@ scaleRect(NSRect rect, double scale)
 
   [dict setObject: NSNUMBER(NSMaxRange(viewPageRange))
            forKey: @"NSPrintTotalPages"];
-  allPages = [[dict objectForKey: NSPrintAllPages] boolValue];
+  value = [dict objectForKey: NSPrintAllPages];
+  if (value != nil)
+    allPages = [value boolValue];
+  else
+    allPages = YES;
   if (allPages == YES)
     {
       info.first = viewPageRange.location;
@@ -967,8 +981,18 @@ scaleRect(NSRect rect, double scale)
     }
   else
     {
-      info.first = [[dict objectForKey: NSPrintFirstPage] intValue];
-      info.last  = [[dict objectForKey: NSPrintLastPage] intValue];
+      value = [dict objectForKey: NSPrintFirstPage];
+      if (value != nil)
+	info.first = [value intValue];
+      else 
+	info.first = 1;
+
+      value = [dict objectForKey: NSPrintLastPage];
+      if (value != nil)
+	info.last  = [value intValue];
+      else
+	info.last  = INT_MAX;
+
       info.first = MAX(info.first, (int)viewPageRange.location);
       info.first = MIN(info.first, (int)(NSMaxRange(viewPageRange) - 1));
       info.last = MAX(info.last, info.first);
