@@ -29,23 +29,25 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include <Foundation/NSArray.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSForm.h"
-#include "AppKit/NSNibLoading.h"
-#include "AppKit/NSPrinter.h"
-#include "AppKit/NSPrintPanel.h"
-#include "AppKit/NSPrintInfo.h"
-#include "AppKit/NSPrintOperation.h"
-#include "AppKit/NSPopUpButton.h"
-#include "AppKit/NSSavePanel.h"
-#include "AppKit/NSView.h"
-#include "GSGuiPrivate.h"
-#include "GNUstepGUI/GSPrinting.h"
+#import "config.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSForm.h"
+#import "AppKit/NSNib.h"
+#import "AppKit/NSNibLoading.h"
+#import "AppKit/NSPrinter.h"
+#import "AppKit/NSPrintPanel.h"
+#import "AppKit/NSPrintInfo.h"
+#import "AppKit/NSPrintOperation.h"
+#import "AppKit/NSPopUpButton.h"
+#import "AppKit/NSSavePanel.h"
+#import "AppKit/NSView.h"
+#import "GSGuiPrivate.h"
+#import "GNUstepGUI/GSPrinting.h"
 
 static NSPrintPanel *shared_instance;
 
@@ -137,15 +139,16 @@ static NSPrintPanel *shared_instance;
   if (self == nil)
     return nil;
 
-  panel = [GSGuiBundle() pathForResource: GSPANELNAME ofType: @"gorm"
-		      inDirectory: nil];
+  // self will come from a bundle, to get the panel from the GUI library 
+  // we have to select that bundle explicitly
+  panel = [GSGuiBundle() pathForNibResource: GSPANELNAME];
   if (panel == nil)
     {
       NSRunAlertPanel(@"Error", @"Could not find print panel resource", 
 		      @"OK", NULL, NULL);
       return nil;
     }
-  table = [NSDictionary dictionaryWithObject: self forKey: @"NSOwner"];
+  table = [NSDictionary dictionaryWithObject: self forKey: NSNibOwner];
   if ([NSBundle loadNibFile: panel 
 	  externalNameTable: table
 		withZone: [self zone]] == NO)
@@ -155,6 +158,9 @@ static NSPrintPanel *shared_instance;
       return nil;
     }
 
+  /* Set the title */
+  [self setTitle: _(@"Print Panel")];
+
   /* Transfer the objects to us. FIXME: There must be a way to 
      instantiate the panel directly */
   subviews = [[_panel contentView] subviews];
@@ -166,8 +172,8 @@ static NSPrintPanel *shared_instance;
 
   /* Setup the layout popup */
   control = CONTROL(self, NSPPLayoutButton);
-  list = [NSArray arrayWithObjects: @"1 up", @"2 up", @"4 up", @"6 up", 
-		  @"8 up", nil];
+  list = [NSArray arrayWithObjects: _(@"1 up"), _(@"2 up"), _(@"4 up"), _(@"6 up"), 
+		  _(@"8 up"), nil];
   [control removeAllItems];
   for (i = 0; i < [list count]; i++)
     {
@@ -281,8 +287,8 @@ static NSPrintPanel *shared_instance;
   else if (tag ==NSFaxButton)
     {
       _picked = NSFaxButton;
-      NSRunAlertPanel(@"Sorry", @"Faxing of print file not implemented", 
-		      @"OK", NULL, NULL);
+      NSRunAlertPanel(_(@"Sorry"), _(@"Faxing of print file not implemented"), 
+		      _(@"OK"), NULL, NULL);
       /* Don't stop the modal session */
       return;
     }
@@ -297,7 +303,6 @@ static NSPrintPanel *shared_instance;
   else if (tag == NSPPOptionsButton)
     {
       /* Open the options panel */
-      NSLog(@"Running _optionPanel modal");
       [NSApp runModalForWindow: _optionPanel];
       [_optionPanel orderOut: self];
       return;
@@ -452,7 +457,7 @@ static NSPrintPanel *shared_instance;
 	}
     }
   else
-    [control addItemWithTitle: @"Unknown"];
+    [control addItemWithTitle: _(@"Unknown")];
 
   /* Setup the paper feed popup */
   control = CONTROL(_optionPanel, NSPPPaperFeedButton);
@@ -465,7 +470,7 @@ static NSPrintPanel *shared_instance;
 
       manual = [printer stringForKey:@"DefaultManualFeed" inTable: @"PPD"];
       if (manual)
-	[control addItemWithTitle: @"Manual"];
+	[control addItemWithTitle: _(@"Manual")];
       list = [printer stringListForKey:@"InputSlot" inTable: @"PPD"];
       if ([list count])
 	{
@@ -498,7 +503,7 @@ static NSPrintPanel *shared_instance;
 	}
     }
   else
-    [control addItemWithTitle: @"Unknown"];
+    [control addItemWithTitle: _(@"Unknown")];
 
 }
 
@@ -600,14 +605,14 @@ static NSPrintPanel *shared_instance;
       if (manual)
 	{
 	  if (selected == 0)
-	    sel = @"Manual";
+	    sel = _(@"Manual");
 	  else
 	    selected--;
 	}
       if (sel == nil)
 	sel = [list objectAtIndex: selected];
       def = [printer stringForKey:@"DefaultInputSlot" inTable: @"PPD"];
-      if ([sel isEqual: @"Manual"] == YES)
+      if ([sel isEqual: _(@"Manual")] == YES)
 	{
 	  [dict setObject: NSPrintManualFeed forKey: NSPrintPaperFeed];
 	  /* FIXME: This needs to be more robust. I'm just assuming
