@@ -32,31 +32,32 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSAutoreleasePool.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSURL.h>
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSBox.h"
-#include "AppKit/NSBrowser.h"
-#include "AppKit/NSBrowserCell.h"
-#include "AppKit/NSButton.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSForm.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSImageView.h"
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSPasteboard.h"
-#include "AppKit/NSDragging.h"
-#include "AppKit/NSSavePanel.h"
-#include "AppKit/NSTextField.h"
-#include "AppKit/NSWorkspace.h"
+#import <Foundation/NSAutoreleasePool.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSURL.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSBox.h"
+#import "AppKit/NSBrowser.h"
+#import "AppKit/NSBrowserCell.h"
+#import "AppKit/NSButton.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSForm.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSImageView.h"
+#import "AppKit/NSMatrix.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSDragging.h"
+#import "AppKit/NSSavePanel.h"
+#import "AppKit/NSTextField.h"
+#import "AppKit/NSWorkspace.h"
 
 #include "GSGuiPrivate.h"
+#include "GNUstepGUI/GSTheme.h"
 
 #define _SAVE_PANEL_X_PAD	5
 #define _SAVE_PANEL_Y_PAD	4
@@ -117,6 +118,7 @@ setPath(NSBrowser *browser, NSString *path)
 - (void) _unmountMedia;
 - (void) _selectTextInColumn: (int)column;
 - (void) _selectCellName: (NSString *)title;
+- (void) _setFileName: (NSString *)name;
 - (void) _setupForDirectory: (NSString *)path file: (NSString *)name;
 - (BOOL) _shouldShowExtension: (NSString *)extension;
 - (void) _windowResized: (NSNotification*)n;
@@ -232,13 +234,14 @@ setPath(NSBrowser *browser, NSString *path)
 
   r = NSMakeRect (8, 39, 291, 21);
   _form = [NSForm new];
-  [_form addEntry: @"Name:"];
+  [_form addEntry: _(@"Name:")];
   [_form setFrame: r];
   // Force the size we want
   [_form setCellSize: NSMakeSize (291, 21)];
   [_form setEntryWidth: 291];
   [_form setInterlineSpacing: 0];
   [_form setAutosizesCells: YES];
+  [_form setDrawsBackground: NO];
   [_form setTag: NSFileHandlingPanelForm];
   [_form setAutoresizingMask: NSViewWidthSizable];
   [_form setDelegate: self];
@@ -295,7 +298,7 @@ setPath(NSBrowser *browser, NSString *path)
   r = NSMakeRect (148, 6, 71, 27);
   button = [[NSButton alloc] initWithFrame: r]; 
   [button setBordered: YES];
-  [button setTitle:  @"Cancel"];
+  [button setTitle:  _(@"Cancel")];
   [button setImagePosition: NSNoImage]; 
   [button setTarget: self];
   [button setAction: @selector(cancel:)];
@@ -311,7 +314,7 @@ setPath(NSBrowser *browser, NSString *path)
   r = NSMakeRect (228, 6, 71, 27);
   _okButton = [[NSButton alloc] initWithFrame: r]; 
   [_okButton setBordered: YES];
-  [_okButton setTitle:  @"OK"];
+  [_okButton setTitle:  _(@"OK")];
   [_okButton setImagePosition: NSImageRight]; 
   [_okButton setImage: [NSImage imageNamed: @"common_ret"]];
   [_okButton setAlternateImage: [NSImage imageNamed: @"common_retH"]];
@@ -409,9 +412,10 @@ setPath(NSBrowser *browser, NSString *path)
 - (void) _resetDefaults
 {
   [self _setDefaultDirectory];
-  [self setPrompt: @"Name:"];
-  [self setTitle: @"Save"];
+  [self setPrompt: _(@"Name:")];
+  [self setTitle: _(@"Save")];
   [self setAllowedFileTypes: nil];
+  [self setAllowsOtherFileTypes: NO];
   [self setTreatsFilePackagesAsDirectories: NO];
   [self setDelegate: nil];
   [self setAccessoryView: nil];
@@ -559,6 +563,14 @@ selectCellWithString: (NSString*)title
   return YES;
 }
 
+- (void) _setFileName: (NSString *)filename
+{
+  [self _selectCellName: filename];
+  [[_form cellAtIndex: 0] setStringValue: filename];
+  [_form selectTextAtIndex: 0];
+  [_form setNeedsDisplay: YES];
+}
+
 - (void) _setupForDirectory: (NSString *)path file: (NSString *)filename
 {
   if (path == nil)
@@ -576,13 +588,7 @@ selectCellWithString: (NSString*)title
     filename = @"";
   ASSIGN(_fullFileName, [_directory stringByAppendingPathComponent: filename]);
   setPath(_browser, _fullFileName);
-
-  [self _selectCellName: filename];
-  [_form abortEditing]; // FIXME This is necessary because we cannot set a
-                        // cell's value while it is being edited.
-  [[_form cellAtIndex: 0] setStringValue: filename];
-  [_form selectTextAtIndex: 0];
-  [_form setNeedsDisplay: YES];
+  [self _setFileName: filename];
 
   [self _browser: _browser
 	selectCellWithString: [[_browser selectedCell] stringValue] 
@@ -666,7 +672,8 @@ selectCellWithString: (NSString*)title
 {
   if (_gs_gui_save_panel == nil)
     {
-      _gs_gui_save_panel = [[NSSavePanel alloc] init];
+      Class savePanelClass = [[GSTheme theme] savePanelClass];
+      _gs_gui_save_panel = [[savePanelClass alloc] init]; 
     }
 
   [_gs_gui_save_panel _resetDefaults];
@@ -1079,6 +1086,8 @@ selectCellWithString: (NSString*)title
 - (int) runModalForDirectory: (NSString*)path file: (NSString*)filename
 {
   [self _setupForDirectory: path file: filename];
+  if ([filename length] > 0)
+    [_okButton setEnabled: YES];
   return [NSApp runModalForWindow: self];
 }
 
@@ -1087,6 +1096,8 @@ selectCellWithString: (NSString*)title
 	    relativeToWindow: (NSWindow*)window
 {
   [self _setupForDirectory: path file: filename];
+  if ([filename length] > 0)
+    [_okButton setEnabled: YES];
   return [NSApp runModalForWindow: self
 		relativeToWindow: window];
 }
@@ -1099,6 +1110,8 @@ selectCellWithString: (NSString*)title
 		    contextInfo: (void *)contextInfo
 {
   [self _setupForDirectory: path file: filename];
+  if ([filename length] > 0)
+    [_okButton setEnabled: YES];
   [NSApp beginSheet: self
 	 modalForWindow: docWindow
 	 modalDelegate: delegate
@@ -1118,6 +1131,11 @@ selectCellWithString: (NSString*)title
     return @"";
 }
 
+- (NSURL *) directoryURL
+{
+  return [NSURL fileURLWithPath: [self directory]];
+}
+
 /**<p> Returns the absolute filename choosen by the user.  Do not invoke
    filename within a modal loop because the information that these
    methods fetch is updated only upon return.</p>
@@ -1133,18 +1151,21 @@ selectCellWithString: (NSString*)title
       [_allowedFileTypes indexOfObject: @""] != NSNotFound)
     return _fullFileName;
 
-  /* add filetype extension only if the filename does not include an
-     allowed one already */
+  /* add file type extension if the file name does not have an extension or
+     the file name's extension is not one of the allowed extensions and the
+     save panel does not allow other extensions */
   fileType = [_fullFileName pathExtension];
-  if ([_allowedFileTypes indexOfObject: fileType] != NSNotFound)
-    {
-      return _fullFileName;
-    }
-  else
+  if ([fileType length] == 0 ||
+      ((!_allowsOtherFileTypes &&
+	[_allowedFileTypes indexOfObject: fileType] == NSNotFound)))
     {
       fileType = [_allowedFileTypes objectAtIndex: 0];
       return [_fullFileName stringByAppendingPathExtension: fileType];
     }
+  else
+    {
+      return _fullFileName;
+    }
 }
 
 - (NSURL *) URL
@@ -1208,13 +1229,71 @@ selectCellWithString: (NSString*)title
 	  ASSIGN (_directory, [filename stringByDeletingLastPathComponent]);
 	  ASSIGN (_fullFileName, filename);
 	  setPath(_browser, _fullFileName);
+	  [self _setFileName: [_fullFileName lastPathComponent]];
+	}
+    }
 
-	  filename = [_fullFileName lastPathComponent];
+  /* Warn user if a wrong extension was entered */
+  if (_allowedFileTypes != nil &&
+      [_allowedFileTypes indexOfObject: @""] == NSNotFound)
+    {
+      NSString *fileType = [_fullFileName pathExtension];
+      if ([fileType length] != 0 &&
+	  [_allowedFileTypes indexOfObject: fileType] == NSNotFound)
+	{
+	  int result;
+	  NSString *msgFormat, *butFormat;
+	  NSString *altType, *requiredType;
 
-	  [self _selectCellName: filename];
-	  [[_form cellAtIndex: 0] setStringValue: filename];
-	  [_form selectTextAtIndex: 0];
-	  [_form setNeedsDisplay: YES];
+	  requiredType = [self requiredFileType];
+	  if ([self allowsOtherFileTypes])
+	    {
+	      msgFormat =
+		_(@"You have used the extension '.%@'.\n"
+		  @"The standard extension is '.%@'.'");
+	      butFormat = _(@"Use .%@");
+	      altType = fileType;
+	    }
+	  else
+	    {
+	      msgFormat =
+		_(@"You cannot save this document with extension '.%@'.\n"
+		  @"The required extension is '.%@'.");
+	      butFormat = _(@"Use .%@");
+	      altType = [fileType stringByAppendingPathExtension: requiredType];
+	    }
+
+	  result = NSRunAlertPanel(_(@"Save"),
+		     msgFormat,
+		     [NSString stringWithFormat: butFormat, requiredType],
+		     _(@"Cancel"),
+		     [NSString stringWithFormat: butFormat, altType],
+		     fileType, requiredType);
+	  switch (result)
+	    {
+	    case NSAlertDefaultReturn:
+	      filename = [_fullFileName stringByDeletingPathExtension];
+	      filename =
+		[filename stringByAppendingPathExtension: requiredType];
+
+	      ASSIGN (_fullFileName, filename);
+	      setPath(_browser, _fullFileName);
+	      [self _setFileName: [_fullFileName lastPathComponent]];
+	      break;
+	    case NSAlertOtherReturn:
+	      if (altType != fileType)
+		{
+		  filename =
+		    [_fullFileName stringByAppendingPathExtension: requiredType];
+
+		  ASSIGN (_fullFileName, filename);
+		  setPath(_browser, _fullFileName);
+		  [self _setFileName: [_fullFileName lastPathComponent]];
+		}
+	      break;
+	    default:
+	      return;
+	    }
 	}
     }
 
@@ -1479,15 +1558,10 @@ createRowsForColumn: (int)column
           column: (int)column;
 @end 
 
-struct NSSavePanel_struct
-{
-  @defs (NSSavePanel)
-};
-
 static int compareFilenames (id elem1, id elem2, void *context)
 {
   /* TODO - use IMP optimization here.  */
-  struct NSSavePanel_struct *s = (struct NSSavePanel_struct *)context;
+  NSSavePanel *s = context;
   NSSavePanel *self = (NSSavePanel *)context;
 
   return (int)[s->_delegate panel: self
@@ -1628,7 +1702,7 @@ createRowsForColumn: (int)column
     {
       display_progress = YES;
       base_frac = count / 4;
-      progressString = [@"Reading Directory " stringByAppendingString: path];
+      progressString = [_(@"Reading Directory ") stringByAppendingString: path];
       [super setTitle: progressString];
       // Is the following really safe? 
       [self flushWindow];
@@ -1777,7 +1851,7 @@ createRowsForColumn: (int)column
   NSComparisonResult  result;
   NSRange             range;
 
-  s = [[[aNotification userInfo] objectForKey:@"NSFieldEditor"] string];
+  s = [[[aNotification userInfo] objectForKey: @"NSFieldEditor"] string];
 
   /*
    * If the user typed in an absolute path, display it.
