@@ -33,6 +33,7 @@
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSNotification.h>
+#include <Foundation/NSUserDefaults.h>
 
 #include "AppKit/NSColor.h"
 #include "AppKit/NSColorList.h"
@@ -145,6 +146,8 @@ static float scrollerWidth;
 {
   NSSize size = frameSize;
   NSSize border = [[GSTheme theme] sizeForBorderType: borderType];
+  float innerBorderWidth = [[NSUserDefaults standardUserDefaults]
+			      boolForKey: @"GSScrollViewNoInnerBorder"] ? 0.0 : 1.0;
 
   /*
    * Substract 1 from the width and height of
@@ -153,11 +156,11 @@ static float scrollerWidth;
    */
   if (hFlag)
     {
-      size.height -= scrollerWidth + 1;
+      size.height -= scrollerWidth + innerBorderWidth;
     }
   if (vFlag)
     {
-      size.width -= scrollerWidth + 1;
+      size.width -= scrollerWidth + innerBorderWidth;
     }
 
   size.width -= 2 * border.width;
@@ -173,6 +176,8 @@ static float scrollerWidth;
 {
   NSSize size = contentSize;
   NSSize border = [[GSTheme theme] sizeForBorderType: borderType];
+  float innerBorderWidth = [[NSUserDefaults standardUserDefaults]
+			      boolForKey: @"GSScrollViewNoInnerBorder"] ? 0.0 : 1.0;
 
   /*
    * Add 1 to the width and height for the line that separates the
@@ -180,11 +185,11 @@ static float scrollerWidth;
    */
   if (hFlag)
     {
-      size.height += scrollerWidth + 1;
+      size.height += scrollerWidth + innerBorderWidth;
     }
   if (vFlag)
     {
-      size.width += scrollerWidth + 1;
+      size.width += scrollerWidth + innerBorderWidth;
     }
 
   size.width += 2 * border.width;
@@ -387,7 +392,8 @@ static float scrollerWidth;
 - (void) scrollWheel: (NSEvent *)theEvent
 {
   NSRect clipViewBounds;
-  float delta = [theEvent deltaY];
+  float deltaY = [theEvent deltaY];
+  float deltaX = [theEvent deltaX];
   float amount;
   NSPoint point;
 
@@ -401,46 +407,49 @@ static float scrollerWidth;
     }
   point = clipViewBounds.origin;
 
-
-  if (_hasHorizScroller == YES
-    && ([theEvent modifierFlags] & NSShiftKeyMask) == NSShiftKeyMask)
+  // Holding shift converts vertical scrolling to horizontal
+  if (([theEvent modifierFlags] & NSShiftKeyMask) == NSShiftKeyMask)
     {
-      if (([theEvent modifierFlags] & NSAlternateKeyMask) == NSAlternateKeyMask)
-        {
-          amount = - (clipViewBounds.size.width - _hPageScroll) * delta;
-        }
-      else
-        {
-          amount = - _hLineScroll * delta;
-        }
-      NSDebugLLog (@"NSScrollView", 
-        @"increment/decrement: amount = %f, horizontal", amount);
+      deltaX = -deltaY;
+      deltaY = 0;
+    }
 
-      point.x = clipViewBounds.origin.x + amount;
+  // Scroll horizontally
+  if (([theEvent modifierFlags] & NSAlternateKeyMask) == NSAlternateKeyMask)
+    {
+      amount = (clipViewBounds.size.width - _hPageScroll) * deltaX;
     }
   else
     {
-      if (([theEvent modifierFlags] & NSAlternateKeyMask) == NSAlternateKeyMask)
-        {
-          amount = - (clipViewBounds.size.height - _vPageScroll) * delta;
-        }
-      else
-        {
-          amount = - _vLineScroll * delta;
-        }
-
-      if (_contentView != nil && !_contentView->_rFlags.flipped_view)
-        {
-          /* If view is flipped reverse the scroll direction */
-          amount = -amount;
-        }
-      NSDebugLLog (@"NSScrollView", 
-        @"increment/decrement: amount = %f, flipped = %d",
-        amount, _contentView ? _contentView->_rFlags.flipped_view : 0);
-
-      point.y = clipViewBounds.origin.y + amount;
+      amount = _hLineScroll * deltaX;
     }
 
+  NSDebugLLog (@"NSScrollView", 
+    @"increment/decrement: amount = %f, horizontal", amount);
+
+  point.x = clipViewBounds.origin.x + amount;
+
+  // Scroll vertically
+  if (([theEvent modifierFlags] & NSAlternateKeyMask) == NSAlternateKeyMask)
+    {
+      amount = - (clipViewBounds.size.height - _vPageScroll) * deltaY;
+    }
+  else
+    {
+      amount = - _vLineScroll * deltaY;
+    }
+
+  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+    {
+      /* If view is flipped reverse the scroll direction */
+      amount = -amount;
+    }
+  NSDebugLLog (@"NSScrollView", 
+    @"increment/decrement: amount = %f, flipped = %d",
+    amount, _contentView ? _contentView->_rFlags.flipped_view : 0);
+
+  point.y = clipViewBounds.origin.y + amount;
+
   /* scrollToPoint: will call reflectScrolledClipView:, which will
    * update rules, headers, and scrollers.  */
   [_contentView _scrollToPoint: point];
@@ -726,6 +735,56 @@ static float scrollerWidth;
   [_contentView _scrollToPoint: point];
 }
 
+- (void) scrollToBeginningOfDocument: (id)sender
+{
+  NSRect  clipViewBounds, documentRect;
+  NSPoint point;
+
+  if (_contentView == nil)
+    {
+      clipViewBounds = NSZeroRect;
+      documentRect = NSZeroRect;
+    }
+  else
+    {
+      clipViewBounds = [_contentView bounds];
+      documentRect = [_contentView documentRect];
+    }
+  point = documentRect.origin;
+  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+    {
+      point.y = NSMaxY(documentRect) - NSHeight(clipViewBounds);
+      if (point.y < 0)
+        point.y = 0;
+    }
+  [_contentView _scrollToPoint: point];
+}
+
+- (void) scrollToEndOfDocument: (id)sender
+{
+  NSRect  clipViewBounds, documentRect;
+  NSPoint point;
+
+  if (_contentView == nil)
+    {
+      clipViewBounds = NSZeroRect;
+      documentRect = NSZeroRect;
+    }
+  else
+    {
+      clipViewBounds = [_contentView bounds];
+      documentRect = [_contentView documentRect];
+    }
+  point = documentRect.origin;
+  if (_contentView == nil || _contentView->_rFlags.flipped_view)
+    {
+      point.y = NSMaxY(documentRect) - NSHeight(clipViewBounds);
+      if (point.y < 0)
+        point.y = 0;
+    }
+  [_contentView _scrollToPoint: point];
+}
+
 //
 // This method is here purely for nib compatibility.  This is the action
 // connected to by NSScrollers in IB when building a scrollview.
@@ -1004,6 +1063,8 @@ static float scrollerWidth;
   float headerViewHeight = 0;
   NSRectEdge verticalScrollerEdge = NSMinXEdge;
   NSInterfaceStyle style;
+  float innerBorderWidth = [[NSUserDefaults standardUserDefaults]
+			      boolForKey: @"GSScrollViewNoInnerBorder"] ? 0.0 : 1.0;
 
   style = NSInterfaceStyleForKey(@"NSScrollViewInterfaceStyle", nil);
 
@@ -1073,8 +1134,9 @@ static float scrollerWidth;
       [_vertScroller setFrame: vertScrollerRect];
 
       /* Substract 1 for the line that separates the vertical scroller
-       * from the clip view (and eventually the horizontal scroller).  */
-      NSDivideRect (contentRect, NULL, &contentRect, 1, verticalScrollerEdge);
+       * from the clip view (and eventually the horizontal scroller),
+       * unless the GSScrollViewNoInnerBorder default is set. */
+      NSDivideRect (contentRect, NULL, &contentRect, innerBorderWidth, verticalScrollerEdge);
     }
 
   /* Prepare the horizontal scroller.  */
@@ -1088,8 +1150,9 @@ static float scrollerWidth;
       [_horizScroller setFrame: horizScrollerRect];
 
       /* Substract 1 for the width for the line that separates the
-       * horizontal scroller from the clip view.  */
-      NSDivideRect (contentRect, NULL, &contentRect, 1, bottomEdge);
+       * horizontal scroller from the clip view,
+       * unless the GSScrollViewNoInnerBorder default is set. */
+      NSDivideRect (contentRect, NULL, &contentRect, innerBorderWidth, bottomEdge);
     }
 
   /* Now place and size the header view to be exactly above the
@@ -1148,13 +1211,15 @@ static float scrollerWidth;
   GSTheme	*theme = [GSTheme theme];
   NSColor	*color;
   NSString	*name;
+  BOOL hasInnerBorder = ![[NSUserDefaults standardUserDefaults]
+			   boolForKey: @"GSScrollViewNoInnerBorder"];
 
   name = [theme nameForElement: self];
   if (name == nil)
     {
       name = @"NSScrollView";
     }
-  color = [theme colorNamed: name state: GSThemeNormalState cache: YES];
+  color = [theme colorNamed: name state: GSThemeNormalState];
   if (color == nil)
     {
       color = [NSColor controlDarkShadowColor];
@@ -1179,46 +1244,49 @@ static float scrollerWidth;
         break;
     }
 
-  [color set];
-  DPSsetlinewidth(ctxt, 1);
-
-  if (_hasVertScroller)
+  if (hasInnerBorder)
     {
-      NSInterfaceStyle style;
+      [color set];
+      DPSsetlinewidth(ctxt, 1);
 
-      style = NSInterfaceStyleForKey(@"NSScrollViewInterfaceStyle", nil);
-      if (style == NSMacintoshInterfaceStyle
-        || style == NSWindows95InterfaceStyle)
-        {
-          DPSmoveto(ctxt, [_vertScroller frame].origin.x - 1, 
-            [_vertScroller frame].origin.y - 1);
-        }
-      else
-        {
-          DPSmoveto(ctxt, [_vertScroller frame].origin.x + scrollerWidth, 
-            [_vertScroller frame].origin.y - 1);
-        }
-      DPSrlineto(ctxt, 0, [_vertScroller frame].size.height + 1);
-      DPSstroke(ctxt);
-    }
+      if (_hasVertScroller)
+	{
+	  NSInterfaceStyle style;
 
-  if (_hasHorizScroller)
-    {
-      float ypos;
-      float scrollerY = [_horizScroller frame].origin.y;
+	  style = NSInterfaceStyleForKey(@"NSScrollViewInterfaceStyle", nil);
+	  if (style == NSMacintoshInterfaceStyle
+	      || style == NSWindows95InterfaceStyle)
+	    {
+	      DPSmoveto(ctxt, [_vertScroller frame].origin.x - 1, 
+			[_vertScroller frame].origin.y - 1);
+	    }
+	  else
+	    {
+	      DPSmoveto(ctxt, [_vertScroller frame].origin.x + scrollerWidth, 
+			[_vertScroller frame].origin.y - 1);
+	    }
+	  DPSrlineto(ctxt, 0, [_vertScroller frame].size.height + 1);
+	  DPSstroke(ctxt);
+	}
 
-      if (_rFlags.flipped_view)
-        {
-          ypos = scrollerY - 1;
-        }
-      else
-        {
-          ypos = scrollerY + scrollerWidth + 1;
-        }
+      if (_hasHorizScroller)
+	{
+	  float ypos;
+	  float scrollerY = [_horizScroller frame].origin.y;
 
-      DPSmoveto(ctxt, [_horizScroller frame].origin.x - 1, ypos);
-      DPSrlineto(ctxt, [_horizScroller frame].size.width + 1, 0);
-      DPSstroke(ctxt);
+	  if (_rFlags.flipped_view)
+	    {
+	      ypos = scrollerY - 1;
+	    }
+	  else
+	    {
+	      ypos = scrollerY + scrollerWidth + 1;
+	    }
+
+	  DPSmoveto(ctxt, [_horizScroller frame].origin.x - 1, ypos);
+	  DPSrlineto(ctxt, [_horizScroller frame].size.width + 1, 0);
+	  DPSstroke(ctxt);
+	}
     }
 }
 
@@ -1290,7 +1358,7 @@ static float scrollerWidth;
 
 - (BOOL) isOpaque
 {
-  return YES;
+  return [_contentView isOpaque];
 }
 
 - (NSBorderType) borderType
@@ -1536,6 +1604,8 @@ static float scrollerWidth;
       _vLineScroll = 10;
       _vPageScroll = 10;
       _scrollsDynamically = YES;
+      /* _autohidesScroller, _rulersVisible, _hasHorizRuler and _hasVertRuler 
+         implicitly set to NO */
 
       if ([aDecoder containsValueForKey: @"NSsFlags"])
         {
@@ -1550,33 +1620,35 @@ static float scrollerWidth;
 
           _hasVertScroller = scrollViewFlags.hasVScroller;
           _hasHorizScroller = scrollViewFlags.hasHScroller;
-          _autohidesScrollers = scrollViewFlags.autohidesScrollers;
-          _scrollsDynamically = (!scrollViewFlags.nonDynamic);
-          _rulersVisible = scrollViewFlags.showRulers;
-          _hasHorizRuler = scrollViewFlags.hasHRuler;
-          _hasVertRuler = scrollViewFlags.hasVRuler;
+          // TODO: Enable once we encode the next values in 
+          // -encodeWithCoder:, but not before otherwise we read random memory.
+          //_autohidesScrollers = scrollViewFlags.autohidesScrollers;
+          //_scrollsDynamically = (!scrollViewFlags.nonDynamic);
+          //_rulersVisible = scrollViewFlags.showRulers;
+          //_hasHorizRuler = scrollViewFlags.hasHRuler;
+          //_hasVertRuler = scrollViewFlags.hasVRuler;
           // [self setDrawsBackground: (!scrollViewFlags.doesNotDrawBack)];
           _borderType = scrollViewFlags.border;
         }
 
-      if (content != nil)
+      /* FIXME: This should only happen when we load a Mac NIB file.
+         And as far as I can tell tile is handling this correctly.
+      if (vScroller != nil && _hasVertScroller && content != nil)
         {
           // Move the content view since it is not moved when we retile.
           NSRect frame = [content frame];
           float w = [vScroller frame].size.width;
 
-          if (_hasVertScroller)
-            {
-              //
-              // Slide the content view over, since on Mac OS X the scroller is on the
-              // right, the content view is not properly positioned since our scroller
-              // is on the left.
-              //
-              frame.origin.x += w;
-              [content setFrame: frame];
-            }
+          //
+          // Slide the content view over, since on Mac OS X the scroller is on the
+          // right, the content view is not properly positioned since our scroller
+          // is on the left.
+          //
+          frame.origin.x += w;
+          [content setFrame: frame];
         }
-     
+      */
+
       if (hScroller != nil && _hasHorizScroller)
         {
           [self setHorizontalScroller: hScroller];
@@ -1595,10 +1667,10 @@ static float scrollerWidth;
           _headerClipView = [aDecoder decodeObjectForKey: @"NSHeaderClipView"];
         }
 
-      // [self tile];
       // set the document view into the content.
-      [self addSubview: vScroller];
       [self setContentView: content];
+      [self tile];
+      // Reenable notification sending.
       [docView setPostsFrameChangedNotifications: post_frame];
       [docView setPostsBoundsChangedNotifications: post_bound];
     }
@@ -1648,6 +1720,7 @@ static float scrollerWidth;
         {
           NSLog(@"unknown NSScrollView version (%d)", version);
           DESTROY(self);
+          return nil;
         }
       [self tile];
       
