@@ -30,29 +30,36 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include "config.h"
-#include <Foundation/NSArray.h>
-#include <Foundation/NSSet.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSProxy.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSSpellServer.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/NSNibLoading.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSSpellChecker.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSTextField.h"
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSBrowser.h"
-#include "AppKit/NSBrowserCell.h"
-#include "AppKit/NSPopUpButton.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSPanel.h"
-#include "GSGuiPrivate.h"
-#include "GNUstepGUI/GSServicesManager.h"
+#import "config.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSConnection.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSDistantObject.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSProxy.h>
+#import <Foundation/NSSet.h>
+#import <Foundation/NSSpellServer.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSBrowser.h"
+#import "AppKit/NSBrowserCell.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMatrix.h"
+#import "AppKit/NSNib.h"
+#import "AppKit/NSNibLoading.h"
+#import "AppKit/NSPanel.h"
+#import "AppKit/NSPopUpButton.h"
+#import "AppKit/NSSpellChecker.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSTextField.h"
+#import "AppKit/NSWindow.h"
+#import "GSGuiPrivate.h"
+#import "GNUstepGUI/GSServicesManager.h"
 
 // prototype for function to create name for server
 NSString *GSSpellServerName(NSString *checkerDictionary, NSString *language);
@@ -235,7 +242,8 @@ static int __documentTag = 0;
 - (void)_populateDictionaryPulldown: (NSArray *)dictionaries
 {
   [_dictionaryPulldown removeAllItems];
-  [_dictionaryPulldown addItemsWithTitles: dictionaries];
+  [_dictionaryPulldown addItemsWithTitles:
+    [dictionaries sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)]];
   [_dictionaryPulldown selectItemWithTitle: _language];
 }
 
@@ -257,10 +265,12 @@ static int __documentTag = 0;
 - init
 {
   NSArray *userLanguages = [NSUserDefaults userLanguages];  
-  NSString *panel = nil;
+
+  self = [super init];
+  if (self == nil)
+    return nil;
 
   // Set the language to the default for the user.
-  self = [super init];
   _language = [userLanguages objectAtIndex: 0];
   _wrapFlag = NO;
   _position = 0;
@@ -269,11 +279,9 @@ static int __documentTag = 0;
   _currentTag = 0;
   _ignoredWords = [NSMutableDictionary new];
 
-  // Load the gmodel file
-  panel = [GSGuiBundle() pathForResource: @"GSSpellPanel" ofType: @"gorm"
-		      inDirectory: nil];
-  if (![NSBundle loadNibFile: panel
-	  externalNameTable: [NSDictionary dictionaryWithObject: self forKey: @"NSOwner"]
+  // Load the NIB file from the GUI bundle
+  if (![GSGuiBundle() loadNibFile: @"GSSpellPanel"
+	  externalNameTable: [NSDictionary dictionaryWithObject: self forKey: NSNibOwner]
 	           withZone: [self zone]])
     {
       NSLog(@"Model file load failed for GSSpellPanel");
@@ -533,9 +541,9 @@ inSpellDocumentWithTag:(int)tag
 
 - _findNext: (id)sender
 {
-  BOOL processed = [NSApp sendAction: @selector(checkSpelling:) 
-			  to: nil 
-			  from: _spellPanel];
+  BOOL processed = [[[NSApp mainWindow] firstResponder]
+		       tryToPerform: @selector(checkSpelling:)
+			       with: _spellPanel];
 
   if (!processed)
     {
@@ -592,9 +600,9 @@ inSpellDocumentWithTag:(int)tag
 
 - _ignore: (id)sender
 {
-  BOOL processed = [NSApp sendAction: @selector(ignoreSpelling:) 
-			  to: nil 
-			  from: _wordField];
+  BOOL processed = [[[NSApp mainWindow] firstResponder]
+		       tryToPerform: @selector(ignoreSpelling:)
+			       with: _wordField];
 
   if (!processed)
     {
@@ -615,9 +623,9 @@ inSpellDocumentWithTag:(int)tag
 
 - _correct: (id)sender
 {
-  BOOL processed = [NSApp sendAction: @selector(changeSpelling:) 
-			  to: nil 
-			  from: _wordField];
+  BOOL processed = [[[NSApp mainWindow] firstResponder]
+		       tryToPerform: @selector(changeSpelling:)
+			       with: _wordField];
 
   if (!processed)
     {
@@ -642,7 +650,7 @@ inSpellDocumentWithTag:(int)tag
       if (proxy != nil)
 	{
 	  ASSIGN(_language, language);
-	  ASSIGN(_serverProxy, proxy);
+	  ASSIGN(_serverProxy, (id)proxy);
 	}
       else
 	{
@@ -674,15 +682,22 @@ inSpellDocumentWithTag:(int)tag
   [_accessoryView setDoubleAction: @selector(_correct:)];
 
   [_findNextButton setKeyEquivalent: @"n"];
+  [_findNextButton setKeyEquivalentModifierMask: NSCommandKeyMask];
   [_ignoreButton setKeyEquivalent: @"i"];
+  [_ignoreButton setKeyEquivalentModifierMask: NSCommandKeyMask];
   [_learnButton setKeyEquivalent: @"l"];
+  [_learnButton setKeyEquivalentModifierMask: NSCommandKeyMask];
   [_forgetButton setKeyEquivalent: @"f"];
+  [_forgetButton setKeyEquivalentModifierMask: NSCommandKeyMask];
   [_guessButton setKeyEquivalent: @"g"];
+  [_guessButton setKeyEquivalentModifierMask: NSCommandKeyMask];
   [_correctButton setKeyEquivalent: @"c"];
+  [_correctButton setKeyEquivalentModifierMask: NSCommandKeyMask];
   [_correctButton setImagePosition: NSImageRight];
   [_correctButton setImage: [NSImage imageNamed: @"common_ret"]];
   [_correctButton setAlternateImage: [NSImage imageNamed: @"common_retH"]];
   [_spellPanel makeFirstResponder: _correctButton];
+  [_spellPanel setBecomesKeyOnlyIfNeeded: YES];
 }
 @end
 
@@ -729,7 +744,7 @@ inSpellDocumentWithTag:(int)tag
 
 - (NSString*) browser: (NSBrowser*)sender titleOfColumn: (int)column
 {
-  return @"Guess";
+  return _(@"Guess");
 }
 
 - (void) browser: (NSBrowser *)sender 
