@@ -28,21 +28,14 @@
 
 #include "config.h"
 
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSControl.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSGraphicsContext.h"
-#include "AppKit/NSStepperCell.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/PSOperators.h"
+#import <Foundation/NSValue.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSControl.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSStepperCell.h"
+#import "AppKit/NSWindow.h"
 #include "GNUstepGUI/GSTheme.h"
 
-// Hard coded values for button sizes
-#define STEPPER_WIDTH 15
-#define STEPPER_HEIGHT 11
-
 @interface NSStepperCell (Private)
 - (void) _increment;
 - (void) _decrement;
@@ -50,8 +43,6 @@
 	       upButton: (BOOL)upButton
 	      withFrame: (NSRect)frame
 		 inView: (NSView*)controlView;
-- (NSRect) upButtonRectWithFrame: (NSRect)frame;
-- (NSRect) downButtonRectWithFrame: (NSRect)frame;
 @end
 
 @implementation NSStepperCell
@@ -75,14 +66,16 @@
 - (id) init
 {
   self = [super init];
+  if (!self)
+    return nil;
 
   [self setIntValue: 0];
   [self setAlignment: NSRightTextAlignment];
   [self setWraps: NO];
   _autorepeat = YES;
   _valueWraps = YES;
-  _maxValue = 59;
-  _minValue = 0;
+  [self setMaxValue: 59];
+  [self setMinValue: 0];
   _increment = 1;
 
   highlightUp = NO;
@@ -99,6 +92,8 @@
 - (void) setMaxValue: (double)maxValue
 {
   _maxValue = maxValue;
+  if ([self doubleValue] > _maxValue)
+    [self setDoubleValue: _maxValue];
 }
 
 - (double) minValue
@@ -109,6 +104,31 @@
 - (void) setMinValue: (double)minValue
 {
   _minValue = minValue;
+  if ([self doubleValue] < _minValue)
+    [self setDoubleValue: _minValue];
+}
+
+- (void) setObjectValue: (id)anObject
+{
+  // NOTE: valueWraps has no effect on setObjectValue:
+  // FIXME: Copied from NSSliderCell.. can we share the code somehow?
+
+  // If the provided object doesn't respond to doubeValue, or our minValue
+  // is greater than our maxValue, we set our value to our minValue
+  // (this arbitrary choice matches OS X)
+  if ([anObject respondsToSelector: @selector(doubleValue)] == NO ||
+      _minValue > _maxValue)
+    [super setObjectValue: [NSNumber numberWithDouble: _minValue]];
+  else
+    {
+      double aDouble = [anObject doubleValue];
+      if (aDouble < _minValue)
+        [super setObjectValue: [NSNumber numberWithDouble: _minValue]];
+      else if (aDouble > _maxValue)
+        [super setObjectValue: [NSNumber numberWithDouble: _maxValue]];
+      else
+        [super setObjectValue: anObject];
+    }
 }
 
 - (double) increment
@@ -121,169 +141,34 @@
   _increment = increment;
 }
 
-- (BOOL)autorepeat
+- (BOOL) autorepeat
 {
   return _autorepeat;
 }
 
-- (void)setAutorepeat: (BOOL)autorepeat
+- (void) setAutorepeat: (BOOL)autorepeat
 {
   _autorepeat = autorepeat;
 }
 
-- (BOOL)valueWraps
+- (BOOL) valueWraps
 {
   return _valueWraps;
 }
 
-- (void)setValueWraps: (BOOL)valueWraps
+- (void) setValueWraps: (BOOL)valueWraps
 {
   _valueWraps = valueWraps;
 }
 
-static inline NSRect DrawLightButton(NSRect border, NSRect clip)
-{
-/*
-  NSRect highlightRect = NSInsetRect(border, 1., 1.);
-  [[GSTheme theme] drawButton: border : clip];
-  return highlightRect;
-*/
-  NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, 
-			   NSMinXEdge, NSMaxYEdge}; 
-  NSRectEdge dn_sides[] = {NSMaxXEdge, NSMaxYEdge, 
-			   NSMinXEdge, NSMinYEdge}; 
-  // These names are role names not the actual colours
-  NSColor *dark = [NSColor controlShadowColor];
-  NSColor *white = [NSColor controlLightHighlightColor];
-  NSColor *colors[] = {dark, dark, white, white};
-
-  if ([[NSView focusView] isFlipped] == YES)
-    {
-      return NSDrawColorTiledRects(border, clip, dn_sides, colors, 4);
-    }
-  else
-    {
-      return NSDrawColorTiledRects(border, clip, up_sides, colors, 4);
-    }
-}
-
-static inline void DrawUpButton(NSRect aRect)
-{
-  NSRect unHighlightRect = DrawLightButton(aRect, NSZeroRect);
-  [[NSColor controlBackgroundColor] set];
-  NSRectFill(unHighlightRect);
-      
-  PSsetlinewidth(1.0);
-  [[NSColor controlShadowColor] set];
-  PSmoveto(NSMaxX(aRect) - 5, NSMinY(aRect) + 3);
-  PSlineto(NSMaxX(aRect) - 8, NSMinY(aRect) + 9);
-  PSstroke();
-  [[NSColor controlDarkShadowColor] set];
-  PSmoveto(NSMaxX(aRect) - 8, NSMinY(aRect) + 9);
-  PSlineto(NSMaxX(aRect) - 11, NSMinY(aRect) + 4);
-  PSstroke();
-  [[NSColor controlLightHighlightColor] set];
-  PSmoveto(NSMaxX(aRect) - 11, NSMinY(aRect) + 3);
-  PSlineto(NSMaxX(aRect) - 5, NSMinY(aRect) + 3);
-  PSstroke();
-}
-
-static inline void HighlightUpButton(NSRect aRect)
-{
-  NSRect highlightRect = DrawLightButton(aRect, NSZeroRect);
-  [[NSColor selectedControlColor] set];
-  NSRectFill(highlightRect);
-  
-  PSsetlinewidth(1.0);
-  [[NSColor controlHighlightColor] set];
-  PSmoveto(NSMaxX(aRect) - 5, NSMinY(aRect) + 3);
-  PSlineto(NSMaxX(aRect) - 8, NSMinY(aRect) + 9);
-  PSstroke();
-  [[NSColor controlDarkShadowColor] set];
-  PSmoveto(NSMaxX(aRect) - 8, NSMinY(aRect) + 9);
-  PSlineto(NSMaxX(aRect) - 11, NSMinY(aRect) + 4);
-  PSstroke();
-  [[NSColor controlHighlightColor] set];
-  PSmoveto(NSMaxX(aRect) - 11, NSMinY(aRect) + 3);
-  PSlineto(NSMaxX(aRect) - 5, NSMinY(aRect) + 3);
-  PSstroke();
-}
-
-static inline void DrawDownButton(NSRect aRect)
-{
-  NSRect unHighlightRect = DrawLightButton(aRect, NSZeroRect);
-  [[NSColor controlBackgroundColor] set];
-  NSRectFill(unHighlightRect);
-
-  PSsetlinewidth(1.0);
-  [[NSColor controlShadowColor] set];
-  PSmoveto(NSMinX(aRect) + 4, NSMaxY(aRect) - 3);
-  PSlineto(NSMinX(aRect) + 7, NSMaxY(aRect) - 8);
-  PSstroke();
-  [[NSColor controlLightHighlightColor] set];
-  PSmoveto(NSMinX(aRect) + 7, NSMaxY(aRect) - 8);
-  PSlineto(NSMinX(aRect) + 10, NSMaxY(aRect) - 3);
-  PSstroke();
-  [[NSColor controlDarkShadowColor] set];
-  PSmoveto(NSMinX(aRect) + 10, NSMaxY(aRect) - 2);
-  PSlineto(NSMinX(aRect) + 4, NSMaxY(aRect) - 2);
-  PSstroke();
-}
-
-static inline void HighlightDownButton(NSRect aRect)
-{
-  NSRect highlightRect = DrawLightButton(aRect, NSZeroRect);
-  [[NSColor selectedControlColor] set];
-  NSRectFill(highlightRect);
-  
-  PSsetlinewidth(1.0);
-  [[NSColor controlHighlightColor] set];
-  PSmoveto(NSMinX(aRect) + 4, NSMaxY(aRect) - 3);
-  PSlineto(NSMinX(aRect) + 7, NSMaxY(aRect) - 8);
-  PSstroke();
-  [[NSColor controlHighlightColor] set];
-  PSmoveto(NSMinX(aRect) + 7, NSMaxY(aRect) - 8);
-  PSlineto(NSMinX(aRect) + 10, NSMaxY(aRect) - 3);
-  PSstroke();
-  [[NSColor controlDarkShadowColor] set];
-  PSmoveto(NSMinX(aRect) + 10, NSMaxY(aRect) - 2);
-  PSlineto(NSMinX(aRect) + 4, NSMaxY(aRect) - 2);
-  PSstroke();
-}
-
 - (void) drawInteriorWithFrame: (NSRect)cellFrame
 			inView: (NSView*)controlView
 {
-  NSRect upRect;
-  NSRect downRect;
-  NSRect twoButtons;
-
-  upRect = [self upButtonRectWithFrame: cellFrame];
-  downRect = [self downButtonRectWithFrame: cellFrame];
-  
-  twoButtons = downRect;
-  twoButtons.origin.y--;
-  twoButtons.size.width++;
-  twoButtons.size.height = 2 * STEPPER_HEIGHT + 1;
-
-  if (highlightUp)
-    HighlightUpButton(upRect);
-  else
-    DrawUpButton(upRect);
-
-  if (highlightDown)
-    HighlightDownButton(downRect);
-  else
-    DrawDownButton(downRect);
-
-  {
-    NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge};
-    NSColor *black = [NSColor controlDarkShadowColor];
-    NSColor *grays[] = {black, black}; 
-    
-    NSDrawColorTiledRects(twoButtons, NSZeroRect,
-			  up_sides, grays, 2);
-  }
+  [[GSTheme theme] drawStepperCell: self
+                   withFrame: cellFrame
+                   inView: controlView
+                   highlightUp: highlightUp
+                   highlightDown: highlightDown];
 }
 
 - (void) getPeriodicDelay: (float*)delay interval: (float*)interval
@@ -325,8 +210,8 @@ static inline void HighlightDownButton(NSRect aRect)
   if ([theEvent type] != NSLeftMouseDown)
     return NO;
 
-  upRect = [self upButtonRectWithFrame: cellFrame];
-  downRect = [self downButtonRectWithFrame: cellFrame];
+  upRect = [[GSTheme theme] stepperUpButtonRectWithFrame: cellFrame];
+  downRect = [[GSTheme theme] stepperDownButtonRectWithFrame: cellFrame];
   
   // Did the mouse go down in the up or in the down part?
   if (NSMouseInRect(point, upRect, NO))
@@ -534,13 +419,6 @@ static inline void HighlightDownButton(NSRect aRect)
       else if (newValue < minValue)
 	newValue = newValue + maxValue - minValue + 1;
     }
-  else
-    {
-      if (newValue > maxValue)
-	newValue = maxValue;
-      else if (newValue < minValue)
-	 newValue = minValue;
-    }
   [self setDoubleValue: newValue];
 }
 
@@ -559,13 +437,6 @@ static inline void HighlightDownButton(NSRect aRect)
       else if (newValue < minValue)
 	newValue = newValue + maxValue - minValue + 1;
     }
-  else
-    {
-      if (newValue > maxValue)
-	newValue = maxValue;
-      else if (newValue < minValue)
-	 newValue = minValue;
-    }
   [self setDoubleValue: newValue];
 }
 
@@ -586,26 +457,4 @@ static inline void HighlightDownButton(NSRect aRect)
   [controlView setNeedsDisplayInRect: frame];
 }
 
-- (NSRect) upButtonRectWithFrame: (NSRect)frame
-{
-  NSRect upRect;
-
-  upRect.size.width = STEPPER_WIDTH;
-  upRect.size.height = STEPPER_HEIGHT;
-  upRect.origin.x = NSMaxX(frame) - STEPPER_WIDTH - 1;
-  upRect.origin.y = NSMinY(frame) + ((int)frame.size.height / 2) + 1;
-  return upRect;
-}
-
-- (NSRect) downButtonRectWithFrame: (NSRect)frame
-{
-  NSRect downRect;
-
-  downRect.size.width = STEPPER_WIDTH;
-  downRect.size.height = STEPPER_HEIGHT;
-  downRect.origin.x = NSMaxX(frame) - STEPPER_WIDTH - 1;
-  downRect.origin.y = NSMinY(frame) + ((int)frame.size.height / 2) - STEPPER_HEIGHT + 1;
-  return downRect;
-}
-
 @end
