@@ -32,15 +32,15 @@
 
 #include <math.h>
 
-#include <Foundation/NSException.h>
-#include <Foundation/NSLock.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSLock.h>
 
-#include "AppKit/NSAffineTransform.h"
-#include "AppKit/NSLayoutManager.h"
-#include "AppKit/NSTextContainer.h"
-#include "AppKit/NSTextStorage.h"
-#include "AppKit/DPSOperators.h"
-#include "GSTextStorage.h"
+#import "AppKit/NSAffineTransform.h"
+#import "AppKit/NSLayoutManager.h"
+#import "AppKit/NSStringDrawing.h"
+#import "AppKit/NSTextContainer.h"
+#import "AppKit/NSTextStorage.h"
+#import "AppKit/DPSOperators.h"
 
 /*
 TODO:
@@ -456,7 +456,7 @@ glyphs to be drawn upside-down, so we need to tell NSFont to flip the fonts.
   cache_t *c;
 
   NSRange r;
-  BOOL need_clip;
+  BOOL need_clip = NO;
   NSGraphicsContext *ctxt = GSCurrentContext();
 
   if (rect.size.width <= 0 || rect.size.height <= 0)
@@ -540,7 +540,7 @@ glyphs to be drawn upside-down, so we need to tell NSFont to flip the fonts.
 - (NSSize) size
 {
   int ci;
-  NSSize result;
+  NSSize result = NSZeroSize;
 
   cache_lock();
   NS_DURING
@@ -569,7 +569,7 @@ glyphs to be drawn upside-down, so we need to tell NSFont to flip the fonts.
                         options: (NSStringDrawingOptions)options
 {
   int ci;
-  NSRect result;
+  NSRect result = NSZeroRect;
 
   cache_lock();
   NS_DURING
@@ -660,7 +660,7 @@ NSAttributedString to do the job.
   cache_t *c;
 
   NSRange r;
-  BOOL need_clip;
+  BOOL need_clip = NO;
   NSGraphicsContext *ctxt = GSCurrentContext();
 
   if (rect.size.width <= 0 || rect.size.height <= 0)
@@ -743,7 +743,7 @@ NSAttributedString to do the job.
 - (NSSize) sizeWithAttributes: (NSDictionary *)attrs
 {
   int ci;
-  NSSize result;
+  NSSize result = NSZeroSize;
 
   cache_lock();
   NS_DURING
@@ -773,7 +773,7 @@ NSAttributedString to do the job.
                      attributes: (NSDictionary *)attributes
 {
   int ci;
-  NSRect result;
+  NSRect result = NSZeroRect;
 
   cache_lock();
   NS_DURING
