@@ -8,7 +8,7 @@
    Date: 1996
    Author: Felipe A. Rodriguez <far@ix.netcom.com>
    Date: July 1998
-   Author:  Daniel Bhringer <boehring@biomed.ruhr-uni-bochum.de>
+   Author:  Daniel Boehringer <boehring@biomed.ruhr-uni-bochum.de>
    Date: August 1998
    Author: Fred Kiefer <FredKiefer@gmx.de>
    Date: March 2000
@@ -20,23 +20,25 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02111 - 1307, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #include "AppKit/NSText.h"
 
+#include "AppKit/NSAttributedString.h"
 #include "AppKit/NSPanel.h"
 #include "AppKit/NSSpellChecker.h"
 #include "AppKit/NSTextView.h"
