@@ -35,6 +35,7 @@
 #include "AppKit/NSTextStorage.h"
 #include "AppKit/NSTextView.h"
 #include "GNUstepGUI/GSLayoutManager.h"
+#include "GSGuiPrivate.h"
 
 @interface NSTextContainer (TextViewObserver)
 - (void) _textViewFrameChanged: (NSNotification*)aNotification;
@@ -114,6 +115,11 @@ use bounds rectangle instead of frame? */
   return self;
 }
 
+- (id) init
+{
+  return [self initWithContainerSize: NSZeroSize];
+}
+
 - (void) dealloc
 {
   if (_textView != nil)
@@ -160,6 +166,7 @@ framework intact.
       GSLayoutManager *oldLayoutManager = _layoutManager;
 
       RETAIN(oldLayoutManager);
+      RETAIN(textStorage);
       [textStorage removeLayoutManager: _layoutManager];
       [textStorage addLayoutManager: aLayoutManager];
 
@@ -180,6 +187,7 @@ framework intact.
           [[container textView] setTextContainer: container];
           RELEASE(container);
         }
+      RELEASE(textStorage);
       RELEASE(oldLayoutManager);
     }
 }
@@ -212,6 +220,9 @@ framework intact.
               selector: @selector(_textViewFrameChanged:)
               name: NSViewFrameDidChangeNotification
               object: _textView];
+          [self _textViewFrameChanged: 
+                  [NSNotification notificationWithName: NSViewFrameDidChangeNotification
+                                                object: _textView]];
         }
     }
 
@@ -454,8 +465,8 @@ framework intact.
 {
   if ([aDecoder allowsKeyedCoding])
     {
-      NSTextView *view = [aDecoder decodeObjectForKey: @"NSTextView"];
       NSSize size = NSZeroSize;
+      // NSTextView *view = [aDecoder decodeObjectForKey: @"NSTextView"];
 
       if ([aDecoder containsValueForKey: @"NSWidth"])
         {
@@ -473,8 +484,12 @@ framework intact.
         }
 
       // decoding the manager adds this text container automatically...
-      [aDecoder decodeObjectForKey: @"NSLayoutManager"];
-      [self setTextView: view];
+      if ([aDecoder containsValueForKey: @"NSLayoutManager"])
+	{
+	  _layoutManager = [aDecoder decodeObjectForKey: @"NSLayoutManager"];
+	}
+
+      // [self setTextView: view];
       return self;
     }
   else
