@@ -439,7 +439,7 @@ static Class textFieldCellClass;
 
 -(BOOL) needsPanelToBecomeKey
 {
-  return [self isEditable];
+  return [self isEditable] || [self isSelectable];
 }
 
 - (BOOL) abortEditing
@@ -571,33 +571,46 @@ static Class textFieldCellClass;
 - (void) textDidEndEditing: (NSNotification *)aNotification
 {
   id textMovement;
+  int movement;
 
   [super textDidEndEditing: aNotification];
 
   textMovement = [[aNotification userInfo] objectForKey: @"NSTextMovement"];
+
   if (textMovement)
     {
-      switch ([(NSNumber *)textMovement intValue])
+      movement = [(NSNumber *)textMovement intValue];
+    }
+  else
+    {
+      movement = 0;
+    }
+
+  if (movement == NSReturnTextMovement)
+    {
+      if ([self sendAction: [self action] to: [self target]] == NO)
+        {
+          [self performKeyEquivalent: [_window currentEvent]];
+        }
+      if ([_window firstResponder] == _window)
+	[self selectText: self];
+    }
+  else
+    {
+      if ([[self cell] sendsActionOnEndEditing])
+        [self sendAction: [self action] to: [self target]];
+
+      if (movement == NSTabTextMovement)
         {
-        case NSReturnTextMovement:
-          if ([self sendAction: [self action] to: [self target]] == NO)
-            {
-              if ([self performKeyEquivalent: [_window currentEvent]] == NO)
-                [self selectText: self];
-            }
-          break;
-        case NSTabTextMovement:
           [_window selectKeyViewFollowingView: self];
-
           if ([_window firstResponder] == _window)
             [self selectText: self];
-          break;
-        case NSBacktabTextMovement:
+        }
+      else if (movement == NSBacktabTextMovement)
+        {
           [_window selectKeyViewPrecedingView: self];
-
           if ([_window firstResponder] == _window)
             [self selectText: self];
-          break;
         }
     }
 }
