@@ -46,15 +46,16 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSLayoutManager.h"
-#include "AppKit/NSScrollView.h"
-#include "AppKit/NSTextStorage.h"
-#include "AppKit/NSTextView.h"
-#include "AppKit/NSParagraphStyle.h"
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSLayoutManager.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSScrollView.h"
+#import "AppKit/NSTextStorage.h"
+#import "AppKit/NSTextView.h"
+#import "AppKit/NSParagraphStyle.h"
 
 /*
 These methods are for user actions, ie. they are normally called from
@@ -114,6 +115,12 @@ send -shouldChangeTextInRange:replacementString: or -didChangeText.
 
 */
 
+/* global kill buffer shared between all text views */
+/* Note: I'm not using an attributed string here because Apple apparently is
+   using a plain string either. Maybe this is because NeXT was using the X11
+   cut buffer for the kill buffer, which can hold only plain strings? */
+static NSString *killBuffer = @"";
+  
 /** First some helpers **/
 
 @interface NSTextView (UserActionHelpers)
@@ -548,10 +555,21 @@ static NSNumber *float_plus_one(NSNumber *cur)
   //[self insertText: @"\t"];
 }
 
+- (void) insertNewlineIgnoringFieldEditor: (id)sender
+{
+  [self insertText: @"\n"];
+}
+
+- (void) insertTabIgnoringFieldEditor: (id)sender
+{
+  [self insertText: @"\t"];
+}
+
 
 - (void) deleteForward: (id)sender
 {
   NSRange range = [self rangeForUserTextChange];
+  NSDictionary *attributes;
   
   if (range.location == NSNotFound)
     {
@@ -575,21 +593,31 @@ static NSNumber *float_plus_one(NSNumber *cur)
 	  return;
 	}
     }
+  else if ([self smartInsertDeleteEnabled] &&
+	   [self selectionGranularity] == NSSelectByWord)
+    {
+      range = [self smartDeleteRangeForProposedRange: range];
+    }
   
   if (![self shouldChangeTextInRange: range  replacementString: @""])
     {
       return;
     }
 
+  attributes = RETAIN([_textStorage attributesAtIndex: range.location
+				       effectiveRange: NULL]);
   [_textStorage beginEditing];
   [_textStorage deleteCharactersInRange: range];
   [_textStorage endEditing];
+  [self setTypingAttributes: attributes];
+  RELEASE(attributes);
   [self didChangeText];
 }
 
 - (void) deleteBackward: (id)sender
 {
   NSRange range = [self rangeForUserTextChange];
+  NSDictionary *attributes;
   
   if (range.location == NSNotFound)
     {
@@ -614,12 +642,19 @@ static NSNumber *float_plus_one(NSNumber *cur)
 	  return;
 	}
     }
+  else if ([self smartInsertDeleteEnabled] &&
+	   [self selectionGranularity] == NSSelectByWord)
+    {
+      range = [self smartDeleteRangeForProposedRange: range];
+    }
   
   if (![self shouldChangeTextInRange: range  replacementString: @""])
     {
       return;
     }
 
+  attributes = RETAIN([_textStorage attributesAtIndex: range.location
+				       effectiveRange: NULL]);
   [_textStorage beginEditing];
   [_textStorage deleteCharactersInRange: range];
   [_textStorage endEditing];
@@ -632,6 +667,7 @@ static NSNumber *float_plus_one(NSNumber *cur)
   NSRange linerange;
   unsigned maxRange;
   unsigned endCorrection = 0;
+  NSDictionary *attributes;
 
   if (range.location == NSNotFound)
     {
@@ -676,12 +712,22 @@ static NSNumber *float_plus_one(NSNumber *cur)
       return;
     }
 
+  ASSIGN(killBuffer, [[_textStorage string] substringWithRange: range]);
+  attributes = RETAIN([_textStorage attributesAtIndex: range.location
+				       effectiveRange: NULL]);
   [_textStorage beginEditing];
   [_textStorage deleteCharactersInRange: range];
   [_textStorage endEditing];
+  [self setTypingAttributes: attributes];
+  RELEASE(attributes);
   [self didChangeText];
 }
 
+- (void) yank: (id)sender
+{
+  [self insertText: killBuffer];
+}
+
 
 /*
 TODO: find out what affinity is supposed to mean
@@ -1322,6 +1368,16 @@ and layout is left-to-right */
   [[self enclosingScrollView] scrollPageUp: sender];
 }
 
+- (void) scrollToBeginningOfDocument: (id)sender
+{
+  [[self enclosingScrollView] scrollToBeginningOfDocument: sender];
+}
+
+- (void) scrollToEndOfDocument: (id)sender
+{
+  [[self enclosingScrollView] scrollToEndOfDocument: sender];
+}
+
 - (void) centerSelectionInVisibleArea: (id)sender
 {
   NSRange range;
