@@ -29,31 +29,32 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSKeyValueCoding.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSNull.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSTimer.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSDecimalNumber.h>
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSNibLoading.h"
-#include "AppKit/NSToolbar.h"
-#include "AppKit/NSToolbarItem.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSWindow.h"
-#include "GNUstepGUI/GSToolbarView.h"
+#import <Foundation/NSObject.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSKeyValueCoding.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSNull.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSTimer.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSDecimalNumber.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSNibLoading.h"
+#import "AppKit/NSToolbar.h"
+#import "AppKit/NSToolbarItem.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSToolbarView.h"
 
-#include "NSToolbarFrameworkPrivate.h"
-#include "GSToolbarCustomizationPalette.h"
+#import "NSToolbarFrameworkPrivate.h"
+#import "GSToolbarCustomizationPalette.h"
 
 // internal
 static NSNotificationCenter *nc = nil;
@@ -775,7 +776,7 @@ static GSValidationCenter *vc = nil;
   [self _build];
 }
 
-- (void) setSelectedItemIdentifier: (NSString *)itemIdentifier
+- (void) setSelectedItemIdentifier: (NSString *)identifier
 {
   NSArray *selectedItems;
   NSArray *itemsToSelect;
@@ -815,7 +816,7 @@ static GSValidationCenter *vc = nil;
        return;
      }
    
-   itemsToSelect = [_items objectsWithValue: itemIdentifier 
+   itemsToSelect = [_items objectsWithValue: identifier 
                                      forKey: @"_itemIdentifier"]; 
    e = [itemsToSelect objectEnumerator];
    while ((item = [e nextObject]) != nil)
@@ -830,7 +831,7 @@ static GSValidationCenter *vc = nil;
    
    if (updated)
      {
-       ASSIGN(_selectedItemIdentifier, itemIdentifier);
+       ASSIGN(_selectedItemIdentifier, identifier);
      }
    else
      {
@@ -898,11 +899,12 @@ static GSValidationCenter *vc = nil;
   NSString *tableKey = 
     [NSString stringWithFormat: @"NSToolbar Config %@",_identifier];
   NSDictionary *config = [defaults objectForKey: tableKey];
-  NSToolbarDisplayMode displayMode = 0;
-  NSToolbarSizeMode sizeMode = 0;
 
-  if(config)
+  if (config)
     {
+      NSToolbarDisplayMode displayMode = 0;
+      NSToolbarSizeMode sizeMode = 0;
+
       displayMode = (NSToolbarDisplayMode)[[config objectForKey: @"displayMode"] intValue];
       [self setDisplayMode: displayMode];
       sizeMode = (NSToolbarDisplayMode)[[config objectForKey: @"sizeMode"] intValue];
@@ -941,7 +943,7 @@ static GSValidationCenter *vc = nil;
   while ((itemIdentifier = [e nextObject]) != nil) 
     {
       [self _insertItemWithItemIdentifier: itemIdentifier 
-                                  atIndex: i 
+                                  atIndex: [_items count]
                                 broadcast: NO];
       i++;
     }
@@ -1174,6 +1176,7 @@ static GSValidationCenter *vc = nil;
                             object: self
                           userInfo: [NSDictionary dictionaryWithObject: item  forKey: @"item"]];
           [item _setToolbar: self];
+		  [item _layout];
           [_items insertObject: item atIndex: index];
           
           // We reload the toolbarView each time a new item is added except when
@@ -1319,14 +1322,53 @@ static GSValidationCenter *vc = nil;
     }
 }
 
-// This method wont make a toolbar visible or invisible by itself.
-// Use [NSWindow toggleToolbarShown:]
+
+- (NSWindow*) _window
+{
+  NSWindow *window = [_toolbarView window];
+  NSEnumerator *wenum;
+
+  if (window)
+    return window;
+
+  wenum = [GSAllWindows() objectEnumerator];
+  while ((window = [wenum nextObject]))
+    {
+      if ([window toolbar] == self)
+        return window;
+    }
+
+  return nil;
+}
+
 - (void) _setVisible: (BOOL)shown broadcast: (BOOL)broadcast
 {
   if (_visible != shown)
     {  
        _visible = shown; 
-    
+
+       if (shown)
+         [self _build];
+
+       if (shown)
+         {
+           if ((_toolbarView == nil) || ([_toolbarView superview] == nil))
+             {
+               NSWindow *w = [self _window];
+
+               [(id)[w _windowView] addToolbarView: [self _toolbarView]];
+             }
+         }
+       else
+         {
+           if ((_toolbarView != nil) && ([_toolbarView superview] != nil))
+             {
+               NSWindow *w = [self _window];
+
+               [(id)[w _windowView] removeToolbarView: [self _toolbarView]];
+             }
+         }
+
        if (broadcast) 
          {
            TRANSMIT(_setVisible: _visible broadcast: NO);
