@@ -26,17 +26,18 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSString.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSString.h>
 
-#include "AppKit/NSNibLoading.h"
-#include "AppKit/NSPanel.h"
-#include "AppKit/NSWindowController.h"
-#include "NSDocumentFrameworkPrivate.h"
+#import "AppKit/NSNib.h"
+#import "AppKit/NSNibLoading.h"
+#import "AppKit/NSPanel.h"
+#import "AppKit/NSWindowController.h"
+#import "NSDocumentFrameworkPrivate.h"
 
 @implementation NSWindowController
 
@@ -261,6 +262,7 @@
 	 nothing to do here. */
       if ([_window isReleasedWhenClosed])
 	{
+	  RETAIN(self);
 	  if ([_window delegate] == self) 
 	    {
 	      [_window setDelegate: nil];
@@ -281,6 +283,7 @@
 	  [self setWindow: nil];
 
 	  [_document _removeWindowController: self];
+	  AUTORELEASE(self);
 	}
     }
 }
@@ -299,13 +302,15 @@
 	}
 
       [self loadWindow];
-
-      [self _windowDidLoad];
-      if ([_document respondsToSelector:
-		       @selector(windowControllerDidLoadNib:)])
-	{
-	  [_document windowControllerDidLoadNib:self];
-	}
+      if ([self isWindowLoaded]) 
+      {
+        [self _windowDidLoad];
+        if ([_document respondsToSelector:
+	  	       @selector(windowControllerDidLoadNib:)])
+        {
+          [_document windowControllerDidLoadNib:self];
+        }
+      }
     }
 
   return _window;
@@ -397,7 +402,11 @@
       NSString *title = [self windowTitleForDocumentDisplayName: displayName];
 
       /* If they just want to display the filename, use the fancy method */
-      if (filename != nil && [title isEqualToString: filename])
+      /* NB For compatibility with Mac OS X, a document display name is equal
+         to its last path component, so we check for that here too */
+      if (filename != nil &&
+	  ([title isEqualToString: filename] ||
+	   [title isEqualToString: [filename lastPathComponent]]))
         {
           [_window setTitleWithRepresentedFilename: filename];
         }
@@ -462,8 +471,7 @@
       return;
     }
 
-  table = [NSDictionary dictionaryWithObject: _owner forKey: @"NSOwner"];
-  
+  table = [NSDictionary dictionaryWithObject: _owner forKey: NSNibOwner];
   if ([NSBundle loadNibFile: [self windowNibPath]
 		externalNameTable: table
 		withZone: [_owner zone]])
