@@ -51,7 +51,7 @@
 #include <Foundation/NSPathUtilities.h>
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSTask.h>
-#include <GNUstepBase/NSTask+GS.h>
+#include <GNUstepBase/NSTask+GNUstepBase.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSNotificationQueue.h>
@@ -72,6 +72,7 @@
 #include "AppKit/NSScreen.h"
 #include "GNUstepGUI/GSServicesManager.h"
 #include "GNUstepGUI/GSDisplayServer.h"
+#include "GSGuiPrivate.h"
 
 /* Informal protocol for method to ask an app to open a URL.
  */
@@ -1448,6 +1449,17 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   return YES;
 }
 
+- (BOOL) launchAppWithBundleIdentifier: (NSString *)bundleIdentifier
+			       options: (NSWorkspaceLaunchOptions)options 
+	additionalEventParamDescriptor: (NSAppleEventDescriptor *)descriptor 
+		      launchIdentifier: (NSNumber **)identifier 
+{
+  // TODO: full implementation
+  return [self launchApplication: bundleIdentifier
+			showIcon: YES
+		      autolaunch: NO];
+}
+
 /**
  * Returns a description of the currently active application, containing
  * the name (NSApplicationName), path (NSApplicationPath) and process
@@ -1478,8 +1490,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 /**
  * Returns an array listing all the applications known to have been
  * launched.  Each entry in the array is a dictionary providing
- * the name, path and process identfier of an application.<br />
- * NB. The contents of this array are not guaranteed to be up to date.
+ * the name, path and process identfier of an application.
  */
 - (NSArray*) launchedApplications
 {
@@ -1632,8 +1643,8 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 - (NSArray*) mountedLocalVolumePaths
 {
   NSMutableArray	*names;
-  NSFileManager		*mgr = [NSFileManager defaultManager];
 #if	defined(__MINGW32__)
+  NSFileManager		*mgr = [NSFileManager defaultManager];
   unsigned		max = BUFSIZ;
   unichar		buf[max];
   unichar		*base = buf;
@@ -1668,6 +1679,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
     }
 
 #elif	defined(HAVE_GETMNTENT) && defined (MNT_MEMB)
+  NSFileManager		*mgr = [NSFileManager defaultManager];
   FILE		*fptr = fopen("/etc/mtab", "r");
   struct mntent	*m;
 
@@ -2136,10 +2148,29 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 {
   NSDictionary	*typeInfo = [extInfo objectForKey: appName];
   NSString	*file = [typeInfo objectForKey: @"NSIcon"];
-
+  
+  //
+  // If the NSIcon entry isn't there and the CFBundle entries are,
+  // get the first icon in the list if it's an array, or assign
+  // the icon to file if it's a string.
+  //
+  // FIXME: CFBundleTypeExtensions/IconFile can be arrays which assign
+  // multiple types to icons.  This needs to be handled eventually.
+  //
   if(file == nil)
     {
-      file = [typeInfo objectForKey: @"CFBundleTypeIconFile"];
+      id icon = [typeInfo objectForKey: @"CFBundleTypeIconFile"];
+      if([icon isKindOfClass: [NSArray class]])
+	{
+	  if([icon count])
+	    {
+	      file = [icon objectAtIndex: 0];
+	    }
+	}
+      else
+	{
+	  file = icon;
+	}
     }
 
   if (file && [file length] != 0)
@@ -2556,8 +2587,8 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 
 - (id) _connectApplication: (NSString*)appName
 {
-  NSTimeInterval        replyTimeout;
-  NSTimeInterval        requestTimeout;
+  NSTimeInterval        replyTimeout = 0.0;
+  NSTimeInterval        requestTimeout = 0.0;
   NSString	*host;
   NSString	*port;
   NSDate	*when = nil;
