@@ -76,6 +76,13 @@ NSString *NSWordTablesWriteException = @"WordTablesWrite";
 
 NSString *GSWindowServerInternalException = @"WindowServerInternal";
 
+// NSAnimation
+NSString* NSAnimationProgressMarkNotification
+= @"NSAnimationProgressMarkNotification";
+NSString *NSAnimationProgressMark = @"NSAnimationProgressMark";
+NSString *NSAnimationTriggerOrderIn = @"NSAnimationTriggerOrderIn"; 
+NSString *NSAnimationTriggerOrderOut = @"NSAnimationTriggerOrderOut"; 
+
 // Application notifications
 NSString *NSApplicationDidBecomeActiveNotification
               = @"ApplicationDidBecomeActive";
@@ -384,6 +391,15 @@ NSString *NSViewBoundsDidChangeNotification
 NSString *NSViewGlobalFrameDidChangeNotification
     = @"NSViewGlobalFrameDidChangeNotification";
 
+// NSViewAnimation 
+NSString *NSViewAnimationTargetKey     = @"NSViewAnimationTargetKey";
+NSString *NSViewAnimationStartFrameKey = @"NSViewAnimationStartFrameKey";
+NSString *NSViewAnimationEndFrameKey   = @"NSViewAnimationEndFrameKey";
+NSString *NSViewAnimationEffectKey     = @"NSViewAnimationEffectKey";
+NSString *NSViewAnimationFadeInEffect  = @"NSViewAnimationFadeInEffect";
+NSString *NSViewAnimationFadeOutEffect = @"NSViewAnimationFadeOutEffect";
+
+
 // NSMenu notifications
 NSString* const NSMenuDidSendActionNotification = @"MenuDidSendAction";
 NSString* const NSMenuWillSendActionNotification = @"MenuWillSendAction";
@@ -567,7 +583,7 @@ NSString *NSToolbarPrintItemIdentifier = @"NSToolbarPrintItemIdentifier";
 NSString *NSOldSelectedCharacterRange = @"NSOldSelectedCharacterRange";
 
 /* NSFont matrix */
-const float NSFontIdentityMatrix[] = {1, 0, 0, 1, 0, 0};
+const CGFloat NSFontIdentityMatrix[] = {1, 0, 0, 1, 0, 0};
 
 /* Drawing engine externs */
 NSString *NSBackendContext = @"NSBackendContext";
@@ -626,12 +642,14 @@ NSString *NSValueTransformerNameBindingOption = @"NSValueTransformerName";
 NSString *NSValueTransformerBindingOption = @"NSValueTransformer";
  
 NSString *NSAlignmentBinding = @"alignment";
+NSString *NSContentObjectBinding = @"contentObject";
 NSString *NSEditableBinding = @"editable";
 NSString *NSEnabledBinding = @"enabled";
 NSString *NSFontBinding = @"font";
 NSString *NSHiddenBinding = @"hidden";
 NSString *NSSelectedIndexBinding = @"selectedIndex";
 NSString *NSTextColorBinding = @"textColor";
+NSString *NSTitleBinding = @"title";
 NSString *NSToolTipBinding = @"toolTip";
 NSString *NSValueBinding = @"value";
 
@@ -641,6 +659,10 @@ id NSNoSelectionMarker = @"NSNoSelectionMarker";
 id NSNotApplicableMarker = @"NSNotApplicableMarker";
 
 
+// NSNib
+NSString *NSNibTopLevelObjects = @"NSTopLevelObjects";
+NSString *NSNibOwner = @"NSOwner";
+
 extern void __objc_gui_force_linking (void);
 
 void
