@@ -34,24 +34,23 @@
 
 /*  we have to satisfy the scanner with a stream reading function */
 typedef struct {
-  NSString	*string;
+  const char	*string;
   int		position;
   int		length;
 } StringContext;
 
 static void	
-initStringContext (StringContext *ctxt, NSString *string)
+initStringContext (StringContext *ctxt, NSData *data)
 {
-  ctxt->string = string;
+  ctxt->string = [data bytes];
   ctxt->position = 0;
-  ctxt->length = [string length];
+  ctxt->length = [data length];
 }
 
 static int	
-readNSString (StringContext *ctxt)
+readString (StringContext *ctxt)
 {
-  return (ctxt->position < ctxt->length )
-    ? [ctxt->string characterAtIndex:ctxt->position++]: EOF;
+  return ctxt->position < ctxt->length ? ctxt->string[ctxt->position++] : EOF;
 }
 
 // Hold the attributs of the current run
@@ -279,22 +278,30 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 
   if (flag)
     {
-      if ([format isEqual: @"RTFD"])
+      if (([format isEqual: NSRTFDTextDocumentType]) ||
+          ([format isEqual: @"com.apple.rtfd"]) ||
+          ([format isEqual: @"rtfd"]))
 	{
 	  cClass = [RTFDProducer class];
 	}
-      else if ([format isEqual: @"RTF"])
+      else if (([format isEqual: NSRTFTextDocumentType]) ||
+               ([format isEqual: @"public.rtf"]) ||
+               ([format isEqual: @"rtf"]))
 	{
 	  cClass = [RTFProducer class];
 	}
     }
   else
     {
-      if ([format isEqual: @"RTFD"])
+      if (([format isEqual: NSRTFDTextDocumentType]) ||
+          ([format isEqual: @"com.apple.rtfd"]) ||
+          ([format isEqual: @"rtfd"]))
 	{
 	  cClass = [RTFDConsumer class];
 	}
-      else if ([format isEqual: @"RTF"])
+      else if (([format isEqual: NSRTFTextDocumentType]) ||
+               ([format isEqual: @"public.rtf"]) ||
+               ([format isEqual: @"rtf"]))
 	{
 	  cClass = [RTFConsumer class];
 	}
@@ -303,7 +310,9 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 }
 
 + (NSAttributedString*) parseFile: (NSFileWrapper *)wrapper
+                          options: (NSDictionary *)options
 	       documentAttributes: (NSDictionary **)dict
+                            error: (NSError **)error
 			    class: (Class)class
 {
   NSAttributedString *text = nil;
@@ -337,7 +346,9 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 }
 
 + (NSAttributedString*) parseData: (NSData *)rtfData 
+                          options: (NSDictionary *)options
 	       documentAttributes: (NSDictionary **)dict
+                            error: (NSError **)error
 			    class: (Class)class
 {
   RTFConsumer *consumer = [RTFConsumer new];
@@ -355,6 +366,7 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 {
   ignore = 0;  
   result = nil;
+  encoding = NSISOLatin1StringEncoding;
   documentAttributes = nil;
   fonts = nil;
   attrs = nil;
@@ -406,14 +418,20 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 }
 
 + (NSAttributedString*) parseData: (NSData *)rtfData 
+                          options: (NSDictionary *)options
 	       documentAttributes: (NSDictionary **)dict
+                            error: (NSError **)error
 			    class: (Class)class
 {
   NSAttributedString *str;
   NSFileWrapper *wrapper = [[NSFileWrapper alloc] 
 			     initWithSerializedRepresentation: rtfData];
   
-  str = [self parseFile: wrapper  documentAttributes: dict  class: class];
+  str = [self parseFile: wrapper 
+              options: options 
+              documentAttributes: dict 
+              error: error 
+              class: class];
   RELEASE (wrapper);
 
   return str;
@@ -433,13 +451,21 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
       if (wrapper != nil)
         {
           NSImage* image = [[NSImage alloc] initWithData: [wrapper regularFileContents]];
-          NSTextAttachmentCell* attachedCell = [[NSTextAttachmentCell alloc] initImageCell: image];
-          NSTextAttachment* attachment = [[NSTextAttachment alloc] initWithFileWrapper: wrapper];
+          NSTextAttachment* attachment;
           RTFAttribute* attr = [self attr];
           NSMutableDictionary* attributes = nil;
           NSMutableAttributedString* str = nil;
 
-          [attachment setAttachmentCell: attachedCell];
+          if (image != nil)
+            {
+              [wrapper setIcon: image];
+            }
+          attachment = [[NSTextAttachment alloc] initWithFileWrapper: wrapper];
+          if (attachment == nil)
+            {
+              NSLog(@"No attachment at %d", oldPosition);
+              return;
+            }
         
           attributes = [[NSMutableDictionary alloc]
 			 initWithObjectsAndKeys:
@@ -453,10 +479,9 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
           [str addAttributes: attributes range: NSMakeRange (0, [str length])];
           
           [result replaceCharactersInRange: insertionRange withAttributedString: str];
-          
+          attr->changed = YES;
           RELEASE(attributes);
           RELEASE(attachment);
-          RELEASE(attachedCell);
           RELEASE(image);
         }
     }
@@ -521,18 +546,42 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
   CREATE_AUTORELEASE_POOL(pool);
   RTFscannerCtxt scanner;
   StringContext stringCtxt;
-  // We should read in the first few characters to find out which
+  char buffer[5];
+
+  // We read in the first few characters to find out which
   // encoding we have
-  NSString *rtfString = [[NSString alloc] 
-			  initWithData: rtfData
-			  encoding: NSASCIIStringEncoding];
+  if ([rtfData length] < 10)
+    {
+      // Too short to be an RTF
+      return nil;
+    }
+  [rtfData getBytes: buffer range: NSMakeRange(7, 3)];
+  if (strncmp(buffer, "mac", 3) == 0)
+    {
+      encoding = NSMacOSRomanStringEncoding;
+    }
+  else if (strncmp(buffer, "pc", 2) == 0)
+    {
+      // FIXME: Code page 437 kCFStringEncodingDOSLatinUS
+      encoding = NSISOLatin1StringEncoding;
+    }
+  else if (strncmp(buffer, "pca", 3) == 0)
+    {
+      // FIXME: Code page 850 kCFStringEncodingDOSLatin1
+      encoding = NSISOLatin1StringEncoding;
+    }
+  else
+    {
+      encoding = NSISOLatin1StringEncoding;
+    }
+
 
   // Reset this RFTConsumer, as it might already have been used!
   _class = class;
   [self reset];
 
-  initStringContext(&stringCtxt, rtfString);
-  lexInitContext(&scanner, &stringCtxt, (int (*)(void*))readNSString);
+  initStringContext(&stringCtxt, rtfData);
+  lexInitContext(&scanner, &stringCtxt, (int (*)(void*))readString);
   [result beginEditing];
   NS_DURING
     GSRTFparse((void *)self, &scanner);
@@ -543,7 +592,6 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
   NS_ENDHANDLER
   [result endEditing];
 
-  RELEASE(rtfString);
   RELEASE(pool);
   // document attributes
   if (dict)
@@ -623,6 +671,7 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 #define IGNORE	((RTFConsumer *)ctxt)->ignore
 #define TEXTPOSITION [RESULT length]
 #define DOCUMENTATTRIBUTES ((RTFConsumer*)ctxt)->documentAttributes
+#define ENCODING ((RTFConsumer *)ctxt)->encoding
 
 #define FILES ((RTFDConsumer*)ctxt)->files
 
@@ -706,7 +755,7 @@ void GSRTFmangleText (void *ctxt, const char *text)
   NSData *data = [[NSData alloc] initWithBytes: (void*)text 
 				 length: strlen(text)];
   NSString *str = [[NSString alloc] initWithData: data
-				    encoding: NSISOLatin1StringEncoding];
+				    encoding: ENCODING];
 
   [(RTFConsumer *)ctxt appendString: str];
   DESTROY(str);
@@ -715,11 +764,15 @@ void GSRTFmangleText (void *ctxt, const char *text)
 
 void GSRTFunicode (void *ctxt, int uchar)
 {
-  unichar chars = uchar;
-  NSString *str = [[NSString alloc] initWithCharacters: &chars 
-				    length: 1];
-  [(RTFConsumer *)ctxt appendString: str];
-  DESTROY(str);
+  // Don't add the attachment character, this gets handled separatly
+  if (uchar != (int)NSAttachmentCharacter)
+    {
+      unichar chars = uchar;
+      NSString *str = [[NSString alloc] initWithCharacters: &chars 
+                                                    length: 1];
+      [(RTFConsumer *)ctxt appendString: str];
+      DESTROY(str);
+    }
 }
 
 void GSRTFregisterFont (void *ctxt, const char *fontName, 
